/*
 * apm.c:
 *      this contains the XFolder APM interface for
 *      automatically turning the computer off after
 *      shutdown has completed. You must fill in
 *      something meaningful into the following
 *      three functions here, or APM power-off will
 *      not work.
 *
 *      Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define INCL_DOS
#define INCL_DOSDEVICES
#define INCL_DOSDEVIOCTL
#include <os2.h>

#include "string.h"
#include "apm.h"

/*
 * apmQueryVersion:
 *      this function should return a PSZ to an APM
 *      version string, e.g. "1.1"
 */

CHAR szAPMVersion[10];

PSZ apmQueryVersion(VOID)
{
    strcpy(szAPMVersion, "?.?");
    return (szAPMVersion);
}

/*
 * apmPowerOffSupported:
 *      this function must return TRUE if the computer
 *      supports the APM V1.2 power-off function. This
 *      is called by the "XDesktop" page in the Desktop's
 *      settings notebook to determine if the "APM power
 *      off" checkbox will be enabled or not (i.e. cannot
 *      be selected).
 */

BOOL apmPowerOffSupported(VOID) {
    // note: this function can be called several times,
    // so make sure you leave no open files here
    return (FALSE);
}

/*
 * apmPreparePowerOff:
 *      this function is called _once_ by XShutdown while
 *      the system is being shut down, if apmPowerOff
 *      above returned TRUE. This call happens
 *      after all windows have been closed, but
 *      before DosShutdown is called, so you may
 *      open the APM.SYS device driver here and
 *      do other preparations you might need.
 *      After this, you must return _one_ of the
 *      following flags:
 *          APM_OK   go ahead with XShutdown and call
 *                   apmDoPowerOff later.
 *          APM_IGNORE go ahead with XShutdown and do
 *                   _not_ call apmDoPowerOff later;
 *                   this will lead to the normal
 *                   "Press Ctrl+Alt+Del" window
 *          APM_CANCEL cancel shutdown and present the
 *                   error message which you must
 *                   then copy to pszError, which
 *                   points to a buffer 500 bytes in
 *                   size
 *      ORed with _one_ or _none_ of the following:
 *          APM_DOSSHUTDOWN_0
 *                   set this flag if XFolder should
 *                   call DosShutdown(0); this is
 *                   recommended
 *          APM_DOSSHUTDOWN_1
 *                   the same for DosShutdown(1)
 *      If you return APM_OK only without either
 *      APM_DOSSHUTDOWN_0 or APM_DOSSHUTDOWN_1,
 *      XFolder will call apmDoPowerOff later,
 *      but without having called DosShutdown.
 *      You MUST do this yourself then.
 *      This will however prevent XFolder from
 *      presenting the proper informational windows
 *      to the user.
 *      The buffer that pszError points to is only
 *      evaluated if you return APM_CANCEL.
 */

ULONG apmPreparePowerOff(PSZ pszError) {
    // example for error message; this should not be
    // displayed, because we returned FALSE for
    // apmPowerOffSupported
    strcpy(pszError, "Sorry, APM power-off is not yet implemented.");
    return (APM_CANCEL);

    // instead, you could do the following:
    // declare
    //  HFILE hfAPMSys = NULLHANDLE
    // as a global variable and open APM.SYS here,
    // then return (APM_OK | APM_DOSSHUTDOWN_0)
}

/*
 * apmDoPowerOff:
 *      if you have set the APM_OK flag above,
 *      XFolder calls this function after it is
 *      done with XShutdown. In this function,
 *      you should call the APM function which
 *      turns off the computer's power. If you
 *      have not specified one of the APM_DOSSHUTDOWN
 *      flags above, you must call DosShutdown
 *      yourself here.
 *      You must then return from this function
 *      to let XFolder block the system, in case
 *      power-off didn't work.
 */

VOID apmDoPowerOff(VOID) {
    // call APM.SYS func
    DosBeep(1000, 1000);
    DosExit(EXIT_PROCESS, 0);
}

