/*
 * helpers.h:
 *      header file for helpers.c.
 *      helpers.c contains helper functions that are independent
 *      of a single application, i.e. you can use them in any
 *      program. Parts of these functions require PM, others
 *      the OS/2 kernel, others are just C helper routines.
 *
 *      #include this file to access helper functions.
 *
 *      Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define DebugBox(title, text) \
         WinMessageBox(HWND_DESKTOP, HWND_DESKTOP,   \
            (PSZ)text, (PSZ)title,    \
            0, MB_OK | MB_ICONEXCLAMATION | MB_MOVEABLE)

#define YesNoBox(title, text) \
         WinMessageBox(HWND_DESKTOP, HWND_DESKTOP,   \
            (PSZ)text, (PSZ)title,    \
            0, MB_YESNO | MB_ICONQUESTION | MB_MOVEABLE)

#define MPNULL                 (MPFROMP(NULL))
#define MPZERO                 (MPFROMSHORT(0))
#define MRTRUE                 (MRFROMSHORT((SHORT) TRUE))
#define MRFALSE                (MRFROMSHORT((SHORT) FALSE))
#define BM_UNCHECKED           0   // for checkboxes: disabled
#define BM_CHECKED             1   // for checkboses: enabled
#define BM_INDETERMINATE       2   // for tri-state checkboxes: indeterminate

#define WinSetDlgItemChecked(hwnd, id, bCheck) \
        WinSendDlgItemMsg(hwnd, id, BM_SETCHECK, MPFROMSHORT(bCheck), MPNULL)

#define WinIsDlgItemChecked(hwnd, id) \
        (SHORT1FROMMR(WinSendDlgItemMsg((hwnd), (id), BM_QUERYCHECK, MPNULL, MPNULL)))

#define WinSetMenuItemChecked(hwndMenu, usId, bCheck) \
        WinSendMsg(hwndMenu, MM_SETITEMATTR, MPFROM2SHORT(usId, TRUE), \
                MPFROM2SHORT(MIA_CHECKED, ((bCheck) ? MIA_CHECKED : FALSE)))

/* #define WinEnableMenuItem(hwndMenu, usId, bEnable) \
        WinSendMsg(hwndMenu, MM_SETITEMATTR, MPFROM2SHORT(usId, TRUE), \
                MPFROM2SHORT(MIA_DISABLED, ((!bCheck) ? MIA_DISABLED : FALSE))) */

#define WinShowDlgItem(hwnd, id, show) \
        WinShowWindow(WinWindowFromID(hwnd, id), show)

#define WinSetWindowFont(hwnd, szFont)\
        (WinSetPresParam(hwnd),\
        PP_FONTNAMESIZE, (ULONG)strlen(szFont)+1, szFont))

#define WinSetDlgItemFont(hwnd, usId, szFont)\
        (WinSetPresParam(WinWindowFromID(hwnd, usId),\
        PP_FONTNAMESIZE, (ULONG)strlen(szFont)+1, szFont))

#define WinEnableDlgItem(hwndDlg, ulId, Enable) \
        WinEnableWindow(WinWindowFromID(hwndDlg, ulId), Enable)

#define WinIsDlgItemEnabled(hwndDlg, ulId) \
        WinIsWindowEnabled(WinWindowFromID(hwndDlg, ulId))

#define WinSetDlgItemFocus(hwndDlg, ulId) \
        WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwndDlg, ulId))

#define WinQueryCnrInfo(hwndCnr, CnrInfo) \
        WinSendMsg(hwndCnr, CM_QUERYCNRINFO,  \
                (MPARAM)&CnrInfo,             \
                (MPARAM)sizeof(CnrInfo))

#define WinCnrSelectRecord(hwndCnr, precc) \
        WinSendMsg(hwndCnr, CM_SETRECORDEMPHASIS, \
                (MPARAM)precc, \
                MPFROM2SHORT(TRUE, CRA_SELECTED))

// provoke trap macro
#define CRASH {PSZ p = NULL; *p = 'a'; }

typedef struct LIST_STRUCT {
    struct LIST_STRUCT   *pNext, *pPrevious;
    ULONG                ulSize;
} LISTITEM, *PLISTITEM;

typedef struct
{
  BYTE bFlags;
  BYTE bNameLength;
  USHORT usValueLength;
  PSZ pszName;
  PSZ pszValue;
} EABINDING, *PEABINDING;

typedef struct _EALIST
{
  EABINDING * binding;
  struct _EALIST * next;
} EALIST, *PEALIST;

typedef struct MENUITEM2_STRUCT {
    USHORT  usStyle;    // menu item styles (MIS_*)
    USHORT  usAttribute;// Menu item attributes (MIA_*)
    USHORT  usItemId;   // application-provided identifier for the menu item
    PSZ     pszTitle;
    // Variable data:
    // following the identifier is a variable data structure
    // whose format depends upon the value of Style:
    //     MIS_TEXT
    //          Text (PSZ) Null-terminated text string
    //     MIS_SUBMENU
    //          A menu template structure.
    //     MIS_BITMAP
    //          Text (PCH)
    //              Null-terminated text string to find resource
} TEXTMENUITEM, *PTEXTMENUITEM;

/* Menu template structure (not defined by PM) */
typedef struct MENUTEMPLATE_STRUCT {
    USHORT  usLength;        // length of the structure
    USHORT  usVersion;      // template version; 0 and 1 are valid
    USHORT  usCodepage;     // identifier of the code page used for the text items
                            // within the menu (but not any submenus, which each have their own
                            // code pages
    USHORT  usItemOffset;   // offset of the items from the start of the template, in bytes
    USHORT  usItemCount;    // count of menu items
    USHORT  usPPOffset;     // Offset of presentation parameters from the start of the template,
                            // in bytes. This field is only present for version 1 of the template
    TEXTMENUITEM MenuItems[1]; // here comes a variable array of menu items
} MENUTEMPLATE, *PMENUTEMPLATE;

#define WM_UPDATEPROGRESSBAR    WM_USER+1000

/* flags for ulAttr */
#define PBA_NOPERCENTAGE        0x0000
#define PBA_ALIGNLEFT           0x0001
#define PBA_ALIGNRIGHT          0x0002
#define PBA_ALIGNCENTER         0x0003
#define PBA_PERCENTFLAGS        0x0003
#define PBA_BUTTONSTYLE         0x0010

/* thread info structure passed to secondary threads */
typedef struct _THREADINFO {
        /* initialized by main thread */
        ULONG   cb;
        HWND    hwndOwner;
        BOOL    fExit;
        /* data maintained by secondary thread */
        HAB     hab;
        TID     tid;
        BOOL    fExitComplete;
        ULONG   ulResult;
        ULONG   ulFuncInfo;
} THREADINFO, *PTHREADINFO;

/********************************************************************
 *                                                                  *
 *   Functions for maintaining linked pointer lists                 *
 *                                                                  *
 ********************************************************************/

/* The following functions are used commonly to maintain linked
   pointer lists: we have functions for appending, counting and
   removing list items, plus one for deleting a whole list.
   All these function names are introduced by "lst".

   Note that all these functions assume the passed pointers to
   be pointers to the class LISTITEM below. You may, of course,
   use different structures, but the first item in your structure
   must be a *pNext to make these functions work.

   All of the following functions work in two modes:
   -- "one-way" lists, i.e. list which can only be accessed through
      the first item on the list;
   -- "two-way" lists, i.e. list which can be accessed both through
      the first item and the last item on the list; in this case,
      you must also specify *ppLast, the pointer of the last item,
      which will also be maintained by these functions. "Two-way"
      lists are much faster when appending many items, because we
      don't have to go through the whole list to find the last item.

   You will have to cast your structure pointers manually
   to PLISTITEM before using these functions.

   All of the following functions support mutex semaphores to make
   them thread-safe. You can pass an existing mutex semaphore (of
   which you created one for each maintained list) in hmtx and
   specify the timeout in ulTimeout; if your list will only be used
   by one thread, simply pass 0, 0, and no semaphores will be used.  */

/*
 * lstAppendItem:
 *      this will create an item for a linked list starting at
 *      *ppFirst; this must be a pointer to a pointer to the
 *      first list item; if *ppFirst is NULL (= list is empty),
 *      *ppFirst will be adjusted so that it points to the new
 *      item. If ppLast == NULL, the list is assumed to be
 *      a "one-way" list, otherwise it must point to a pointer
 *      for the last list item, which will be adjusted.
 *      This function returns the address of the new list item.
 */

PLISTITEM lstAppendItem(PLISTITEM *ppFirst, // first list item
            PLISTITEM *ppLast,              // last list item (or NULL)
            PLISTITEM pNewItem,             // item to insert
            HMTX hmtx,                      // mutex semaphore for this list
            ULONG ulTimeout);               // mutex timeout in ms

/*
 * lstInsertItemAt:
 *      does just the same, but does not append the new item
 *      at the end of the list, but BEFORE the specified index
 *      lIndex. If lIndex is larger than the number of items
 *      on the list or == -1, the new item will be appended.
 */

PLISTITEM lstInsertItemAt(PLISTITEM *ppFirst, PLISTITEM *ppLast,
            PLISTITEM pNewItem, LONG lIndex, HMTX hmtx, ULONG ulTimeout);

/*
 * lstRemoveItem:
 *      reversely, this will remove a given pRemoveItem from
 *      the linked list beginning at *ppFirst. Returns TRUE if
 *      successful, FALSE if pRemoveItem was not found.
 */

BOOL lstRemoveItem(PLISTITEM *ppFirst, PLISTITEM *ppLast, PLISTITEM pRemoveItem, HMTX hmtx, ULONG ulTimeout);

/*
 * lstSwapItems:
 *      this will swap the items ppItem1 and ppItem2 in the linked
 *      list starting at *ppFirst; useful for sorting.
 *      Note that this function will not only adjust the pNext
 *      and pPrevious pointers of the LISTITEMs, but also
 *      swap the *ppItem1 and *ppItem2 pointers.
 */

BOOL lstSwapItems(PLISTITEM *ppFirst, PLISTITEM *ppLast,
            PLISTITEM *ppItem1, PLISTITEM *ppItem2,
            HMTX hmtx, ULONG ulTimeout);

typedef SHORT (EXPENTRY FNSORTLIST)(PVOID, PVOID, PVOID);
typedef FNSORTLIST *PFNSORTLIST;

/*
 * lstQuickSort:
 *      this will sort the list starting at *ppFirst using the
 *      sort function pfnSort, which works similar to those of the
 *      container sorts, i.e. it must be declared as a
 *          SHORT EXPENTRY fnSort(PVOID pItem1, PVOID pItem1, PVOID pStorage)
 *      These sort functions need to return the following:
 *          0   pItem1 == pItem2
 *         -1   pItem1 <  pItem2
 *         +1   pItem1 >  pItem2
 *      This function uses the "quick sort" algorithm, which is one of
 *      the fastest sort algorithms available. However, this is an
 *      "instable" sort algorithm, meaning that list items considered
 *      equal by pfnSort do _not_ retain their position, but might be
 *      changed. If you need these to remain where they are, use
 *      lstBubbleSort.
 */

BOOL lstQuickSort(PLISTITEM *ppFirst, PLISTITEM *ppLast,
            PFNSORTLIST pfnSort, PVOID pStorage,
            HMTX hmtx, ULONG ulTimeout);

/*
 * lstBubbleSort:
 *      this will sort the list starting at *ppFirst using the
 *      sort function pfnSort, which works similar to those of the
 *      container sorts, i.e. it must be declared as a
 *          SHORT EXPENTRY fnSort(PVOID pItem1, PVOID pItem1, PVOID pStorage)
 *      These sort functions need to return the following:
 *          0   pItem1 == pItem2
 *         -1   pItem1 <  pItem2
 *         +1   pItem1 >  pItem2
 *      This function uses the "bubble sort" algorithm, which will cause a
 *      lot of calls to pfnSort and which is really ineffective for lists
 *      with more than 100 items.
 */

BOOL lstBubbleSort(PLISTITEM *ppFirst, PLISTITEM *ppLast,
            PFNSORTLIST pfnSort, PVOID pStorage,
            HMTX hmtx, ULONG ulTimeout);

/*
 * lstCountItems:
 *      this will count the number of items from pFirst;
 *      returns 0 if error occured.
 */

ULONG lstCountItems(PLISTITEM pFirst, HMTX hmtx, ULONG ulTimeout);

/*
 * lstItemFromIndex:
 *      returns the list item with a given ulIndex
 *      (counting from 0), or NULL if ulIndex is too large
 */

PLISTITEM lstItemFromIndex(PLISTITEM pFirst, ULONG ulIndex, HMTX hmtx, ULONG ulTimeout);

/*
 * lstClear:
 *      this will free the whole list starting
 *      at *ppFirst and set *ppFirst/*ppLast to NULL
 */

void lstClear(PLISTITEM *ppFirst, PLISTITEM *ppLast, HMTX hmtx, ULONG ulTimeout);

/********************************************************************
 *                                                                  *
 *   Functions for synchronizing threads                            *
 *                                                                  *
 ********************************************************************/

/*
 * thrCreate:
 *      this function creates a THREADINFO structure in *ppti;
 *      you must pass the thread function in pfn, which will
 *      then be executed; specification of hwndOwner is mandatory
 */

BOOL thrCreate(PTHREADINFO *ppti, PFNTHREAD pfn, HWND hwndOwner);

/*
 * thrClose:
 *      this functions sets a thread's "Close" flag to
 *      TRUE; the thread should monitor this flag
 *      periodically and then terminate itself
 */

BOOL thrClose(PTHREADINFO pti);

/*
 * thrWait:
 *      this function waits for a thread to end
 */

BOOL thrWait(PTHREADINFO pti);

/*
 * thrFree:
 *      this is a combination of thrClose and
 *      thrWait; the THREADINFO block is then freed
 *      and *ppti set to NULL
 */

BOOL thrFree(PTHREADINFO *ppti);

/*
 * thrKill:
 *      just like thrFree, but the thread is
 *      brutally killed
 */

BOOL thrKill(PTHREADINFO *ppti);

/*
 * thrQueryID:
 *      returns thread ID
 */

TID thrQueryID(PTHREADINFO pti);

/********************************************************************
 *                                                                  *
 *   String functions                                               *
 *                                                                  *
 ********************************************************************/

/*
 * strhexcat:
 *      this converts a ULONG into a hex string
 *      and appends the result to the buffer pszTarget
 */

void strhexcat(PSZ pszTarget, ULONG ul);

/*
 * strrpl:
 *      replace oldStr by newStr in str.
 *
 *      str should have enough allocated space for the replacement, no check
 *      is made for this. str and OldStr/NewStr should not overlap.
 *      The empty string ("") is found at the beginning of every string.
 *
 *      Returns: pointer to first location behind where NewStr was inserted
 *      or NULL if OldStr was not found.
 *      This is useful for multiple replacements also.
 *      (be careful not to replace the empty string this way !)
 *
 *      Author:     Gilles Kohl
 *      Started:    09.06.1992   12:16:47
 *      Modified:   09.06.1992   12:41:41
 *      Subject:    Replace one string by another in a given buffer.
 *                  This code is public domain. Use freely.
 */

char* strrpl(char* str, char* oldStr, char* newStr);

/*
 * strinsert:
 *      inserts to_insert into in_str at position place and
 *      copies the result to out_str
 *
 *      Author:     Gilles Kohl
 *      Started:    09.06.1992   12:16:47
 *      Modified:   09.06.1992   12:41:41
 *      Subject:    Replace one string by another in a given buffer.
 *                  This code is public domain. Use freely.
 */

char* strinsert(char* out_str, char* in_str, char* to_insert, int place);

/*
 * stristr:
 *      like strstr, but case-insensitive.
 */

PSZ stristr(PSZ string1, PSZ string2);

/*
 * strths:
 *      converts a ULONG into a decimal string, while
 *      inserting thousands separators into it. Specify
 *      the separator char in cThousands.
 *      Returns pszTarget so you can use it directly
 *      with sprintf and the "%s" flag.
 */

CHAR* strths(PSZ pszTarget, ULONG ul, CHAR cThousands);

/*
 * strFileDate:
 *      converts FILEFINDBUF4 date data to a string (to pszBuf).
 *      ulDateFormat can be queried using
 *              PrfQueryProfileInt(HINI_USER, "PM_National", "iDate", 0);
 *      meaning:
 *          0   mm.dd.yyyy  (English)
 *          1   dd.mm.yyyy  (German)
 *          2   yyyy.mm.dd  (Japanese)
 *          3   yyyy.dd.mm
 *      cDateSep is used as a date separator (e.g. '.').
 *              PrfQueryProfileString(HINI_USER, "PM_National", "sDate", "/",
 *                  szDateSep, sizeof(szDateSep)-1);
 */

VOID strFileDate(PSZ pszBuf, FILEFINDBUF4 *pffb4, ULONG ulDateFormat, CHAR cDateSep);

/*
 * strFileTime:
 *      converts FILEFINDBUF4 time data to a string (to pszBuf).
 *      ulTimeFormat can be queried using
 *              PrfQueryProfileInt(HINI_USER, "PM_National", "iTime", 0);
 *      meaning:
 *          0   12-hour clock
 *          >0  24-hour clock
 *      cDateSep is used as a date separator (e.g. ':').
 *              PrfQueryProfileString(HINI_USER, "PM_National", "sTime", ":",
 *                  szTimeSep, sizeof(szTimeSep)-1);
 */

VOID strFileTime(PSZ pszBuf, FILEFINDBUF4 *pffb4, ULONG ulTimeFormat, CHAR cTimeSep);

/*
 * strFindKey:
 *      finds pszKey in pszSearchIn; similar to stristr,
 *      but this one makes sure the key is at the beginning
 *      of a line. Spaces before the key are tolerated.
 *      Used by strGet/SetParameter
 */

PSZ strFindKey(PSZ pszSearchIn, PSZ pszKey);

/*
 * strGetParameter:
 *      searches pszSearchIn for the key pszKey; if found, it
 *      returns a pointer to the following characters in pszSearchIn
 *      and, if pszCopyTo != NULL, copies the rest of the line to
 *      that buffer, of which cbCopyTo specified the size.
 *      If the key is not found, NULL is returned.
 *      This is useful for querying CONFIG.SYS settings.
 */

PSZ strGetParameter(PSZ pszSearchIn, PSZ pszKey, PSZ pszCopyTo, ULONG cbCopyTo);

/*
 * strSetParameter:
 *      searches pszSearchIn for the key pszKey; if found, it
 *      replaces the characters following this key up to the
 *      end of the line by pszParam. If pszKey is not found in
 *      pszSearchIn, it is appended in a new line. You must
 *      ensure that *pszSearchin is large enough for the
 *      manipulation, i.e. it must be at least of the following
 *      size:
 *          strlen(pszSearchIn) + strlen(pszNewParam)
 *      in case the key was not found in the text.
 *      This function searches w/out case sensitivity.
 *      This function returns a pointer to the new parameter.
 */

PSZ strSetParameter(PSZ pszSearchIn, PSZ pszKey, PSZ pszNewParam);

/*
 * strDeleteLine:
 *      this deletes the line in pszSearchIn which starts with
 *      the key pszKey. Returns TRUE if the line was found and
 *      deleted.
 */

BOOL strDeleteLine(PSZ pszSearchIn, PSZ pszKey);

/*
 * strBeautifyTitle:
 *      replaces line breaks (0xd, 0xa) with spaces
 */

BOOL strBeautifyTitle(PSZ psz);

/********************************************************************
 *                                                                  *
 *   Common helper routines                                         *
 *                                                                  *
 ********************************************************************/

/*
 * UL2H:
 *      converts a ULONG into an 8-digit hex string
 */

void UL2H(PSZ szMsg, ULONG ul);

/*
 * DosGetULongTime:
 *      this returns the current time as a ULONG value (in milliseconds).
 *      Useful for stopping how much time the machine has spent in
 *      a certain function.
 */

ULONG DosGetULongTime(VOID);

/*
 * printfRtl:
 *      printf's rectangle information
 */

VOID printfRtl(PSZ psz, PRECTL prcl);

/*
 * DosQueryThreadPriority:
 *          returns the priority of the calling thread.
 *          The low byte of the low word is a hexadecimal value
 *          representing a rank (value 0 to 31) within a priority class.
 *          Class values, found in the high byte of the low word, are as follows:
 *              0x01  idle
 *              0x02  regular
 *              0x03  time-critical.
 *              0x04  server
 */

ULONG DosQueryThreadPriority(VOID);

/*
 * DosBrutallyKillProcess:
 *      this kills a process by using the new OS/2 "kill, kill"
 *      interface of Warp 3 FP17 or Warp 4; requires that
 *      XF86SUP.SYS be installed (from XFREE86/OS2)
 *
 *      (W) (C) Holger Veit.
 *      Taken from the XFree86/OS/2 packages.
 *      Public domain (I guess).
 */

APIRET DosBrutallyKillProcess(PID pid);

/*
 * DosQueryShiftState:
 *      returns TRUE if any of the SHIFT keys are
 *      currently pressed
 */

BOOL DosQueryShiftState(VOID);

/*
 * DosIsWarp4:
 *      returns TRUE if at least OS/2 Warp 4 is running, FALSE otherwise
 */

BOOL DosIsWarp4(VOID);

/*
 * DosQueryBootDrive:
 *      returns the letter of the boot drive
 */

CHAR DosQueryBootDrive(VOID);

/*
 * DosQueryFileSize:
 *      returns the size of an already opened file
 */

ULONG DosQueryFileSize(HFILE hFile);

/*
 * DosQueryPathSize:
 *      returns the size of any file
 */

ULONG DosQueryPathSize(PSZ pszFile);

/*
 * DosReadTextFile:
 *      reads a text file from disk, allocates memory
 *      via malloc() and returns a pointer to this
 *      buffer (or NULL upon errors). Specify in
 *      ulExtraMemory how much extra memory (in addition
 *      to the file's size) should be allocated to
 *      allow text manipulation.
 *      Returns NULL if an error occured.
 */

PSZ DosReadTextFile(PSZ pszFile, ULONG ulExtraMemory);

/*
 * DosCreateBackupFileName:
 *      creates a new PSZ (malloc) with a proper
 *      backup filename which does not exist yet.
 */

PSZ DosCreateBackupFileName(PSZ pszExisting);

/*
 * DosWriteTextFile:
 *      writes a text file to disk; pszFile must contain the
 *      whole path and filename.
 *      An existing file will be backed up if (fBackup).
 *      Returns the number of bytes written.
 */

ULONG DosWriteTextFile(PSZ pszFile, PSZ pszContent, BOOL fBackup);

/*
 * DosOpenLogFile:
 *      this opens a log file in the root directory of
 *      the boot drive; it is titled pszFilename, and
 *      the file handle is returned
 */

HFILE DosOpenLogFile(PSZ pszFileName);

/*
 * DosWriteToLogFile
 *      writes a string to a log file
 */

APIRET DosWriteToLogFile(HFILE hfLog, PSZ psz);

/*
 * DosQuickStartSession:
 *      shortcut to DosStartSession w/out having to deal
 *      with all the parameters; this one needs the following:
 *          PSZ pszPath     the executable (e.g. "F:\OS2\CMD.EXE")
 *          PSZ pszParams   parameters to pass (e.g. "/c blah.cmd")
 *          BOOL Visible    make program visible or not
 *          BOOL Wait       if TRUE: this function does not return until
 *                          the program has finished; DO NOT use this
 *                          on a PM main thread, or everything will block
 *          PULONG pulSID   session ID to be returned by DosStartSession
 *          PPID   ppid     process ID to be returned by DosStartSession
 */

APIRET DosQuickStartSession(PSZ pszPath, PSZ pszParams, BOOL Visible, BOOL Wait,
            PULONG pulSID, PPID ppid);

/*
 * DosSetPathAttr:
 *      sets the file attributes of pszFile,
 *      which must be fully qualified.
 *      fAttr can be:
 *              FILE_ARCHIVED
 *              FILE_READONLY
 *              FILE_SYSTEM
 *              FILE_HIDDEN
 */

APIRET DosSetPathAttr(PSZ pszFile, ULONG fAttr);

/********************************************************************
 *                                                                  *
 *   Extended Attribute handling                                    *
 *                                                                  *
 ********************************************************************/

/* All the following code dealing with Extended Attributes is
   written (w) by Chris Hanson (cph@zurich.ai.mit.edu)
   Copyright (c) 1995 Massachusetts Institute of Technology

   This material was developed by the Scheme project at the Massachusetts
   Institute of Technology, Department of Electrical Engineering and
   Computer Science.  Permission to copy this software, to redistribute
   it, and to use it for any purpose is granted, subject to the following
   restrictions and understandings.

   1. Any copy made of this software must include this copyright notice
   in full.

   2. Users of this software agree to make their best efforts (a) to
   return to the MIT Scheme project any improvements or extensions that
   they make, so that these may be included in future releases; and (b)
   to inform MIT of noteworthy uses of this software.

   3. All materials developed as a consequence of the use of this
   software shall duly acknowledge such use, in accordance with the usual
   standards of acknowledging credit in academic research.

   4. MIT has made no warrantee or representation that the operation of
   this software will be error-free, and MIT is under no obligation to
   provide any services, by way of maintenance, update, or otherwise.

   5. In conjunction with products arising from the use of this material,
   there shall be no use of the name of the Massachusetts Institute of
   Technology nor of any adaptation thereof in any advertising,
   promotional, or sales literature without prior written consent from
   MIT in each case. */

/* The following procedures deallocate the EA binding memory
   structures that are generated by the `read_...' procedures
   described above.  These procedures assume that `malloc' is used for
   allocation of all storage, and that the `name' and `value' fields
   of each binding are also allocated with `malloc'.  The `value'
   field may also be 0.  */

void EAFreeBinding(PEABINDING binding);

void EAFreeList(PEALIST list);

/*
 * EAHFReadAll:
 * Read all of the extended attributes, and return them as a list.
 * Returns 0 if there are no attributes.  The returned list of
 * attributes should be freed by calling `EAFreeList'.
 */

PEALIST EAHFileReadAll(HFILE file_handle);

PEALIST EAPathReadAll(PSZ path);

/*
 * EAHFReadOneByIndex:
 * Return the extended attribute specified by a given index.  The
 * index of the first attribute is 1.  Returns 0 if the index is
 * larger than the number of attributes attached to the file; in
 * particular, returns 0 for index 1 if there are no attributes.  The
 * returned binding should be freed by calling `EAFreeBinding'.
 */

PEABINDING   EAHFileReadOneByIndex(HFILE file_handle, ULONG index);

PEABINDING   EAPathReadOneByIndex(PSZ path, ULONG index);

/*
 * EAPReadOneByName:
 * Return the extended attribute specified by the given name.  If
 * there is no such attribute, returns 0.  The returned binding should
 * be freed by calling `EAFreeBinding'.
 */

PEABINDING   EAHFileReadOneByName(HFILE file_handle, PSZ name);

PEABINDING   EAPathReadOneByName(PSZ path, PSZ name);

/* The following procedures write extended attributes to a given file.
   A given attribute is deleted if its `usValueLength' is 0; in that
   case the `value' field is also permitted to be 0.  */

/*
 * EAHFWriteAll:
 * Write a list of extended attributes to a given file.  These
 * attributes add to any attributes already stored on the file.
 */

void EAHFileWriteAll(HFILE file_handle, PEALIST list);

void EAPathWriteAll(PSZ path, PEALIST list);

/*
 * EAHFWriteOne:
 * Write a single extended attribute to a given file.  This attribute
 * is added to any attributes already stored on the file.
 */

void EAHFileWriteOne(HFILE file_handle, PEABINDING   binding);

void EAPathWriteOne(PSZ path, PEABINDING   binding);

/*
 * DumbDlgProc:
 *      this is what the name says: a really dumb dlg proc
 */

MRESULT EXPENTRY DumbDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);

/*
 * PrfCopyProfile:
 *      this function copies a given profile into a
 *      new file. hOld is the handle, pszOld the filename
 *      of the profile to be copied (e.g. HINI_USERPROFILE
 *      and "?:\OS2\OS2.INI"; pszNew must point to a
 *      buffer which will contain the new filename ending
 *      in ".XFL" after PrfCopyProfile returns
 *      (e.g. "?:\OS2\OS2.XFL").
 *      You may specify a Callback procedure which gets
 *      called upon every copied application in the INI
 *      file; this way, you can provide a progress bar.
 *      fncbError gets called upon errors.
 */

BOOL PrfCopyProfile(HAB hab, HFILE hfLog,
                               HINI hOld, PSZ pszOld,
                               PSZ pszNew,
                               PFNWP fncbUpdate,
                               HWND hwnd, ULONG msg, ULONG ulCount, ULONG ulMax,
                               PFNWP fncbError, PULONG pulFunc2);

/*
 * PrfSaveINIs:
 *      this will enforce saving of OS2.INI and OS2SYS.INI
 *      by calling PrfCopyProfile (above) on them; the therefrom
 *      resulting ".XFL" files will replace the original
 *      ".INI" files.
 *      Specify fncbUpdate and fncbError like in PrfCopyProfile.
 */

APIRET PrfSaveINIs(HAB hab, HFILE hfLog,
        PFNWP fncbUpdate, HWND hwnd, ULONG msg,
        PFNWP fncbError, PULONG pulFunc2);

/*
 * PrfQueryColor:
 *      returns a system color in OS2.INI's PM_Colors as a LONG
 */

LONG PrfQueryColor(PSZ pszKeyName, PSZ pszDefault);

/*
 * WinInsertMenuItem:
 *      this inserts one one menu item into a given menu; parameters:
 *          HWND hwndMenu       the menu to add the submenu to
 *          ULONG iPosition     the index at which to insert
 *          SHORT sItemId       menu ID of the new item
 *          PSZ pszItemTitle    title of this item
 */

SHORT WinInsertMenuItem(HWND hwndMenu, SHORT iPosition,
            SHORT sItemId, PSZ pszItemTitle,
            SHORT afStyle, SHORT afAttr);

/*
 * WinInsertSubmenu:
 *      this inserts a submenu into a given menu and inserts one item
 *      into this new submenu; parameters:
 *          HWND hwndMenu       the menu to add the submenu to
 *          ULONG iPosition     the index at which to insert
 *          SHORT sMenuId       menu ID of the new menu
 *          PSZ pszSubmenuTitle title of the submenu
 *          USHORT afMenuStyle  style flags of the submenu; MIS_SUBMENU will
 *                              always be set by this func
 *          USHORT sItemId      menu ID of the first item in the new submenu
 *          PSZ pszItemTitle    title of this item
 *          USHORT afItemStyle  style flags of this item, e.g. MIS_TEXT
 *          USHORT afAttribute  attributes of this item, e.g. MIA_DISABLED
 *      Returns the HWND of the new submenu.
 */

HWND WinInsertSubmenu(HWND hwndMenu, ULONG iPosition,
                 SHORT sMenuId, PSZ pszSubmenuTitle, USHORT afMenuStyle,
                 SHORT sItemId, PSZ pszItemTitle, USHORT afItemStyle, USHORT afAttribute);

/*
 * WinInsertMenuSeparator:
 *      this inserts a separator into a given menu at
 *      the given position (which may be MIT_END);
 *      returns the position at which the item was
 *      inserted
 */

SHORT WinInsertMenuSeparator(HWND hMenu, SHORT iPosition, SHORT sId);

/*
 * WinMenuRemoveEllipse:
 *      removes a "..." substring from a menu item
 *      title, if found
 */

VOID WinMenuRemoveEllipse(HWND hwndMenu, USHORT usItemId);

/*
 * WinSetDlgItemSpinData:
 *          sets a spin button's data within a dialog window
 */

VOID WinSetDlgItemSpinData(HWND hwndDlg,
                           ULONG idSpinButton,
                           ULONG min,       // minimum allowed value
                           ULONG max,       // maximum allowed value
                           ULONG current);  // current value

/*
 * WinCnrScrollToRecord:
 *      scrolls a given container control to make a given
 *      record visible; returns TRUE if actual scrolling was
 *      performed, FALSE if not neccessary or on errors.
 *      Parameters:
 *          HWND hwndCnr     container window
 *          PRECORDCORE precc the record to scroll to; this can also
 *                           be a PMINIRECORDCORE, we don't care
 *          ULONG fsExtent   determines the record core coordinates to
 *                           take into account (ORed):
 *                           CMA_ICON    icon rectangle
 *                           CMA_TEXT    text rectangle
 *                           CMA_TREEICON "plus" sign in tree view
 *          BOOL KeepParent  for tree views only: keep parent record of
 *                           the record visible; in case pRec is not
 *                           currently visible, we will scroll so that
 *                           the parent record is at the top of the
 *                           visible records (Win95 style)
 *      Returns:
 *          0           scrolled
 *          >0          not scrolled for the following reason:
 *                  1       record rectangle query failed (error)
 *                  2       cnr viewport query failed (error)
 *                  3       record is already visible
 *                  4       cnrinfo query failed (error)
 *                  5       parent record rectangle query failed (error)
 *      This function an improved version of the one (W)(C) Dan Libby, found at
 *      http://zebra.asta.fh-weingarten.de/os2/Snippets/Howt6364.HTML
 *      Improvements (C) 1998 Ulrich Mller.
 */

ULONG WinCnrScrollToRecord(HWND hwndCnr, PRECORDCORE pRec, ULONG fsExtent, BOOL KeepParent);

/*
 * WinCnrAllocRecord:
 *      this is a shortcut to allocating memory for
 *      a container record. Returns the recc.
 *          ULONG cbrecc    total size of your record core
 *                          or sizeof(RECORDCORE), if you have
 *                          no additional recc data
 */

PRECORDCORE WinCnrAllocRecord(HWND hwndCnr, ULONG cbrecc);

/*
 * WinCnrInsertRecord:
 *      shortcut to inserting a previously allocated recc
 *      into a container. This func returns the recordcore.
 *      Params:
 *          HWND hwndCnr    the container window
 *          PRECORDCORE preccParent
 *                          for tree views: parent record
 *          PSZ pszText     text for record
 *          ULONG flRecordAttr
 *                  recc flags, as explained in pmstddlg.h:
 *                          CRA_SELECTED       record is selected
 *                          CRA_TARGET         record has target emphasis
 *                          CRA_CURSORED       cursor is on the record
 *                          CRA_INUSE          record has in-use emphasis
 *                          CRA_FILTERED       record has been filtered
 *                          CRA_DROPONABLE     record can be dropped on
 *                          CRA_RECORDREADONLY record is read-only
 *                          CRA_EXPANDED       record is expanded
 *                          CRA_COLLAPSED      record is collapsed
 *                          CRA_PICKED         record picked (Lazy Drag)
 *                  plus the following undocumented:
 *                          CRA_IGNORE
 *                          CRA_SOURCE
 *                          CRA_DISABLED
 *                          CRA_OWNERFREE
 *                          CRA_OWNERDRAW
 */

PRECORDCORE WinCnrInsertRecord(HWND hwndCnr, PRECORDCORE preccParent, PRECORDCORE precc,
            PSZ pszText, ULONG flRecordAttr);

/*
 * WinCenterWindow:
 *      centers a window within its parent window.
 *      The window should not be visible to avoid flickering.
 */

void WinCenterWindow(HWND hwnd);

/*
 * WinCenteredDlgBox:
 *      just like WinDlgBox, but the dlg box is centered on the screen;
 *      you should mark the dlg template as not visible in the dlg
 *      editor, or display will flicker
 */

ULONG WinCenteredDlgBox(HWND hwndParent, HWND hwndOwner,
              PFNWP pfnDlgProc, HMODULE hmod, ULONG idDlg, PVOID pCreateParams);

/*
 * WinQueryPresColor:
 *      returns the specified color. This is queried in the
 *      following order:
 *      1)  hwnd's pres params are searched for ulPP;
 *      2)  if this fails, PM_Colors in OS2.INI is searched
 *          for pszKeyName;
 *      3)  if this fails also, pszDefault is evaluated.
 *      The return value is always an RGB LONG.
 *      The following ulPP / pszKeyName pairs are useful:
 *          PP_FOREGROUNDCOLOR             "WindowText"
 *             Foreground color
 *          PP_BACKGROUNDCOLOR             "Window" or "DialogBackground"
 *             Background color
 *          PP_HILITEFOREGROUNDCOLOR       "HiliteForeground" or "MenuHiliteText"
 *             Highlighted foreground color, for example for selected menu
 *          PP_HILITEBACKGROUNDCOLOR       "HiliteBackground" or "MenuHilite"
 *             Highlighted background color
 *          PP_DISABLEDFOREGROUNDCOLOR
 *             Disabled foreground color
 *          PP_DISABLEDBACKGROUNDCOLOR
 *             Disabled background color
 *          PP_BORDERCOLOR                 "WindowFrame"
 *             Border color
 *          PP_ACTIVECOLOR
 *             Active color
 *          PP_INACTIVECOLOR
 *             Inactive color
 *          PP_ACTIVETEXTFGNDCOLOR
 *             Active text foreground color
 *          PP_ACTIVETEXTBGNDCOLOR
 *             Active text background color
 *          PP_INACTIVETEXTFGNDCOLOR
 *             Inactive text foreground color
 *          PP_INACTIVETEXTBGNDCOLOR
 *             Inactive text background color
 */

LONG WinQueryPresColor(HWND hwnd, ULONG ulPP, PSZ pszKeyName, PSZ pszDefault);

/*
 * WinCreateFakeDesktop:
 *      this routine creates and displays a frameless window over
 *      the whole screen to fool the user that all windows
 *      have been closed (which in fact might not be the
 *      case); the window background color is set to the Desktop's
 *      (PM's one, not the WPS's one); returns the hwnd of this window
 */

HWND WinCreateFakeDesktop(HWND hwndSibling);

/*
 * WinQueryCnrFromFrame:
 *         find the handle of a frame window's container; we do this
 *         by enumerating all the frame's child windows and comparing
 *         their window classes to the WC_CONTAINER code;
 *         returns NULLHANDLE if not found
 */

HWND WinQueryCnrFromFrame(HWND hwndFrame);

/*
 * WinQueryItemUnderMouse:
 *      this queries the menu item which corresponds
 *      to the given mouse coordinates
 */

SHORT WinQueryItemUnderMouse(HWND hwndMenu, POINTL *pptlMouse, RECTL *prtlItem);

/*
 * IsObjectAbstract:
 *      this macro returns TRUE if the passed hObject points
 *      to an abstract object; the WPS sets the HIWORD of
 *      an abstract hObject to 2 or 3 for these, to 1 for
 *      file system objects
 */

#define IsObjectAbstract(h)     (BOOL)((HIUSHORT(h)) != 3)

/********************************************************************
 *                                                                  *
 *   Common routines for progress bars                              *
 *                                                                  *
 ********************************************************************/

/*
 * WinProgressBarFromStatic:
 *      this function turns an existing static rectangle control
 *      into a progress bar by subclassing its window procedure;
 *      this way you can easily create a progress bar in the Dialog
 *      Editor; after loading the dlg template, simply call this
 *      function with the hwnd of the static control.
 *      In order to update the progress bar, simply post or send
 *      WM_UPDATEPROGRESSBAR to the static (= progress bar) window;
 *      this message is equal to WM_USER and needs the following
 *      parameters:
 *          mp1     ULONG ulNow, the current progress
 *          mp2     ULONG ulMax, the maximally possible progress
 *                               (= 100%)
 *      The progress bar automatically calculates the current progress
 *      display. For example, if ulMax == 8192 and ulNow == 4096,
 *      a progress of 50% will be shown. It is possible to change
 *      ulMax after the progress bar has started display. If ulMax
 *      is 0, a progress of 0% will be shown (to avoid dividision
 *      by zero traps).
 *
 *      ulAttr accepts of the following:
 *          PBA_NOPERCENTAGE    do not display percentage
 *          PBA_ALIGNLEFT       left-align percentage
 *          PBA_ALIGNRIGHT      right-align percentage
 *          PBA_ALIGNCENTER     center percentage
 */

BOOL WinProgressBarFromStatic(HWND hwndStatic, ULONG ulAttr);

/*
 * WinAnimatedIconFromStatic:
 *      turns a static control into one which
 *      properly displays icons when given a
 *      SM_SETHANDLE msg; you should perform this
 *      function on all your animation icons, because
 *      regular static controls do not properly display
 *      transparency
 */

BOOL WinAnimatedIconFromStatic(HWND hwndStatic);

/*
 * DebugNum:
 *      displays a msgbox with a ULONG for debugging
 */

void DebugNum(PSZ pszTitle, ULONG ul);

/*
 * WinDrawFormattedText:
 *      this func takes a rectangle and draws pszText into
 *      it, breaking the words as neccessary. The line spacing
 *      is determined from the currently selected font.
 *      Parameters:
 *          HPS hps         the presentation space, whose settings are
 *                          not altered;
 *          PRECTL prcl     the rectangle to draw text in; the y coords
 *                          are modified and return the space actually
 *                          used for drawing
 *          PSZ pszText     the text to draw;
 *          flCmd0          may be the ORed flags like in WinDrawText. Tested
 *                          are: DT_TOP, DT_LEFT.
 *                          DT_WORDBREAK | DT_TEXTATTRS are automatically
 *                          set.
 *                          If you specify DT_QUERYEXTENT, no drawing is performed,
 *                          but only the above prcl calculation.
 *      Returns the number of lines drawn.
 */

ULONG WinDrawFormattedText(HPS hps, PRECTL prcl, PSZ pszText, ULONG flCmd);

/*
 * GpiQueryLineSpacing:
 *      this returns the optimal line spacing for text
 *      output with the current HPS; this is computed
 *      by evaluating those incredible FONTMETRICS
 */

LONG GpiQueryLineSpacing(HPS hps, PSZ pszText);

/*
 * GpiDraw3DFrame:
 *      this draws a rectl in 3D style with a given line width;
 *      if (fRaised), the 3D line is given a "raised" look,
 *      "sunken" otherwise
 */

VOID GpiDraw3DFrame(HPS hps, PRECTL prcl, USHORT usWidth, BOOL fRaised);

/*
 * GpiCreateBmpFromPS:
 *      this creates a new bitmap, which is compatible with the
 *      device associated with hpsScreen, and then copies the
 *      rectangle prcl into it; returns the handle of the bitmap,
 *      which can then be used for WinDrawBitmap and such
 */

HBITMAP GpiCreateBmpFromPS(HAB hab, HPS hpsScreen, PRECTL prcl);

/*
 * GpiPowerOffAnim:
 *      displays an animation that looks like a
 *      monitor being turned off; hps must have
 *      been acquired using WinGetScreenPS,
 *      ulSteps should be around 40-50
 */

VOID GpiPowerOffAnim(HPS hps, ULONG ulSteps);


