/*
 * procstat.c:
 *      functions for querying process information.
 *      Based on Kai Uwe Rommel's "dosqproc" package
 *      available at Hobbes:
 *      Kai Uwe Rommel - Wed 25-Mar-1992
 *                       Sat 13-Aug-1994
 *
 *      Modifications Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define INCL_DOS
#define INCL_DOSMODULEMGR
#include <os2.h>

#include <stdlib.h>      // already #include'd
#include <string.h>      // already #include'd
#include <stdio.h>

#include "procstat.h"

/********************************************************************
 *                                                                  *
 *   Functions for querying process information                     *
 *                                                                  *
 ********************************************************************/

/*
 * qpsTimestr:
 *
 */

void qpsTimestr(ULONG time, char *buffer)
{
  ULONG seconds, hundredths;

  seconds = time / 32;
  hundredths = (time % 32) * 100 / 32;

  sprintf(buffer, "%ld:%02ld.%02ld", seconds / 60, seconds % 60, hundredths);
}

/*
 * qpsReport:
 *      fill QPSPROCESS structure
 */

VOID qpsReport(PPROCESS pProcess, PQPSPROCESS pqpsp)
{
    USHORT usPriority = 0;
    PTHREAD pThread;
    int i;

    DosQueryModuleName(pProcess->usHModule, sizeof(pqpsp->szModuleName), pqpsp->szModuleName);
    DosGetPrty(PRTYS_PROCESS, &(pqpsp->usPriority), pProcess->usPID);

    for ( pqpsp->ulCPU = 0, i = 0, pThread = PTR(pProcess->ulThreadList, 0);
              i < pProcess->usThreads;
              i++, pThread++ )
        pqpsp->ulCPU += (pThread->ulSysTime + pThread->ulUserTime);

    pqpsp->usPID            = pProcess->usPID;
    pqpsp->usParentPID      = pProcess->usParentPID;
    pqpsp->ulSID            = pProcess->ulSID;
    pqpsp->usThreads        = pProcess->usThreads;
}

/*
 * qpsSearchProcTree:
 *
 */

BOOL qpsSearchProcTree(PPROCSTAT pps, USHORT usSearchPID, USHORT usCheckPID, PQPSPROCESS pqpsp)
{
    PPROCESS pProcess;

    for ( pProcess = PTR(pps->ulProcesses, 0);
          pProcess->ulType != 3; // not sure if there isn't another termination method
          pProcess = PTR(pProcess->ulThreadList,
                     pProcess->usThreads * sizeof(THREAD))
        )
    {
        if (    (pProcess->usParentPID == usCheckPID)
            &&  (pProcess->ulType != 0)
           )
        {
            if (usSearchPID == pProcess->usPID) {
                qpsReport(pProcess, pqpsp);
                return (TRUE);
            }
            pProcess->ulType = 0;  // kludge, mark it as already printed
            if (qpsSearchProcTree(pps, usSearchPID, pProcess->usPID, pqpsp))
                return (TRUE);
        }
    }

    if ( usCheckPID != 0 )
        return (FALSE);

    // if at the root level, check for those processes that have lost
    // their parent process and show them as if they were childs of 0

    for ( pProcess = PTR(pps->ulProcesses, 0);
          pProcess->ulType != 3; // not sure if there isn't another termination method
          pProcess = PTR(pProcess->ulThreadList,
                     pProcess->usThreads * sizeof(THREAD))
        )
    {
        if ( pProcess->ulType != 0 )
        {
            if (usSearchPID == pProcess->usPID) {
                qpsReport(pProcess, pqpsp);
                return (TRUE);
            }
            if (qpsSearchProcTree(pps, usSearchPID, pProcess->usPID, pqpsp))
                return (TRUE);
        }

        pProcess->ulType = 1;    // kludge, reset mark
    }
    return (FALSE);
}

/*
 * qpsListThreads:
 *
 */

void qpsListThreads(PPROCSTAT pps)
{
    PPROCESS pProcess;
    PTHREAD pThread;
    UCHAR aucName[256], aucSysTime[32], aucUserTime[32];
    USHORT usCount;
    char *pcStatus;

    for ( pProcess = PTR(pps->ulProcesses, 0);
          pProcess->ulType != 3;
          pProcess = PTR(pProcess->ulThreadList,
                     pProcess->usThreads * sizeof(THREAD))
        )
    {
        DosQueryModuleName(pProcess->usHModule, sizeof(aucName), aucName);
        // printf("%s\n", aucName);

        for ( usCount = 0, pThread = PTR(pProcess->ulThreadList, 0);
              usCount < pProcess->usThreads; usCount++, pThread++ )
        {
          qpsTimestr(pThread->ulSysTime, aucSysTime);
          qpsTimestr(pThread->ulUserTime, aucUserTime);
          // pcStatus = threadstat[pThread->ucStatus];
          /* printf("%5d  %5d  %04lX %-7.7s %08lX %10s %10s\n",
             pThread->usTID, pThread->usThreadSlotID, pThread->ulPriority,
             pcStatus ? pcStatus : "Unknown", pThread->ulBlockID, aucSysTime, aucUserTime); */
        }
    }
}

/*
 * qpsListModules:
 *
 */

void qpsListModules(PPROCSTAT pps, int imports)
{
  PMODULE pModule;
  UCHAR name[256];
  ULONG ulCount;

  for ( pModule = PTR(pps->ulModules, 0); ;
        pModule = PTR(pModule->nextmodule, 0)
      )
  {
    if ( !imports || pModule->submodules )
      if ( imports )
      {
        // printf("%04X  %s\n", pModule->modhandle, (char *) PTR(pModule->namepointer, 0));

        for ( ulCount = 0; ulCount < pModule->submodules; ulCount++ )
        {
          DosQueryModuleName(pModule->submodule[ulCount], sizeof(name), name);
          // printf("  %04X  %s\n", pModule->submodule[ulCount], name);
        }
      }
      else
        /* printf(" %04X   %2d  %7ld  %6ld  %s\n", pModule->modhandle,
               pModule->modtype ? 32 : 16, pModule->segments,
               pModule->submodules, (char *) PTR(pModule->namepointer, 0)); */

    if ( pModule->nextmodule == 0L )
      break;
  }
}

/*
 * qpsListSemaphores:
 *
 */

void qpsListSemaphores(PPROCSTAT pps)
{
  PSEMAPHORE pSemaphore;

  for ( pSemaphore = PTR(pps->ulSemaphores, 16);
        pSemaphore->nextsem != 0L;
        pSemaphore = PTR(pSemaphore->nextsem, 0)
      )
    /* printf("%3d  %3d   %02X  %4d   %04X  \\SEM%s\n",
           pSemaphore->refs, pSemaphore->requests, pSemaphore->flag,
           pSemaphore->owner, pSemaphore->index, pSemaphore->name); */
    ;
}


/*
 * qpsListSharedMem:
 *
 */

void qpsListSharedMem(PPROCSTAT pps)
{
  PSHAREDMEM pSharedMem;
  UCHAR name[256];
  ULONG size;
  ULONG attrib;
  PVOID base;

  for ( pSharedMem = PTR(pps->ulSharedMem, 0);
        pSharedMem->nextseg != 0L;
        pSharedMem = PTR(pSharedMem->nextseg, 0)
      )
  {
    strcpy(name, "\\SHAREMEM\\");
    strcat(name, pSharedMem->name);

    DosGetNamedSharedMem(&base, name, PAG_READ);
    size = -1;
    attrib = PAG_SHARED | PAG_READ;
    if ( DosQueryMem(base, &size, &attrib) )
      size = 0;
    DosFreeMem(base);

    /* printf(" %04X   %04X   %08lX  %4d  %s\n", pSharedMem->handle,
           pSharedMem->selector, size,
           pSharedMem->refs, name); */
  }
}

/*
 * qpsQueryProcessInfo:
 *      this searches for a given process ID (usPID) and
 *      fills a given QPSPROCESS structure with lots of
 *      information about this process
 */

BOOL qpsQueryProcessInfo(USHORT usPID, PQPSPROCESS pqpsp)
{
    BOOL rc = FALSE;
    PPROCESS pProcess;

    PPROCSTAT pps = malloc(0x8000);
    DosQProcStatus(pps, 0x8000);

    for ( pProcess = PTR(pps->ulProcesses, 0);
          pProcess->ulType != 3;
          pProcess = PTR(pProcess->ulThreadList,
                     pProcess->usThreads * sizeof(THREAD))
        )
    {
        if (pProcess->usPID == usPID) {
            qpsReport(pProcess, pqpsp);
            rc = TRUE;
            break;
        }
    }

    free(pps);
    return (rc);
}

/*
 * qpsForEachProcess:
 *      this calls a given callback func for each running
 *      process; this func will be passed the following params:
 *          HWND hwnd       like hwnd passed to qpsForEachProcess
 *          ULONG msg       like msg passed to qpsForEachProcess
 *          MPARAM mp1      like mp1 passed to qpsForEachProcess
 *          PQPSPROCESS mp2 pointer to a QPSPROCESS struct for each process
 *      This function returns the number of running processes on the
 *      system. If pfnwpCallback is NULL, only this number will be
 *      returned.
 */

ULONG qpsForEachProcess(PFNWP pfnwpCallback, HWND hwnd, ULONG ulMsg, MPARAM mp1)
{
    ULONG ulrc = 0;
    PPROCESS pProcess;
    QPSPROCESS qpsp;
    PPROCSTAT pps = malloc(0x8000);
    DosQProcStatus(pps, 0x8000);

    for ( pProcess = PTR(pps->ulProcesses, 0);
          pProcess->ulType != 3;
          pProcess = PTR(pProcess->ulThreadList,
                     pProcess->usThreads * sizeof(THREAD))
        )
    {
        if (pfnwpCallback) {
            qpsReport(pProcess, &qpsp);
            (*pfnwpCallback)(hwnd, ulMsg, mp1, &qpsp);
        }
        ulrc++;
    }

    free(pps);
    return (ulrc);
}

/*
 * qpsQueryThreadCount:
 *      returns the total number of running threads
 *      on the system
 */

ULONG qpsQueryThreadCount(VOID)
{
    PPROCSTAT pps = malloc(0x8000);
    PGLOBAL   pg;
    DosQProcStatus(pps, 0x8000);
    pg = PTR(pps->ulGlobal, 0);
    free(pps);
    return (pg->ulThreads);
}

