/*
 * procstat.c:
 *      header file for procstat.h (querying process information).
 *      Based on Kai Uwe Rommel's "dosqproc" package
 *      available at Hobbes:
 *      Kai Uwe Rommel - Wed 25-Mar-1992
 *                       Sat 13-Aug-1994
 *
 *      Modifications Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define PTR(ptr, ofs)  ((void *) ((char *) (ptr) + (ofs)))

/* DosQProcStatus() = DOSCALLS.154 */
USHORT APIENTRY16 DosQProcStatus(PVOID pBuf, USHORT cbBuf);
/* DosGetPrty = DOSCALLS.9 */
USHORT APIENTRY16 DosGetPrty(USHORT usScope, PUSHORT pusPriority, USHORT pid);

typedef struct _PROCSTAT
{
    ULONG  ulGlobal;        // offset to global data section
    ULONG  ulProcesses;     // pffset to process data section
    ULONG  ulSemaphores;    // offset to semaphore data section
    ULONG  ulUnknown1;
    ULONG  ulSharedMem;     // offset to shared mem data section
    ULONG  ulModules;       // offset to DLL data section
    ULONG  ulUnknown2;
    ULONG  ulUnknown3;
} PROCSTAT, *PPROCSTAT;

typedef struct _GLOBAL {
    ULONG  ulThreads;       // total number of threads;
    ULONG  ulReserved1,
           ulReserved2;
} GLOBAL, *PGLOBAL;

typedef struct _PROCESS {
    ULONG  ulType;          // 1 for processes
    ULONG  ulThreadList;
    USHORT usPID;
    USHORT usParentPID;
    ULONG  ulSessionType;
    ULONG  ulStatus;        // see status #define's below
    ULONG  ulSID;
    USHORT usHModule;
    USHORT usThreads;
    ULONG  ulReserved1;
    ULONG  ulReserved2;
    USHORT usSemaphores;    // # of 16-bit semaphores
    USHORT usDLLs;          // # of linked DLLs
    USHORT usSharedMems;
    USHORT usReserved3;
    ULONG  ulSemList;       // offset to semaphore list
    ULONG  ulDLLList;       // offset to DLL list
    ULONG  ulSharedMemList; // offset to shared mem list
    ULONG  ulReserved4;
} PROCESS, *PPROCESS;

#define STAT_EXITLIST 0x01
#define STAT_EXIT1    0x02
#define STAT_EXITALL  0x04
#define STAT_PARSTAT  0x10
#define STAT_SYNCH    0x20
#define STAT_DYING    0x40
#define STAT_EMBRYO   0x80

typedef struct _THREAD
{
    ULONG  ulType;          // 0x100 for thread records
    USHORT usTID;
    USHORT usThreadSlotID;
    ULONG  ulBlockID;
    ULONG  ulPriority;
    ULONG  ulSysTime;
    ULONG  ulUserTime;
    UCHAR  ucStatus;        // see status #define's below
    UCHAR  ucReserved1;
    USHORT usReserved2;
} THREAD, *PTHREAD;

#define TSTAT_READY   1
#define TSTAT_BLOCKED 2
#define TSTAT_RUNNING 5
#define TSTAT_LOADED  9

typedef struct _MODULE
{
  ULONG  nextmodule;
  USHORT modhandle;
  USHORT modtype;
  ULONG  submodules;
  ULONG  segments;
  ULONG  reserved;
  ULONG  namepointer;
  USHORT submodule[1];      // varying, see member submodules */
} MODULE, *PMODULE;

typedef struct _SEMAPHORE
{
  ULONG  nextsem;
  USHORT owner;
  UCHAR  flag;
  UCHAR  refs;
  UCHAR  requests;
  UCHAR  reserved1;
  USHORT reserved2;
  USHORT index;
  USHORT dummy;
  UCHAR  name[1];       /* varying */
} SEMAPHORE, *PSEMAPHORE;

typedef struct _SHAREDMEM
{
  ULONG  nextseg;
  USHORT handle;
  USHORT selector;
  USHORT refs;
  UCHAR  name[1];       /* varying */
} SHAREDMEM, *PSHAREDMEM;

typedef struct _QPSPROCESS {
    CHAR   szModuleName[CCHMAXPATH];
    USHORT usPID,
           usParentPID,
           usThreads;
    ULONG  ulSID;
    USHORT usPriority;
    ULONG  ulCPU;
} QPSPROCESS, *PQPSPROCESS;

/*
 * qpsQueryProcessInfo:
 *      this searches for a given process ID (usPID) and
 *      fills a given QPSPROCESS structure with lots of
 *      information about this process
 */

BOOL qpsQueryProcessInfo(USHORT usPID, PQPSPROCESS pqpsp);

/*
 * qpsForEachProcess:
 *      this calls a given callback func for each running
 *      process; this func will be passed the following params:
 *          HWND hwnd       like hwnd passed to qpsForEachProcess
 *          ULONG msg       like msg passed to qpsForEachProcess
 *          MPARAM mp1      like mp1 passed to qpsForEachProcess
 *          PQPSPROCESS mp2 pointer to a QPSPROCESS struct for each process
 *      This function returns the number of running processes on the
 *      system. If pfnwpCallback is NULL, only this number will be
 *      returned.
 */

ULONG qpsForEachProcess(PFNWP pfnwpCallback, HWND hwnd, ULONG ulMsg, MPARAM mp1);


/*
 * qpsQueryThreadCount:
 *      returns the total number of running threads
 *      on the system
 */

ULONG qpsQueryThreadCount(VOID);

