/*
 * wphandles.c:
 *      This file contains the logic for dealing with
 *      those annoying WPS object handles in OS2SYS.INI.
 *
 *      This code is mostly written by Henk Kelder and published
 *      with his permission.
 *
 *      Copyright (C) 1993-98 Henk Kelder
 *
 *      Modifications Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define INCL_DOS
#define INCL_GPI
#define INCL_WINSHELLDATA
#include <os2.h>
#include <pmbitmap.h>

#define OPTIONS_SIZE 32767

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>

#include "wphandles.ih"

static ULONG ulHandlesSize = 0L;
static PBYTE pHandlesBuffer = NULL;
static BYTE  szActiveHandles[100];
static BYTE  szCurHandles[100];

static BOOL  fNewFormat;

static USHORT _System objGetFSObjectID(PBYTE pHandlesBuffer, ULONG ulBufSize, USHORT usParent, PSZ pszFname);
PNODE _System objFindPartName(PBYTE pHandlesBuffer, ULONG ulBufSize, USHORT usID, PSZ pszFname, USHORT usMax);

#define MakeDiskHandle(usObjID) (usObjID | 0x30000)

#define IsObjectDisk(hObject) ((hObject & 0x30000) == 0x30000)

/*
 * objQueryProfileData:
 *      like PrfQueryProfileData, but allocates sufficient
 *      memory and returns a pointer to that buffer.
 *      pulSize must point to a ULONG which will then
 *      contain the no. of copied bytes.
 */

PBYTE objQueryProfileData(PSZ pApp, PSZ pKey, HINI hIniSystem, PULONG pulSize)
// Expl: pvData = objQueryProfileData(LOCATION, pszName, hIniUser, &ulProfileSize);
{
    PBYTE pData = NULL;
    // printf("PrfQueryProfileSize(%lX, %s, %s, %d)\n", hIniSystem, pApp, pKey, *pulSize);
    if (PrfQueryProfileSize(hIniSystem, pApp, pKey, pulSize)) {
        pData = malloc(*pulSize);
        PrfQueryProfileData(hIniSystem, pApp, pKey, pData, pulSize);
    }
    return (pData);
}

/*
 * objEnumProfileKeys:
 *      allocates memory for a buffer and copies the keys
 *      for pApp into it.
 *      Returns the pointer to the buffer.
 *      pulKeysSize must point to a ULONG which will then
 *      contain the no. of copied bytes.
 */

PBYTE objEnumProfileKeys(PSZ pApp, HINI hIniSystem, PULONG pulKeysSize)
{
    PBYTE pszKeys = NULL;
    if (PrfQueryProfileSize(hIniSystem, pApp, NULL, pulKeysSize)) {
        pszKeys = malloc(*pulKeysSize);
        if (pszKeys) {
            PrfQueryProfileData(hIniSystem, pApp, NULL, pszKeys, pulKeysSize);
        }
    }
    return (pszKeys);
}

/*
 * objResetBlockBuffer:
 *      Reset the block buffer, make sure the buffer is re-read.
 */

VOID _System objResetBlockBuffer(VOID)
{
   if (pHandlesBuffer)
   {
       free(pHandlesBuffer);
       pHandlesBuffer = NULL;
   }
}

/*
 * objGetActiveHandles:
 *
 */

BOOL _System objGetActiveHandles(HINI hIniSystem, PSZ pszHandlesAppName, USHORT usMax)
{
    PBYTE pszHandles;
    ULONG ulProfileSize;

    pszHandles = objQueryProfileData(ACTIVEHANDLES, HANDLESAPP,
         hIniSystem , &ulProfileSize);
    if (!pszHandles)
    {
        strncpy(pszHandlesAppName, HANDLES, usMax-1);
        return TRUE;
    }
    fNewFormat = TRUE;
    strncpy(pszHandlesAppName, pszHandles, usMax-1);
    free(pszHandles);
    return TRUE;
}

/*
 * objfReadAllBlocks:
 *      Read all BLOCK's.
 */

BOOL _System objfReadAllBlocks(HINI hiniSystem, PSZ pszActiveHandles, PBYTE * ppBlock, PULONG pulSize)
{
    PBYTE pbAllBlocks,
          pszBlockName,
          p;
    ULONG ulBlockSize;
    ULONG ulTotalSize;
    BYTE  rgfBlock[100];
    BYTE  szAppName[10];
    USHORT usBlockNo;

    pbAllBlocks = objEnumProfileKeys(pszActiveHandles, hiniSystem, &ulBlockSize);
    if (!pbAllBlocks)
       return FALSE;

    // query size of all individual blocks and calculate total
    memset(rgfBlock, 0, sizeof rgfBlock);
    ulTotalSize = 0L;
    pszBlockName = pbAllBlocks;
    while (*pszBlockName)
    {
        if (!memicmp(pszBlockName, "BLOCK", 5))
        {
           usBlockNo = atoi(pszBlockName + 5);
           if (usBlockNo < 100)
           {
               rgfBlock[usBlockNo] = TRUE;

               if (!PrfQueryProfileSize(hiniSystem,
                         pszActiveHandles,
                         pszBlockName,
                         &ulBlockSize))
               {
                   free(pbAllBlocks);
                   return FALSE;
               }
               ulTotalSize += ulBlockSize;
           }
        }
        pszBlockName += strlen(pszBlockName) + 1;
    }

    // *pulSize contains total size of all blocks
    *ppBlock = malloc(ulTotalSize);
    if (!(*ppBlock))
    {
       /* MessageBox("objfReadAllBlocks", "Not enough memory for profile data!"); */
         free(pbAllBlocks);
         return FALSE;
    }

    // now get all blocks
    p = *ppBlock;
    (*pulSize) = 0;
    for (usBlockNo = 1; usBlockNo < 100; usBlockNo++)
    {
        if (!rgfBlock[usBlockNo])
            break;

        sprintf(szAppName, "BLOCK%u", (UINT)usBlockNo);
        ulBlockSize = ulTotalSize;
        if (!PrfQueryProfileData(hiniSystem,
                  pszActiveHandles,
                  szAppName,
                  p,
                  &ulBlockSize))
        {
            free(pbAllBlocks);
            free(*ppBlock);
            return FALSE;
        }
        p += ulBlockSize;
        (*pulSize) += ulBlockSize;
        ulTotalSize -= ulBlockSize;
    }

    free(pbAllBlocks);
    return TRUE;
}

/****************************************************
 *                                                  *
 *  get hObject from filename                       *
 *                                                  *
 ****************************************************/

/*
 * objGetFSObjectID:
 *      returns the NODE ID which corresponds to pszFname
 *      searching pHandlesBuffer.
 */

USHORT _System objGetFSObjectID(PBYTE pHandlesBuffer, // in: handles buffer
                                ULONG ulBufSize,      // in: sizeof(pHandlesBuffer)
                                USHORT usParent,      // in: parent NODE ID
                                PSZ pszFname)
{
    PDRIV pDriv;
    PNODE pNode;
    PBYTE pCur;                 // current
    PBYTE p,
          pEnd;                 // *end of buffer
    USHORT usPartSize;

    // printf("Entering objGetFSObjectID for %s\n", pszFname);
    p = strchr(pszFname, '\\');
    if (p)
        usPartSize = p - pszFname;
    else
        usPartSize = strlen(pszFname);

    pEnd = pHandlesBuffer + ulBufSize;
    pCur = pHandlesBuffer + 4;
    while (pCur < pEnd)
    {
        if (!memicmp(pCur, "DRIV", 4))
        {
            // pCur points to a DRIVE node:
            pDriv = (PDRIV)pCur;
            pCur += sizeof(DRIV) + strlen(pDriv->szName);
            // printf("  DRIV: %s\n", pDriv->szName);
        }
        else if (!memicmp(pCur, "NODE", 4))
        {
            // pCur points to a regular NODE:
            pNode = (PNODE)pCur;
            pCur += sizeof (NODE) + pNode->usNameSize;
            // printf("    NODE: %s\n", pNode->szName);

            if (    (usParent == pNode->usParentID)
                 && (pNode->usNameSize == usPartSize)
                 // compare names
                 && (!memicmp(pszFname, pNode->szName, usPartSize))
               )
            {
                if (strlen(pszFname) == usPartSize)
                   // found:
                   return (pNode->usID);

                pszFname += usPartSize + 1;
                p = strchr(pszFname, '\\');
                if (p)
                   usPartSize = p - pszFname;
                else
                   usPartSize = strlen(pszFname);
                usParent = pNode->usID;
            }
        }
        else
            // error: neither DRIVE nor NODE
            return (0);
    }
    // not found: end of buffer reached
    return (0);
}

/*
 * objQueryHandleFromPath:
 *      find the object handle for pszName; this
 *      can either be an object ID ("<WP_DESKTOP>")
 *      or a fully qualified filename.
 *      Returns:
 *          -1      file does not exist
 *          -2      error querying handles buffer
 *          -3      object handle not found; might not exist
 *      other:      found object handle
 */

HOBJECT _System objQueryHandleFromPath(HINI hIniUser,     // in: user ini file
                                 HINI hIniSystem,   // in: system ini file
                                 PSZ pszName)       //
{
    USHORT usObjID;
    PVOID  pvData;
    ULONG  ulProfileSize;
    HOBJECT hObject = 0L;
    BYTE    szFullPath[300];

    // try to get the objectID via PM_Workplace:Location
    pvData = objQueryProfileData(LOCATION, pszName, hIniUser, &ulProfileSize);
    if (pvData) {
        if (ulProfileSize >= sizeof(HOBJECT))
            hObject = *(PULONG)pvData;
        free(pvData);
    }
    if (hObject)
        return (hObject);

    // not found there: is pszName an existing pathname?
    if (access(pszName, 0))     // POSIX C for checking existence
        // == -1: file does not exist
        return (-1);
        // return (hObject); // == 0;

    // else: make full path for pszName
    _fullpath(szFullPath, pszName, sizeof(szFullPath));

    // check if the HandlesBlock is valid
    objGetActiveHandles(hIniSystem, szCurHandles, sizeof(szCurHandles));
    if (strcmp(szCurHandles, szActiveHandles))
    {
        if (pHandlesBuffer)
           free(pHandlesBuffer);
        pHandlesBuffer = NULL;
        strcpy(szActiveHandles, szCurHandles);
    }

    if (!pHandlesBuffer)
        pHandlesBuffer = objQueryProfileData(szActiveHandles, HANDLEBLOCK,
                                        hIniSystem, &ulHandlesSize);
    if (!pHandlesBuffer)
        return (-2);
        // return (hObject); // == 0;

    usObjID = objGetFSObjectID(pHandlesBuffer,
                                ulHandlesSize,
                                0,
                                szFullPath);

    if (!usObjID)
        return (-3);
        // return (hObject); // == 0;

    hObject = MakeDiskHandle(usObjID);
    // hObject = usObjID; // (UM)
    return (hObject);
}

/****************************************************
 *                                                  *
 *  get filename from hObject                       *
 *                                                  *
 ****************************************************/

/*
 * objFindPartName:
 *      this searches pHandlesBuffer for usID and, if found,
 *      appends the object partname to pszFname.
 *      This function recurses, if neccessary.
 */

PNODE _System objFindPartName(PBYTE pHandlesBuffer,  // in: handles buffer
                             ULONG ulBufSize,       // in: buffer size
                             USHORT usID,           // in: NODE ID to search for
                             PSZ pszFname,          // out: object partname
                             USHORT usMax)          // in: sizeof(pszFname)
{
    PDRIV pDriv;
    PNODE pNode;
    PBYTE p, pEnd;
    USHORT usSize;

    pEnd = pHandlesBuffer + ulBufSize;
    p = pHandlesBuffer + 4;
    while (p < pEnd)
    {
        if (!memicmp(p, "DRIV", 4))
        {
            pDriv = (PDRIV)p;
            p += sizeof(DRIV) + strlen(pDriv->szName);
        }
        else if (!memicmp(p, "NODE", 4))
        {
            pNode = (PNODE)p;
            p += sizeof (NODE) + pNode->usNameSize;
            if (pNode->usID == usID)
            {
                usSize = usMax - strlen(pszFname);
                if (usSize > pNode->usNameSize)
                    usSize = pNode->usNameSize;
                if (pNode->usParentID)
                {
                    if (!objFindPartName(pHandlesBuffer, ulBufSize, pNode->usParentID, pszFname, usMax))
                       return (NULL);
                    strcat(pszFname, "\\");
                    strncat(pszFname, pNode->szName, usSize);
                    return pNode;
                } else {
                    strncpy(pszFname, pNode->szName, usSize);
                    return pNode;
                }
            }
        }
        else
           return (NULL);
    }
    return (NULL);
}

/*
 * objQueryPathFromHandle:
 *      get the filename for hObject.
 */

PNODE _System objQueryPathFromHandle(HINI hIniSystem,
                                HOBJECT hObject,    // in: object handle
                                PSZ pszFname,       // out: filename, if found
                                USHORT usMax)       // in: sizeof (pszFname)
{
    USHORT usObjID;

    objGetActiveHandles(hIniSystem, szCurHandles, sizeof(szCurHandles));
    if (strcmp(szCurHandles, szActiveHandles))
    {
        // active handles changed? => adjust
        if (pHandlesBuffer)
           free(pHandlesBuffer);
        pHandlesBuffer = NULL;
        strcpy(szActiveHandles, szCurHandles);
    }

    // use lower byte only
    usObjID = LOUSHORT(hObject);

    if (!IsObjectDisk(hObject))
       return NULL;

    if (!pHandlesBuffer)
       if (!objfReadAllBlocks(hIniSystem, szActiveHandles,
                &pHandlesBuffer, &ulHandlesSize))
          return NULL;

    memset(pszFname, 0, usMax);

    return (objFindPartName(pHandlesBuffer, ulHandlesSize, usObjID, pszFname, usMax));
}


