
/*
 * xfdisk.c:
 *      This file contains the following major XFolder parts:
 *
 *      -  XFldDisk SOM stuff
 *
 *      The XFldDisk introduces a few new methods to simplify
 *      querying information about a disk object plus some
 *      status bar logic. Otherwise, it mostly routes context
 *      menu handling to the counterparts in xfldr.c, because
 *      disk and folder context menus should be pretty similar.
 *
 *      Check the other files starting with xf* for the
 *      other XFolder classes.
 *
 *      Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

/*
 *  This file was generated by the SOM Compiler and Emitter Framework.
 *  Generated using:
 *      SOM Emitter emitctm: 2.42
 */

#ifndef SOM_Module_xfdisk_Source
#define SOM_Module_xfdisk_Source
#endif
#define XFldDisk_Class_Source
#define M_XFldDisk_Class_Source

#define INCL_DOSPROCESS
#define INCL_DOSSEMAPHORES
#define INCL_DOSEXCEPTIONS
#define INCL_DOSERRORS
#define INCL_DOSDEVICES
#define INCL_DOSDEVIOCTL
#define INCL_WIN
#include  <os2.h>

#include "xfdisk.ih"
#include "xfldr.h"

#include "common.h"           // this includes other libs

XFldDisk* apDrives[27];       // global array of all awake Disk objects

/********************************************************************
 *                                                                  *
 *   here come the XFldDsk instance methods                         *
 *                                                                  *
 ********************************************************************/

/*
 * xfQueryFreeSpace:
 *           returns the number of bytes remaining on this disk
 */

SOM_Scope ULONG  SOMLINK xfdisk_xfQueryFreeSpace(XFldDisk *somSelf)
{
    FSALLOCATE fsa;
    ULONG ulrc = -1, ulDrive = 0;
    /* XFldDiskData *somThis = XFldDiskGetData(somSelf); */
    XFldDiskMethodDebug("XFldDisk","xfdisk_xfQueryFreeSpace");

    // DosError(FERR_DISABLEHARDERR | FERR_DISABLEEXCEPTION);
    ulDrive = _wpQueryLogicalDrive(somSelf);
    if (ulDrive)
        if (DosQueryFSInfo(ulDrive, FSIL_ALLOC, &fsa, sizeof(fsa))
                == NO_ERROR)
            ulrc = (fsa.cSectorUnit * fsa.cbSector * fsa.cUnitAvail);

    // DosError(FERR_ENABLEHARDERR | FERR_ENABLEEXCEPTION);
    return (ulrc);
}

/*
 * xfQueryFSType:
 *           returns the file-system type of this disk
 *           (HPFS, FAT, CDFS etc.)
 */

SOM_Scope PSZ  SOMLINK xfdisk_xfQueryFSType(XFldDisk *somSelf)
{
    XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_xfQueryFSType");

    if (_szFSType[0] == '\0')
    {
        // if we haven't queried the FS type yet, do it now
        // by calling DosQueryFSAttach; we only need to do this
        // once, because the FS type cannot change once the
        // system has booted

        // CHAR szName[CCHMAXPATH];
        ULONG ulLogicalDrive = 0;
        APIRET arc = NO_ERROR;

        // DosError(FERR_DISABLEHARDERR | FERR_DISABLEEXCEPTION);

        ulLogicalDrive = _wpQueryLogicalDrive(somSelf);

        if (ulLogicalDrive) {
            // check if drive is removeable
            ULONG parmsize;
            ULONG datasize;
            UCHAR ucNonRemoveable;

            // parameter packet
            struct {
                UCHAR command, drive;
            } parms;

            // data packet
            /* struct {
                UCHAR   aucBPB[31];         // BIOS parameter block
                USHORT  usCylinders;
                UCHAR   ucDeviceType;
                USHORT  usDeviceAttrs;
            } data;

            parms.command = 1; // read currently inserted media
            parms.drive=(UCHAR)(ulLogicalDrive-1);

            parmsize = sizeof(parms);
            datasize = sizeof(data);
            arc = DosDevIOCtl((HFILE)-1,
                            IOCTL_DISK,
                            DSK_GETDEVICEPARAMS,
                            &parms, parmsize, &parmsize,
                            &data,  datasize, &datasize);

            _Pmpf(("GetDeviceParams APIRET: %d", arc));
            _Pmpf(("  Device type: %d", data.ucDeviceType));
            _Pmpf(("  Device Attrs: 0x%lX", data.usDeviceAttrs)); */

            ULONG parmSize = sizeof(parms);
            ULONG dataLen = sizeof(ucNonRemoveable);

            parms.drive = (UCHAR)(ulLogicalDrive-1);
            arc = DosDevIOCtl((HFILE)-1,
                            IOCTL_DISK,
                            DSK_BLOCKREMOVABLE,
                            &parms,
                            parmSize,
                            &parmSize,
                            &ucNonRemoveable,
                            dataLen,
                            &dataLen);

            if (ucNonRemoveable)
            // if (arc == NO_ERROR)
            {
                CHAR szDrives[] = " ABCDEFGHIJKLMNOPQRSTUVWXYZ",
                     szName[5];

                szName[0] = szDrives[ulLogicalDrive];
                szName[1] = ':';
                szName[2] = '\0';

                {
                    ULONG  ulOrdinal       = 0;     // ordinal of entry in name list
                    // PBYTE  pszFSDName      = NULL;  // pointer to FS name
                    // PBYTE  prgFSAData      = NULL;  // pointer to FS data
                    APIRET rc              = NO_ERROR; // return code
                    BYTE   fsqBuffer[sizeof(FSQBUFFER2) + (3 * CCHMAXPATH)] = {0};
                    ULONG  cbBuffer   = sizeof(fsqBuffer);        // Buffer length)
                    PFSQBUFFER2 pfsqBuffer = (PFSQBUFFER2)fsqBuffer;

                    rc = DosQueryFSAttach(
                                    szName,          // logical drive of attached FS
                                    ulOrdinal,       // ignored for FSAIL_QUERYNAME
                                    FSAIL_QUERYNAME, // return data for a Drive or Device
                                    pfsqBuffer,      // returned data
                                    &cbBuffer);      // returned data length

                    if (rc == NO_ERROR)
                    {
                        // The data for the last three fields in the FSQBUFFER2
                        // structure are stored at the offset of fsqBuffer.szName.
                        // Each data field following fsqBuffer.szName begins
                        // immediately after the previous item.
                        PSZ pszFSDName = pfsqBuffer->szName + pfsqBuffer->cbName + 1;
                        strncpy(_szFSType, pszFSDName, sizeof(_szFSType)-1);
                        _szFSType[sizeof(_szFSType)-1] = '\0';
                    }
                }
            }
        }
        // DosError(FERR_ENABLEHARDERR | FERR_ENABLEEXCEPTION);
    }

    return (_szFSType);
}

/*
 * wpFilterPopupMenu:
 *           remove default entries for disks according to Global Settings
 */

SOM_Scope ULONG  SOMLINK xfdisk_wpFilterPopupMenu(XFldDisk *somSelf,
                                                ULONG ulFlags,
                                                HWND hwndCnr,
                                                BOOL fMultiSelect)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    // XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpFilterPopupMenu");

    return (XFldDisk_parent_WPDisk_wpFilterPopupMenu(somSelf,
                                                     ulFlags,
                                                     hwndCnr,
                                                     fMultiSelect)
        & ~(pGlobalSettings->DefaultMenuItems)
        );
}

/*
 * wpModifyPopupMenu:
 *           add the variable XFolder menu entries;
 *           have this done by the common XFolder function
 */

SOM_Scope BOOL  SOMLINK xfdisk_wpModifyPopupMenu(XFldDisk *somSelf,
                                               HWND hwndMenu,
                                               HWND hwndCnr,
                                               ULONG iPosition)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    BOOL            rc;
    XFolder         *pFolder = _wpQueryRootFolder(somSelf);

    // XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpModifyPopupMenu");

    XFldDisk_parent_WPDisk_wpModifyPopupMenu(somSelf,hwndMenu,hwndCnr,iPosition);

    if (pGlobalSettings->RemoveFormatDisk) {
        // we need a handle to the WPDisk's "Format Disk"

        WinSendMsg(hwndMenu, MM_REMOVEITEM,
                MPFROM2SHORT(ID_WPMI_FORMATDISK, FALSE),
                0);
    }

    if (pGlobalSettings->RemoveCheckDisk) {
        // we need a handle to the WPDisk's "Check Disk"

        WinSendMsg(hwndMenu, MM_REMOVEITEM,
                MPFROM2SHORT(ID_WPMI_CHECKDISK, FALSE),
                0);
    }

    rc = cmnModifyPopupMenu(pFolder, hwndMenu, hwndCnr, iPosition);

    return (rc);
}

/*
 * wpMenuItemSelected:
 *           process input when any menu item was selected;
 *           pass input to common XFolder function
 */

SOM_Scope BOOL  SOMLINK xfdisk_wpMenuItemSelected(XFldDisk *somSelf,
                                                HWND hwndFrame,
                                                ULONG ulMenuId)
{
    XFolder         *pFolder = _wpQueryRootFolder(somSelf);
    // XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpMenuItemSelected");

    if (cmnMenuItemSelected(pFolder, hwndFrame, ulMenuId))
        return TRUE;
    else
        return (XFldDisk_parent_WPDisk_wpMenuItemSelected(somSelf, hwndFrame,
                                                   ulMenuId));
}

/*
 * wpMenuItemHelpSelected:
 *           display help for a context menu item
 */

SOM_Scope BOOL  SOMLINK xfdisk_wpMenuItemHelpSelected(XFldDisk *somSelf,
                                                    ULONG MenuId)
{
    XFolder         *pFolder = _wpQueryRootFolder(somSelf);
    // XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpMenuItemHelpSelected");

    if (cmnMenuItemHelpSelected(pFolder, MenuId))
        return TRUE;
    else
        return (XFldDisk_parent_WPDisk_wpMenuItemHelpSelected(somSelf,
                                                           MenuId));
}

/*
 * wpOpen:
 *           we call the parent first and then subclass the
 *           resulting frame window
 */

SOM_Scope HWND  SOMLINK xfdisk_wpOpen(XFldDisk *somSelf, HWND hwndCnr,
                                       ULONG ulView, ULONG param)
{
    HWND                hwndNewFrame;
    PSUBCLASSEDLISTITEM psli;
    PGLOBALSETTINGS     pGlobalSettings = cmnQueryGlobalSettings();

    // query root folder (WPRootFolder class, which is a descendant
    // of WPFolder/XFolder); each WPDisk is paired with one of those,
    // and the actual display is done by this class, so will will pass
    // pRootFolder instead of somSelf to most following method calls
    XFolder*            pRootFolder = _wpQueryRootFolder(somSelf);

    // XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpOpen");

    // call parent to get frame handle
    hwndNewFrame = XFldDisk_parent_WPDisk_wpOpen(somSelf, hwndCnr, ulView,
                                          param);

    // subclass frame window (use pRootFolder)
    psli = cmnSubclassFrameWnd(hwndNewFrame, pRootFolder, ulView);

    // add status bar, if allowed:
    // 1) status bar only if allowed for the current folder
    if (pGlobalSettings->StatusBar)
        // 2) check that subclassed list item is valid
        if (psli)
            // 4) status bar only if allowed for the current view type
            if (    (   (ulView == OPEN_CONTENTS)
                     && (pGlobalSettings->SBForViews & SBV_ICON)
                    )
                 || (   (ulView == OPEN_TREE)
                     && (pGlobalSettings->SBForViews & SBV_TREE)
                    )
                 || (   (ulView == OPEN_DETAILS)
                     && (pGlobalSettings->SBForViews & SBV_DETAILS)
                    )
                )
                _xfShowStatusBar(pRootFolder, hwndNewFrame, psli, TRUE);

    // extended sort functions
    if (    (pGlobalSettings->ReplaceSort)
         && (   (ulView == OPEN_CONTENTS)
             || (ulView == OPEN_TREE)
             || (ulView == OPEN_DETAILS)
            )
       )
    {
        hwndCnr = WinQueryCnrFromFrame(hwndNewFrame);
        cmnSetCnrSort(pRootFolder, hwndCnr);
        // let Worker thread reset the cnr sort, because the WPS messes
        // with it somewhere
        cmnPostWorkerMsg(WOM_CORRECTCNRSORT, (MPARAM)pRootFolder, (MPARAM)hwndCnr);
    }

    return (hwndNewFrame);
}

/*
 * wpInitData:
 *           set up WPDisk instance data
 */

SOM_Scope void  SOMLINK xfdisk_wpInitData(XFldDisk *somSelf)
{
    XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpInitData");

    XFldDisk_parent_WPDisk_wpInitData(somSelf);
    XFldDisk_parent_XFldObject_wpInitData(somSelf);

    _szFSType[0] = '\0';
}

/*
 * wpObjectReady:
 *           this instance method is called by the system when
 *           object awakening is complete and object data seems
 *           to be stable in memory;
 *           we will have this object's pointer stored in a
 *           global array to be able to find the Disk object for
 *           a given object (wpclsQueryDiskObject).
 */

SOM_Scope void  SOMLINK xfdisk_wpObjectReady(XFldDisk *somSelf,
                                             ULONG ulCode, WPObject* refObject)
{
    ULONG ulDisk;
    // XFldDiskData *somThis = XFldDiskGetData(somSelf);
    XFldDiskMethodDebug("XFldDisk","xfdisk_wpObjectReady");

    XFldDisk_parent_WPDisk_wpObjectReady(somSelf, ulCode, refObject);

    ulDisk = _wpQueryLogicalDrive(somSelf);
    apDrives[ulDisk] = somSelf;

    /* XFldDisk_parent_XFldObject_wpObjectReady(somSelf, ulCode,
                                             refObject); */
}

/*
 * xfTranslateStatusBarMnemonics:
 *           this method is called on an object to retrieve
 *           the "single-object" status bar info. pszText
 *           originally contains the (class-specific) mnemonic
 *           string returned by xfclsQueryStatusBarMnemonics.
 *           As opposed to the other mnemonics funcs, this is
 *           an _instance_ method because some translation
 *           might already be performed depending on the
 *           selected object.
 */

SOM_Scope ULONG  SOMLINK xfdisk_xfTranslateStatusBarMnemonics(XFldDisk *somSelf,
                                                              PSZ pszText,
                                                              ULONG cbText)
/* single-object status bar text mnemonics understood by XFldDisk:

     $F      file system type (HPFS, FAT, CDFS, ...)

     $fb     free space on drive in bytes
     $fk     free space on drive in kBytes
     $fK     free space on drive in KBytes
     $fm     free space on drive in mBytes
     $fM     free space on drive in MBytes

    NOTE: the $f keys are also handled by cmnComposeStatusBarText, but
          those only work properly for file-system objects, so we need
          to calculate these values for these (abstract) disk objects here
 */

{
    CHAR        // szSizeSelected[20] = "0",
                szThousand[10],
                szTemp[300];
    BOOL        fCont = TRUE;
    PSZ         p;

    /* XFldDiskData *somThis = XFldDiskGetData(somSelf); */
    XFldDiskMethodDebug("XFldDisk","xfdisk_xfTranslateStatusBarMnemonics");

    PrfQueryProfileString(HINI_USER, "PM_Default_National", "sThousand",
            ",", szThousand, sizeof(szThousand)-1);

    do {
        strcpy(szTemp, "?");
        fCont = FALSE;

        // the following are for free space on drive
        if (p = strstr(pszText, "$fb"))
        {
            strths(szTemp,
                    _xfQueryFreeSpace(somSelf),
                    szThousand[0]);
            strrpl(pszText, "$fb", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fk"))
        {
            strths(szTemp,
                    ((_xfQueryFreeSpace(somSelf) + 500) / 1000),
                    szThousand[0]);
            strrpl(pszText, "$fk", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fK"))
        {
            strths(szTemp,
                    ((_xfQueryFreeSpace(somSelf) + 512) / 1024),
                    szThousand[0]);
            strrpl(pszText, "$fK", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fm"))
        {
            strths(szTemp,
                    ((_xfQueryFreeSpace(somSelf) +500000) / 1000000),
                    szThousand[0]);
            strrpl(pszText, "$fm", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fM"))
        {
            strths(szTemp,
                    ((_xfQueryFreeSpace(somSelf) + (1024*1024/2)) / (1024*1024)),
                    szThousand[0]);
            strrpl(pszText, "$fM", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$F"))  // file-system type (HPFS, ...)
        {
            strrpl(pszText, "$F", _xfQueryFSType(somSelf));
            fCont = TRUE;
        }
    } while (fCont);

    return (XFldDisk_parent_XFldObject_xfTranslateStatusBarMnemonics(somSelf, pszText, cbText));
}

/********************************************************************
 *                                                                  *
 *   here come the xfdisk class methods                            *
 *                                                                  *
 ********************************************************************/

CHAR    szWPDiskStatusBarMnemonics[CCHMAXMNEMONICS] = "";

/*
 * xfclsQueryDisk:
 *           since I have no idea what the default wpQueryDisk
 *           is for (why the hell does it return a WPFileSystem?),
 *           this is a proper method which returns the disk object
 *           which corresponds to the drive on which the object
 *           resides. This works for both abstract and file-system
 *           objects.
 *           Returns NULL upon errors. This might happen if you
 *           call this method before the Drives folder is populated
 *           and the disk objects are thus not instantiated yet,
 *           because this class method relies on XFldDisk::wpObjectReady.
 *           The WPS normally populates that folder a few seconds
 *           after the Desktop is up.
 */

SOM_Scope XFldDisk*  SOMLINK xfdiskM_xfclsQueryDiskObject(M_XFldDisk *somSelf,
                                                          WPObject* pObject)
{
    ULONG ulLogicalDrive = _xfQueryLogicalDriveNumber(pObject);
    /* M_XFldDiskData *somThis = M_XFldDiskGetData(somSelf); */
    M_XFldDiskMethodDebug("M_XFldDisk","xfdiskM_xfclsQueryDiskObject");

    _Pmpf(("xfclsQueryDiskObject: ulDrive = %d", ulLogicalDrive));
    return (apDrives[ulLogicalDrive]);
}

/*
 * wpclsInitData:
 *           initialize class instance data
 */

SOM_Scope void  SOMLINK xfdiskM_wpclsInitData(M_XFldDisk *somSelf)
{
    // M_XFldDiskData *somThis = M_XFldDiskGetData(somSelf);
    M_XFldDiskMethodDebug("M_XFldDisk","xfdiskM_wpclsInitData");

    M_XFldDisk_parent_M_WPDisk_wpclsInitData(somSelf);
    M_XFldDisk_parent_M_XFldObject_wpclsInitData(somSelf);

    szWPDiskStatusBarMnemonics[0] = '\0';
    memset(&apDrives, 0, sizeof(apDrives));
}

/*
 * wpclsQueryTitle:
 *           tell the WPS the new class name: Disk (XFolder)
 */

SOM_Scope PSZ  SOMLINK xfdiskM_wpclsQueryTitle(M_XFldDisk *somSelf)
{
    // M_XFldDiskData *somThis = M_XFldDiskGetData(somSelf);
    M_XFldDiskMethodDebug("M_XFldDisk","xfdiskM_wpclsQueryTitle");

    /* strcpy(&(_szTitle[0]), M_XFldDisk_parent_M_WPDisk_wpclsQueryTitle(somSelf));
    strcat(&(_szTitle[0]), " (XFolder)");

    return (&(_szTitle[0])); */
    return (M_XFldDisk_parent_M_WPDisk_wpclsQueryTitle(somSelf));
}

/*
 * xfclsAddsNewStatusBarMnemonics:
 *           this method returns flags for whether this class
 *           supports specific status bar mnemonics, i.e. objects
 *           of this class (and subclasses) display different
 *           information in status bar "single-object" mode
 *           than objects of parent classes. This method is
 *           called from the "Status bar" settings page in
 *           order to find out if different mnemonics can be
 *           set for this class.
 *           ORed return values are:
 *                SBS_ADDSMNEMONICS   class has own mnemonics
 *                SBS_MODIFIABLE      these mnemonics are modifiable
 *                                    (if not: they're static)
 */

SOM_Scope ULONG  SOMLINK xfdiskM_xfclsAddsNewStatusBarMnemonics(M_XFldDisk *somSelf)
{
    // M_XFldDiskData *somThis = M_XFldDiskGetData(somSelf);
    M_XFldDiskMethodDebug("M_XFldDisk","xfdiskM_xfclsAddsNewStatusBarMnemonics");

    return ( (somSelf == _XFldDisk)
             ? (SBS_ADDSMNEMONICS | SBS_MODIFIABLE)
             : 0);
}

/*
 * xfclsQueryStatusBarMnemonics:
 *           this returns the status bar mnemonics for "single object"
 *           info for objects of this class and subclasses. This string
 *           is either what has been specified on the "Status bar"
 *           notebook page or a default string from the XFolder NLS DLL.
 *           The WPObject version returns the generic string for all
 *           objects, but this is overridden for disk objects to
 *           return something more meaningful.
 *           Note that for WPDisk, this affects only the status bar of
 *           the "Drives" folder, because root drive views are instances
 *           of WPRootFolder instead of WPDisk.
 */

SOM_Scope ULONG  SOMLINK xfdiskM_xfclsQueryStatusBarMnemonics(M_XFldDisk *somSelf,
                                                              PSZ pszText,
                                                              ULONG cbText)
{
    // M_XFldDiskData *somThis = M_XFldDiskGetData(somSelf);
    M_XFldDiskMethodDebug("M_XFldDisk","xfdiskM_xfclsQueryStatusBarMnemonics");

    if (szWPDiskStatusBarMnemonics[0] == '\0')
        // load string if this is the first time
        if (PrfQueryProfileString(HINI_USERPROFILE, INIAPP, INIKEY_SBTEXT_WPDISK,
                     NULL, &(szWPDiskStatusBarMnemonics), sizeof(szWPDiskStatusBarMnemonics))
                == 0)
            // string not found in profile: load default from NLS resources
            WinLoadString(WinQueryAnchorBlock(HWND_DESKTOP), NLS_MODULE,
                        ID_XSSI_SBTEXTWPDISK,
                        sizeof(szWPDiskStatusBarMnemonics),
                        szWPDiskStatusBarMnemonics);

    strncpy(pszText, szWPDiskStatusBarMnemonics, cbText);
    pszText[cbText-1] = '\0';

    return (cbText);
}

/*
 * xfclsSetStatusBarMnemonics:
 *           change the status bar mnemonics for "single object"
 *           info for objects of this class and subclasses to
 *           pszText. If *pszText points to null-length string, no
 *           status bar info will be displayed; if pszText is NULL,
 *           the menmonics will be reset to the default value.
 */

SOM_Scope ULONG  SOMLINK xfdiskM_xfclsSetStatusBarMnemonics(M_XFldDisk *somSelf,
                                                            PSZ pszText)
{
    // M_XFldDiskData *somThis = M_XFldDiskGetData(somSelf);
    M_XFldDiskMethodDebug("M_XFldDisk","xfdiskM_xfclsSetStatusBarMnemonics");

    /* strncpy(_szStatusBarMnemonics, pszText, sizeof(_szStatusBarMnemonics));
    _szStatusBarMnemonics[sizeof(_szStatusBarMnemonics)-1] = '\0'; */

    // provoke a reload of the settings
    // in xfclsQueryStatusBarMnemonics
    szWPDiskStatusBarMnemonics[0] = '\0';

    // set the class mnemonics in OS2.INI; if
    // pszText == NULL, the key will be deleted,
    // and xfclsQueryStatusBarMnemonics will use
    // the default value
    return (ULONG)PrfWriteProfileString(HINI_USERPROFILE, INIAPP, INIKEY_SBTEXT_WPDISK,
            pszText);
}


