#
# Makefile for the XFolder project w/out NLS support.
#
# For use with IBM NMAKE, which is part of IBM C-Set/2 and VisualAge C++.
# This file will probably not work with other MAKE utilities, such as
# GNU make or DMAKE.
#
# (w) (c) 1997-98 Ulrich Mller. All rights reserved.
#

#
# First define the suffixes for files which NMAKE will work on.
# .SUFFIXES is a reserved NMAKE keyword ("pseudotarget") for
# defining file extensions that NMAKE will recognize.
#

.SUFFIXES: .c .obj .dll .idl .h .ih .rc .res

#
# Now set up a few environment variables. This is done with
# "macros" (in NMAKE terminology), which are assumed when
# a "=" is encountered.
#

# Change this to the directory where you installed XFolder.
# The output files will be copied there to.
!ifndef XFOLDERPATH
XFOLDERPATH=G:\develop\C\XFolder\running
!endif
XFRUNNING = $(XFOLDERPATH)

SOMTEMP = L:\var\temp

!if [set SOMTMP=L:\var\temp] || \
    [set SMTMP=$(SOMTEMP)] || \
    [set SMEMIT=ih;h;c] || \
    [set SMADDSTAR=1] || \
    [set SMNOTC=1]
!endif

# STEM is the stem for the dll which we want to create.
STEM = xfldr

#
# Compiler macro. This is used for each .OBJ file to be created.
# "\" is the line-continuation mark.
#

CC      = icc /c /gd- /ge- /gi+ /gm+ /o+ /ol+ /q+ /se /ss

# /oi- /w2 /fi+ /si+ /ti+

# Some VisualAge C++ compiler options explained:
# /c:   compile only, no link
# /fi+: precompile header files
# /gd-: link runtime statically
# /ge-: create DLL
# /gi+: fast integer execution
# /gm+: multithread libraries
# /kc+: produce preprocessor warnings
# /o+:  optimization (inlining etc.)
# /oi-: no inlining (?)
# /ol+: use intermediate linker
# /q+:  suppress icc logo
# /se:  all language extensions
# /si+: allow use of precompiled header files
# /ss:  allow double slashes
# /ti+: debug code
# /Wcnd: conditional exprs problems (= / == etc.)
# /Wgen: generic debugging msgs
# /Wcmp: possible unsigned comparison redundancies
# /Wcns: operations involving constants
# /Wpar: list not-referenced parameters (annoying)
# /Wppc: list possible preprocessor problems (.h dependencies)
# /Wpro: warn if funcs have not been prototyped
# /Wrea: mark code that cannot be reached
# /Wret: check consistency of return levels
# /w2:   produce error and warning messages, but no infos

#
# Link macro. This is used for final linking.
#

# LINK    = J:\develop\WarpToolkit\bin\LINK386 /align:16 /map /noi /nol /nod /packcode /packdata
LINK =  ilink /optfunc

# Some LINK386 cmd line options
# (Visual Age ILINK understands these too if /nofree is specified):
# /align:  executable pages align factor (in bytes)
# /noi:    no ignore case (necc. for C)
# /map:    list public symbols
# /nod:    no default library search (explicitly specify libs)
# /nol:    no logo (link386 startup)
# /packcode: group neighboring code segments
# /packdata: group neighboring data segments
# /de:     include debug code

#
# Resource compiler macro.
#

RC      = rc -p -x2

LIBS = somtk.lib mmpm2.lib pmprintf.lib

#
# The OBJS macro contains all the .OBJ files which belong to this project.
#

OBJS =  apm.obj wphandles.obj common.obj helpers.obj xfsort.obj procstat.obj xfpgm.obj xfsys.obj xfobj.obj xfldr.obj xfdisk.obj xfdesk.obj xffsys.obj xfdataf.obj objdlgs.obj

#
# Now define inference rules for what to do with certain file
# types, based on their file extension.
# The syntax we need here is ".fromext.toext".
# So whenever NMAKE encounters a .toext file, it
# executes what we specify here.
#
# Special macros used here: $* is the current target w/out ext.
#

# 1) compile C files to .OBJ files, using the CC macro above
.c.obj:
        @ echo Compiling $*.c:
        $(CC) -I$(INCLUDE) $*.c

# 2) create/update .H, .IH, .C files when .IDL has changed
.idl.h:
        sc -p -v -maddstar -mnoint -S128000 -C128000  $*.idl
        sc -p -v -maddstar -mnoint -S128000 -C128000 -sc $*.idl
        sc -p -v -maddstar -mnoint -S128000 -C128000 -s def $*.idl

# The main target file is xfldr.dll in the installation directors.
all: $(XFRUNNING)\bin\xfldr.dll

# If this is older than the one in the source directory, update it;
# this also sets the dependency on xfldr.dll at the bottom.
$(XFRUNNING)\bin\xfldr.dll: xfldr.dll
        unlock $(XFRUNNING)\bin\xfldr.dll
        cmd.exe /c copy xfldr.dll $(XFRUNNING)\bin

#
# Now define inference rules for invoking the SOM compiler (sc.exe),
# using the .idl.h rule defined above. E.g., xfobj.h/.ih/.c will be
# updated by the SOM compiler when xfobj.idl has changed.
#

# XFldObject class
xfobj.h:   xfobj.idl

# XFolder classes
xfldr.h:   xfldr.idl

# XFldDisk class
xfdisk.h:   xfdisk.idl

# XFldDesktop class
xfdesk.h:   xfdesk.idl

# XFldProgram class
xfpgm.h:   xfpgm.idl

# XFldSystem/XFldWPS classes
xfsys.h:   xfsys.idl

# XFldFileSystem class
xffsys.h:   xffsys.idl

# XFldDataFile class
xfdataf.h:   xfdataf.idl

#
# Now define inference rules for all our .obj files by telling NMAKE
# which .OBJ file depends on what. For each .obj we define what files
# will enforce a recompilation when they've been changed.
#

# XFolder "library" .OBJ files used by several classes

common.obj: common.c common.h dlgids.h helpers.h xfsort.h xfldr.h

helpers.obj: helpers.c helpers.h

xfsort.obj: xfsort.c xfsort.h xfldr.h helpers.h

procstat.obj: procstat.c procstat.h

objdlgs.obj: objdlgs.c objdlgs.h xfldr.h helpers.h xfsort.h dlgids.h wphandles.h

apm.obj: apm.h

wphandles.obj: wphandles.c wphandles.h helpers.h

# Now follow the .OBJs for the SOM/WPS classes. Each class needs to be
# recompiled when its SOM header files change (due to SOM recompilation)
# or when xfobj.h has changed, which is #included in most .c files, or
# when the "library" header files have changed.

xfobj.obj: xfobj.c xfobj.h xfobj.h xfldr.h xfdesk.h wphandles.h dlgids.h common.h helpers.h

xfldr.obj: $(STEM).c $(STEM).h $(STEM).h xfdesk.h wphandles.h objdlgs.h xfsort.h \
                                                         xfobj.h dlgids.h common.h helpers.h

xfdisk.obj: xfdisk.c xfdisk.h xfdisk.h      xfobj.h     xfldr.h dlgids.h common.h helpers.h

xfdesk.obj: xfdesk.c xfdesk.h xfdesk.h      xfobj.h     xfldr.h dlgids.h common.h helpers.h apm.h

xfpgm.obj: xfpgm.c xfpgm.h xfpgm.h                      xfobj.h dlgids.h common.h helpers.h

xfsys.obj: xfsys.c xfsys.h xfsys.h                      xfldr.h dlgids.h common.h helpers.h objdlgs.h

xffsys.obj: xffsys.c xffsys.h xffsys.h      xfdesk.h    xfobj.h dlgids.h common.h helpers.h

xfdataf.obj: xfdataf.c xfdataf.h xfdataf.h              xfobj.h dlgids.h common.h helpers.h

# XFolder language-independent resource file
xfldr.res: $*.rc dlgids.h *.bmp *.ico
         $(RC) -r $(STEM).rc $*.res

#
# Finally, define rule for linking the whole thing.
#
# xfjoint.def is the common .DEF file w/ all the class info.
# SOM class DLLs must export certain structures so that the
# SOM kernel knows what classes it can find in a DLL. Since the SOM
# compiler cannot create a common .DEF file for one DLL when
# several IDL files are used (but creates a different DEF file for
# each class instead), we have assembled xfjoint.def manually
# from the class-specific .DEF files created by the SOM compiler.
#
xfldr.dll: xfjoint.def $(OBJS) $(STEM).res
         $(LINK) /OUT:$(STEM).dll xfjoint.def $(LIBS) $(OBJS)
         $(RC) -x2 $(STEM).res $(STEM).dll


