/*
 * xfdesk.c:
 *      This file contains the following major XFolder parts:
 *
 *      -  XFldDesktop SOM stuff
 *      -  all the XShutdown routines
 *      -  the XFolder Worker thread (for doing stuff in the background)
 *      -  the XFolder Quick thread (for playing sounds in the background)
 *      -  the XFolder Watchdog thread (for terminating hanging apps)
 *
 *      Check the other files starting with xf* for the
 *      other XFolder classes.
 *
 *      Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

/*
 *  This file was generated by the SOM Compiler and Emitter Framework.
 *  Generated using:
 *      SOM Emitter emitctm: 2.41
 */

#pragma langlvl(extended)

#ifndef SOM_Module_xfdesk_Source
#define SOM_Module_xfdesk_Source
#endif
#define XFldDesktop_Class_Source
#define M_XFldDesktop_Class_Source

#include <process.h>

// #define INCL_DOSFILEMGR
#define INCL_DOSMODULEMGR
#define INCL_DOSPROCESS
#define INCL_DOSSEMAPHORES
#define INCL_DOSEXCEPTIONS
#define INCL_DOSERRORS
#define INCL_DOSDEVICES
#define INCL_DOSDEVIOCTL

// pmwin.h
#define INCL_WINWINDOWMGR
#define INCL_WINFRAMEMGR        // WM_FORMATFRAME etc.
#define INCL_WINPOINTERS

#define INCL_WINCOUNTRY         // WinCompareStrings
#define INCL_WINTIMER
#define INCL_WINSYS             // presparams

// dialogs and controls therein
#define INCL_WINDIALOGS
#define INCL_WINSTATICS
#define INCL_WINMENUS
#define INCL_WINENTRYFIELDS
#define INCL_WINBUTTONS
#define INCL_WINLISTBOXES

// pmstddlg.h
#define INCL_WINSTDBOOK         // notebooks

#define INCL_WINSHELLDATA       // profile funcs
#define INCL_WINSWITCHLIST
#define INCL_WINPROGRAMLIST     // PROGDETAILS etc.

#define INCL_WINWORKPLACE

#define INCL_GPILOGCOLORTABLE
#define INCL_GPIPRIMITIVES

#include <os2.h>

// multimedia includes
#define INCL_MACHDR
#define INCL_MCIOS2
#define INCL_OS2MM
#include <os2me.h>

#include "xfdesk.ih"

// we need to include the following also, because
// SOM doesn't know that WPFolder and WPObject
// have been replaced
#include "xfldr.h"
#include "xfobj.h"

#include "common.h"           // this includes other libs

#include "apm.h"              // APM power-off for XShutdown

#include <wpshadow.h>      // WPShadow

#pragma  linkage (main, optlink)

// thread IDs
TID             tidWorkerThread = NULLHANDLE,       // XFolder Worker thread,
                tidQuickThread = NULLHANDLE,
                tidWorkplaceThread = NULLHANDLE;    // Workplace thread (PMSHELL.EXE)

// thread object windows
HWND            hwndWorkerObject = NULLHANDLE,
                hwndQuickObject = NULLHANDLE;

BOOL            WorkerThreadHighPriority = FALSE;

ULONG           ulAwakeMsgCount = 0;

// this flag is initially set to TRUE and will cause the
// exit list to write a respective key to the INI files
// so that the startup folder shan't be processed upon the
// next WPS initialization; the flag is only set to FALSE
// if WPSRESET closed all programs successfully
BOOL            PreventNextStartup = TRUE;

// address of awake WarpCenter
WPObject        *pAwakeWarpCenter = NULL;

LONG            lAwakeObjectsCount = 0;

// shutdown threads
PTHREADINFO     ptiUpdateThread = NULL,
                ptiShutdownThread = NULL,
                ptiWatchdogThread = NULL;

BOOL            ShutdownRunning = FALSE;

// debug flags for trap log
ULONG           ulShutdownFunc = 0,
                ulShutdownFunc2 = 0,
                ulWorkerFunc = 0,
                ulWorkerFunc2 = 0,
                ulQuickFunc = 0,
                ulQuickFunc2 = 0;

/* root of linked list to remember awake objects which
   need to be saved at shutdown; this list is maintained
   by the Worker thread and evaluated by fnwpShutdown */

// root of this linked list
POBJECTLISTITEM
                pliAwakeObjectsFirst = NULL,
                pliAwakeObjectsLast = NULL;
// mutex semaphore for access to this list
HMTX            hmtxAwakeObjects = NULLHANDLE;

/*
 * cmnDescribeThreadInfo:
 *      this copies information about the running XFolder
 *      threads into szBuffer; used by the exception handlers
 *      and exit lists for ?:\XFLDTRAP.LOG
 */

void cmnDescribeThreadInfo(PSZ szBuffer)
{
    PTIB    ptib = NULL;
    PPIB    ppib = NULL;
    CHAR    szBuffer2[CCHMAXPATH];

    if (DosGetInfoBlocks(&ptib, &ppib) == NO_ERROR) {
        // process info
        if (ppib) {
            DosQueryModuleName(ppib->pib_hmte, sizeof(szBuffer2), szBuffer2);
            sprintf(szBuffer+strlen(szBuffer),
                "\nProcess information: PID 0x%lX, module handle: 0x%lX (%s)\n",
                ppib->pib_ulpid, ppib->pib_hmte, szBuffer2);
        } else
            strcat(szBuffer, "\nProcess information was not available.");

        // thread info
        if (ptib) {
            if (ptib->tib_ptib2) {

                sprintf(szBuffer+strlen(szBuffer),
                    "Thread information: TID 0x%lX",
                    ptib->tib_ptib2->tib2_ultid);

                if (ptib->tib_ptib2->tib2_ultid == tidWorkerThread) {
                        strcat(szBuffer, " (XFolder's Worker thread)\n");
                        sprintf(szBuffer+strlen(szBuffer), "Function code 2: %d\n", ulWorkerFunc2);
                } else if (ptib->tib_ptib2->tib2_ultid == tidQuickThread) {
                        strcat(szBuffer, " (XFolder's Quick thread)\n");
                        sprintf(szBuffer+strlen(szBuffer), "Function code 2: %d\n", ulQuickFunc2);
                } else if (ptib->tib_ptib2->tib2_ultid == thrQueryID(ptiUpdateThread)) {
                        strcat(szBuffer, " (XFolder's Update thread)\n");
                        sprintf(szBuffer+strlen(szBuffer), "Function code 2: %d\n", 0);
                } else if (ptib->tib_ptib2->tib2_ultid == thrQueryID(ptiShutdownThread)) {
                        strcat(szBuffer, " (XFolder's Shutdown thread)\n");
                        sprintf(szBuffer+strlen(szBuffer), "Function code 2: %d\n", ulShutdownFunc2);
                } else if (ptib->tib_ptib2->tib2_ultid == tidWorkplaceThread) {
                        strcat(szBuffer, " (PMSHELL's Workplace thread)\n");
                        sprintf(szBuffer+strlen(szBuffer), "Function code 2: %d\n",
                                *cmnGetWorkplaceFunc2());
                } else {
                    strcat(szBuffer, " (unknown identity)\n");
                }
            }
            else
                strcat(szBuffer, "Thread ID cannot be determined.\n");
        } else
            strcat(szBuffer, "Thread information was not available.\n");

        strcat(szBuffer, "\nThe following running XFolder threads could be identified:\n");
        sprintf(szBuffer+strlen(szBuffer), "  Workplace thread ID: 0x%lX\n", tidWorkplaceThread);
        if (tidWorkerThread) {
            sprintf(szBuffer+strlen(szBuffer), "  XFolder Worker thread ID: 0x%lX\n",
                tidWorkerThread);
        }
        if (tidQuickThread) {
            sprintf(szBuffer+strlen(szBuffer), "  XFolder Quick thread ID: 0x%lX\n",
                tidQuickThread);
        }
        if (thrQueryID(ptiShutdownThread)) {
            sprintf(szBuffer+strlen(szBuffer), "  Shutdown thread ID: 0x%lX\n",
                thrQueryID(ptiShutdownThread));
        }
        if (thrQueryID(ptiUpdateThread)) {
            sprintf(szBuffer+strlen(szBuffer), "  Update thread ID: 0x%lX\n",
                thrQueryID(ptiUpdateThread));
        }
    }
}

/*
 * cmnRegisterFunc:
 *
 */

typedef struct _FUNCINFO {
    PSZ     pszFile;
    PSZ     pszFunction;
    ULONG   ulLine;
} FUNCINFO, *PFUNCINFO;

PFUNCINFO   pfiWorkerFirst = NULL,      pfiWorkerLast = NULL,
            pfiQuickFirst = NULL,       pfiQuickLast = NULL,
            pfiUpdateFirst = NULL,      pfiUpdateLast = NULL,
            pfiShutdownFirst = NULL,    pfiShutdownLast = NULL,
            pfiWorkplaceFirst = NULL,   pfiWorkplaceLast = NULL;

VOID cmnRegisterFunc(PSZ pszFile, PSZ pszFunction, ULONG ulLine)
{
    PFUNCINFO    *ppFuncInfoFirst = NULL, *ppFuncInfoLast = NULL;
    PFUNCINFO    pfiNew = NULL;
    PTIB         ptib;
    PPIB         ppib;

    if (DosGetInfoBlocks(&ptib, &ppib) == NO_ERROR) {
        // thread info
        if (ptib) {
            if (ptib->tib_ptib2) {
                if (ptib->tib_ptib2->tib2_ultid == tidWorkerThread) {
                        ppFuncInfoFirst = &pfiWorkerFirst;
                        ppFuncInfoLast = &pfiWorkerLast;
                } else if (ptib->tib_ptib2->tib2_ultid == tidWorkplaceThread) {
                        ppFuncInfoFirst = &pfiWorkplaceFirst;
                        ppFuncInfoLast = &pfiWorkplaceLast;
                } else if (ptib->tib_ptib2->tib2_ultid == tidQuickThread) {
                        ppFuncInfoFirst = &pfiQuickFirst;
                        ppFuncInfoLast = &pfiQuickLast;
                } else if (ptib->tib_ptib2->tib2_ultid == thrQueryID(ptiUpdateThread)) {
                        ppFuncInfoFirst = &pfiUpdateFirst;
                        ppFuncInfoLast = &pfiUpdateLast;
                } else if (ptib->tib_ptib2->tib2_ultid == thrQueryID(ptiShutdownThread)) {
                        ppFuncInfoFirst = &pfiShutdownFirst;
                        ppFuncInfoLast = &pfiShutdownLast;
                }

                if (ppFuncInfoFirst) {
                    pfiNew = malloc(sizeof(PFUNCINFO));
                    if (pfiNew) {
                        pfiNew->pszFile = pszFile;
                        pfiNew->pszFunction = pszFunction;
                        pfiNew->ulLine = ulLine;

                        lstAppendItem((PLISTITEM*)ppFuncInfoFirst,
                                      (PLISTITEM*)ppFuncInfoLast,
                                (PLISTITEM)pfiNew, 0, 0);
                    }
                }
            }
        }
    }
}

VOID cmnDeregisterFunc(VOID)
{
    PFUNCINFO    *ppFuncInfoFirst = NULL, *ppFuncInfoLast = NULL;
    PTIB         ptib;
    PPIB         ppib;

    if (DosGetInfoBlocks(&ptib, &ppib) == NO_ERROR) {
        // thread info
        if (ptib) {
            if (ptib->tib_ptib2) {
                if (ptib->tib_ptib2->tib2_ultid == tidWorkerThread) {
                        ppFuncInfoFirst = &pfiWorkerFirst;
                        ppFuncInfoLast = &pfiWorkerLast;
                } else if (ptib->tib_ptib2->tib2_ultid == tidWorkplaceThread) {
                        ppFuncInfoFirst = &pfiWorkplaceFirst;
                        ppFuncInfoLast = &pfiWorkplaceLast;
                } else if (ptib->tib_ptib2->tib2_ultid == tidQuickThread) {
                        ppFuncInfoFirst = &pfiQuickFirst;
                        ppFuncInfoLast = &pfiQuickLast;
                } else if (ptib->tib_ptib2->tib2_ultid == thrQueryID(ptiUpdateThread)) {
                        ppFuncInfoFirst = &pfiUpdateFirst;
                        ppFuncInfoLast = &pfiUpdateLast;
                } else if (ptib->tib_ptib2->tib2_ultid == thrQueryID(ptiShutdownThread)) {
                        ppFuncInfoFirst = &pfiShutdownFirst;
                        ppFuncInfoLast = &pfiShutdownLast;
                }

                if (ppFuncInfoFirst) {
                    lstRemoveItem((PLISTITEM*)ppFuncInfoFirst,
                                  (PLISTITEM*)ppFuncInfoLast,
                            (PLISTITEM)*ppFuncInfoFirst, 0, 0);
                }
            }
        }
    }
}

/********************************************************************
 *                                                                  *
 *   here come the XFldDesktop instance methods                     *
 *                                                                  *
 ********************************************************************/

void _System fntShutdownThread(ULONG ulData);

BOOL        optReboot, optConfirm, optDebug, optRestartWPS,
            optWPSCloseWindows, optAutoCloseVIO, optLog,
            optAnimate, optAPMPowerOff,
            optWPSReuseStartupFolder;

CHAR        szRebootCommand[CCHMAXPATH];

HWND        hwndActiveDesktop;
WPDesktop   *pActiveDesktop;

HPOINTER    hSDIcon;

SHORT       sAniCount = 0;

/*
 * funcs for loading / freeing the shutdown animation
 *
 */

VOID cmnLoadShutdownAnimation(HPOINTER ahptr[ANIMAX], HMODULE hmod)
{
    ahptr[0] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM1);
    ahptr[1] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM2);
    ahptr[2] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM3);
    ahptr[3] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM4);
    ahptr[4] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM5);
    ahptr[5] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM4);
    ahptr[6] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM3);
    ahptr[7] = WinLoadPointer(HWND_DESKTOP, hmod, ID_ICONSDANIM2);
}

VOID cmnFreeShutdownAnimation(HPOINTER ahptr[ANIMAX])
{
    USHORT us;
    for (us = 0; us < ANIMAX; us++)
        WinDestroyPointer(ahptr[us]);
}

/*
 * fnwpConfirm:
 *      dlg proc for XShutdown confirmation windows
 */

MRESULT EXPENTRY fnwpConfirm(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = MPNULL;
    switch (msg) {
         case WM_CONTROL: {
            if (SHORT1FROMMP(mp1) == ID_SDDI_REBOOT)
                WinEnableDlgItem(hwndDlg, ID_SDDI_BOOTMGR,
                        WinIsDlgItemChecked(hwndDlg, ID_SDDI_REBOOT));
            mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
        break; } //continue

        default:
            mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
    }
    return (mrc);
}

/*
 * cmnConfirmShutdown:
 *
 */

ULONG cmnConfirmShutdown(VOID) {
    PANIMATION  pAnimation;
    ULONG       ulReturn;
    HWND        hwndConfirm;
    HMODULE     hmodResource = NLS_MODULE;
    CHAR        szDefault[100];
    ULONG       ulKeyLength;
    PSZ         p, pINI = NULL;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    cmnSetHelpPanel(ID_XMH_XSHUTDOWN);
    hwndConfirm = WinLoadDlg(HWND_DESKTOP, NULLHANDLE,
                    fnwpConfirm,
                    hmodResource,
                    ID_SDD_CONFIRM,
                    NULL);

    hSDIcon = WinLoadPointer(HWND_DESKTOP, hmodResource,
                    ID_SDICON);

    WinPostMsg(hwndConfirm,
               WM_SETICON,
               (MPARAM)hSDIcon,
                NULL);

    // prepare confirmation box items
    WinSetDlgItemChecked(hwndConfirm, ID_SDDI_REBOOT, optReboot);
    WinSetDlgItemChecked(hwndConfirm, ID_SDDI_MESSAGEAGAIN, optConfirm);
    WinEnableDlgItem(hwndConfirm, ID_SDDI_BOOTMGR, optReboot);

    // insert ext reboot items into combo box;
    // first: "Default"
    WinLoadString(WinQueryAnchorBlock(HWND_DESKTOP),
        NLS_MODULE,
        ID_SDSI_DEFAULT,
        sizeof(szDefault)-1,
        szDefault);

    WinSendDlgItemMsg(hwndConfirm, ID_SDDI_BOOTMGR,
                    LM_INSERTITEM,
                    (MPARAM)LIT_END,
                    (MPARAM)szDefault);

    // now check for more items in OS2.INI
    if (PrfQueryProfileSize(HINI_USER,
                INIAPP, INIKEY_BOOTMGR,
                &ulKeyLength))
    {
        // _Pmpf(( "Size: %d", ulKeyLength ));
        // items exist: evaluate
        pINI = malloc(ulKeyLength);
        if (pINI)
        {
            PrfQueryProfileData(HINI_USER,
                        INIAPP, INIKEY_BOOTMGR,
                        pINI,
                        &ulKeyLength);
            p = pINI;
            // _Pmpf(( "%s", p ));
            while (strlen(p)) {
                WinSendDlgItemMsg(hwndConfirm, ID_SDDI_BOOTMGR,
                                LM_INSERTITEM,
                                (MPARAM)LIT_END,
                                (MPARAM)p);
                p += (strlen(p)+1);
                p += (strlen(p)+1);
            }
        }
        WinSendDlgItemMsg(hwndConfirm, ID_SDDI_BOOTMGR,
                        LM_SELECTITEM,
                        (MPARAM)pGlobalSettings->LastRebootExt, // item index
                        (MPARAM)TRUE); // select (not deselect)

    } else {
        // no items found: disable
        WinSendDlgItemMsg(hwndConfirm, ID_SDDI_BOOTMGR,
                        LM_SELECTITEM,
                        (MPARAM)0, // item index
                        (MPARAM)TRUE); // select (not deselect)
        // WinEnableDlgItem(hwndConfirm, ID_SDDI_BOOTMGR, FALSE);
    }

    WinCenterWindow(hwndConfirm);

    pAnimation = malloc( sizeof(ANIMATION) + (7 * sizeof(HPOINTER)) );

    pAnimation->cb = sizeof(ANIMATION) + (7 * sizeof(HPOINTER));
    pAnimation->hwndIcon = WinWindowFromID(hwndConfirm, ID_SDDI_ICON);
    pAnimation->usAniCount = 8;
    pAnimation->ulDelay = 150;
    cmnLoadShutdownAnimation(&(pAnimation->ahptrAniIcons[0]),
            _xfclsQueryModuleHandle(_XFldObject));

    WinPostMsg(hwndConfirm, WM_ANIMATE, (MPARAM)pAnimation, MPNULL);

    ulReturn = WinProcessDlg(hwndConfirm);

    cmnFreeShutdownAnimation(&(pAnimation->ahptrAniIcons[0]));
    free(pAnimation);

    if (ulReturn == DID_OK) {
        optReboot = (WinIsDlgItemChecked(hwndConfirm, ID_SDDI_REBOOT));
        if (!(WinIsDlgItemChecked(hwndConfirm, ID_SDDI_MESSAGEAGAIN)))
        {
            pGlobalSettings->XShutdown &= ~XSD_CONFIRM;
        }
        if (optReboot) {
            USHORT usSelected = (USHORT)WinSendDlgItemMsg(hwndConfirm, ID_SDDI_BOOTMGR,
                    LM_QUERYSELECTION, (MPARAM)LIT_CURSOR, MPNULL);
            if (usSelected > 0) {
                USHORT us;
                p = pINI;
                for (us = 0; us < (usSelected-1); us++) {
                    p += (strlen(p)+1);
                    p += (strlen(p)+1);
                }
                p += (strlen(p)+1);
                strcpy(szRebootCommand, p);
            }
            pGlobalSettings->LastRebootExt = usSelected;
        }
        PrfWriteProfileData(HINI_USERPROFILE, INIAPP, INIKEY_GLOBALSETTINGS,
             pGlobalSettings, sizeof(GLOBALSETTINGS));
    }

    if (pINI)
        free(pINI);

    WinDestroyWindow(hwndConfirm);

    return (ulReturn);
}

/*
 * xfInitiateShutdown:
 *           Shutdown entry point; checks pGlobalSettings->XShutdown for settings:
 *                -- XSD_CONFIRM: display confirmation dlg box
 *                -- XSD_REBOOT:  reboot after shutdown
 *                -- XSD_DEBUG:   display debug window, only simulate shutdown
 *           If compiled with XFLDR_DEBUG defined (at top of this file), debug mode
 *           will also be turned on if the SHIFT key is pressed at call time.
 *
 *           This routine will display a confirmation box, if the settings want
 *           it, and then start the main shutdown thread (fntShutdownThread),
 *           which will keep running even after shutdown is complete, unless the
 *           user presses the "Cancel shutdown" button.
 *
 *           Although xfInitiateShutdown does return almost immediately (after the
 *           confirmation dlg is dismissed), the shutdown will proceed in the separate thread.
 */

SOM_Scope BOOL  SOMLINK xfdesk_xfInitiateShutdown(XFldDesktop *somSelf,
                                                     HWND hwndCallingDesktop)
{
    ULONG ulSpooled = 0;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    // XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_xfInitiateShutdown");

    if (thrQueryID(ptiShutdownThread))
        // shutdown thread already running: return!
        return FALSE;

    ShutdownRunning = TRUE;

    hwndActiveDesktop = hwndCallingDesktop;
    pActiveDesktop = somSelf;

    optReboot =  ((pGlobalSettings->XShutdown & XSD_REBOOT) != 0);
    optRestartWPS = FALSE;
    optWPSCloseWindows = TRUE;
    optWPSReuseStartupFolder = optWPSCloseWindows;
    optConfirm = ((pGlobalSettings->XShutdown & XSD_CONFIRM) != 0);
    optAutoCloseVIO = ((pGlobalSettings->XShutdown & XSD_AUTOCLOSEVIO) != 0);
    optLog = ((pGlobalSettings->XShutdown & XSD_LOG) != 0);
    optAnimate = ((pGlobalSettings->XShutdown & XSD_ANIMATE) != 0);
    optAPMPowerOff = (  ((pGlobalSettings->XShutdown & XSD_APMPOWEROFF) != 0)
                      && (apmPowerOffSupported())
                     );

    #ifdef DEBUG_SHUTDOWN
        optDebug = DosQueryShiftState();
    #else
        optDebug = FALSE;
    #endif

    strcpy(szRebootCommand, "");

    if (optConfirm) {
        ULONG ulReturn = cmnConfirmShutdown();
        if (ulReturn != DID_OK) {
            ShutdownRunning = FALSE;
            return FALSE;
        }
    }

    // check for pending spool jobs
    ulSpooled = cmnQueryPendingSpoolJobs();
    if (ulSpooled) {
        CHAR szTemp[20];
        PSZ pTable[1];
        sprintf(szTemp, "%d", ulSpooled);
        pTable[0] = szTemp;
        if (cmnMessageBoxMsgExt(HWND_DESKTOP, 114, pTable, 1, 115,
                        MB_YESNO | MB_DEFBUTTON2)
                    != MBID_YES)
        {
            ShutdownRunning = FALSE;
            return (FALSE);
        }
    }

    // everything OK: create shutdown thread
    thrCreate(&ptiShutdownThread, fntShutdownThread, NULLHANDLE);
    cmnPlaySystemSound(MMSOUND_XFLD_SHUTDOWN);
    return TRUE;
}

/*
 * cmnInitiateShutdownExt:
 *      just like the XFldDesktop method, but this one
 *      allows setting all the shutdown parameters by
 *      using the SHUTDOWNPARAMS structure. This is used
 *      for calling XShutdown externally, which is done
 *      by sending XOM_EXTERNALSHUTDOWN to the XFolder
 *      object window (see xfobj.c).
 *      NOTE: The memory block pointed to by psdp is
 *      not released by this function.
 */

BOOL cmnInitiateShutdownExt(PSHUTDOWNPARAMS psdp)
{
    if (thrQueryID(ptiShutdownThread))
        // shutdown thread already running: return!
        return FALSE;

    if (psdp == NULL)
        return FALSE;

    ShutdownRunning = TRUE;

    hwndActiveDesktop = _wpclsQueryActiveDesktopHWND(_WPDesktop);
    pActiveDesktop = _wpclsQueryActiveDesktop(_WPDesktop);

    optReboot =  psdp->optReboot;
    optRestartWPS = psdp->optRestartWPS;
    optWPSCloseWindows = psdp->optWPSCloseWindows;
    optConfirm = psdp->optConfirm;
    optAutoCloseVIO = psdp->optAutoCloseVIO;
    optLog = psdp->optLog;
    optAnimate = psdp->optAnimate;

    optDebug = psdp->optDebug;

    strcpy(szRebootCommand, psdp->szRebootCommand);

    if (optConfirm) {
        ULONG ulReturn = cmnConfirmShutdown();
        if (ulReturn != DID_OK) {
            ShutdownRunning = FALSE;
            return FALSE;
        }
    }

    // everything OK: create shutdown thread
    thrCreate(&ptiShutdownThread, fntShutdownThread, NULLHANDLE);
    cmnPlaySystemSound(MMSOUND_XFLD_SHUTDOWN);
    return TRUE;
}

/*
 * xfRestartWPS:
 *           pretty similar to xfInitiateShutdown, i.e. will also show a
 *           confirmation box and start the Shutdown thread, except that flags
 *           are set differently so that after closing all windows, no shutdown
 *           is performed, but only the WPS is restarted.
 */

SOM_Scope BOOL  SOMLINK xfdesk_xfRestartWPS(XFldDesktop *somSelf,
                                               HWND hwndCallingDesktop)
{
    PGLOBALSETTINGS pGlobalSettings =
            cmnQueryGlobalSettings();

    // XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_xfRestartWPS");

    if (thrQueryID(ptiShutdownThread))
        // shutdown thread already running: return!
        return FALSE;

    ShutdownRunning = TRUE;

    hwndActiveDesktop = hwndCallingDesktop;
    pActiveDesktop = somSelf;

    optReboot =  FALSE;
    optRestartWPS = TRUE;
    optWPSCloseWindows = ((pGlobalSettings->XShutdown & XSD_WPS_CLOSEWINDOWS) != 0);
    optWPSReuseStartupFolder = optWPSCloseWindows;
    optConfirm = ((pGlobalSettings->XShutdown & XSD_CONFIRM) != 0);
    optAutoCloseVIO = ((pGlobalSettings->XShutdown & XSD_AUTOCLOSEVIO) != 0);
    optLog =  ((pGlobalSettings->XShutdown & XSD_LOG) != 0);
    #ifdef DEBUG_SHUTDOWN
        optDebug = DosQueryShiftState();
    #else
        optDebug = FALSE;
    #endif

    if (optConfirm) {
        PANIMATION  pAnimation;
        ULONG       ulReturn;
        HWND        hwndConfirm;
        HMODULE     hmodResource = NLS_MODULE;

        cmnSetHelpPanel(ID_XMH_RESTARTWPS);
        hwndConfirm = WinLoadDlg(HWND_DESKTOP, NULLHANDLE,
                        fnwpConfirm,
                        hmodResource,
                        ID_SDD_CONFIRMWPS,
                        NULL);

        hSDIcon = WinLoadPointer(HWND_DESKTOP, hmodResource,
                        ID_SDICON);

        WinPostMsg(hwndConfirm,
                   WM_SETICON,
                   (MPARAM)hSDIcon,
                    NULL);

        WinSetDlgItemChecked(hwndConfirm, ID_SDDI_WPS_CLOSEWINDOWS, optWPSCloseWindows);
        WinSetDlgItemChecked(hwndConfirm, ID_SDDI_WPS_STARTUPFOLDER, optWPSCloseWindows);
        WinSetDlgItemChecked(hwndConfirm, ID_SDDI_MESSAGEAGAIN, optConfirm);
        WinCenterWindow(hwndConfirm);

        pAnimation = malloc( sizeof(ANIMATION) + (7 * sizeof(HPOINTER)) );

        pAnimation->cb = sizeof(ANIMATION) + (7 * sizeof(HPOINTER));
        pAnimation->hwndIcon = WinWindowFromID(hwndConfirm, ID_SDDI_ICON);
        pAnimation->usAniCount = 8;
        pAnimation->ulDelay = 150;
        cmnLoadShutdownAnimation(&(pAnimation->ahptrAniIcons[0]),
                _xfclsQueryModuleHandle(_XFldObject));

        WinPostMsg(hwndConfirm, WM_ANIMATE, (MPARAM)pAnimation, MPNULL);

        // *** go!
        ulReturn = WinProcessDlg(hwndConfirm);

        cmnFreeShutdownAnimation(&(pAnimation->ahptrAniIcons[0]));
        free(pAnimation);

        if (ulReturn == DID_OK) {
            optWPSCloseWindows =
                    WinIsDlgItemChecked(hwndConfirm, ID_SDDI_WPS_CLOSEWINDOWS);
            if (optWPSCloseWindows)
                pGlobalSettings->XShutdown |= XSD_WPS_CLOSEWINDOWS;
            else
                pGlobalSettings->XShutdown &= ~XSD_WPS_CLOSEWINDOWS;
            optWPSReuseStartupFolder =
                    WinIsDlgItemChecked(hwndConfirm, ID_SDDI_WPS_STARTUPFOLDER);
            if (!(WinIsDlgItemChecked(hwndConfirm, ID_SDDI_MESSAGEAGAIN)))
                pGlobalSettings->XShutdown &= ~XSD_CONFIRM;
            PrfWriteProfileData(HINI_USERPROFILE, INIAPP, INIKEY_GLOBALSETTINGS,
                     pGlobalSettings, sizeof(GLOBALSETTINGS));
        }

        WinDestroyWindow(hwndConfirm);

        if (ulReturn != DID_OK) {
            ShutdownRunning = FALSE;
            return FALSE;
        }
    }

    thrCreate(&ptiShutdownThread, fntShutdownThread, NULLHANDLE);

    cmnPlaySystemSound(MMSOUND_XFLD_RESTARTWPS);

    ShutdownRunning = FALSE;
    return TRUE;
}

/*
 * wpModifyPopupMenu:
 *           play with the Desktop menu entries
 *           (Shutdown and such)
 */

SOM_Scope BOOL  SOMLINK xfdesk_wpModifyPopupMenu(XFldDesktop *somSelf,
                                                    HWND hwndMenu,
                                                    HWND hwndCnr,
                                                    ULONG iPosition)
{
    BOOL rc;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    // XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_wpModifyPopupMenu");

    /* calling the parent (which is XFolder!) will insert all the
       variable menu items */
    rc = XFldDesktop_parent_WPDesktop_wpModifyPopupMenu(somSelf,
                                                           hwndMenu,
                                                           hwndCnr,
                                                           iPosition);

    if (_wpIsCurrentDesktop(somSelf))
    {
        if (pGlobalSettings->XShutdown & XSD_RESTARTWPS) {
            WinInsertMenuItem(hwndMenu,
                        (SHORT)WinSendMsg(hwndMenu,   // position
                                MM_ITEMPOSITIONFROMID,
                                MPFROM2SHORT(WPMENUID_SHUTDOWN, FALSE),
                                MPNULL) + 1,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_RESTARTWPS),
                        (cmnQueryNLSStrings())->pszRestartWPS,
                        MIS_TEXT,
                        ((thrQueryID(ptiShutdownThread) || (ShutdownRunning))
                            ? MIA_DISABLED : 0));

            if ((pGlobalSettings->XShutdown & XSD_CONFIRM) == 0)
                WinMenuRemoveEllipse(hwndMenu, (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_RESTARTWPS));
        }

        if (pGlobalSettings->XShutdown & XSD_DTM_LOCKUP)
            WinSendMsg(hwndMenu, MM_REMOVEITEM,
                MPFROM2SHORT(WPMENUID_LOCKUP, FALSE),
                MPNULL);
        if (pGlobalSettings->XShutdown & XSD_DTM_SYSTEMSETUP)
            WinSendMsg(hwndMenu, MM_REMOVEITEM,
                MPFROM2SHORT(WPMENUID_SYSTEMSETUP, FALSE),
                MPNULL);

        if (pGlobalSettings->XShutdown & XSD_DTM_SHUTDOWN)
            WinSendMsg(hwndMenu, MM_REMOVEITEM,
                MPFROM2SHORT(WPMENUID_SHUTDOWN, FALSE),
                MPNULL);
        else {
            if (    (ShutdownRunning)
                 || (thrQueryID(ptiShutdownThread))
               )
            { // disable "Shutdown"
                WinSendMsg(hwndMenu, MM_SETITEMATTR,
                    MPFROM2SHORT(WPMENUID_SHUTDOWN, FALSE),
                    MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
            }
            if ((pGlobalSettings->XShutdown & (XSD_CONFIRM | XSD_ENABLED)) == XSD_ENABLED)
                WinMenuRemoveEllipse(hwndMenu, WPMENUID_SHUTDOWN);
        }
    }

    return (rc);
}

/*
 * wpMenuItemSelected:
 *           process input when any menu item was selected;
 *           intercept Shutdown and such
 */

SOM_Scope BOOL  SOMLINK xfdesk_wpMenuItemSelected(XFldDesktop *somSelf,
                                                     HWND hwndFrame,
                                                     ULONG ulMenuId)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    // XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_wpMenuItemSelected");

    if (!ShutdownRunning)
    {
        if ((ulMenuId - (pGlobalSettings->VarMenuOffset)) == ID_XFMI_OFS_RESTARTWPS)
        {
            _xfRestartWPS(somSelf, hwndFrame);
            return (TRUE);
        }
        else if (    (ulMenuId == WPMENUID_SHUTDOWN)
                 &&  ((pGlobalSettings->XShutdown & XSD_ENABLED) != 0)
                 &&  (pGlobalSettings->NoWorkerThread == 0)
                )
        {
            _xfInitiateShutdown(somSelf, hwndFrame);
            return (TRUE);
        }
    }

    return (XFldDesktop_parent_WPDesktop_wpMenuItemSelected(somSelf,
                                                            hwndFrame,
                                                            ulMenuId));
}

/*
 * wpAddSettingsPages:
 *           insert "XFldDesktop" settings page into notebook; as opposed
 *           to the "XFolder" page, which deals with instance data, we save
 *           these settings in the GlobalSettings structure, because there
 *           should ever be only one desktop
 */

SOM_Scope BOOL  SOMLINK xfdesk_wpAddSettingsPages(XFldDesktop *somSelf,
                                                     HWND hwndNotebook)
{
    BOOL            rc;
    PAGEINFO        pi;

    // XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_wpAddSettingsPages");

    rc = (XFldDesktop_parent_WPDesktop_wpAddSettingsPages(somSelf,
                                                            hwndNotebook));
    if (_wpIsCurrentDesktop(somSelf))
    {
        PSZ pszHelpLibrary = cmnQueryHelpLibrary();

        // insert Shutdown settings page 2
        memset((PCH)&pi, 0, sizeof(PAGEINFO));
        pi.cb                  = sizeof(PAGEINFO);
        pi.hwndPage            = NULLHANDLE;
        // the dlg wnd proc is def'd in xfldr.c
        pi.pfnwp               = fnwpSettingsDlg_Desktop2;
        pi.resid               = NLS_MODULE;
        pi.pCreateParams       = somSelf;
        pi.dlgid               = ID_XSD_SETTINGS_DTP2;
        pi.usPageStyleFlags    = BKA_STATUSTEXTON;      // no major tab;
        pi.usPageInsertFlags   = BKA_FIRST;
        pi.usSettingsFlags     = SETTINGS_PAGE_NUMBERS; // enumerate in status line
        pi.pszName             = "~XDesktop";

        pi.pszHelpLibraryName  = pszHelpLibrary;
        pi.idDefaultHelpPanel  = ID_XSH_SETTINGS_DTP1+1;

        _wpInsertSettingsPage(somSelf, hwndNotebook, &pi);

        // insert Shutdown settings page 1
        memset((PCH)&pi, 0, sizeof(PAGEINFO));
        pi.cb                  = sizeof(PAGEINFO);
        pi.hwndPage            = NULLHANDLE;
        // the dlg wnd proc is def'd in xfldr.c
        pi.pfnwp               = fnwpSettingsDlg_Desktop1;
        pi.resid               = NLS_MODULE;
        pi.pCreateParams       = somSelf;
        pi.dlgid               = ID_XSD_SETTINGS_DTP1;
        pi.usPageStyleFlags    = BKA_STATUSTEXTON | BKA_MAJOR;   // major tab;
        pi.usPageInsertFlags   = BKA_FIRST;
        pi.usSettingsFlags     = SETTINGS_PAGE_NUMBERS; // enumerate in status line
        pi.pszName             = "~XDesktop";

        pi.pszHelpLibraryName  = pszHelpLibrary;
        pi.idDefaultHelpPanel  = ID_XSH_SETTINGS_DTP1;

        _wpInsertSettingsPage(somSelf, hwndNotebook, &pi);
    }
    return (rc);
}

/*
 * wpOpen:
 *           after the Desktop is opened, we begin the the startup process,
 *           so we need to give a signal to the Worker thread
 */

SOM_Scope HWND  SOMLINK xfdesk_wpOpen(XFldDesktop *somSelf,
                                         HWND hwndCnr, ULONG ulView,
                                         ULONG param)
{
    HWND hwnd;
    XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_wpOpen");

    hwnd = XFldDesktop_parent_WPDesktop_wpOpen(somSelf, hwndCnr,
                                                ulView, param);
    if (!_DesktopOpen)
        if (_wpIsCurrentDesktop(somSelf)) {
            _DesktopOpen = TRUE;
            cmnPostWorkerMsg(WOM_DESKTOPREADY, (MPARAM)somSelf, MPNULL);
            // the worker thread will now loop until the Desktop
            // is populated also
        }

    return (hwnd);
}

/*
 * wpInitData:
 *           initialize XFldDesktop instance data
 */

SOM_Scope void  SOMLINK xfdesk_wpInitData(XFldDesktop *somSelf)
{
    XFldDesktopData *somThis = XFldDesktopGetData(somSelf);
    XFldDesktopMethodDebug("XFldDesktop","xfdesk_wpInitData");

    _DesktopOpen = FALSE;

    XFldDesktop_parent_WPDesktop_wpInitData(somSelf);
}

/*
 * wpclsQueryTitle:
 *           tell the WPS the new class name: Desktop (XFolder)
 */

SOM_Scope PSZ  SOMLINK xfdeskM_wpclsQueryTitle(M_XFldDesktop *somSelf)
{

    M_XFldDesktopData *somThis = M_XFldDesktopGetData(somSelf);
    M_XFldDesktopMethodDebug("M_XFldDesktop","xfdeskM_wpclsQueryTitle");

    strcpy(_szTitle, M_XFldDesktop_parent_M_WPDesktop_wpclsQueryTitle(somSelf));
    strcat(_szTitle, " (XFolder)");

    return (_szTitle);
}

/********************************************************************
 *                                                                  *
 *   here come all the different shutdown functions / threads       *
 *                                                                  *
 ********************************************************************/

MRESULT EXPENTRY fnwpShutdown(HWND hwndFrame, ULONG msg, MPARAM mp1, MPARAM mp2);
void _System fntUpdateThread(HWND hwndFrame);

HWND        hwndMain = NULLHANDLE,
            hwndShutdownStatus = NULLHANDLE;
PFNWP       SysWndProc;
BOOL        fAllWindowsClosed = FALSE,
            fClosingApps = FALSE,
            fShutdownBegun = FALSE;

PID         pidWPS, pidPM;
ULONG       sidWPS, sidPM;

ULONG       ulMaxItemCount,
            ulLastItemCount;

HFILE       hfShutdownLog = NULLHANDLE;

/*
 * here come the neccessary definitions and functions
 * for maintaining pliShutdownFirst, the global list of
 * windows which need to be closed; these lists are
 * maintained according to the system Tasklist
 */

typedef struct SHUTLIST_STRUCT {
    struct SHUTLIST_STRUCT  *pNext, *pPrevious;
    ULONG                   ulSize;
    SWCNTRL                 swctl;          // system tasklist structure (see PM ref.)
    WPObject                *pObject;       // NULL for non-WPS windows
    CHAR                    szClass[100];   // window class of the task's main window
} SHUTLISTITEM, *PSHUTLISTITEM;

// this is the global shutlist
PSHUTLISTITEM   pliShutdownFirst = NULL,
                pliShutdownLast = NULL,
                pliSkipped = NULL;
HMTX            hmtxShutdown = NULLHANDLE,
                hmtxSkipped = NULLHANDLE;
HEV             hevUpdated = NULLHANDLE;

// temporary storage for closing VIOs
HWND        hwndVioDlg = NULLHANDLE;
CHAR        szVioTitle[1000] = "";
SHUTLISTITEM VioItem;

// this is the user-defined list of auto-close items
typedef struct AUTOCLOSELISTITEM_STRUCT {
    struct AUTOCLOSELISTITEM_STRUCT
                        *pNext, *pPrevious;
    ULONG               ulSize;
    CHAR                szItemName[100];
    USHORT              usAction;
} AUTOCLOSELISTITEM, *PAUTOCLOSELISTITEM;

typedef struct _AUTOCLOSEWINDATA
{
    PAUTOCLOSELISTITEM  pliAutoClose;
    PANIMATION          pAnimation;
    SHORT               sSelected;
    PAUTOCLOSELISTITEM  pliSelected;
    USHORT              usItemCount;
} AUTOCLOSEWINDATA, *PAUTOCLOSEWINDATA;

PAUTOCLOSELISTITEM pliAutoCloseFirst = NULL;

/*
 * fnwpAutoCloseDetails:
 *      dlg func for "Auto-Close Details", accessed from
 *      xfldr.c
 */

MRESULT EXPENTRY fnwpAutoCloseDetails(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = (MPARAM)NULL;

    switch (msg) {
        case WM_INITDLG: {
            ULONG       ulKeyLength;
            PSZ         p, pINI;

            // create window data in QWL_USER
            PAUTOCLOSEWINDATA pData = malloc(sizeof(AUTOCLOSEWINDATA));
            memset(pData, 0, sizeof(AUTOCLOSEWINDATA));

            // set animation
            pData->pAnimation = malloc( sizeof(ANIMATION) + (7 * sizeof(HPOINTER)) );

            pData->pAnimation->cb = sizeof(ANIMATION) + (7 * sizeof(HPOINTER));
            pData->pAnimation->hwndIcon = WinWindowFromID(hwndDlg, ID_SDDI_ICON);
            pData->pAnimation->usAniCount = 8;
            pData->pAnimation->ulDelay = 150;
            cmnLoadShutdownAnimation(&(pData->pAnimation->ahptrAniIcons[0]),
                    _xfclsQueryModuleHandle(_XFldObject));
            WinPostMsg(hwndDlg, WM_ANIMATE, (MPARAM)pData->pAnimation, MPNULL);

            pData->usItemCount = 0;

            // get existing items from INI
            if (PrfQueryProfileSize(HINI_USER,
                        INIAPP, INIKEY_AUTOCLOSE,
                        &ulKeyLength))
            {
                //printf("Size: %d\n", ulKeyLength);
                // items exist: evaluate
                pINI = malloc(ulKeyLength);
                if (pINI)
                {
                    PrfQueryProfileData(HINI_USER,
                                INIAPP, INIKEY_AUTOCLOSE,
                                pINI,
                                &ulKeyLength);
                    p = pINI;
                    //printf("%s\n", p);
                    while (strlen(p)) {
                        PAUTOCLOSELISTITEM pliNew = malloc(sizeof(AUTOCLOSELISTITEM));
                        strcpy(pliNew->szItemName, p);
                        lstAppendItem((PLISTITEM*)&(pData->pliAutoClose), NULL,
                                (PLISTITEM)pliNew, 0, 0);

                        WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_LISTBOX,
                                        LM_INSERTITEM,
                                        (MPARAM)LIT_END,
                                        (MPARAM)p);
                        p += (strlen(p)+1);

                        if (strlen(p)) {
                            pliNew->usAction = *((PUSHORT)p);
                            p += sizeof(USHORT);
                        }
                        pData->usItemCount++;
                    }
                    free(pINI);
                }
            }

            WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_ITEMNAME,
                EM_SETTEXTLIMIT, (MPARAM)(100-1), MPNULL);

            WinSetWindowULong(hwndDlg, QWL_USER, (ULONG)pData);

            WinPostMsg(hwndDlg, WM_UPDATE, MPNULL, MPNULL);
        break; }

        case WM_CONTROL: {
            switch (SHORT1FROMMP(mp1)) {

                case ID_XSDI_XRB_LISTBOX: {
                    if (SHORT2FROMMP(mp1) == LN_SELECT)
                        // listbox was clicked on: update other
                        // controls with new data
                        WinSendMsg(hwndDlg, WM_UPDATE, MPNULL, MPNULL);
                break; }

                case ID_XSDI_XRB_ITEMNAME: {
                    if (SHORT2FROMMP(mp1) == EN_KILLFOCUS) {
                        // user changed item title: update data
                        PAUTOCLOSEWINDATA pData =
                                (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
                        //printf("WM_CONTROL ID_XSDI_XRB_ITEMNAME EN_KILLFOCUS\n");
                        if (pData) {
                            if (pData->pliSelected) {
                                WinQueryDlgItemText(hwndDlg, ID_XSDI_XRB_ITEMNAME,
                                    sizeof(pData->pliSelected->szItemName)-1,
                                    pData->pliSelected->szItemName);
                                WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_LISTBOX,
                                    LM_SETITEMTEXT,
                                    (MPARAM)pData->sSelected,
                                    (MPARAM)(pData->pliSelected->szItemName));
                            }
                        }
                    }
                break; }

                // radio buttons
                case ID_XSDI_ACL_WMCLOSE:
                case ID_XSDI_ACL_CTRL_C:
                case ID_XSDI_ACL_KILLSESSION:
                case ID_XSDI_ACL_SKIP: {
                    if (SHORT2FROMMP(mp1) == BN_CLICKED) {
                        PAUTOCLOSEWINDATA pData =
                                (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
                        //printf("WM_CONTROL ID_XSDI_XRB_ITEMNAME EN_KILLFOCUS\n");
                        if (pData) {
                            if (pData->pliSelected) {
                                pData->pliSelected->usAction =
                                    (SHORT1FROMMP(mp1) == ID_XSDI_ACL_WMCLOSE)
                                        ? ACL_WMCLOSE
                                    : (SHORT1FROMMP(mp1) == ID_XSDI_ACL_CTRL_C)
                                        ? ACL_CTRL_C
                                    : (SHORT1FROMMP(mp1) == ID_XSDI_ACL_KILLSESSION)
                                        ? ACL_KILLSESSION
                                    : ACL_SKIP;
                            }
                        }
                    }
                break; }

                default:
                    mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
            }
        break; }

        case WM_UPDATE: {
            // posted from various locations to wholly update
            // the dlg items
            PAUTOCLOSEWINDATA pData =
                    (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
            //printf("WM_CONTROL ID_XSDI_XRB_LISTBOX LN_SELECT\n");
            if (pData) {
                pData->pliSelected = NULL;
                pData->sSelected = (USHORT)WinSendDlgItemMsg(hwndDlg,
                        ID_XSDI_XRB_LISTBOX,
                        LM_QUERYSELECTION,
                        (MPARAM)LIT_CURSOR,
                        MPNULL);
                //printf("  Selected: %d\n", pData->sSelected);
                if (pData->sSelected != LIT_NONE) {
                    pData->pliSelected = (PAUTOCLOSELISTITEM)lstItemFromIndex(
                            (PLISTITEM)(pData->pliAutoClose),
                            pData->sSelected, 0, 0);
                }

                if (pData->pliSelected) {
                    WinSetDlgItemText(hwndDlg, ID_XSDI_XRB_ITEMNAME,
                            pData->pliSelected->szItemName);
                    switch (pData->pliSelected->usAction) {
                        case ACL_WMCLOSE:
                            WinSetDlgItemChecked(hwndDlg,
                                ID_XSDI_ACL_WMCLOSE, 1); break;
                        case ACL_CTRL_C:
                            WinSetDlgItemChecked(hwndDlg,
                                ID_XSDI_ACL_CTRL_C, 1); break;
                        case ACL_KILLSESSION:
                            WinSetDlgItemChecked(hwndDlg,
                                ID_XSDI_ACL_KILLSESSION, 1); break;
                        case ACL_SKIP:
                            WinSetDlgItemChecked(hwndDlg,
                                ID_XSDI_ACL_SKIP, 1); break;
                    }
                } else {
                    WinSetDlgItemText(hwndDlg, ID_XSDI_XRB_ITEMNAME,
                            "");
                }
                WinEnableDlgItem(hwndDlg, ID_XSDI_XRB_ITEMNAME,
                            (pData->pliSelected != NULL));

                WinEnableDlgItem(hwndDlg, ID_XSDI_ACL_WMCLOSE,
                            (pData->pliSelected != NULL));
                WinEnableDlgItem(hwndDlg, ID_XSDI_ACL_CTRL_C,
                            (pData->pliSelected != NULL));
                WinEnableDlgItem(hwndDlg, ID_XSDI_ACL_KILLSESSION,
                            (pData->pliSelected != NULL));
                WinEnableDlgItem(hwndDlg, ID_XSDI_ACL_SKIP,
                            (pData->pliSelected != NULL));

                WinEnableDlgItem(hwndDlg, ID_XSDI_XRB_DELETE,
                            (   (pData->usItemCount > 0)
                             && (pData->pliSelected)
                            )
                        );
            }
        break; }

        case WM_COMMAND: {
            switch (SHORT1FROMMP(mp1)) {

                /*
                 * ID_XSDI_XRB_NEW:
                 *      create new item
                 */

                case ID_XSDI_XRB_NEW: {
                    PAUTOCLOSEWINDATA pData =
                            (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
                    PAUTOCLOSELISTITEM pliNew = malloc(sizeof(AUTOCLOSELISTITEM));
                    //printf("WM_COMMAND ID_XSDI_XRB_NEW BN_CLICKED\n");
                    strcpy(pliNew->szItemName, "???");
                    pliNew->usAction = ACL_SKIP;
                    lstAppendItem((PLISTITEM*)&(pData->pliAutoClose), NULL,
                            (PLISTITEM)pliNew, 0, 0);

                    pData->usItemCount++;
                    WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_LISTBOX,
                                    LM_INSERTITEM,
                                    (MPARAM)LIT_END,
                                    (MPARAM)pliNew->szItemName);
                    WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_LISTBOX,
                                    LM_SELECTITEM, // will cause WM_UPDATE
                                    (MPARAM)(lstCountItems(
                                            (PLISTITEM)(pData->pliAutoClose),
                                            0, 0) - 1),
                                    (MPARAM)TRUE);
                    WinSetDlgItemFocus(hwndDlg, ID_XSDI_XRB_ITEMNAME);
                    WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_ITEMNAME,
                            EM_SETSEL,
                            MPFROM2SHORT(0, 1000), // select all
                            MPNULL);
                break; }

                /*
                 * ID_XSDI_XRB_DELETE:
                 *      delete selected item
                 */

                case ID_XSDI_XRB_DELETE: {
                    PAUTOCLOSEWINDATA pData =
                            (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
                    //printf("WM_COMMAND ID_XSDI_XRB_DELETE BN_CLICKED\n");
                    if (pData) {
                        if (pData->pliSelected) {
                            lstRemoveItem((PLISTITEM*)&(pData->pliAutoClose), NULL,
                                    (PLISTITEM)pData->pliSelected, 0, 0);
                            WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_LISTBOX,
                                    LM_DELETEITEM,
                                    (MPARAM)pData->sSelected,
                                    MPNULL);
                        }
                        WinPostMsg(hwndDlg, WM_UPDATE, MPNULL, MPNULL);
                    }
                    WinSetDlgItemFocus(hwndDlg, ID_XSDI_XRB_LISTBOX);
                break; }

                /*
                 * DID_OK:
                 *      store data in INI and dismiss dlg
                 */

                case DID_OK: {
                    PAUTOCLOSEWINDATA pData =
                            (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
                    PSZ     pINI, p;
                    BOOL    fValid = TRUE;

                    //printf("WM_COMMAND DID_OK BN_CLICKED\n");
                    // store data in INI
                    if (pData->pliAutoClose) {
                        pINI = malloc(
                                    sizeof(AUTOCLOSELISTITEM)
                                  * lstCountItems((PLISTITEM)(pData->pliAutoClose), 0, 0)
                               );
                        memset(pINI, 0,
                                    sizeof(AUTOCLOSELISTITEM)
                                  * lstCountItems((PLISTITEM)(pData->pliAutoClose), 0, 0));

                        if (pINI) {
                            PAUTOCLOSELISTITEM pli = pData->pliAutoClose;
                            USHORT          usCurrent = 0;
                            p = pINI;
                            while (pli) {
                                if (    (strlen(pli->szItemName) == 0)
                                   )
                                {
                                    WinAlarm(HWND_DESKTOP, WA_ERROR);
                                    WinSendDlgItemMsg(hwndDlg, ID_XSDI_XRB_LISTBOX,
                                                    LM_SELECTITEM,
                                                    (MPARAM)usCurrent,
                                                    (MPARAM)TRUE);
                                    fValid = FALSE;
                                    break;
                                }
                                strcpy(p, pli->szItemName);
                                p += (strlen(p)+1);
                                *((PUSHORT)p) = pli->usAction;
                                p += sizeof(USHORT);

                                pli = pli->pNext;
                                usCurrent++;
                            }

                            PrfWriteProfileData(HINI_USER,
                                        INIAPP, INIKEY_AUTOCLOSE,
                                        pINI,
                                        (p - pINI + 2));

                            free (pINI);
                        }
                    } // end if (pData->pliAutoClose)
                    else
                        PrfWriteProfileData(HINI_USER,
                                    INIAPP, INIKEY_AUTOCLOSE,
                                    NULL, 0);

                    // dismiss dlg
                    if (fValid)
                        mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
                break; }

                default: { // includes DID_CANCEL
                    mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
                break; }
            }
        break; }

        /*
         * WM_ANIMATE:
         *      initiate animation
         */

        case WM_ANIMATE: {
            PAUTOCLOSEWINDATA pData =
                    (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
            if (pData)
                if (pData->pAnimation) {
                    pData->pAnimation->usAniCurrent = 0;
                    WinStartTimer(WinQueryAnchorBlock(hwndDlg), hwndDlg, 1,
                            pData->pAnimation->ulDelay);
                }
        break; }

        /*
         * WM_TIMER:
         *      proceed to next animation
         */

        case WM_TIMER: {
            PAUTOCLOSEWINDATA pData =
                    (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
            PANIMATION pa = NULL;
            if (pData)
                if (pData->pAnimation)
                    pa = pData->pAnimation;

            cmnAnimate(pa);
        break; }

        /*
         * WM_DESTROY:
         *      clean up allocated memory
         */

        case WM_DESTROY: {
            PAUTOCLOSEWINDATA pData = (PAUTOCLOSEWINDATA)WinQueryWindowULong(hwndDlg, QWL_USER);
            WinStopTimer(WinQueryAnchorBlock(hwndDlg), hwndDlg, 1);
            cmnFreeShutdownAnimation(&(pData->pAnimation->ahptrAniIcons[0]));
            free(pData->pAnimation);
            lstClear((PLISTITEM*)&(pData->pliAutoClose), NULL, 0, 0);
            mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
        break; } // continue

        default:
            mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
        break;
    }
    return (mrc);
}

/*
 * xsdItemFromPID:
 *      searches a pliShutdownFirst for a process ID
 */

PSHUTLISTITEM xsdItemFromPID(PSHUTLISTITEM pFirst, PID pid, HMTX hmtx, ULONG ulTimeout)
{
    PSHUTLISTITEM pItem = NULL;
    BOOL          fAccess;

    if (hmtx)
        fAccess = (DosRequestMutexSem(hmtx, ulTimeout) == NO_ERROR);
    else
        fAccess = TRUE;

    if (fAccess)
    {
        if (pFirst) {
            pItem = pFirst;
            while (pItem)
                if (pItem->swctl.idProcess == pid)
                    break;
                else pItem = pItem->pNext;
        }
    }
    if (hmtx)
        DosReleaseMutexSem(hmtx);

    return (pItem);
}

/*
 * xsdItemFromSID:
 *      searches a pliShutdownFirst for a session ID
 */

PSHUTLISTITEM xsdItemFromSID(PSHUTLISTITEM pFirst, ULONG sid, HMTX hmtx, ULONG ulTimeout)
{
    PSHUTLISTITEM pItem = NULL;
    BOOL          fAccess;

    if (hmtx)
        fAccess = (DosRequestMutexSem(hmtx, ulTimeout) == NO_ERROR);
    else
        fAccess = TRUE;

    if (fAccess)
    {
        if (pFirst) {
            pItem = pFirst;
            while (pItem)
                if (pItem->swctl.idSession == sid)
                    break;
                else pItem = pItem->pNext;
        }
    }
    if (hmtx)
        DosReleaseMutexSem(hmtx);

    return (pItem);
}

/*
 * xsdCountRemainingItems:
 *      counts the items left to be closed by counting
 *      the window list items and subtracting the items
 *      which were skipped
 */

ULONG xsdCountRemainingItems(VOID)
{
    return (
                lstCountItems((PLISTITEM)pliShutdownFirst, hmtxShutdown, 4000)
              - lstCountItems((PLISTITEM)pliSkipped, hmtxSkipped, 4000)
           );
}

/*
 * xsdLongTitle:
 *      creates a descriptive string in pszTitle from pItem
 *      (for the main (debug) window listbox)
 */

void xsdLongTitle(PSZ pszTitle, PSHUTLISTITEM pItem)
{
    sprintf(pszTitle, "%s%s",
            pItem->swctl.szSwtitle,
            (pItem->swctl.uchVisibility == SWL_VISIBLE)
                ? (", visible")
                : ("")
        );

    strcat(pszTitle, ", ");
    switch (pItem->swctl.bProgType) {
        case PROG_DEFAULT: strcat(pszTitle, "default"); break;
        case PROG_FULLSCREEN: strcat(pszTitle, "OS/2 FS"); break;
        case PROG_WINDOWABLEVIO: strcat(pszTitle, "OS/2 win"); break;
        case PROG_PM:
            strcat(pszTitle, "PM, class: ");
            strcat(pszTitle, pItem->szClass);
        break;
        case PROG_VDM: strcat(pszTitle, "VDM"); break;
        case PROG_WINDOWEDVDM: strcat(pszTitle, "VDM win"); break;
        default: {
            strcat(pszTitle, "? (");
            strhexcat(pszTitle, pItem->swctl.bProgType);
            strcat(pszTitle, ")");
        break;}
    }
    sprintf(pszTitle+strlen(pszTitle),
            ", hwnd: 0x%lX, pid: 0x%lX, sid: 0x%lX, pObj: 0x%lX",
            (ULONG)pItem->swctl.hwnd,
            (ULONG)pItem->swctl.idProcess,
            (ULONG)pItem->swctl.idSession,
            (ULONG)pItem->pObject
        );
}

/*
 * xsdQueryCurrentItem:
 *      returns the next PSHUTLISTITEM to be
 *      closed (skipping the items that were
 *      skipped)
 */

PSHUTLISTITEM xsdQueryCurrentItem(VOID)
{
    PSHUTLISTITEM   pliShutItem = NULL, pliSkipItem = NULL;
    CHAR            szShutItem[1000],
                    szSkipItem[1000];

    if (    (DosRequestMutexSem(hmtxShutdown, 4000) == NO_ERROR)
         && (DosRequestMutexSem(hmtxSkipped, 4000) == NO_ERROR)
       )
    {
        pliShutItem = pliShutdownFirst;
        while (pliShutItem) {
            pliSkipItem = pliSkipped;
            while (pliSkipItem) {
                xsdLongTitle(szShutItem, pliShutItem);
                xsdLongTitle(szSkipItem, pliSkipItem);
                if (strcmp(szShutItem, szSkipItem) == 0)
                    /* current shut item is on skip list:
                       break (==> take next shut item */
                    break;
                else
                    pliSkipItem = pliSkipItem->pNext;
            }
            if (pliSkipItem != NULL)
                // current item was skipped: take next one
                pliShutItem = pliShutItem->pNext;
            else
                /* current item is not on the skip list:
                   return this item */
                break;
        }
    }
    DosReleaseMutexSem(hmtxSkipped);
    DosReleaseMutexSem(hmtxShutdown);

    return (pliShutItem);
}

/*
 * xsdAppendShutListItem:
 *      this appends a new PSHUTLIST item to a given list
 *      and returns the address of the new item; the list
 *      to append to must be specified in *ppFirst / *ppLast,
 *      for which you may specify a mutex semaphore; the
 *      item to add is to be specified by swctl and possibly
 *      *pObject (if swctl describes a WPS object)
 */

PSHUTLISTITEM xsdAppendShutListItem(PSHUTLISTITEM *ppFirst, PSHUTLISTITEM *ppLast,
    SWCNTRL swctl, WPObject *pObject, HMTX hmtx, ULONG ulTimeout)  //, PSZ pszTitle, PSZ pszClass
{
    PSHUTLISTITEM  pNewItem = NULL;

    pNewItem = malloc(sizeof(SHUTLISTITEM));

    if (pNewItem) {
        pNewItem->pObject = pObject;
        pNewItem->swctl = swctl;

        strcpy(pNewItem->szClass, "unknown");

        if (pObject)
            if (cmnCheckObject(pObject)) {
                strncpy(pNewItem->szClass, (PSZ)_somGetClassName(pObject), sizeof(pNewItem->szClass)-1);

                pNewItem->swctl.szSwtitle[0] = '\0';
                strncpy(pNewItem->swctl.szSwtitle, _wpQueryTitle(pObject), sizeof(pNewItem->swctl.szSwtitle)-1);

                pNewItem->swctl.idProcess = pidWPS;
                pNewItem->swctl.idSession = sidWPS;
                if (_wpFindUseItem(pObject, USAGE_OPENVIEW, NULL)) {
                    pNewItem->swctl.hwnd = (_wpFindViewItem(pObject, VIEW_ANY, NULL))->handle;
                }
            }
            else pNewItem->pObject = NULL;
        else {
            WinQueryClassName(swctl.hwnd,
                sizeof(pNewItem->szClass)-1,
                pNewItem->szClass);
        }

        lstAppendItem((PLISTITEM*)ppFirst, (PLISTITEM*)ppLast,
            (PLISTITEM)pNewItem,
            hmtx, ulTimeout);
    }
    return (pNewItem);
}

/*
 * xsdBuildShutList:
 *      this routine builds a new ShutList by evaluating the
 *      system task list; this list is built in *ppFirst / *ppLast,
 *      for which you may specify a mutex semaphore
 */

PSHUTLISTITEM xsdBuildShutList(PSHUTLISTITEM *ppFirst, PSHUTLISTITEM *ppLast,
                        HMTX hmtx, ULONG ulTimeout)
{
    // SWCNTRL         swctl;
    CHAR            szSwUpperTitle[100];

    WPObject        *pObj;

    ULONG           ul;
    ULONG           cbItems    = 0,            // Number of items in list
                    ulBufSize  = 0;            // Size of buffer for information
    PSWBLOCK        pSwBlock   = NULL;         // Pointer to information returned

    HAB             habDesktop = WinQueryAnchorBlock(HWND_DESKTOP);
    BOOL            Append;

    cbItems = WinQuerySwitchList(NULLHANDLE, NULL, 0);
    ulBufSize = (cbItems * sizeof(SWENTRY)) + sizeof(HSWITCH);

    // Allocate the buffer
    pSwBlock = (PSWBLOCK)malloc(ulBufSize);

    // Call WinQuerySwitchList again to fill our buffer with information
    cbItems = WinQuerySwitchList(NULLHANDLE, pSwBlock, ulBufSize);

    // Loop through all enumerated windows.
    for (ul = 0; ul < (pSwBlock->cswentry); ul++)
    {
        strcpy(szSwUpperTitle, pSwBlock->aswentry[ul].swctl.szSwtitle);
        WinUpper(habDesktop, 0, 0, szSwUpperTitle);

        // now we check which windows we add to the shutdown list
        if (        // PID must be != 0
                (pSwBlock->aswentry[ul].swctl.idProcess != 0)
                    // skip ourselves
             && (pSwBlock->aswentry[ul].swctl.hwnd != hwndMain)
             && (pSwBlock->aswentry[ul].swctl.hwnd != hwndVioDlg)
             && (pSwBlock->aswentry[ul].swctl.hwnd != hwndShutdownStatus)
                    // skip PROTSHELL PMSHELL.EXE
//                 && (strcmp(szSwUpperTitle, "PMSHELL.EXE") != 0)
                    // skip PMWORKPLACE cmd.exe, which is invisible
             && (pSwBlock->aswentry[ul].swctl.uchVisibility == SWL_VISIBLE)
                    /* skip all windows of the Workplace process,
                       which are mainly WPS folders; we will add these
                       later on */
             #ifdef DEBUG_SHUTDOWN
                 && (strcmp(szSwUpperTitle, "PMPRINTF - 2.56") != 0)
             #endif
             // && (pSwBlock->aswentry[ul].swctl.idProcess != pidWPS)
           )
        {
            Append = TRUE;
            pObj = NULL;

            if (pSwBlock->aswentry[ul].swctl.bProgType == PROG_WINDOWEDVDM)
                WinQueryWindowProcess(pSwBlock->aswentry[ul].swctl.hwnd,
                    &(pSwBlock->aswentry[ul].swctl.idProcess),
                    &(pSwBlock->aswentry[ul].swctl.idSession));
            else if (pSwBlock->aswentry[ul].swctl.idProcess == pidWPS)
            {
                // PID == Workplace Shell PID: get SOM pointer from hwnd
                pObj = _wpclsQueryObjectFromFrame(_WPDesktop,
                            pSwBlock->aswentry[ul].swctl.hwnd);
                if (!pObj) {
                    if (pSwBlock->aswentry[ul].swctl.hwnd == hwndActiveDesktop) {
                        pObj = pActiveDesktop;
                    }
                    else
                        Append = FALSE;
                }

                // check for special objects
                if (    (pObj == pActiveDesktop)
                     || (pObj == pAwakeWarpCenter)
                   )
                    Append = FALSE;
            }

            if (Append)
                // if we have Restart WPS && ~ close all windows, append
                // only open WPS objects
                if ( (!optWPSCloseWindows) && (pObj == NULL) )
                    Append = FALSE;

            if (Append)
                xsdAppendShutListItem(ppFirst, ppLast,
                    pSwBlock->aswentry[ul].swctl,
                    pObj,
                    hmtx, ulTimeout);
        }
    }

    free(pSwBlock);
    return (*ppFirst);
}

/*
 * xsdUpdateListBox:
 *      this routine builds a new PSHUTITEM list from the
 *      pointer to the pointer of the first item (*ppliShutdownFirst)
 *      by setting its value to xsdBuildShutList's return value;
 *      it also fills the listbox in the (debug) main window.
 *      If *ppliShutdownFirst is != NULL,
 *      the old shutlist is cleared also.
 */

void xsdUpdateListBox(HWND hwndListbox)
{
    PSHUTLISTITEM   pItem;
    CHAR            szTitle[1024];

    if (DosRequestMutexSem(hmtxShutdown, 4000) == NO_ERROR)
    {
        if (pliShutdownFirst)
            lstClear((PLISTITEM*)&pliShutdownFirst, (PLISTITEM*)&pliShutdownLast, 0, 0);

        xsdBuildShutList(&pliShutdownFirst, &pliShutdownLast, 0, 0);

        WinEnableWindowUpdate(hwndListbox, FALSE);
        WinSendMsg(hwndListbox, LM_DELETEALL, MPNULL, MPNULL);

        pItem = pliShutdownFirst;
        while (pItem) {
            xsdLongTitle(szTitle, pItem);
            WinInsertLboxItem(hwndListbox, 0, szTitle);
            pItem = pItem->pNext;
        }
        WinEnableWindowUpdate(hwndListbox, TRUE);
    }
    DosReleaseMutexSem(hmtxShutdown);
}

/*
 * fncbUpdateINIStatus:
 *      this is a callback func for PrfSaveINIs
 */

MRESULT EXPENTRY fncbUpdateINIStatus(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    if (hwnd)
        WinSendMsg(hwnd, msg, mp1, mp2);
    return ((MPARAM)TRUE);
}

/*
 * fncbINIError:
 *      callback func for PrfSaveINIs (helpers.c)
 */

MRESULT EXPENTRY fncbSaveINIError(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    return ((MRESULT)cmnMessageBox(HWND_DESKTOP, "XShutdown: Error",
            (PSZ)mp1,     // error text
            (ULONG)mp2)   // MB_ABORTRETRYIGNORE or something
       );
}

/*
 * xsdRestartWPS:
 *      terminated the WPS process, which will lead
 *      to a WPS restart.
 */

VOID xsdRestartWPS(VOID)
{
    DosExit(EXIT_PROCESS, 0);
}

/*
 * fntShutdownThread:
 *      this is the main shutdown thread which is created by
 *      xfInitiateShutdown / xfRestartWPS when shutdown is about
 *      to begin.
 *
 *      Shutdown / Restart WPS runs in the following phases:
 *
 *      1)  First, all neccessary preparations are done, i.e. two
 *          windows are created (the status window with the progress
 *          bar and the "main" window, which is only visible in debug
 *          mode, but processes all the messages). These two methods
 *          daringly share the same msg proc (fnwpShutdown below).
 *
 *      2)  fntShutdownThread remains in a message loop until shutdown
 *          is cancelled by the user or all windows have been closed.
 *          In both cases, fnwpShutdown posts a WM_QUIT then.
 *
 *          The order of msg processing in fntShutdownThread / fnwpShutdown
 *          is the following:
 *
 *          a)  Upon WM_INITDLG, fnwpShutdown will create the Update
 *              thread and do nothing more. This thread is responsible
 *              for monitoring the task list; every time an item is
 *              closed (or even opened!), fntUpdateThread will post
 *              a ID_SDMI_UPDATESHUTLIST command to fnwpShutdown.
 *
 *          b)  ID_SDMI_UPDATESHUTLIST will update the list of currently
 *              open windows (which is not touched by any other thread).
 *              If we're not in debug mode, the first time ID_SDMI_UPDATESHUTLIST
 *              is received, we will post ID_SDDI_BEGINSHUTDOWN (go to c)).
 *              Otherwise (subsequent calls), we post ID_SDMI_CLOSEITEM
 *              (go to e)).
 *
 *          c)  ID_SDDI_BEGINSHUTDOWN will begin processing the contents
 *              of the Shutdown folder. After this is done,
 *              ID_SDMI_BEGINCLOSINGITEMS is posted.
 *
 *          d)  ID_SDMI_BEGINCLOSINGITEMS will prepare closing all windows
 *              by setting flagClosingItems to TRUE and then post the
 *              first ID_SDMI_CLOSEITEM.
 *
 *          e)  ID_SDMI_CLOSEITEM will now undertake the neccessary
 *              actions for closing the first / next item on the list
 *              of items to close. After this, this thread is idle.
 *              If no more items are left to close, we post
 *              ID_SDMI_BEGINSAVEWPS (go to g)).
 *
 *          f)  When the window has actually closed, the Update thread
 *              realizes this because the task list will have changed.
 *              The next ID_SDMI_UPDATESHUTLIST will be posted by the
 *              Update thread then. Go back to b).
 *
 *          g)  ID_SDMI_BEGINSAVEWPS will save the state of all currently
 *              awake WPS objects by using the list maintained by the
 *              Worker thread.
 *
 *          i)  After this, ID_SDMI_FLUSHBUFFERS is posted, which
 *              will either restart the WPS (in Restart WPS mode) and
 *              remain idle, or post a WM_QUIT (in Shutdown mode), which
 *              will end the msg loop. In this case, fAllWindowsClosed
 *              is set to TRUE in order to allow continuation.
 *
 *      3)  Depending on whether fnwpShutdown set fAllWindowsClosed to
 *          TRUE, we will then actually shut down the system or exit
 *          this thread (= shutdown cancelled), and the user may continue work.
 *
 *          Shutting down the system is done via DosShutdown;
 *          this main Shutdown thread will then  either reboot the
 *          machine or run in an endless loop, if no reboot was desired.
 *
 *          When shutdown is cancelled by pressing the respective button,
 *          the Update thread is killed, all shutdown windows are closed,
 *          and then this thread also terminates.
 */

HAB         habShutdownThread;
HMQ         hmqShutdownThread;
HPOINTER    ahptrShutdown[ANIMAX];

void _System fntShutdownThread(ULONG ulptiMyself)
{
    HMODULE     hmodResource = NLS_MODULE;
    PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
    QMSG        qmsg;

    CHAR        szLog[1000] = "";

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    PSZ                   pszErrMsg = NULL;

    begin:

    // set our extended exception handler
    RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
    DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);

    // store a known thread state
    ulShutdownFunc2 = 500;
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt)
    {
        HWND hwndDesktop;
        // *** exception occurred:
        // get the handler off the chain
        DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
        // clean up here: free memory allocations, release mutex sems, etc.

        ulShutdownFunc2 = 515;
        if (hmtxAwakeObjects)
            DosReleaseMutexSem(hmtxAwakeObjects);
        if (hmtxShutdown)
            DosReleaseMutexSem(hmtxShutdown);
        if (hmtxSkipped)
            DosReleaseMutexSem(hmtxSkipped);

        // check for the killed-thread case
        /* switch (ulExcpt) {
            case XCPT_PROCESS_TERMINATE:
            case XCPT_ASYNC_PROCESS_TERMINATE:
                DosExit(EXIT_THREAD, 0);
        } */

        hwndDesktop = _wpclsQueryActiveDesktopHWND(_WPDesktop);
        if (    (hwndDesktop)
             && (pszErrMsg == NULL)
           )
        {
            // only report the first error, or otherwise we will
            // jam the system with msg boxes
            ulShutdownFunc2 = 516;
            pszErrMsg = malloc(1000);
            if (pszErrMsg) {
                strcpy(pszErrMsg, "An error occured in the XFolder Shutdown thread. "
                        "In the root directory of your boot drive, you will find a "
                        "file named XFLDTRAP.LOG, which contains debugging information. "
                        "If you had shutdown logging enabled, you will also find the "
                        "file XSHUTDWN.LOG there. If not, please enable shutdown "
                        "logging in the Desktop's settings notebook. "
                        "\n\nThe XShutdown procedure will be terminated now. We can "
                        "now also restart the Workplace Shell. This is recommended if "
                        "your Desktop has already been closed or if "
                        "the error occured during the saving of the INI files. In these "
                        "cases, please disable XShutdown and perform a regular OS/2 "
                        "shutdown to prevent loss of your WPS data."
                        "\n\nRestart the Workplace Shell now?");
                cmnPostObjectMsg(XOM_EXCEPTIONCAUGHT, (MPARAM)pszErrMsg,
                        (MPARAM)1); // enforce WPS restart

                if (hfShutdownLog) {
                    DosWriteToLogFile(hfShutdownLog, "\n*** CRASH\n");
                    DosWriteToLogFile(hfShutdownLog, pszErrMsg);
                    DosWriteToLogFile(hfShutdownLog, "\n");
                }
            }
        }

        // now clean up
        ulShutdownFunc2 = 515;
        goto cleanup;
    }

    fAllWindowsClosed = FALSE;
    fShutdownBegun = FALSE;
    fClosingApps = FALSE;

    hwndVioDlg = NULLHANDLE;
    strcpy(szVioTitle, "");

    // prepare shutdown log file
    if (optLog)
        hfShutdownLog = DosOpenLogFile(XFOLDER_SHUTDOWNLOG);

    if (hfShutdownLog) {
        DATETIME DT;
        DosGetDateTime(&DT);
        sprintf(szLog, "\n\nXFolder shutdown log -- Date: %02d/%02d/%04d, Time: %02d:%02d:%02d\n",
            DT.month, DT.day, DT.year,
            DT.hours, DT.minutes, DT.seconds);
        strcat(szLog, "--------------------------------------------------------\n");
        sprintf(szLog+strlen(szLog), "\nXFolder version: %s\n", XFOLDER_VERSION);
        sprintf(szLog+strlen(szLog), "\nShutdown thread started, TID: 0x%lX\n", thrQueryID(ptiShutdownThread));
        sprintf(szLog+strlen(szLog), "Settings: RestartWPS %s, Confirm %s, Reboot %s, WPSCloseWnds %s, CloseVIOs %s, APMPowerOff %s\n\n",
                (optRestartWPS) ? "ON" : "OFF",
                (optConfirm) ? "ON" : "OFF",
                (optReboot) ? "ON" : "OFF",
                (optWPSCloseWindows) ? "ON" : "OFF",
                (optAutoCloseVIO) ? "ON" : "OFF",
                (optAPMPowerOff) ? "ON" : "OFF");
        DosWriteToLogFile(hfShutdownLog, szLog);
    }

    //  raise our own priority
    DosSetPriority(PRTYS_THREAD,
                   PRTYC_REGULAR,
                   PRTYD_MAXIMUM, // priority delta
                   0);

    /* event semaphore which signals to the Update thread
       that the Shutlist has been updated by fnwpShutdown */
    DosCreateEventSem(
        NULL,         // unnamed
        &hevUpdated,
        0,            // unshared
        FALSE         // not posted
    );

    // mutex semaphores for linked lists
    if (hmtxShutdown == NULLHANDLE) {
        DosCreateMutexSem("\\sem32\\ShutdownList", &hmtxShutdown, 0, FALSE);  // unnamed, unowned
        DosCreateMutexSem("\\sem32\\SkippedList", &hmtxSkipped, 0, FALSE);  // unnamed, unowned
    }

    if (habShutdownThread = WinInitialize(0))
    {
        if (hmqShutdownThread = WinCreateMsgQueue(habShutdownThread, 0))
        {
            SWCNTRL     swctl;
            HSWITCH     hswitch;
            ULONG       ul;
            ULONG       ulKeyLength = 0;

            WinCancelShutdown(hmqShutdownThread, TRUE);

            // check for auto-close items in OS2.INI
            // and build pliAutoClose list accordingly
            if (PrfQueryProfileSize(HINI_USER,
                        INIAPP, INIKEY_AUTOCLOSE,
                        &ulKeyLength))
            {
                //printf("Size: %d\n", ulKeyLength);
                // items exist: evaluate
                PSZ p;
                PSZ pINI = malloc(ulKeyLength);

                if (hfShutdownLog)
                    DosWriteToLogFile(hfShutdownLog, "Auto-close items found: \n");

                if (pINI)
                {
                    PrfQueryProfileData(HINI_USER,
                                INIAPP, INIKEY_AUTOCLOSE,
                                pINI,
                                &ulKeyLength);
                    p = pINI;
                    //printf("%s\n", p);
                    while (strlen(p)) {
                        PAUTOCLOSELISTITEM pliNew = malloc(sizeof(AUTOCLOSELISTITEM));
                        strcpy(pliNew->szItemName, p);
                        lstAppendItem((PLISTITEM*)&(pliAutoCloseFirst), NULL,
                                (PLISTITEM)pliNew, 0, 0);

                        p += (strlen(p)+1);

                        if (strlen(p)) {
                            pliNew->usAction = *((PUSHORT)p);
                            p += sizeof(USHORT);
                        }

                        if (hfShutdownLog) {
                            sprintf(szLog, "  %s, usAction: %d\n",
                                pliNew->szItemName, pliNew->usAction);
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }
                    }
                    free(pINI);
                }
            }

            if (hfShutdownLog) {
                DosWriteToLogFile(hfShutdownLog, "Creating windows...\n");
            }

            // setup main (debug) window
            hwndMain = WinLoadDlg(HWND_DESKTOP, NULLHANDLE,
                            fnwpShutdown,
                            hmodResource,
                            ID_SDD_MAIN,
                            NULL);
            WinSendMsg(hwndMain,
                       WM_SETICON,
                       (MPARAM)hSDIcon,
                        NULL);

            // query process and session IDs
            WinQueryWindowProcess(hwndMain, &pidWPS, NULL);
            WinQueryWindowProcess(HWND_DESKTOP, &pidPM, NULL);
            sidPM = 1;  // should always be this, I hope */

            // add ourselves to the tasklist
            swctl.hwnd = hwndMain;                 // window handle
            swctl.hwndIcon = hSDIcon;            // icon handle
            swctl.hprog = NULLHANDLE;               // program handle
            swctl.idProcess = pidWPS;                  // process identifier
            swctl.idSession = 0;                    // session identifier
            swctl.uchVisibility = SWL_VISIBLE;      // visibility
            swctl.fbJump = SWL_JUMPABLE;            // jump indicator
            WinQueryWindowText(hwndMain, sizeof(swctl.szSwtitle), (PSZ)&swctl.szSwtitle);
            swctl.bProgType = PROG_DEFAULT;         // program type

            hswitch = WinAddSwitchEntry(&swctl);
            WinQuerySwitchEntry(hswitch, &swctl);
            sidWPS = swctl.idSession;

            // setup status window (always visible)
            hwndShutdownStatus = WinLoadDlg(HWND_DESKTOP, NULLHANDLE,
                            fnwpShutdown,
                            hmodResource,
                            ID_SDD_STATUS,
                            NULL);

            WinSendMsg(hwndShutdownStatus,
                       WM_SETICON,
                       (MPARAM)hSDIcon,
                       NULL);

            /* subclass the static rectangle control in the dialog to make
               it a progress bar */
            WinProgressBarFromStatic(
                    WinWindowFromID(hwndShutdownStatus, ID_SDDI_PROGRESSBAR),
                    PBA_ALIGNCENTER | PBA_BUTTONSTYLE);

            WinShowWindow(hwndShutdownStatus, TRUE);

            WinSetActiveWindow(HWND_DESKTOP, hwndShutdownStatus);

            cmnLoadShutdownAnimation(&(ahptrShutdown[0]), _xfclsQueryModuleHandle(_XFldObject));
            sAniCount = 0;
            WinStartTimer(habShutdownThread, hwndShutdownStatus, 1, 150);

            // now start!
            if (optDebug) {
                WinCenterWindow(hwndMain);
                WinShowWindow(hwndMain, TRUE);
            }

            if (hfShutdownLog) {
                DosWriteToLogFile(hfShutdownLog, "Now entering message loop...\n");
            }

            // now enter the common message loop for the main (debug) and
            // status windows (fnwpShutdown); this will keep running
            // until closing all windows is cancelled or complete
            while (WinGetMsg(habShutdownThread, &qmsg, NULLHANDLE, 0, 0))
            {
                WinDispatchMsg(habShutdownThread, &qmsg);
            }

            if (hfShutdownLog) {
                DosWriteToLogFile(hfShutdownLog, "Done with message loop\n");
            }

            // OK, shutdown is either cancelled or complete; clean up
            ulShutdownFunc2 = 2000;
            WinStopTimer(habShutdownThread, hwndShutdownStatus, 1);
            for (ul = 0; ul < ANIMAX; ul++)
                WinDestroyPointer(ahptrShutdown[ul]);
            WinDestroyWindow(hwndMain);

            if (fAllWindowsClosed) {
                // (fAllWindowsClosed = TRUE) only if shutdown was not cancelled; this
                // means that all windows have been successfully closed and we can
                // actually shut down the system

                HWND        hwndCADMessage;
                HPS         hpsScreen;
                HPOINTER    hptrOld, hptrWait;
                HFILE       hIOCTL;
                ULONG       ulAction;
                ULONG       ulrcAPM = 0;

                if (optRestartWPS) {
                    WinDestroyWindow(hwndShutdownStatus);
                    DosExit(EXIT_PROCESS, 0);
                }

                // else change the mouse pointer to wait state
                ulShutdownFunc2 = 2010;
                hptrOld = WinQueryPointer(HWND_DESKTOP);
                hptrWait = WinQuerySysPointer(HWND_DESKTOP,
                   SPTR_WAIT, FALSE);
                WinSetPointer(HWND_DESKTOP, hptrWait);

                // allocate some memory and free it again to make
                // sure we won't swap later, because after DosShutdown,
                // even the swapper file is blocked, and the system
                // would just halt
                {
                    PSZ p = calloc(100*1000, 1);   // 100 KB, initialize to 0
                    ulShutdownFunc2 = 2020;
                    free(p);
                }

                // setup Ctrl+Alt+Del message window; this needs to be done
                // before DosShutdown, because we won't be able to reach the
                // resource files after that
                ulShutdownFunc2 = 2030;
                hwndCADMessage = WinLoadDlg(HWND_DESKTOP, NULLHANDLE,
                                DumbDlgProc,
                                hmodResource,
                                ID_SDD_CAD,
                                NULL);
                WinCenterWindow(hwndCADMessage);  // wnd is still invisible

                if (optAnimate)
                    // hps for animation later
                    hpsScreen = WinGetScreenPS(HWND_DESKTOP);

                // enforce saving of INIs
                ulShutdownFunc2 = 2040;
                WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                        pNLSStrings->pszSDSavingProfiles);
                ulShutdownFunc2 = 2050;

                if (hfShutdownLog)
                    DosWriteToLogFile(hfShutdownLog, "Saving INIs...\n");

                ulShutdownFunc2 = 2060;
                if (PrfSaveINIs(habShutdownThread, hfShutdownLog,
                        (PFNWP)fncbUpdateINIStatus,
                        WinWindowFromID(hwndShutdownStatus, ID_SDDI_PROGRESSBAR),
                        WM_UPDATEPROGRESSBAR,
                        (PFNWP)fncbSaveINIError, &ulShutdownFunc2) != NO_ERROR)
                    if (cmnMessageBoxMsg(hwndShutdownStatus, 110, 111, MB_YESNO)
                                == MBID_YES)
                        xsdRestartWPS();

                ulShutdownFunc2 = 2070;
                if (hfShutdownLog)
                    DosWriteToLogFile(hfShutdownLog, "Done saving INIs; now preparing shutdown\n");

                // say "flushing buffers"
                WinShowPointer(HWND_DESKTOP, FALSE);
                ulShutdownFunc2 = 2080;
                WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                        pNLSStrings->pszSDFlushing);

                // if (optReboot), open DOS.SYS
                if ( (optReboot) && (strlen(szRebootCommand) == 0) ) {
                    ulShutdownFunc2 = 2090;
                    if (DosOpen("\\DEV\\DOS$", &hIOCTL, &ulAction, 0L,
                        FILE_NORMAL,
                        OPEN_ACTION_OPEN_IF_EXISTS,
                        OPEN_SHARE_DENYNONE | OPEN_ACCESS_READWRITE,
                        0L) != NO_ERROR)
                    {
                        DebugBox("XShutdown", "The DOS.SYS device driver could not be opened. "
                                "XShutdown will be unable to reboot your computer. "
                                "Please consult the XFolder Online Reference for a remedy of this problem.");
                    }
                    if (hfShutdownLog) {
                        sprintf(szLog, "Opened DOS.SYS, hFile: 0x%lX\n", hIOCTL);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }
                }

                if (optAPMPowerOff) {
                    CHAR szAPMError[500];
                    // prepare APM power off
                    ulrcAPM = apmPreparePowerOff(szAPMError);
                    if (hfShutdownLog) {
                        sprintf(szLog, "apmPreparePowerOff returned 0x%lX\n",
                                ulrcAPM);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    // if this returns APM_IGNORE, we continue
                    // with the regular shutdown w/out APM
                    if (ulrcAPM & APM_IGNORE)
                        optAPMPowerOff = FALSE;
                    // APM_CANCEL means cancel shutdown
                    else if (ulrcAPM & APM_CANCEL) {
                        WinShowPointer(HWND_DESKTOP, TRUE);
                        cmnMessageBox(HWND_DESKTOP, "APM Error", szAPMError,
                                MB_CANCEL | MB_SYSTEMMODAL);
                        // restart WPS
                        xsdRestartWPS();
                    }
                    // APM_OK means preparing went alright
                }

                // **** DosShutdown ****
                if (!optDebug) {
                    ulShutdownFunc2 = 2095;
                    if (strlen(szRebootCommand) == 0) {
                        if (    (!optAPMPowerOff)
                             || (ulrcAPM & APM_DOSSHUTDOWN_0)
                           )
                        {
                            if (hfShutdownLog) {
                                sprintf(szLog, "Calling DosShutdown(0), closing log.\n");
                                DosWriteToLogFile(hfShutdownLog, szLog);
                                DosClose(hfShutdownLog);
                                hfShutdownLog = NULLHANDLE;
                            }
                            DosShutdown(0);
                        }
                        // if apmPreparePowerOff requested this,
                        // do DosShutdown(1)
                        else if (   (optAPMPowerOff)
                                 && (ulrcAPM & APM_DOSSHUTDOWN_1)
                                )
                        {
                            if (hfShutdownLog) {
                                sprintf(szLog, "Calling DosShutdown(1), closing log.\n");
                                DosWriteToLogFile(hfShutdownLog, szLog);
                                DosClose(hfShutdownLog);
                                hfShutdownLog = NULLHANDLE;
                            }
                            DosShutdown(1);
                        }
                        // or if apmPreparePowerOff requestet this,
                        // do no DosShutdown()
                    }
                }
                else {
                    // debug!
                    DosSleep(2000);
                }

                WinShowPointer(HWND_DESKTOP, TRUE);
                ulShutdownFunc2 = 2096;
                WinSetPointer(HWND_DESKTOP, hptrOld);

                if (optReboot)
                {
                    // reboot:
                    ulShutdownFunc2 = 2097;
                    if (!optDebug) {
                        ulShutdownFunc2 = 2098;
                        if (strlen(szRebootCommand) == 0) {
                            // restart the machine via DOS.SYS (opened above)
                            WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                                    pNLSStrings->pszSDRebooting);
                            DosSleep(500);
                            DosDevIOCtl(hIOCTL, 0xd5, 0xab, NULL, 0, NULL, NULL, 0, NULL);
                        } else {
                            // user config: use user item
                            CHAR    szTemp[CCHMAXPATH];
                            PID     pid;
                            ULONG   sid;

                            sprintf(szTemp,
                                    pNLSStrings->pszStarting, szRebootCommand);
                            WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                                    szTemp);

                            if (hfShutdownLog) {
                                sprintf(szLog, "Trying to start user reboot cmd: %s, closing log.\n", szRebootCommand);
                                DosWriteToLogFile(hfShutdownLog, szLog);
                                DosClose(hfShutdownLog);
                                hfShutdownLog = NULLHANDLE;
                            }

                            sprintf(szTemp, "/c %s", szRebootCommand);
                            if (DosQuickStartSession("cmd.exe", szTemp,
                                FALSE, // show flag
                                TRUE,  // wait flag
                                &sid, &pid) != NO_ERROR)
                            {
                                DebugBox("XShutdown", "The user-defined restart command failed. "
                                    "We will now restart the WPS.");
                                xsdRestartWPS();
                            }
                            else
                                while (TRUE)
                                    DosSleep(1000);
                        }
                    }
                }
                else {
                    // don't reboot: show message only
                    if (optAnimate)
                        GpiPowerOffAnim(hpsScreen, 50);
                    else
                        // only hide the status window if
                        // animation is off, because otherwise
                        // we get screen display errors
                        // (hpsScreen...)
                        WinShowWindow(hwndShutdownStatus, FALSE);

                    if (optAPMPowerOff) {
                        // if APM power-off was properly prepared
                        // above, this flag is still TRUE; we
                        // will now call the function which
                        // actually turns the power off
                        apmDoPowerOff();
                    } else {
                        WinShowWindow(hwndCADMessage, TRUE);
                        // system-modal windows disable the task list ;-)
                        WinSetSysModalWindow(HWND_DESKTOP, hwndCADMessage);
                    }
                    if (optAnimate)
                        WinReleasePS(hpsScreen);

                    if (!optDebug) {
                        DosEnterCritSec();
                        // now loop forever
                        while (TRUE)
                            DosSleep(1000);
                    }
                    else {
                        DosSleep(2000);
                        DosExit(EXIT_PROCESS, 0);
                    }
                }
            } // end if (fAllWindowsClosed)

            // the following code is only reached when shutdown was cancelled;
            // clean up on the way out

            cleanup:
            // the exception handler also jumps here

            if (hfShutdownLog) {
                sprintf(szLog, "Reached cleanup, closing log.\n");
                DosWriteToLogFile(hfShutdownLog, szLog);
                DosClose(hfShutdownLog);
                hfShutdownLog = NULLHANDLE;
            }

            ulShutdownFunc2 = 2100;
            if (thrQueryID(ptiUpdateThread))
            {
                ulShutdownFunc2 = 2101;
                thrClose(ptiUpdateThread);
                ulShutdownFunc2 = 2102;
                thrFree(&ptiUpdateThread);
            }

            ulShutdownFunc2 = 2103;
            DosCloseEventSem(hevUpdated);

            // Clean up on the way out.
            ulShutdownFunc2 = 2105;
            WinDestroyWindow(hwndShutdownStatus);
            WinDestroyMsgQueue(hmqShutdownThread);
        }
        WinTerminate(habShutdownThread);
    }

    ulShutdownFunc2 = 2200;
    if (pliShutdownFirst) {
        lstClear((PLISTITEM*)&pliShutdownFirst, (PLISTITEM*)&pliShutdownLast, hmtxShutdown, 4000);
        lstClear((PLISTITEM*)&pliSkipped, NULL, hmtxSkipped, 4000);
        lstClear((PLISTITEM*)&pliAutoCloseFirst, NULL, 0, 0);
    }

    if (hmtxShutdown != NULLHANDLE) {
        ulShutdownFunc2 = 2201;
        DosCloseMutexSem(hmtxShutdown);
        hmtxShutdown = NULLHANDLE;
    }
    if (hmtxSkipped != NULLHANDLE) {
        ulShutdownFunc2 = 2202;
        DosCloseMutexSem(hmtxSkipped);
        hmtxSkipped = NULLHANDLE;
    }

    // end of Shutdown thread: we need to set our handle to 0
    //   because otherwise xfInitiateShutdown would prohibit a
    //   second shutdown
    ulShutdownFunc2 = 2300;
    ((PTHREADINFO)ulptiMyself)->fExitComplete = TRUE;
    ((PTHREADINFO)ulptiMyself)->tid = NULLHANDLE;
    // thread exits!
    ulShutdownFunc2 = 2301;
    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);

    ShutdownRunning = FALSE;
}

/*
 * fncbShutdown:
 *      callback function when processing Shutdown folder
 */

ULONG   hPOC;

MRESULT EXPENTRY fncbShutdown(HWND hwndStatus, ULONG ulObject, MPARAM mpNow, MPARAM mpMax)
{
    CHAR szStarting2[200];
    HWND rc = 0;
    if (ulObject) {
        WinSetActiveWindow(HWND_DESKTOP, hwndStatus);
        sprintf(szStarting2,
            (cmnQueryNLSStrings())->pszStarting,
            _wpQueryTitle((WPObject*)ulObject));
        WinSetDlgItemText(hwndStatus, ID_SDDI_STATUS, szStarting2);

        rc = _wpViewObject((WPObject*)ulObject, 0, OPEN_DEFAULT, 0);

        if (hfShutdownLog) {
            CHAR szLog[1000];
            sprintf(szLog, "      ""%s""\n", szStarting2);
            DosWriteToLogFile(hfShutdownLog, szLog);
        }

        WinSetActiveWindow(HWND_DESKTOP, hwndStatus);

        WinSendMsg(WinWindowFromID(hwndStatus, ID_SDDI_PROGRESSBAR),
            WM_UPDATEPROGRESSBAR, mpNow, mpMax);
    }
    else {
        WinPostMsg(hwndStatus, WM_COMMAND, MPFROM2SHORT(ID_SDMI_BEGINCLOSINGITEMS, 0),
            MPNULL);
        hPOC = 0;
    }
    return ((MRESULT)rc);
}

/*
 * xsdUpdateClosingStatus:
 *      set status wnd text to "Closing xxx"
 */

VOID xsdUpdateClosingStatus(PSZ pszProgTitle)
{
    CHAR szTitle[300];
    strcpy(szTitle,
        (cmnQueryNLSStrings())->pszSDClosing);
    strcat(szTitle, " \"");
    strcat(szTitle, pszProgTitle);
    strcat(szTitle, "\"...");
    WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
        szTitle);

    WinSetActiveWindow(HWND_DESKTOP, hwndShutdownStatus);
}

/*
 * fnwpShutdown:
 *      window procedure for both the main (debug) window and
 *      the status window; it receives messages from the Update
 *      threas, so that it can update the windows' contents
 *      accordingly; it also controls this thread by suspending
 *      or killing it, if neccessary, and setting semaphores.
 *      Note that the main (debug) window with the listbox is only
 *      visible in debug mode (signalled to xfInitiateShutdown).
 */

ULONG       ulAwakeNow, ulAwakeMax;

MRESULT EXPENTRY fnwpShutdown(HWND hwndFrame, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT         rc = MRFALSE;
    PSHUTLISTITEM   pItem;
    HWND            hwndListbox = WinWindowFromID(hwndMain, ID_SDDI_LISTBOX);
    CHAR            szLog[2000] = "";

    ulShutdownFunc2 = 1;

    switch(msg)
    {
        case WM_INITDLG:
        case WM_CREATE: {

            ulShutdownFunc2 = 2;
            if (hfShutdownLog) {
                CHAR szTemp[2000];
                WinQueryWindowText(hwndFrame, sizeof(szTemp), szTemp);
                sprintf(szLog, "  WM_INITDLG/WM_CREATE (%s, hwnd: 0x%lX)\n",
                        szTemp, hwndFrame);
                DosWriteToLogFile(hfShutdownLog, szLog);
            }

            if (hfShutdownLog) {
                sprintf(szLog, "    HAB: 0x%lX, HMQ: 0x%lX, pidWPS: 0x%lX, pidPM: 0x%lX\n",
                    habShutdownThread, hmqShutdownThread, pidWPS, pidPM);
                DosWriteToLogFile(hfShutdownLog, szLog);
            }

            ulMaxItemCount = 0;
            ulLastItemCount = -1;

            hPOC = 0;

            if (thrQueryID(ptiUpdateThread) == NULLHANDLE) {
                // first call; this one's called twice!

                /* if (pAwakeWarpCenter) {
                    _wpSaveImmediate(pAwakeWarpCenter);
                } */

                thrCreate(&ptiUpdateThread, fntUpdateThread, NULLHANDLE);

                if (hfShutdownLog) {
                    sprintf(szLog, "    Update thread started, tid: 0x%lX\n", thrQueryID(ptiUpdateThread));
                    DosWriteToLogFile(hfShutdownLog, szLog);
                }
            }
        break; }

        /*
         * WM_TIMER:
         *      for animation
         */

        case WM_TIMER: {
            HWND hwndIcon = WinWindowFromID(hwndShutdownStatus, ID_SDDI_ICON);
            sAniCount++;
            if (sAniCount >= ANIMAX)
                sAniCount = 0;

            if (hwndIcon) {
                WinSendMsg(hwndIcon,
                        SM_SETHANDLE,
                        (MPARAM)ahptrShutdown[sAniCount],
                        MPNULL);
            }
        break; }

        case WM_COMMAND: {
            switch (SHORT1FROMMP(mp1)) {
                case ID_SDDI_BEGINSHUTDOWN: {
                    /* this is either posted by the "Begin shutdown"
                       button (in debug mode) or otherwise automatically
                       after the first update initiated by the Update
                       thread; we will look for a Shutdown folder first */

                    XFolder         *pShutdownFolder;

                    ulShutdownFunc2 = 30;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDDI_BEGINSHUTDOWN, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    if (pShutdownFolder = _wpclsQueryFolder(_WPFolder, XFOLDER_SHUTDOWNID, TRUE))
                    {
                        if (hfShutdownLog) {
                            sprintf(szLog, "    Processing shutdown folder...\n");
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }

                        hPOC = _xfBeginProcessOrderedContent(pShutdownFolder, 0, // wait mode
                            &fncbShutdown, (ULONG)hwndShutdownStatus);

                        if (hfShutdownLog) {
                            sprintf(szLog, "    Done processing shutdown folder\n");
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }
                    }
                    else
                        goto beginclosingitems;
                break; }

                case ID_SDMI_BEGINCLOSINGITEMS:
                beginclosingitems: {
                    /* this is posted after processing of the shutdown
                       folder; shutdown actually begins now with closing
                       all windows */
                    ulShutdownFunc2 = 40;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_BEGINCLOSINGITEMS, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }
                    fClosingApps = TRUE;
                    WinShowWindow(hwndShutdownStatus, TRUE);
                    WinEnableDlgItem(hwndMain, ID_SDDI_BEGINSHUTDOWN, FALSE);
                    WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_CLOSEITEM, 0),
                        MPNULL);
                break; }

                /*
                 * ID_SDMI_CLOSEITEM:
                 *     this msg is posted first upon receiving
                 *     ID_SDDI_BEGINSHUTDOWN and subsequently for every
                 *     application that is to be closed; we only INITIATE
                 *     closing the application here by posting messages
                 *     or killing the application; we then rely on the
                 *     update thread to realize that the application has
                 *     actually terminated; the Update thread then post
                 *     an ID_SDMI_UPDATESHUTLIST, which will then in turn
                 *     post another ID_SDMI_CLOSEITEM
                 */

                case ID_SDMI_CLOSEITEM: {
                    ulShutdownFunc2 = 50;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_CLOSEITEM, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    // get task list item to close from linked list
                    pItem = xsdQueryCurrentItem();
                    if (pItem) {
                        CHAR        szTitle[1024];
                        USHORT      usItem;

                        ulShutdownFunc2 = 51;
                        // compose string from item
                        xsdLongTitle(szTitle, pItem);

                        if (hfShutdownLog) {
                            sprintf(szLog, "    Item: %s\n", szTitle);
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }

                        // find string in the invisible list box
                        usItem = (USHORT)WinSendMsg(hwndListbox,
                            LM_SEARCHSTRING,
                            MPFROM2SHORT(0, LIT_FIRST),
                            (MPARAM)szTitle);
                        // and select it
                        if ((usItem != (USHORT)LIT_NONE)) {
                            WinPostMsg(hwndListbox,
                                LM_SELECTITEM,
                                (MPARAM)usItem,
                                (MPARAM)TRUE);
                        }

                        // update status window: "Closing xxx"
                        xsdUpdateClosingStatus(pItem->swctl.szSwtitle);

                        // now check what kind of action needs to be done
                        if (pItem->pObject)
                        {   // we have a WPS window: use proper method
                            // to ensure that window data is saved
                            ulShutdownFunc2 = 53;
                            _wpClose(pItem->pObject);
                            if (hfShutdownLog) {
                                DosWriteToLogFile(hfShutdownLog, "      Open WPS object, called wpClose(pObject)\n");
                            }
                        }
                        else if (pItem->swctl.hwnd) {
                            // no WPS window: differentiate further
                            ulShutdownFunc2 = 55;
                            if (hfShutdownLog) {
                                sprintf(szLog, "      swctl.hwnd found: 0x%lX\n", pItem->swctl.hwnd);
                                DosWriteToLogFile(hfShutdownLog, szLog);
                            }
                            if (    (pItem->swctl.bProgType == PROG_VDM)
                                 || (pItem->swctl.bProgType == PROG_WINDOWEDVDM)
                                 || (pItem->swctl.bProgType == PROG_FULLSCREEN)
                                 || (pItem->swctl.bProgType == PROG_WINDOWABLEVIO)
                                 || (pItem->swctl.bProgType == PROG_DEFAULT)
                               )
                            {
                                CHAR szTest[1000];

                                if (hfShutdownLog) {
                                    sprintf(szLog, "      Seems to be VIO, swctl.bProgType: 0x%lX\n", pItem->swctl.bProgType);
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }

                                xsdLongTitle(szTest, pItem);
                                if (    (hwndVioDlg == NULLHANDLE)
                                     || (strcmp(szTest, szVioTitle) != 0)
                                   )
                                {
                                    if (hfShutdownLog) {
                                        DosWriteToLogFile(hfShutdownLog, "      Posting ID_SDMI_CLOSEVIO\n");
                                    }
                                    // not a PM session: ask what to do (handled below)
                                    WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_CLOSEVIO, 0),
                                            MPNULL);
                                }
                            }
                            else if (WinWindowFromID(pItem->swctl.hwnd, FID_SYSMENU))
                            {   // window has system menu: close PM application
                                if (hfShutdownLog) {
                                    sprintf(szLog, "      Has system menu, posting WM_SAVEAPPLICATION to hwnd 0x%lX\n",
                                            pItem->swctl.hwnd);
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }
                                WinPostMsg(pItem->swctl.hwnd,
                                    WM_SAVEAPPLICATION,
                                    MPNULL, MPNULL);

                                if (hfShutdownLog) {
                                    sprintf(szLog, "      Posting WM_QUIT to hwnd 0x%lX\n",
                                            pItem->swctl.hwnd);
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }
                                WinPostMsg(pItem->swctl.hwnd,
                                    WM_QUIT,
                                    MPNULL, MPNULL);
                            }
                            else { // no system menu: try something more brutal
                                if (hfShutdownLog) {
                                    sprintf(szLog, "      Has no sys menu, posting WM_CLOSE to hwnd 0x%lX\n",
                                            pItem->swctl.hwnd);
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }
                                WinPostMsg(pItem->swctl.hwnd,
                                    WM_CLOSE,
                                    MPNULL,
                                    MPNULL);
                            }
                        }
                        else {
                            if (hfShutdownLog) {
                                sprintf(szLog, "      Helpless... leaving item alone, pid: 0x%lX\n", pItem->swctl.idProcess);
                                DosWriteToLogFile(hfShutdownLog, szLog);
                            }
                            // DosKillProcess(DKP_PROCESS, pItem->swctl.idProcess);
                        }
                    } // end if (pItem)
                    else  { // no more items left: enter phase 2 (save WPS)
                        ulShutdownFunc2 = 56;
                        if (hfShutdownLog) {
                            DosWriteToLogFile(hfShutdownLog, "    All items closed. Posting ID_SDMI_BEGINSAVEWPS\n");
                        }
                        WinPostMsg(hwndMain, WM_COMMAND,
                            MPFROM2SHORT(ID_SDMI_PREPARESAVEWPS, 0),
                            MPNULL);    // first post
                    }
                break; }

                /*
                 * ID_SDDI_SKIPAPP:
                 *     comes from the "Skip" button
                 */

                case ID_SDDI_SKIPAPP: {
                    PSHUTLISTITEM   pSkipItem;
                    ulShutdownFunc2 = 60;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDDI_SKIPAPP, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }
                    pItem = xsdQueryCurrentItem();
                    if (pItem) {
                        if (hfShutdownLog) {
                            sprintf(szLog, "    Adding %s to the list of skipped items\n",
                                pItem->swctl.szSwtitle);
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }
                        pSkipItem = malloc(sizeof(SHUTLISTITEM));
                        *pSkipItem = *pItem;
                        lstAppendItem((PLISTITEM*)&pliSkipped, NULL,
                                    (PLISTITEM)pSkipItem,
                                    hmtxSkipped, 4000);
                    }
                    if (fClosingApps)
                        WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_UPDATEPROGRESSBAR, 0),
                            MPNULL);
                break; }

                /*
                 * ID_SDMI_CLOSEVIO:
                 *     this is posted by ID_SDMI_CLOSEITEM when
                 *     a non-PM session is encountered; we will now
                 *     either close this session automatically or
                 *     open up a confirmation dlg
                 */

                case ID_SDMI_CLOSEVIO: {
                    ULONG   ulReply;
                    CHAR    szText[500];
                    PAUTOCLOSELISTITEM pliAutoClose;

                    ulShutdownFunc2 = 70;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_CLOSEVIO, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    pItem = xsdQueryCurrentItem();
                    if (pItem) {
                        xsdLongTitle(szVioTitle, pItem);
                        VioItem = *pItem;

                        if (hfShutdownLog) {
                            sprintf(szLog, "    VIO item: %s\n", szVioTitle);
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }

                        // activate VIO window
                        WinSetActiveWindow(HWND_DESKTOP, VioItem.swctl.hwnd);

                        // check if VIO window is on auto-close list
                        pliAutoClose = pliAutoCloseFirst;
                        while (pliAutoClose) {
                            if (strnicmp(VioItem.swctl.szSwtitle,
                                        pliAutoClose->szItemName,
                                        strlen(pliAutoClose->szItemName)) == 0)
                                break;
                            else
                                pliAutoClose = pliAutoClose->pNext;
                        }

                        if (pliAutoClose) {
                            if (hfShutdownLog) {
                                DosWriteToLogFile(hfShutdownLog, "    Found on auto-close list\n");
                            }
                            // item was found on auto-close list:
                            // perform user-configured action
                            switch (pliAutoClose->usAction) {
                                case ACL_WMCLOSE:
                                    WinPostMsg((WinWindowFromID(VioItem.swctl.hwnd, FID_SYSMENU)),
                                        WM_SYSCOMMAND,
                                        (MPARAM)SC_CLOSE, MPNULL);
                                    if (hfShutdownLog) {
                                        sprintf(szLog, "      Posted SC_CLOSE to sysmenu, hwnd: 0x%lX\n",
                                            WinWindowFromID(VioItem.swctl.hwnd, FID_SYSMENU));
                                        DosWriteToLogFile(hfShutdownLog, szLog);
                                    }
                                    break;

                                case ACL_CTRL_C:
                                    DosSendSignalException(VioItem.swctl.idProcess,
                                        XCPT_SIGNAL_INTR);
                                    if (hfShutdownLog) {
                                        sprintf(szLog, "      Sent INTR signal to pid 0x%lX\n",
                                            VioItem.swctl.idProcess);
                                        DosWriteToLogFile(hfShutdownLog, szLog);
                                    }
                                    break;

                                case ACL_KILLSESSION:
                                    DosKillProcess(DKP_PROCESS, VioItem.swctl.idProcess);
                                    if (hfShutdownLog) {
                                        sprintf(szLog, "      Killed pid 0x%lX\n",
                                            VioItem.swctl.idProcess);
                                        DosWriteToLogFile(hfShutdownLog, szLog);
                                    }
                                    break;

                                case ACL_SKIP:
                                    WinPostMsg(hwndMain, WM_COMMAND,
                                            MPFROM2SHORT(ID_SDDI_SKIPAPP, 0),
                                            MPNULL);
                                    if (hfShutdownLog) {
                                        DosWriteToLogFile(hfShutdownLog, "      Posted ID_SDDI_SKIPAPP\n");
                                    }
                                    break;
                            }
                        } else {
                            // not on auto-close list:
                            if (optAutoCloseVIO) {
                                // auto-close enabled globally though:
                                if (hfShutdownLog) {
                                    sprintf(szLog, "    Not found on auto-close list, auto-close is on:\n");
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }
                                // "auto close VIOs" is on
                                if (VioItem.swctl.idSession == 1) {
                                    // for some reason, DOS/Windows sessions always
                                    // run in the Shell process, whose SID == 1
                                    WinPostMsg((WinWindowFromID(VioItem.swctl.hwnd, FID_SYSMENU)),
                                        WM_SYSCOMMAND,
                                        (MPARAM)SC_CLOSE, MPNULL);
                                    if (hfShutdownLog) {
                                        sprintf(szLog, "      Posted SC_CLOSE to hwnd 0x%lX\n",
                                            WinWindowFromID(VioItem.swctl.hwnd, FID_SYSMENU));
                                        DosWriteToLogFile(hfShutdownLog, szLog);
                                    }
                                } else {
                                    // OS/2 windows: kill
                                    DosKillProcess(DKP_PROCESS, VioItem.swctl.idProcess);
                                    if (hfShutdownLog) {
                                        sprintf(szLog, "      Killed pid 0x%lX\n",
                                            VioItem.swctl.idProcess);
                                        DosWriteToLogFile(hfShutdownLog, szLog);
                                    }
                                }
                            } else {
                                // no auto-close: confirmation wnd
                                if (hfShutdownLog) {
                                    sprintf(szLog, "    Not found on auto-close list, query-action dlg:\n");
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }
                                hwndVioDlg = WinLoadDlg(HWND_DESKTOP, hwndShutdownStatus,
                                            fnwpDlgGeneric,
                                            NLS_MODULE,
                                            ID_SDD_CLOSEVIO,
                                            NULL);
                                strcpy(szText, "\"");
                                strcat(szText, VioItem.swctl.szSwtitle);
                                WinQueryDlgItemText(hwndVioDlg, ID_SDDI_VDMAPPTEXT,
                                    100, &(szText[strlen(szText)]));
                                WinSetDlgItemText(hwndVioDlg, ID_SDDI_VDMAPPTEXT,
                                    szText);
                                WinCenterWindow(hwndVioDlg);
                                ulReply = WinProcessDlg(hwndVioDlg);
                                WinDestroyWindow(hwndVioDlg);
                                switch (ulReply) {
                                    case DID_OK:
                                        // "Close" button: kill process
                                        if (VioItem.swctl.idSession == 1) {
                                            // for some reason, DOS/Windows sessions always
                                            // run in the Shell process, whose SID == 1
                                            WinPostMsg((WinWindowFromID(VioItem.swctl.hwnd, FID_SYSMENU)),
                                                WM_SYSCOMMAND,
                                                (MPARAM)SC_CLOSE, MPNULL);
                                            if (hfShutdownLog) {
                                                sprintf(szLog, "      DID_OK = 'Close' pressed, posted SC_CLOSE to hwnd 0x%lX\n",
                                                    WinWindowFromID(VioItem.swctl.hwnd, FID_SYSMENU));
                                                DosWriteToLogFile(hfShutdownLog, szLog);
                                            }
                                        } else {
                                            // OS/2 windows: kill
                                            DosKillProcess(DKP_PROCESS, VioItem.swctl.idProcess);
                                            if (hfShutdownLog) {
                                                sprintf(szLog, "      DID_OK = 'Close' pressed, killed pid 0x%lX\n",
                                                    VioItem.swctl.idProcess);
                                                DosWriteToLogFile(hfShutdownLog, szLog);
                                            }
                                        }
                                        break;

                                    case ID_SDDI_CANCELSHUTDOWN:
                                        // "Cancel shutdown" button: pass to main window
                                        WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDDI_CANCELSHUTDOWN, 0),
                                            MPNULL);
                                        if (hfShutdownLog) {
                                            sprintf(szLog, "      'Cancel shutdown' pressed");
                                            DosWriteToLogFile(hfShutdownLog, szLog);
                                        }
                                        break;

                                    case ID_SDDI_SKIPAPP:
                                        WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDDI_SKIPAPP, 0),
                                            MPNULL);
                                        if (hfShutdownLog) {
                                            sprintf(szLog, "      'Skip' pressed");
                                            DosWriteToLogFile(hfShutdownLog, szLog);
                                        }
                                        break;
                                    /* if the VIO was closed manually, ID_SDMI_UPDATESHUTLIST will post
                                       a WM_CLOSE to the window, so that the return code will be DID_CANCEL;
                                       we need not handle this return code */
                                }

                                hwndVioDlg = NULLHANDLE;
                            } // end else (optAutoCloseVIO)
                        }
                    }

                    if (hfShutdownLog) {
                        sprintf(szLog, "  Done with ID_SDMI_CLOSEVIO\n");
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }
                break; }

                /*
                 * ID_SDMI_UPDATESHUTLIST:
                 *    this cmd comes from the Update thread when
                 *    the task list has changed. This happens when
                 *    1)    something was closed by this function
                 *    2)    the user has closed something
                 *    3)    and even if the user has OPENED something
                 *          new.
                 *    We will then rebuilt the pliShutdownFirst list and
                 *    continue closing items, if Shutdown is currently in progress
                 */

                case ID_SDMI_UPDATESHUTLIST: {
                    #ifdef DEBUG_SHUTDOWN
                        DosBeep(10000, 50);
                    #endif

                    ulShutdownFunc2 = 80;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_UPDATESHUTLIST, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    xsdUpdateListBox(hwndListbox);
                        // this updates the Shutdown linked list
                    DosPostEventSem(hevUpdated);
                        // signal update to Update thread

                    if (hfShutdownLog) {
                        sprintf(szLog, "    Rebuilt shut list, %d items remaining\n",
                                xsdCountRemainingItems());
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    if (hwndVioDlg) {
                        USHORT usItem;
                        // "Close VIO" confirmation window is open:
                        // check if the item that was to be closed
                        // is still in the listbox; if so, exit,
                        // if not, close confirmation window and
                        // continue
                        usItem = (USHORT)WinSendMsg(hwndListbox,
                            LM_SEARCHSTRING,
                            MPFROM2SHORT(0, LIT_FIRST),
                            (MPARAM)&szVioTitle);

                        if ((usItem != (USHORT)LIT_NONE))
                            break;
                        else {
                            WinPostMsg(hwndVioDlg, WM_CLOSE, MPNULL, MPNULL);
                            /* this will result in a DID_CANCEL return code
                               for WinProcessDlg in ID_SDMI_CLOSEVIO above */
                            if (hfShutdownLog) {
                                sprintf(szLog, "    Closed VIO confirm' dlg\n");
                                DosWriteToLogFile(hfShutdownLog, szLog);
                            }
                        }
                    }
                    goto updateprogressbar;
                    // continue with update progress bar
                }

                /*
                 * ID_SDMI_UPDATEPROGRESSBAR:
                 *     well, update the progress bar in the
                 *     status window
                 */

                case ID_SDMI_UPDATEPROGRESSBAR:
                updateprogressbar: {
                    ULONG           ulItemCount, ulMax, ulNow;

                    ulShutdownFunc2 = 90;
                    ulItemCount = xsdCountRemainingItems();
                    if (ulItemCount > ulMaxItemCount) {
                        ulMaxItemCount = ulItemCount;
                        ulLastItemCount = -1; // enforce update
                    }

                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_UPDATEPROGRESSBAR, hwnd: 0x%lX, remaining: %d, total: %d\n",
                                hwndFrame, ulItemCount, ulMaxItemCount);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    if ((ulItemCount) != ulLastItemCount) {
                        ulMax = ulMaxItemCount;
                        ulNow = (ulMax - ulItemCount);

                        WinSendMsg(WinWindowFromID(hwndShutdownStatus, ID_SDDI_PROGRESSBAR),
                            WM_UPDATEPROGRESSBAR,
                            (MPARAM)ulNow, (MPARAM)ulMax);
                        ulLastItemCount = (ulItemCount);
                    }

                    if (fShutdownBegun == FALSE)
                        if (!optDebug) {
                            /* if this is the first time we're here,
                               begin shutdown, unless we're in debug mode */
                            WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDDI_BEGINSHUTDOWN, 0),
                                MPNULL);
                            fShutdownBegun = TRUE;
                            break;
                        }
                    if (fClosingApps)
                        /* if we're already in the process of shutting down, we will
                           initiate closing the next item */
                        WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_CLOSEITEM, 0),
                            MPNULL);
                break; }

                case DID_CANCEL:
                case ID_SDDI_CANCELSHUTDOWN:
                    /* results from the "Cancel shutdown" buttons in both the
                       main (debug) and the status window; we set a semaphore
                       upon which the Update thread will terminate itself,
                       and WM_QUIT is posted to the main (debug) window, so that
                       the message loop in the main Shutdown thread function ends */
                    if (WinIsDlgItemEnabled(hwndShutdownStatus, ID_SDDI_CANCELSHUTDOWN)) {
                        ulShutdownFunc2 = 120;
                        if (hfShutdownLog) {
                            sprintf(szLog, "  DID_CANCEL/ID_SDDI_CANCELSHUTDOWN, hwnd: 0x%lX\n");
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }

                        if (hPOC)
                            ((PPROCESSCONTENTINFO)hPOC)->fCancelled = TRUE;
                        WinEnableDlgItem(hwndShutdownStatus, ID_SDDI_CANCELSHUTDOWN, FALSE);
                        WinEnableDlgItem(hwndShutdownStatus, ID_SDDI_SKIPAPP, FALSE);
                        WinEnableDlgItem(hwndMain, ID_SDDI_BEGINSHUTDOWN, TRUE);
                        fClosingApps = FALSE;

                        WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_CLEANUPANDQUIT, 0),
                            MPNULL);
                    }
                break;

            /* PHASE 2: after all windows have been closed,
               ID_SDMI_BEGINSAVEAVEWPS is posted, so we will now
               call _wpSaveImmediate on all awake objects */

                /*
                 * ID_SDMI_PREPARESAVEWPS:
                 *      the first time, this will be posted with
                 *      mp2 == NULL, 2 or 3 afterwards
                 */

                case ID_SDMI_PREPARESAVEWPS: {
                    CHAR    szTitle[1000];
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_BEGINSAVEWPS, hwnd: 0x%lX, mp2: 0x%lX\n", hwndFrame, mp2);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }

                    if (mp2 == (MPARAM)NULL) {
                        // first post
                        WinSetActiveWindow(HWND_DESKTOP, hwndShutdownStatus);

                        WinEnableDlgItem(hwndShutdownStatus, ID_SDDI_CANCELSHUTDOWN, FALSE);
                        WinEnableDlgItem(hwndShutdownStatus, ID_SDDI_SKIPAPP, FALSE);

                        ulShutdownFunc2 = 100;

                        // close Desktop window
                        xsdUpdateClosingStatus(_wpQueryTitle(pActiveDesktop));
                        if (hfShutdownLog) {
                            sprintf(szLog, "    Closing Desktop window\n");
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }
                        _wpClose(pActiveDesktop);
                        // close WarpCenter next
                        if (pAwakeWarpCenter) {
                            // WarpCenter open?
                            if (_wpFindUseItem(pAwakeWarpCenter,
                                        USAGE_OPENVIEW,
                                        NULL)       // get first useitem
                                )
                            {
                                // if open: close it
                                xsdUpdateClosingStatus(_wpQueryTitle(pAwakeWarpCenter));
                                if (hfShutdownLog) {
                                    sprintf(szLog, "    Closing WarpCenter\n");
                                    DosWriteToLogFile(hfShutdownLog, szLog);
                                }
                                _wpClose(pAwakeWarpCenter);
                            }
                        }

                        WinPostMsg(hwndMain, WM_COMMAND,
                                MPFROM2SHORT(ID_SDMI_PREPARESAVEWPS, 0),
                                (MPARAM)2);    // second post: mp2 == 2
                        // we need to post this msg twice, because
                        // otherwise the status wnd doesn't get updated
                        // properly
                    } else if ((ULONG)mp2 == 2) {
                        // mp2 == 2: second post,
                        // now prepare saving objects
                        ulAwakeMax = lstCountItems((PLISTITEM)pliAwakeObjectsFirst,
                                        hmtxAwakeObjects, 4000);
                        ulAwakeNow = 0;

                        // "Saving xxx awake WPS objects..."
                        sprintf(szTitle, cmnQueryNLSStrings()->pszSDSavingDesktop,
                                ulAwakeMax);
                        WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                            szTitle);

                        if (hfShutdownLog) {
                            sprintf(szLog, "    Saving %d awake WPS objects...\n", ulAwakeMax);
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }

                        // create fake desktop so it looks as though we have
                        // closed all windows, even if we haven't ;-)
                        if ((!optRestartWPS) && (!optDebug))
                            WinCreateFakeDesktop(hwndShutdownStatus);

                        WinPostMsg(hwndMain, WM_COMMAND,
                                MPFROM2SHORT(ID_SDMI_PREPARESAVEWPS, 0),
                                (MPARAM)3);    // third post: mp2 == 3
                    } else if ((ULONG)mp2 == 3) {
                        // reset progress bar
                        WinSendMsg(WinWindowFromID(hwndShutdownStatus, ID_SDDI_PROGRESSBAR),
                            WM_UPDATEPROGRESSBAR,
                            (MPARAM)0, (MPARAM)ulAwakeMax);
                        // and wait a while
                        DosSleep(500);
                        // finally, save WPS
                        WinPostMsg(hwndMain, WM_COMMAND,
                                MPFROM2SHORT(ID_SDMI_SAVEWPSITEM, 0),
                                MPNULL);
                    }
                break; }

                /*
                 * ID_SDMI_SAVEWPSITEM:
                 *      this is then posted for each awake WPS object
                 */

                case ID_SDMI_SAVEWPSITEM: {
                    POBJECTLISTITEM pliAwakeObject;

                    /* now save all currently awake WPS objects; these will
                       be saved after the other windows are closed */

                    ulShutdownFunc2 = 110;

                    pliAwakeObject = (POBJECTLISTITEM)lstItemFromIndex((PLISTITEM)pliAwakeObjectsFirst,
                        ulAwakeNow, hmtxAwakeObjects, 4000);
                    if (pliAwakeObject) {
                        if (pliAwakeObject->pObj) {
                            // save neither Desktop nor WarpCenter
                            if (    (pliAwakeObject->pObj != pActiveDesktop)
                                 && (pliAwakeObject->pObj != pAwakeWarpCenter)
                               )
                                _wpSaveImmediate(pliAwakeObject->pObj);
                        }
                        ulAwakeNow++;
                        // post for next object
                        WinPostMsg(hwndMain, WM_COMMAND,
                                MPFROM2SHORT(ID_SDMI_SAVEWPSITEM, 0),
                                MPNULL);
                    } else {
                        // done
                        if (hfShutdownLog) {
                            DosWriteToLogFile(hfShutdownLog, "    Done saving WPS\n");
                        }
                        WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_FLUSHBUFFERS, 0),
                            MPNULL);
                    }

                    WinSendMsg(WinWindowFromID(hwndShutdownStatus, ID_SDDI_PROGRESSBAR),
                        WM_UPDATEPROGRESSBAR,
                        (MPARAM)ulAwakeNow, (MPARAM)ulAwakeMax);
                break; }

             /* PHASE 3: after saving the WPS objects, ID_SDMI_FLUSHBUFFERS is
                posted, so we will now actually shut down the system or restart
                the WPS */

                case ID_SDMI_FLUSHBUFFERS: {
                    ulShutdownFunc2 = 130;
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_FLUSHBUFFERS, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }
                    /* first close the Update thread to prevent it from interfering
                       with what we're doing now */
                    if (thrQueryID(ptiUpdateThread)) {
                        #ifdef DEBUG_SHUTDOWN
                            WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                                "Waiting for the Update thread to end...");
                        #endif
                        if (hfShutdownLog) {
                            sprintf(szLog, "    Closing Update thread, tid: 0x%lX...\n",
                                    thrQueryID(ptiUpdateThread));
                            DosWriteToLogFile(hfShutdownLog, szLog);
                        }
                        thrFree(&ptiUpdateThread);  // wait
                        if (hfShutdownLog) {
                            DosWriteToLogFile(hfShutdownLog, "    Update thread terminated\n");
                        }
                    }

                    if (optRestartWPS) {
                        // "Restart WPS" mode
                        ULONG i;

                        if (hfShutdownLog) {
                            DosWriteToLogFile(hfShutdownLog, "Restarting WPS, closing log.\n");
                            DosClose(hfShutdownLog);
                            hfShutdownLog = NULLHANDLE;
                        }

                        ulShutdownFunc2 = 131;
                        WinSetDlgItemText(hwndShutdownStatus, ID_SDDI_STATUS,
                                (cmnQueryNLSStrings())->pszSDRestartingWPS);
                        ulShutdownFunc2 = 132;
                        if (optWPSReuseStartupFolder)
                            // set flag to allow startup folder to be
                            // be processed next time (flag is examined
                            // by the XFolder exit list)
                            PreventNextStartup = FALSE;

                        // let the WPS restart be handled by
                        // fntShutdownThread
                        fAllWindowsClosed = TRUE;
                        WinPostMsg(hwndMain, WM_QUIT, MPNULL, MPNULL);
                    } else {
                        // actual "Shutdown" mode:
                        // exit fnwpShutdown and let the rest of the shutdown
                        // be handled by fntShutdown, which will take
                        // over after WM_QUIT

                        ulShutdownFunc2 = 133;
                        fAllWindowsClosed = TRUE;
                        WinPostMsg(hwndMain, WM_COMMAND,
                                MPFROM2SHORT(ID_SDMI_CLEANUPANDQUIT, 0),
                                MPNULL);
                    }
                break; }

                /*
                 * ID_SDMI_CLEANUPANDQUIT:
                 *      for "real" shutdown: let fntShutdownThread handle the rest
                 */

                case ID_SDMI_CLEANUPANDQUIT: {
                    if (hfShutdownLog) {
                        sprintf(szLog, "  ID_SDMI_CLEANUPANDQUIT, hwnd: 0x%lX\n", hwndFrame);
                        DosWriteToLogFile(hfShutdownLog, szLog);
                    }
                    WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_UPDATESHUTLIST, 0),
                        MPNULL);
                    WinPostMsg(hwndMain, WM_QUIT, MPNULL, MPNULL);
                break; }

                default:
                    rc = WinDefDlgProc(hwndFrame, msg, mp1, mp2);
            } // end switch;
        break; } // end case WM_COMMAND

        // other msgs: have them handled by the usual WC_FRAME wnd proc
        default:
           rc = WinDefDlgProc(hwndFrame, msg, mp1, mp2);
        break;
    }

    return (rc);
}

/********************************************************************
 *                                                                  *
 *   here comes the Update thread                                   *
 *                                                                  *
 ********************************************************************/

/*
 * fntUpdateThread:
 *          this thread is responsible for monitoring the window list;
 *          it builds an internal second PSHUTLISTITEM linked list
 *          every 100 ms and then compares it to the global pliShutdownFirst;
 *          if they are different, fnwpShutdown is posted a message
 *          so that it can rebuild pliShutdownFirst and update the windows
 */

void _System fntUpdateThread(ULONG ulptiMyself)
{
    HAB             habUpdateThread;
    HMQ             hmqUpdateThread;
    ULONG
                    ulShutItemCount = 0,
                    ulTestItemCount = 0;
                    // ulLoopCount = 0;

    BOOL            fUpdated = FALSE;
    APIRET          rc = NO_ERROR;

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    PSZ                   pszErrMsg = NULL;

    // set the extended exception handler
    RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
    if ( rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt) {
        HWND hwndDesktop;
        // *** exception occurred:
        // get the handler off the chain
        DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
        // clean up here: free memory allocations, release mutex sems, etc.
        // clean up
        ulShutdownFunc2 = 515;
        if (hmtxShutdown) {
            DosReleaseMutexSem(hmtxShutdown);
        }

        // check for the killed-thread case
        /* switch (ulExcpt) {
            case XCPT_PROCESS_TERMINATE:
            case XCPT_ASYNC_PROCESS_TERMINATE:
                DosExit( EXIT_THREAD, 0 );
                break;
        } */

        hwndDesktop = _wpclsQueryActiveDesktopHWND(_WPDesktop);
        if (    (hwndDesktop)
             && (pszErrMsg == NULL)
           )
        {
            // only report the first error, or otherwise we will
            // jam the system with msg boxes
            ulShutdownFunc2 = 516;
            pszErrMsg = malloc(1000);
            if (pszErrMsg) {
                strcpy(pszErrMsg, "An error occured in the XFolder Update thread. "
                        "In the root directory of your boot drive, you will find a "
                        "file named XFLDTRAP.LOG, which contains debugging information. "
                        "If you had shutdown logging enabled, you will also find the "
                        "file XSHUTDWN.LOG there. If not, please enable shutdown "
                        "logging in the Desktop's settings notebook. ");
                cmnPostObjectMsg(XOM_EXCEPTIONCAUGHT, (MPARAM)pszErrMsg,
                        (MPARAM)0); // don't enforce WPS restart

                if (hfShutdownLog) {
                    DosWriteToLogFile(hfShutdownLog, "\n*** CRASH\n");
                    DosWriteToLogFile(hfShutdownLog, pszErrMsg);
                    DosWriteToLogFile(hfShutdownLog, "\n");
                }
            }
        }
    } else {
        // no exception
        DosSetPriority(PRTYS_THREAD,
                      PRTYC_REGULAR, // PRTYC_IDLETIME,
                       -31,          // priority delta
                       0);           // this thread

        if (habUpdateThread = WinInitialize(0))
        {
            if (hmqUpdateThread = WinCreateMsgQueue(habUpdateThread, 0))
            {
                PSHUTLISTITEM   pTestList = NULL,
                                pTestLastItem = NULL;

                WinCancelShutdown(hmqUpdateThread, TRUE);

                // wait until main shutdown window is up
                while ( (hwndMain == 0) && (!(ptiUpdateThread->fExit)) )
                    DosSleep(100);

                while (!(ptiUpdateThread->fExit)) {
                    // this is the first loop: we arrive here every time
                    // the task list has changed */
                    #ifdef DEBUG_SHUTDOWN
                        _Pmpf(( "UT: Waiting for update..." ));
                    #endif

                    WinPostMsg(hwndMain, WM_COMMAND, MPFROM2SHORT(ID_SDMI_UPDATESHUTLIST, 0),
                        MPNULL);
                    fUpdated = FALSE;
                    do {
                        ULONG   ulUpdate;
                        cmnError("Error querying Update sem: ", DosQueryEventSem(hevUpdated, &ulUpdate));
                        if (ptiUpdateThread->fExit) {
                            fUpdated = TRUE;
                            #ifdef DEBUG_SHUTDOWN
                                _Pmpf(( "UT: Exit recognized" ));
                            #endif
                        }
                        else fUpdated = (ulUpdate > 0);
                        DosSleep(100);
                    } while (!fUpdated);

                    #ifdef DEBUG_SHUTDOWN
                        _Pmpf(( "UT: First update or exit recognized" ));
                    #endif

                    ulTestItemCount = 0;
                    ulShutItemCount = 0;

                    #ifdef DEBUG_SHUTDOWN
                        _Pmpf(( "UT: Waiting for task list change, loop 2..." ));
                    #endif
                    while (     (ulTestItemCount == ulShutItemCount)
                             && (!(ptiUpdateThread->fExit))
                          )
                    {
                        /* this is the second loop: we stay in here until the
                           task list has changed; for monitoring this, we create
                           a second task item list similar to the pliSHutdownFirst
                           list and compare the two */
                        if (pTestList)
                            lstClear((PLISTITEM*)&pTestList, NULL, 0, 0);
                        xsdBuildShutList(&pTestList, NULL, 0, 0);
                        ulTestItemCount = lstCountItems((PLISTITEM)pTestList, 0, 0);
                        ulShutItemCount = lstCountItems((PLISTITEM)pliShutdownFirst,
                                                hmtxShutdown, 4000);

                        if (!(ptiUpdateThread->fExit))
                            DosSleep(100);
                    } /* end while; loop until either the Shutdown thread has set the
                         Exit flag or the list has changed */

                    #ifdef DEBUG_SHUTDOWN
                        _Pmpf(( "UT: Change or exit recognized" ));
                    #endif
                }  // end while; loop until exit flag set

                #ifdef DEBUG_SHUTDOWN
                    _Pmpf(( "UT: Exiting..." ));
                #endif

                lstClear((PLISTITEM*)&pTestList, (PLISTITEM*)&pTestLastItem, 0, 0);
                WinDestroyMsgQueue(hmqUpdateThread);
            }
            WinTerminate(habUpdateThread);
        }
    }

    ((PTHREADINFO)ulptiMyself)->fExitComplete = TRUE;
    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
    #ifdef DEBUG_SHUTDOWN
        DosBeep(100, 100);
    #endif
}

/********************************************************************
 *                                                                  *
 *   here come the XFolder Worker thread functions                  *
 *                                                                  *
 ********************************************************************/

/*
 * cmnRaiseWorkerThreadPriority:
 *      depending on fRaise, raise or lower Worker
 *      thread prty
 */

BOOL fSetting = FALSE;

BOOL cmnRaiseWorkerThreadPriority(BOOL fRaise)
{
    BOOL brc = FALSE;

    while (fSetting)
        DosSleep(1000);

    fSetting = TRUE;
    if (fRaise) {
        if (WorkerThreadHighPriority == FALSE) {
            #ifdef DEBUG_PRIORITY
                DosBeep(1000, 10);
                _Pmpf(( "Raising Worker thread priority" ));
            #endif
            DosSetPriority(PRTYS_THREAD,
                           PRTYC_REGULAR,
                           PRTYD_MAXIMUM, // priority delta
                           tidWorkerThread);
            WorkerThreadHighPriority = TRUE;
            brc = TRUE;
        }
    } else {
        if (WorkerThreadHighPriority == TRUE) {
            #ifdef DEBUG_PRIORITY
                DosBeep(100, 10);
                _Pmpf(( "Lowering Worker thread priority" ));
            #endif
            DosSetPriority(PRTYS_THREAD,
                           PRTYC_IDLETIME,
                           PRTYD_MAXIMUM, // priority delta

                           // PRTYC_IDLETIME,
                           // 0, // priority delta
                           0);
            brc = TRUE;
            WorkerThreadHighPriority = FALSE;
        }
    }
    fSetting = FALSE;
    return (brc);
}

/*
 * cmnPostWorkerMsg:
 *      this posts a msg to the XFolder Worker thread object window
 *      and controls this thread's priority at the same time; use this
 *      function instead of WinPostQueueMsg, cos otherwise the
 *      Worker thread gets confused
 */

BOOL cmnPostWorkerMsg(ULONG msg, MPARAM mp1, MPARAM mp2)
{
    BOOL rc = FALSE;

    if (tidWorkerThread) {
        if (hwndWorkerObject) {
            rc = WinPostMsg(hwndWorkerObject, msg, mp1, mp2);

            if (rc) {
                ulAwakeMsgCount++;
                if (ulAwakeMsgCount > 300)
                    // if the Worker thread msg queue gets congested, boost priority
                    cmnRaiseWorkerThreadPriority(TRUE);
            }
        }
    }
    return (rc);
}

/*
 * cmnPostQuickMsg:
 *      this posts a msg to the XFolder Quick thread object window.
 */

BOOL cmnPostQuickMsg(ULONG msg, MPARAM mp1, MPARAM mp2)
{
    BOOL rc = FALSE;

    if (tidQuickThread) {
        if (hwndQuickObject) {
            rc = WinPostMsg(hwndQuickObject,
                        msg,
                        mp1,
                        mp2);
        }
    }
    return (rc);
}

/*
 * cmnPlaySystemSound:
 *      this posts a msg to the XFolder Quick thread msg queue
 *      to play a system sound. usIndex may be one of the
 *      following:
 *          MMSOUND_WARNING         0
 *          MMSOUND_INFORMATION     1
 *          MMSOUND_ERROR           2
 *          MMSOUND_ANIMATEOPEN     3
 *          MMSOUND_ANIMATECLOSE    4
 *          MMSOUND_DRAG            5
 *          MMSOUND_DROP            6
 *          MMSOUND_SYSTEMSTARTUP   7
 *          MMSOUND_SHUTDOWN        8
 *          MMSOUND_SHREDDER        9
 *          MMSOUND_LOCKUP          10
 *          MMSOUND_ALARMCLOCK      11
 *          MMSOUND_PRINTERROR      12

 *          MMSOUND_XFLD_SHUTDOWN   555
 *          MMSOUND_XFLD_RESTARTWPS 556
 *          MMSOUND_XFLD_SORT       557
 */

BOOL cmnPlaySystemSound(USHORT usIndex)
{
    BOOL rc = FALSE;

    if (tidQuickThread) {
        if (hwndQuickObject) {
            rc = WinPostMsg(hwndQuickObject,
                        QM_PLAYSYSTEMSOUND,
                        (MPARAM)usIndex,
                        MPNULL);
        }
    }
    return (rc);
}

/*
 * fnwpGenericStatus:
 *      dlg func for bootup status wnd and maybe others
 */

MRESULT EXPENTRY fnwpGenericStatus(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = NULL;

    switch (msg) {
        case WM_INITDLG:
            mrc = (MPARAM)TRUE; // focus change flag;
            // mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break;

        /* case WM_FOCUSCHANGE: {
                // swallow this msg
        break; }

        case WM_ACTIVATE: {
            if ( (BOOL)mp1 == TRUE ) {
                // swallow this msg
                mrc = MPNULL;
            } else
                mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; }

        case WM_ADJUSTWINDOWPOS: {
            PSWP pswp = (PSWP)mp1;
            if (pswp->fl & (SWP_ZORDER))
            {
                WinSetWindowPos(hwndDlg, pswp->hwndInsertBehind,
                               0, 0, 0, 0, SWP_ZORDER);
                // this is the trick!
                pswp->hwndInsertBehind = hwndDlg;
                // mrc = (MPARAM)AWP_DEACTIVATE;
            }
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; } */

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break;
    }

    return (mrc);
}

/*
 * cmnCheckObject:
 *      since somIsObj doesn't seem to be working right,
 *      here is a new function which checks if pObject
 *      points to a valid WPS object. This is done by
 *      temporarily installing yet another xcpt handler, so if the
 *      object ain't valid, calling this function doesn't
 *      crash, but returns FALSE only.
 */

BOOL cmnCheckObject(WPObject *pObject)
{
    BOOL                  brc = TRUE;

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    APIRET                rc;

    begin:

    if (pObject == NULL)
        return (FALSE);

    // set our extended exception handler
    RegRec2.pfnHandler = (PFN)ExcptHandlerQuiet;
    if ( rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt)
    {
        brc = FALSE;
        #ifdef DEBUG_AWAKEOBJECTS
            DosBeep(10000, 10);
        #endif
    } else {
        // call an object method; if this doesn't fail,
        // TRUE is returned, otherwise an xcpt occurs and
        // rc is set to FALSE above.
        brc = somIsObj(pObject);
    }

    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
    return (brc);
}

/*
 * cmnQueryAwakeObjectsCount:
 *      returns the current number of awake WPS objects
 *      (maintained by the Worker thread)
 */

LONG cmnQueryAwakeObjectsCount(VOID)
{
    return (lAwakeObjectsCount);
}

/*
 * fnwpWorkerObject:
 *      wnd proc for Worker thread object window (see below)
 */

HAB     habWorkerThread = NULLHANDLE;
HMQ     hmqWorkerThread = NULLHANDLE;
CHAR    szBootupStatus[256];

MRESULT EXPENTRY fnwpWorkerObject(HWND hwndObject, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = NULL;

    ulWorkerFunc2 = 100;
    switch (msg) {

        /*
         * WM_CREATE:
         *      set a timer to periodically free storage
         */

        case WM_CREATE: {
            mrc = WinDefWindowProc(hwndObject, msg, mp1, mp2);
            WinStartTimer(habWorkerThread, hwndObject,
                            2,          // id
                            5*60*1000); // every five minutes
        break; }

        /*
         * WM_TIMER:
         */

        case WM_TIMER: {
            switch ((USHORT)mp1)  {// timer id

                // timer 2: free unused memory
                case 2:
                    #ifdef DEBUG_MEMORYBEEP
                        DosBeep(3000, 20);
                    #endif

                    _heapmin();
                    // _heapmin returns all unused memory from the default
                    // runtime heap to the operating system

                    #ifdef DEBUG_MEMORYBEEP
                        DosBeep(3500, 20);
                    #endif
                break;
            }
        break; }

        /*
         * WOM_DESKTOPREADY:
         *   this msg is posted after the Desktop has been opened;
         *   we will now loop until the Desktop has also been
         *   fully populated; if so, we will either display a
         *   "Welcome" msg or start processing the startup folder.
         *   This msg expects the currently active desktop in
         *   (XFldDesktop*)mp1.
         */

        case WOM_DESKTOPREADY: {
            ULONG ul;
            ULONG ulS = sizeof(ul);
            ulWorkerFunc2 = 1000;

            // _Pmpf(( "WT: WM_DESKTOPREADY" ));
            DosSleep(1000);

            if ((_wpQueryFldrFlags((WPFolder*)mp1)
                        & FOI_POPULATEDWITHALL) == 0)
            {
                DosSleep(100);
                // if Desktop is not populated yet, post this
                // msg again
                cmnPostWorkerMsg(WOM_DESKTOPREADY, mp1, MPNULL);
                // do not continue
                break;
            }

            // else: Desktop is populated

            // if XFolder was just installed, check for
            // existence of config folders and
            // display welcome msg
            if (PrfQueryProfileInt(HINI_USER, INIAPP,
                    INIKEY_JUSTINSTALLED,
                    0x123) != 0x123)
            {   // XFolder was just installed
                PrfWriteProfileString(HINI_USER, INIAPP,
                        INIKEY_JUSTINSTALLED,
                        NULL);
                if (!_wpclsQueryFolder(_WPFolder, XFOLDER_CONFIGID, TRUE))
                    // if config folder does not exist, create it now
                    cmnPostWorkerMsg(WOM_RECREATECONFIGFOLDER,
                        (MPARAM)RCF_RECREATEALL, MPNULL);

                cmnPostWorkerMsg(WOM_WELCOME, MPNULL, MPNULL);
            }

            // initiate processing of startup folder; this is
            // again done by the Workplace thread (XFolder Object wnd)
            if (PrfQueryProfileData(HINI_USERPROFILE,
                        INIAPP, INIKEY_DONTDOSTARTUP,
                        &ul, &ulS) == FALSE)
            {
                // only do startup if we haven't crashed
                // the XFolder Object wnd will check for
                // the existence of the startup folder
                cmnPostObjectMsg(XOM_BEGINSTARTUP, MPNULL, MPNULL);
            } else {
                // delete the INI key, but don't do startup
                PrfWriteProfileData(HINI_USERPROFILE,
                            INIAPP, INIKEY_DONTDOSTARTUP,
                            NULL, 0);
                cmnPostWorkerMsg(WOM_DONEWITHSTARTUP, MPNULL, MPNULL);
            }
        break; }

        /*
         * WOM_WELCOME:
         *     display hello dlg box
         */

        case WOM_WELCOME: {
            HOBJECT hobjIntro = WinQueryObject("<XFOLDER_INTRO>");
            if (hobjIntro) {
                WinOpenObject(hobjIntro, OPEN_DEFAULT, TRUE);
            } else {
                ulWorkerFunc2 = 3000;
                WinCenteredDlgBox(HWND_DESKTOP, HWND_DESKTOP,
                   DumbDlgProc,
                   NLS_MODULE,
                   ID_XFD_WELCOME,
                   0);
            }
        break;}

        /*
         * WOM_DONEWITHSTARTUP:
         *      this is posted after processing of the startup
         *      folder has finished; we will now
         *      populate the config folders in the
         *      background
         */

        case WOM_DONEWITHSTARTUP: {
            XFolder *pFolder = _wpclsQueryFolder(_WPFolder, XFOLDER_CONFIGID, TRUE);
            ulWorkerFunc2 = 4000;
            if (pFolder)
                cmnPopulateTree(pFolder);

            // now go for the "Quick open" folders
            cmnPostObjectMsg(XOM_BEGINQUICKOPEN, MPNULL, MPNULL);
        break; }

        /*
         * WOM_QUICKOPEN:
         *      this is posted by the XFolder Object window
         *      on the main PM thread for each folder with
         *      the QuickOpen flag on: we will populate it
         *      here and load all the icons.
         *      Parameters:
         *          XFolder* mp1    folder pointer
         *          ULONG    mp2    callback function (PFNWP)
         *      The callback will be called if mp2 != NULL
         *      with the following parameters:
         *          HWND    current folder
         *          ULONG   current object therein
         *          MPARAM  current object count
         *          MPARAM  maximum object count
         */

        case WOM_QUICKOPEN: {
            XFolder *pFolder = (XFolder*)mp1;
            _Pmpf(("WOM_QUICKOPEN %lX", mp1));
            if (pFolder) {
                WPObject    *pObject;
                ULONG       ulNow = 0, ulMax = 0;
                BOOL        fContinue = TRUE;

                // populate folder
                cmnCheckIfPopulated(pFolder);

                // count objects
                for (   pObject = _wpQueryContent(pFolder, NULL, (ULONG)QC_FIRST);
                        (pObject);
                        pObject = _wpQueryContent(pFolder, pObject, (ULONG)QC_NEXT)
                    )
                {
                    ulMax++;
                }

                for (   pObject = _wpQueryContent(pFolder, NULL, (ULONG)QC_FIRST);
                        (pObject);
                        pObject = _wpQueryContent(pFolder, pObject, (ULONG)QC_NEXT)
                    )
                {
                    PFNWP pfncb = (PFNWP)mp2;
                    _wpQueryIcon(pObject);
                    if (pfncb) {
                        // callback
                        fContinue = (BOOL)
                                  (*(pfncb))
                                    (
                                        (HWND)pFolder,
                                        (ULONG)pObject,
                                        (MPARAM)ulNow,
                                        (MPARAM)ulMax
                                    );
                        if (!fContinue) {
                            pObject = NULL;
                            cmnPostObjectMsg(XOM_NEXTQUICKOPEN, NULL, MPNULL);
                            break;
                        }
                    }
                    ulNow++;
                }

                if ( (fContinue) && (mp2 != NULL) )
                {
                    // now go for the next folder
                    cmnPostObjectMsg(XOM_NEXTQUICKOPEN, mp1, MPNULL);
                }
            }
        break; }

        /*
         * WOM_RECREATECONFIGFOLDER:
         *    this msg is posted if the
         *    config folder was not found
         */

        case WOM_RECREATECONFIGFOLDER: {
            ULONG   ulAction = (ULONG)mp1;
            ULONG   ulReturn = ulAction;
            cmnSetHelpPanel(ID_XFH_NOCONFIG);

            ulWorkerFunc2 = 2000;
            if (ulAction == RCF_QUERYACTION) {
                DosSleep(1000);
                ulReturn = (   (WinDlgBox(HWND_DESKTOP,      // parent is desktop
                                    HWND_DESKTOP,             // owner is desktop
                                    (PFNWP)fnwpDlgGeneric,
                                    NLS_MODULE,
                                    ID_XFD_NOCONFIG,
                                    (PVOID)NULL)             // no dialog parameters
                           == DID_DEFAULT)
                           ?  RCF_RECREATEALL
                           :  RCF_RECREATEFOLDERONLY);
            }

            if (ulReturn == RCF_RECREATEFOLDERONLY)
                WinCreateObject("WPFolder",             // now create new config folder w/ proper objID
                        "XFolder Configuration",
                        "OBJECTID=<XFOLDER_CONFIG>",
                        "<WP_DESKTOP>",                 // on desktop
                        CO_UPDATEIFEXISTS);
            else if (ulReturn == RCF_RECREATEALL) {
                HWND    hwndCreating;
                CHAR    szPath[CCHMAXPATH], szPath2[CCHMAXPATH], szPath3[CCHMAXPATH];
                ULONG   ulCopied = cmnQueryXFolderBasePath(szPath, sizeof(szPath));
                if (ulCopied)
                {   // INI entry found: append "\" if neccessary
                    PID     pid;
                    ULONG   sid;
                    sprintf(szPath2, "/c %s\\install\\crobj%s.cmd",
                            szPath,
                            cmnQueryLanguageCode());
                    strcpy(szPath3, &(szPath2[3]));

                    hwndCreating = WinLoadDlg(HWND_DESKTOP, HWND_DESKTOP,
                        DumbDlgProc,
                        NLS_MODULE,
                        ID_XFD_CREATINGCONFIG,
                        0);
                    WinShowWindow(hwndCreating, TRUE);
                    DosQuickStartSession("cmd.exe", szPath2,
                                FALSE, // don't show
                                TRUE,  // wait
                                &sid, &pid);

                    if (WinQueryObject(XFOLDER_CONFIGID) == NULLHANDLE) {
                        CHAR    szMsg[1000];
                        sprintf(szMsg, "Oooooops! XFolder could not find %s, which should have recreated "
                                       "your config folder. Please re-install XFolder.", szPath3);
                        DebugBox("XFolder", szMsg);
                    }
                    WinDestroyWindow(hwndCreating);
                }
            }
        break; }

        /*
         * WOM_PROCESSORDEREDCONTENT:
         *      this msg is posted by xfProcessOrderedContent to
         *      begin working our way through the contents of a
         *      folder; from here we will call a callback func
         *      which has been specified with xfProcessOrderedContent.
         *      Msg params:
         *          XFolder *mp1    folder to process
         *          PPROCESSCONTENTINFO
         *                  *mp2    structure with process info,
         *                          composed by xfProcessOrderedContent
         */

        case WOM_PROCESSORDEREDCONTENT: {
            XFolder             *pFolder = (XFolder*)mp1;
            PPROCESSCONTENTINFO pPCI = (PPROCESSCONTENTINFO)mp2;

            #ifdef DEBUG_STARTUP
                _Pmpf(("Entering WOM_PROCESSORDEREDCONTENT..."));
            #endif

            ulWorkerFunc2 = 5000;
            if (cmnCheckObject(pFolder)) {
                if (pPCI) {
                    if (pPCI->ulObjectNow == 0)
                    {   // first call: initialize structure
                        ulWorkerFunc2 = 5010;
                        pPCI->ulObjectMax = 0;
                        // now count objects
                        for (   pPCI->pObject = _xfQueryOrderedContent(pFolder, NULL, QC_FIRST);
                                (pPCI->pObject);
                                pPCI->pObject = _xfQueryOrderedContent(pFolder, pPCI->pObject, QC_NEXT)
                            )
                        {
                            pPCI->ulObjectMax++;
                        }
                        // get first object
                        pPCI->pObject = _xfQueryOrderedContent(pFolder, NULL, QC_FIRST);
                        #ifdef DEBUG_STARTUP
                            _Pmpf(("  Found %d objects", pPCI->ulObjectMax));
                        #endif
                    } else {
                        // subsequent calls: get next object
                        ulWorkerFunc2 = 5020;
                        pPCI->pObject = _xfQueryOrderedContent(pFolder, pPCI->pObject, QC_NEXT);
                    }

                    ulWorkerFunc2 = 5029;
                    pPCI->ulObjectNow++;
                    ulWorkerFunc2 = 5030;
                    if (pPCI->pObject) {
                        // this is not the last object: start it
                        ulWorkerFunc2 = 5040;
                        if (pPCI->pfnwpCallback) {
                            ulWorkerFunc2 = 5042;
                            if (cmnCheckObject(pPCI->pObject))
                            {
                                #ifdef DEBUG_STARTUP
                                    _Pmpf(("  Sending XOM_POCCALLBACK"));
                                #endif
                                cmnSendObjectMsg(XOM_POCCALLBACK, (MPARAM)pPCI, MPNULL);
                            }
                        }
                        ulWorkerFunc2 = 5043;
                        pPCI->ulFirstTime = DosGetULongTime();

                        // wait for next object
                        ulWorkerFunc2 = 5044;
                        cmnPostWorkerMsg(WOM_WAITFORPROCESSNEXT,
                           mp1, mp2);
                    }
                    else {
                        // pPCI->pObject == NULL:
                        // no more objects, call callback with NULL, clean up and stop
                        ulWorkerFunc2 = 5050;
                        if (pPCI->pfnwpCallback) {
                            ulWorkerFunc2 = 5052;
                            #ifdef DEBUG_STARTUP
                                _Pmpf(("  Sending XOM_POCCALLBACK, pObject == NULL"));
                            #endif
                            cmnSendObjectMsg(XOM_POCCALLBACK, (MPARAM)pPCI, MPNULL);
                            /* (*(pPCI->pfnwpCallback))(pPCI->ulCallbackParam,
                                (ULONG)NULL,
                                (MPARAM)pPCI->ulObjectNow, (MPARAM)pPCI->ulObjectMax); */
                            ulWorkerFunc2 = 5055;
                        }

                        ulWorkerFunc2 = 5059;
                        free(pPCI);
                    }
                }
            }
            #ifdef DEBUG_STARTUP
                _Pmpf(("  Done with WOM_PROCESSORDEREDCONTENT"));
            #endif

        break; }

        case WOM_WAITFORPROCESSNEXT: {
            // XFolder             *pFolder = (XFolder*)mp1;
            PPROCESSCONTENTINFO pPCI = (PPROCESSCONTENTINFO)mp2;
            BOOL                OKGetNext = FALSE;

            ulWorkerFunc2 = 5100;
            if (pPCI)
            {
                if (!pPCI->fCancelled) {
                    if (pPCI->ulTiming == 0)
                    {   // wait for close mode
                        ulWorkerFunc2 = 5110;
                        if (_wpWaitForClose(pPCI->pObject,
                                pPCI->hwndView, 0, SEM_IMMEDIATE_RETURN, FALSE) == 0)
                        {
                            OKGetNext = TRUE;
                        }
                    } else {
                        // timing mode
                        ulWorkerFunc2 = 5120;
                        if (DosGetULongTime() > ((pPCI->ulFirstTime) + pPCI->ulTiming))
                        {
                            OKGetNext = TRUE;
                        }
                    }

                    ulWorkerFunc2 = 5130;
                    if (OKGetNext)
                    {
                        ulWorkerFunc2 = 5131;
                        cmnPostWorkerMsg(WOM_PROCESSORDEREDCONTENT, mp1, mp2);
                    } else {
                        ulWorkerFunc2 = 5132;
                        cmnPostWorkerMsg(WOM_WAITFORPROCESSNEXT, mp1, mp2);
                        DosSleep(100);
                    }
                }
            }
            ulWorkerFunc2 = 5139;
        break; }

        /*
         * WOM_ADDAWAKEOBJECT:
         *      this is posted by XFldObject for each
         *      object that is awaked by the WPS; we
         *      need to maintain a list of these objects
         *      for XShutdown
         */

        case WOM_ADDAWAKEOBJECT: {
            WPObject              *pObj = (WPObject*)(mp1);
            POBJECTLISTITEM       poli = NULL;
            REGREC2               RegRec22;
            ULONG                 ulExcpt2;
            APIRET                rc;

            #ifdef DEBUG_AWAKEOBJECTS
               // _PmpfF(("WT: Adding awake object..."));
            #endif

            // increment global count
            lAwakeObjectsCount++;

            ulWorkerFunc2 = 6000;
            // set the quiet exception handler, because
            // sometimes we get a message for an object too
            // late, i.e. it is not awake any more, and then
            // we'll trap
            RegRec22.pfnHandler = (PFN)ExcptHandlerQuiet;
            if ( rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec22) )
                DosBeep(100, 1000);
            else {
                // store a known thread state
                ulExcpt2 = setjmp(RegRec22.jmpThread);
                if (ulExcpt2)
                {
                    #ifdef DEBUG_AWAKEOBJECTS
                        DosBeep(10000, 10);
                    #endif
                } else {
                    ulWorkerFunc2 = 6010;

                    // only save awake abstract and folder objects;
                    // if we included all WPFileSystem objects, all
                    // of these will be saved at XShutdown, that is,
                    // they'll all get .CLASSINFO EAs, which we don't
                    // want
                    if (    (_somIsA(pObj, _WPAbstract))
                         || (_somIsA(pObj, _WPFolder))
                       )
                    {
                        APIRET rc;
                        ulWorkerFunc2 = 6020;
                        if (strcmp(_somGetClassName(pObj), "SmartCenter") == 0)
                        {
                            pAwakeWarpCenter = pObj;
                        }

                        ulWorkerFunc2 = 6030;
                        rc = DosRequestMutexSem(hmtxAwakeObjects, 1000);
                        if (rc == NO_ERROR)
                        {
                            ulWorkerFunc2 = 6040;
                            poli = pliAwakeObjectsFirst;
                            while (poli)
                                if (poli->pObj == pObj)
                                    break;
                                else
                                    poli = poli->pNext;

                            if (poli == NULL)
                            {
                                poli = malloc(sizeof(OBJECTLISTITEM));
                                poli->pObj = pObj;
                                lstAppendItem((PLISTITEM*)&pliAwakeObjectsFirst, (PLISTITEM*)&pliAwakeObjectsLast,
                                    (PLISTITEM)poli,
                                    0, 0);
                            }
                        }

                        ulWorkerFunc2 = 6090;
                        DosReleaseMutexSem(hmtxAwakeObjects);
                    }
                }
            }

            DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec22);

            #ifdef DEBUG_AWAKEOBJECTS
                // _PmpfF(("WT:   leaving."));
            #endif
        break; }

        /*
         * WOM_REMOVEAWAKEOBJECT:
         *      this is posted by WPObject also, but
         *      when an object goes back to sleep.
         *      Be careful: the object pointer in mp1
         *      does not point to a valid SOM object
         *      any more, because the object has
         *      already been freed in memory; so we
         *      must not call any methods here.
         *      We only use the object pointer
         *      for finding the respective object
         *      in the linked list.
         */

        case WOM_REMOVEAWAKEOBJECT:
        {
            WPObject            *pObj = (WPObject*)(mp1);
            POBJECTLISTITEM     poli = NULL;

            #ifdef DEBUG_AWAKEOBJECTS
                // _PmpfF(("WT: Removing asleep object."));
            #endif

            // increment global count
            lAwakeObjectsCount--;

            ulWorkerFunc2 = 7000;
            if (DosRequestMutexSem(hmtxAwakeObjects, 4000)
                == NO_ERROR)
            {
                poli = pliAwakeObjectsFirst;
                while (poli) {
                    if (poli->pObj == pObj) {
                        break;
                    }
                    else
                        poli = poli->pNext;
                }
            }
            DosReleaseMutexSem(hmtxAwakeObjects);

            if (poli) {
                lstRemoveItem((PLISTITEM*)&pliAwakeObjectsFirst, (PLISTITEM*)&pliAwakeObjectsLast,
                            (PLISTITEM)poli,
                            hmtxAwakeObjects, 4000);
            }
        break; }

        /*
         * WOM_REFRESHFOLDERVIEWS:
         *    this one is posted by XFolder's overrides of wpMoveObject,
         *    wpSetTitle or wpRefresh with the calling instance's somSelf
         *    in mp1; we will now update the open frame window titles of both
         *    the caller and its open subfolders with the full folder path
         *    Parameters:
         *    XFolder* mp1  folder to update; if this is NULL, all open
         *                  folders on the system will be updated
         */

        case WOM_REFRESHFOLDERVIEWS: {
            XFolder     *pFolder = NULL,
                        *pCalling = (XFolder*)(mp1);

            ulWorkerFunc2 = 8000;
            if (!cmnCheckObject(pCalling))
                break;

            ulWorkerFunc2 = 8002;
            for ( pFolder = _wpclsQueryOpenFolders(_WPFolder, NULL, QC_FIRST, FALSE);
                  pFolder;
                  pFolder = _wpclsQueryOpenFolders(_WPFolder, pFolder, QC_NEXT, FALSE))
            {
                ulWorkerFunc2 = 8004;
                if (cmnCheckObject(pFolder))
                    if (_somIsA(pFolder, _WPFolder))
                    {
                        ulWorkerFunc2 = 8006;
                        if (pCalling == NULL) {
                            ulWorkerFunc2 = 8010;
                            _xfUpdateAllFrameWndTitles(pFolder);
                        } else if (_xfResidesBelow(pFolder, pCalling)) {
                            ulWorkerFunc2 = 8020;
                            _xfUpdateAllFrameWndTitles(pFolder);
                        }
                    }
            }
        break; }

        /*
         * WOM_UPDATEALLSTATUSBARS:
         *      goes thru all open folder views and updates
         *      status bars if neccessary.
         *      Parameters:
         *      ULONG mp1   do-what flag
         *                  1:  show or hide status bars according
         *                      to folder instance and Global settings
         *                  2:  reformat status bars (e.g. because
         *                      fonts have changed)
         */

        case WOM_UPDATEALLSTATUSBARS: {
            #ifdef DEBUG_STATUSBARS
                _Pmpf(( "WT: WOM_UPDATEALLSTATUSBARS" ));
            #endif
            ulWorkerFunc2 = 8500;
            // for each open folder view, call the callback
            // which updates the status bars
            // (fncbUpdateStatusBars in xfldr.c)
            _xfclsForEachOpenView(_XFolder, (ULONG)mp1,
                (PFNWP)fncbUpdateStatusBars);
        break; }

        /*
         * WM_INVALIDATEORDEREDCONTENT:
         *
         */

        case WOM_INVALIDATEORDEREDCONTENT: {
            /* this one is posted when the .ICONPOS data
               has changed for a folder;
               again, the calling instance's somSelf is in mp1;
               we will now delete our iconpos list, so it can
               be rebuilt when needed */

            // XFolderData *somThis;

            ulWorkerFunc2 = 9000;
            if (!cmnCheckObject((WPObject*)(mp1)))
                break;

            ulWorkerFunc2 = 9001;
            if (_somIsA((WPObject*)(mp1), _XFolder))
                _xfInvalidateOrderedContent((XFolder*)(mp1));
        break; }

        /*
         * WOM_DELETEICONPOSEA:
         *
         */

        case WOM_DELETEICONPOSEA: {
            /* this msg is posted when the .ICONPOS EAs are
               do be deleted; we better do this in the
               background thread */
            EABINDING   eab;
            XFolder     *pFolder = (XFolder*)mp1;
            CHAR        szPath[CCHMAXPATH];
            ULONG       ulSize = sizeof(szPath)-1;

            ulWorkerFunc2 = 9500;
            if (!cmnCheckObject(pFolder))
                break;

            _wpQueryRealName(pFolder, szPath, &ulSize, TRUE); // fully qualfd.
            eab.bFlags = 0;
            eab.pszName = ".ICONPOS";
            eab.bNameLength = strlen(eab.pszName);
            eab.pszValue = "";
            eab.usValueLength = 0;
            EAPathWriteOne(szPath, &eab);
        break; }

        /*
         * WOM_TREEVIEWAUTOSCROLL:
         *     this msg is posted by the fnwpSubclassedFolderFrame
         *     (subclassed folder windows) after the "plus" sign has
         *     been clicked on (WM_CONTROL for containers with
         *     CN_EXPANDTREE notification). Parameters:
         *          HWND mp1:    frame wnd handle
         *          PMINIRECORDCORE mp2:
         *                       the expanded minirecordcore
         */

        case WOM_TREEVIEWAUTOSCROLL: {
            WPObject    *pFolder = OBJECT_FROM_PREC((PMINIRECORDCORE)mp2);
            ulWorkerFunc2 = 10000;

            if (cmnCheckObject(pFolder))
            {
                ulWorkerFunc2 = 10010;
                while (_somIsA(pFolder, _WPShadow))
                    pFolder = _wpQueryShadowedObject(pFolder, TRUE);

                ulWorkerFunc2 = 10020;
                if (!_somIsA(pFolder, _WPFolder)) // check only folders, avoid disks
                    break;

                // now check if the folder whose "plus" sign has been
                // clicked on is already populated: if so, the WPS seems
                // to insert the objects directly (i.e. in the Workplace
                // thread), if not, the objects are inserted by some
                // background populate thread, which we have no control
                // over...
                ulWorkerFunc2 = 10030;
                if ( (_wpQueryFldrFlags(pFolder) & FOI_POPULATEDWITHFOLDERS) == 0)
                {
                    // NOT fully populated: sleep a while, then post
                    // the same msg again, until the "populated" folder
                    // flag has been set
                    DosSleep(100);
                    ulWorkerFunc2 = 10040;
                    cmnPostWorkerMsg(WOM_TREEVIEWAUTOSCROLL, mp1, mp2);
                }
                else {
                    // otherwise: scroll the tree view properly
                    HWND                hwndCnr = WinWindowFromID((HWND)mp1, 0x8008);
                    PMINIRECORDCORE     preccLastChild;
                    ulWorkerFunc2 = 10050;
                    preccLastChild = WinSendMsg(hwndCnr, CM_QUERYRECORD,
                        mp2,   // PMINIRECORDCORE
                        MPFROM2SHORT(CMA_LASTCHILD, CMA_ITEMORDER));
                    WinCnrScrollToRecord(hwndCnr,
                            (PRECORDCORE)preccLastChild,
                            CMA_TEXT,
                            TRUE);
                }
                ulWorkerFunc2 = 10090;
            }
        break; }

        /*
         * WM_CORRECTCNRSORT:
         *      this msg is posted by XFolder's wpOpen if
         *      the WPS's default sort methods have been
         *      replaced; trouble is, when a folder is populated
         *      after having been opened, the WPS resets the
         *      sort flags for the container to something I
         *      don't fully understand, but I know I don't want,
         *      so we wait in here until the folder has finished
         *      populating and then reset the cnr flags properly.
         *      Parameters:
         *          XFolder *mp1    folder to watch
         *          HWND    mp2     cnr wnd handle to update
         *                          (one folder view only)
         */

        case WOM_CORRECTCNRSORT: {
            XFolder *pFolder = (XFolder*)(mp1);
            if ( (_wpQueryFldrFlags(pFolder) & FOI_POPULATEDWITHALL) == 0) {
                // not fully populated yet: sleep for a while and
                // then repost the same msg again
                DosSleep(100);
                cmnPostWorkerMsg(WOM_CORRECTCNRSORT, mp1, mp2);
            } else {
                // populated: correct cnr flags
                cmnSetCnrSort(pFolder, (HWND)(mp2));
            }
        break; }

        /* #ifdef DEBUG_CONTEXT
        case WM_SHOWFOLDERDATA: {
            CHAR    szMsg[1000] = "";
            ULONG   ulFlags = _wpQueryFldrFlags((WPObject*)(mp1));
            // XFolderData *somThis = XFolderGetData((WPObject*)(mp1));

            ulWorkerFunc2 = 11000;

            // ((PLISTITEM)NULL)->pNext = NULL;

            if (ulFlags & FOI_POPULATEDWITHALL)
                strcat(szMsg, "Populated with all\n");
            if (ulFlags & FOI_POPULATEDWITHFOLDERS)
                strcat(szMsg, "Populated with folders\n");
            if (ulFlags & FOI_WORKAREA)
                strcat(szMsg, "Workarea\n");
            if (ulFlags & FOI_CHANGEFONT)
                strcat(szMsg, "FOI_CHANGEFONT\n");
            if (ulFlags & FOI_WAMINIMIZED)
                strcat(szMsg, "FOI_WASTARTONRESTORE\n");
            if (ulFlags & FOI_POPULATEDWITHALL)
                strcat(szMsg, "FOI_WASTARTONRESTORE\n");
            if (ulFlags & FOI_NOREFRESHVIEWS)
                strcat(szMsg, "FOI_NOREFRESHVIEWS\n");
            if (ulFlags & FOI_ASYNCREFRESHONOPEN)
                strcat(szMsg, "FOI_ASYNCREFRESHONOPEN\n");
            if (ulFlags & FOI_TREEPOPULATED)
                strcat(szMsg, "Tree populated\n");
            if (ulFlags & FOI_POPULATEINPROGRESS)
                strcat(szMsg, "Populate in progress\n");
            if (ulFlags & FOI_REFRESHINPROGRESS)
                strcat(szMsg, "Refresh in progress\n");
            if (ulFlags & FOI_FIRSTPOPULATE)
                strcat(szMsg, "FOI_FIRSTPOPULATE: no iconpos data\n");
            if (ulFlags & FOI_WAMCRINPROGRESS)
                strcat(szMsg, "FOI_WAMCRINPROGRESS (Minimize, close, restore in progress)\n");
            if (ulFlags & FOI_CNRBKGNDOLDFORMAT)
                strcat(szMsg, "FOI_CNRBKGNDOLDFORMAT (CnrBkgnd saved in old format)\n");
            if (ulFlags & FOI_DELETEINPROGRESS)
                strcat(szMsg, "FOI_DELETEINPROGRESS (to prevent wpFree from repopulating)\n");

            sprintf(szMsg+strlen(szMsg), "\n_pliOrderedContent list count: %d",
                lstCountItems((PLISTITEM)_pliOrderedContentFirst, _hmtxOrderedContent, 4000));

            sprintf(szMsg+strlen(szMsg), "\npliWndProcs list count: %d",
                lstCountItems((PLISTITEM)pliWndProcs, hmtxWndProcs, 4000));

            sprintf(szMsg+strlen(szMsg), "\npliAwakeObjects list count: %d",
                lstCountItems((PLISTITEM)pliAwakeObjectsFirst, hmtxAwakeObjects, 4000));

            if (tidWorkerThread) {
                sprintf(szMsg+strlen(szMsg), "\nWorker thread ID: 0x0x%lX, prty: %s",
                    tidWorkerThread,
                    (WorkerThreadHighPriority ? "regular" : "idle"));
            }
            if (thrQueryID(ptiWatchdogThread))
                sprintf(szMsg+strlen(szMsg), "\nWatchdog thread ID: 0x0x%lX",
                    thrQueryID(ptiWatchdogThread));
            if (thrQueryID(ptiUpdateThread))
                sprintf(szMsg+strlen(szMsg), "\nUpdate thread ID: 0x0x%lX",
                    thrQueryID(ptiUpdateThread));
            if (thrQueryID(ptiShutdownThread))
                sprintf(szMsg+strlen(szMsg), "\nShutdown thread ID: 0x0x%lX",
                    thrQueryID(ptiShutdownThread));

            DebugBox(_wpQueryTitle((WPObject*)(mp1)), szMsg);
        break; }
        #endif */

        default:
            mrc = WinDefWindowProc(hwndObject, msg, mp1, mp2);

    } // end switch

    ulWorkerFunc2 = 99990;
    ulWorkerFunc2 = 99991;

    return (mrc);
}

/*
 * fntWorkerThread:
 *          this is the thread function for the XFolder
 *          "Worker" (= background) thread, to which
 *          messages for tasks are passed which are too
 *          time-consuming to be processed on the
 *          Workplace thread; it creates an object window
 *          and stores its handle in hwndWorkerObject.
 *          This thread is created by cmnInitializeXFolder.
 */

void _System fntWorkerThread(ULONG ulData)
{
    QMSG                  qmsg;

    // HAB                   habDesktop = WinQueryAnchorBlock(HWND_DESKTOP);

    // HWND                  hwndStartupStatus = NULLHANDLE;
    // HEV                   hev = 0;
    // CHAR                  szHevName[CCHMAXPATH];
    // ULONG                 ulEvents;
    // ULONG                 ulTimerID = 0;

    // PSZ                   pszCreateFolder = NULL;

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    APIRET                rc;
    PSZ                   pszErrMsg = NULL;

    begin:

    // set our extended exception handler
    RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
    if ( rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulWorkerFunc2 = 500;
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt)
    {
        HWND hwndDesktop;

        // the thread exception handler puts us here if an exception occured:
        ulWorkerFunc2 = 510;
        // first get the handler off the chain
        if (rc = DosUnsetExceptionHandler( (PEXCEPTIONREGISTRATIONRECORD)&RegRec2 ))
            DosBeep(100, 1000);

        // clean up
        ulWorkerFunc2 = 515;
        WinDestroyWindow(hwndWorkerObject);
        WinDestroyMsgQueue(hmqWorkerThread);
        WinTerminate(habWorkerThread);
        if (hmtxAwakeObjects) {
            DosReleaseMutexSem(hmtxAwakeObjects);
        }

        // check for the killed-thread case
        /* switch (ulExcpt) {
            case XCPT_PROCESS_TERMINATE:
            case XCPT_ASYNC_PROCESS_TERMINATE:
                DosExit( EXIT_THREAD, 0 );
                break;
        } */

        hwndDesktop = _wpclsQueryActiveDesktopHWND(_WPDesktop);
        if (    (hwndDesktop)
             && (pszErrMsg == NULL)
           )
        {
            // only report the first error, or otherwise we will
            // jam the system with msg boxes
            ulWorkerFunc2 = 516;
            pszErrMsg = malloc(1000);
            if (pszErrMsg) {
                strcpy(pszErrMsg, "An error occured in the XFolder Worker thread. "
                        "Since the error is probably not serious, "
                        "the Worker thread will continue to run. However, if errors "
                        "like these persist, you might want to disable the "
                        "Worker thread in the XFolder Global Settings, "
                        "contact XFolder's author, and send him the file XFLDTRAP.LOG, "
                        "which you will find in the root directory of your boot drive. ");
                cmnPostObjectMsg(XOM_EXCEPTIONCAUGHT, (MPARAM)pszErrMsg, MPNULL);
            }
        }

        // "restart" thread
        goto begin;
    }

    ulWorkerFunc2 = 599;

    /* we will now create the mutex semaphore for access to
       the linked list of awake objects; this is used in
       wpInitData and wpUnInitData */
    DosCreateMutexSem("\\sem32\\AwakeObjectsList",
                        &hmtxAwakeObjects, 0, FALSE);  // unnamed, unowned

    ulWorkerFunc2 = 1;
    if (habWorkerThread = WinInitialize(0))
    {
        if (hmqWorkerThread = WinCreateMsgQueue(habWorkerThread, 4000))
        {
            CHAR szWorkerClass[] = "XFolderWorkerObject";

            ulWorkerFunc2 = 2;
            WinCancelShutdown(hmqWorkerThread, TRUE);

            WorkerThreadHighPriority = FALSE;

            WinRegisterClass(habWorkerThread,
                             szWorkerClass,    // class name
                             (PFNWP)fnwpWorkerObject,    // Window procedure
                             0,                  // class style
                             0);                 // extra window words

            // create object window
            hwndWorkerObject = WinCreateWindow(
                                HWND_OBJECT,
                                szWorkerClass,
                                (PSZ)"",     // title
                                0,           // style
                                0,0,0,0,     // position
                                0,           // owner
                                HWND_BOTTOM, // z-order
                                ID_WORKEROBJECT, // window id
                                NULL,        // create params
                                NULL);       // pres params

            if (!hwndWorkerObject)
                DebugBox("XFolder: Error", "XFolder failed to create the Worker thread object window.");

            /* set ourselved to idle-time priority; this will
               be raised to regular when the msg queue becomes congested */
            cmnRaiseWorkerThreadPriority(FALSE);

            // now enter the message loop
            while (WinGetMsg(habWorkerThread, &qmsg, NULLHANDLE, 0, 0))
            {
                ulWorkerFunc2 = 9;
                if (ulAwakeMsgCount > 0)
                    ulAwakeMsgCount--;

                if (ulAwakeMsgCount < 10)
                    cmnRaiseWorkerThreadPriority(FALSE);

                // dispatch the queue msg
                WinDispatchMsg(habWorkerThread, &qmsg);
                ulWorkerFunc2 = 99992;
            }
            // loop until WM_QUIT

            ulWorkerFunc2 = 99993;
            WinDestroyWindow(hwndWorkerObject);
            hwndWorkerObject = NULLHANDLE;
            WinDestroyMsgQueue(hmqWorkerThread);
            hmqWorkerThread = NULLHANDLE;
            ulWorkerFunc2 = 99994;
        }
        WinTerminate(habWorkerThread);
        ulWorkerFunc2 = 99995;
    }

    ulWorkerFunc2 = 99996;
    DosCloseMutexSem(hmtxAwakeObjects);

    ulWorkerFunc2 = 99997;
    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
    tidWorkerThread = NULLHANDLE;
}

/*
 * fnwpQuickObject:
 *      wnd proc for Sound thread object window (see below)
 */

USHORT  usDeviceID  = 0;
HWND    hwndBootupStatus = NULLHANDLE;

MRESULT EXPENTRY fnwpQuickObject(HWND hwndObject, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = NULL;

    ulQuickFunc2 = 1100;
    switch (msg) {

        /*
         * WM_CREATE:
         *      show XFolder logo at bootup
         */

        case WM_CREATE: {
            // these are already initialized by xfobjM_wpclsInitData
            PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
            if (pGlobalSettings->ShowXFolderAnim) {

                #define BMPSPACING      30
                #define ANIMTIME        1000

                ULONG   ulInitialTime,
                        ulNowTime;
                        // ulCurrentSize = 10;

                HPS hps = WinGetPS(HWND_DESKTOP);
                if (hps) {
                    HBITMAP hbm = GpiLoadBitmap(hps,
                                        _xfclsQueryModuleHandle(_XFldObject),
                                        ID_XFLDRBIGLOGO, 0, 0);
                    if (hbm) {
                        POINTL  ptl = {BMPSPACING, 0};
                        RECTL   rtlStretch;
                        ULONG   ul,
                                ulSteps = 20;
                        BITMAPINFOHEADER bih;
                        GpiQueryBitmapParameters(hbm, &bih);
                        /* ptl.y = WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN)
                                        - BMPSPACING
                                        - bih.cy; */
                        ptl.x = (WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN)
                                        - bih.cx) / 2;
                        ptl.y = (WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN)
                                        - bih.cy) / 2;

                        // we now use ul for the current animation step,
                        // which is a pointer on a scale from 1 to 1000;
                        // ul will be recalculated after each animation
                        // step according to how much time the animation
                        // has cost on this display so far. This has the
                        // following advantages:
                        // 1) no matter how fast the system is, the animation
                        //    will always last ANIMAX milliseconds
                        // 2) since large bitmaps take more time to calculate,
                        //    the animation won't appear to slow down then
                        ulInitialTime = DosGetULongTime();
                        ul = 1;
                        ulSteps = 1000;
                        do {
                            LONG cx = (((bih.cx-20) * ul) / ulSteps) + 20;
                            LONG cy = (((bih.cy-20) * ul) / ulSteps) + 20;
                            rtlStretch.xLeft   = ptl.x + ((bih.cx - cx) / 2);
                            rtlStretch.yBottom = ptl.y + ((bih.cy - cy) / 2);
                            rtlStretch.xRight = rtlStretch.xLeft + cx;
                            rtlStretch.yTop   = rtlStretch.yBottom + cy;

                            WinDrawBitmap(hps, hbm, NULL, (PPOINTL)&rtlStretch,
                                    0, 0,       // we don't need colors
                                    DBM_STRETCH);

                            ulNowTime = DosGetULongTime();

                            // recalculate ul: rule of three based on the
                            // time we've spent on animation so far
                            ul = (ulSteps
                                    * (ulNowTime - ulInitialTime)) // time spent so far
                                    / ANIMTIME;     // time to spend altogether

                        } while (ul < ulSteps);

                        WinDrawBitmap(hps, hbm, NULL, &ptl,
                                0, 0,       // we don't need colors
                                DBM_NORMAL);
                        GpiDeleteBitmap(hbm);
                    }
                    WinReleasePS(hps);
                }
            }

            mrc = WinDefWindowProc(hwndObject, msg, mp1, mp2);
        break; }

        /*
         * QM_BOOTUPSTATUS:
         *      posted by XFldObject::wpclsInitData; mp1
         *      points to a SOM object being initalized;
         *      if NULL, then
         *      the bootup status wnd will be destroyed
         */

        case QM_BOOTUPSTATUS: {
            if (hwndBootupStatus == NULLHANDLE) {
                strcpy(szBootupStatus, "Initialized %s");
                // WinLoadString
                hwndBootupStatus = WinLoadDlg(HWND_DESKTOP, HWND_DESKTOP,
                            fnwpGenericStatus,
                            NLS_MODULE, ID_XFD_BOOTUPSTATUS,
                            NULL);
                WinSetWindowPos(hwndBootupStatus,
                        HWND_TOP,
                        0, 0, 0, 0,
                        /* SWP_DEACTIVATE |*/ SWP_SHOW);
                // WinShowWindow(hwndBootupStatus, TRUE);
            }

            if (hwndBootupStatus)
                if (mp1 == NULL)
                    WinDestroyWindow(hwndBootupStatus);
                else {
                    CHAR szTemp[2000];
                    WPObject* pObj = (WPObject*)mp1;
                    PSZ pszClassTitle = _somGetName(pObj);
                    if (pszClassTitle)
                        sprintf(szTemp, szBootupStatus, pszClassTitle);
                    else sprintf(szTemp, szBootupStatus, "???");

                    WinSetDlgItemText(hwndBootupStatus, ID_XFDI_BOOTUPSTATUSTEXT,
                            szTemp);

                    WinStartTimer(habWorkerThread, hwndObject, 1, 2000);
                }
        break; }

        /*
         * WM_TIMER:
         *      destroy bootup status wnd
         */

        case WM_TIMER: {
            switch ((USHORT)mp1)  {// timer id

                // timer 1: bootup status wnd
                case 1:
                    if (hwndBootupStatus) {
                        WinDestroyWindow(hwndBootupStatus);
                        hwndBootupStatus = NULLHANDLE;
                    }
                    WinStopTimer(habWorkerThread, hwndObject, 1);
                break;
            }
        break; }

        /*
         * QM_PLAYSYSTEMSOUND:
         *      plays system sound specified in MMPM.INI;
         *      (USHORT)mp1 must be the index
         */

        case QM_PLAYSYSTEMSOUND: {
            CHAR    szDescr[CCHMAXPATH];
            PSZ     pszFile = malloc(CCHMAXPATH);
            ULONG   ulVolume;

            ulQuickFunc2 = 1200;
            if (cmnQuerySystemSound((USHORT)mp1,
                    szDescr, pszFile, &ulVolume))
            {
                // _Pmpf(( "AT: Sound %d == %s, %s", mp1, szDescr, pszFile ));
                WinPostMsg(hwndObject,
                            QM_PLAYSOUND,
                            (MPARAM)pszFile, (MPARAM)ulVolume);
            } else
                free(pszFile);
        break; }

        /*
         * QM_PLAYSOUND:
         *      plays the sound file specified in
         *      (PSZ)mp1; this PSZ is assumed to have
         *      been allocated using malloc() and will
         *      be freed afterwards.
         *      (ULONG)mp2 must be the volume (0-100)
         */

        case QM_PLAYSOUND: {
            PSZ     pszFile = (PSZ)mp1;
            if (pszFile)
            {
                ULONG           ulrc;
                // CHAR            szTemp[1000];
                MCI_OPEN_PARMS  mop;
                MCI_PLAY_PARMS  mpp;
                MCI_SET_PARMS   msp;
                MCI_GENERIC_PARMS mgp;

                ulQuickFunc2 = 1300;
                if (usDeviceID) {
                    // we're currently playing: abort current job
                    // and close device first
                    mgp.hwndCallback = hwndObject;
                    mciSendCommand(usDeviceID,
                                MCI_STOP,
                                MCI_WAIT,
                                (ULONG)NULL,
                                (ULONG)NULL);
                    mciSendCommand(usDeviceID,
                                MCI_CLOSE,
                                MCI_WAIT,
                                (ULONG)NULL,
                                (ULONG)NULL);
                }

                // open device
                mop.hwndCallback = hwndObject;       // callback hwnd
                mop.usDeviceID = 0;                  // we don't have one yet
                mop.pszDeviceType = NULL;            // using default device type
                mop.pszElementName = pszFile;        // file name to open

                ulrc = mciSendCommand(0,
                    MCI_OPEN,
                    MCI_WAIT | MCI_OPEN_ELEMENT |
                    MCI_OPEN_SHAREABLE,
                    &mop,
                    0);                              // No user parm

                if (LOUSHORT(ulrc) == MCIERR_SUCCESS) {
                    // successful: go on
                    usDeviceID = mop.usDeviceID;      // remember device ID

                    msp.ulLevel = (ULONG)mp2;            // volume passed in mp2
                    msp.ulAudio = MCI_SET_AUDIO_ALL;
                    mciSendCommand(usDeviceID,
                                   MCI_SET,
                                   MCI_WAIT | MCI_SET_AUDIO |
                                   MCI_SET_VOLUME,
                                   &msp, 0);

                    // play, request callback msg
                    mpp.hwndCallback = (HWND)hwndObject;
                    mciSendCommand(usDeviceID,       // device ID
                                    MCI_PLAY,
                                    MCI_NOTIFY,      // flags
                                    (PVOID)&mpp,
                                    0);              // bo user parm
                } else
                    // no success:
                    usDeviceID = 0;

                free(pszFile);
            }
        break; }

        case MM_MCINOTIFY: {
            // _Pmpf(( "MM_MCINOTIFY: mp1.1 = 0x%lX", SHORT1FROMMP(mp1) ));
            ulQuickFunc2 = 1400;
            if (    (SHORT1FROMMP(mp2) == usDeviceID)
                 && (SHORT1FROMMP(mp1) == MCI_NOTIFY_SUCCESSFUL)
               )
            {
                mciSendCommand(usDeviceID,
                            MCI_CLOSE,
                            MCI_WAIT,
                            (ULONG)NULL,
                            (ULONG)NULL);
                usDeviceID = 0;
            }
        break; }

        default:
            ulQuickFunc2 = 1500;
            mrc = WinDefWindowProc(hwndObject, msg, mp1, mp2);
    }

    ulQuickFunc2 = 1600;
    return (mrc);
}

/*
 * fntQuickThread:
 *          this is the thread function for the XFolder
 *          "Quick" thread, which accepts msgs posted by
 *          cmnPlaySystemSound. As opposed to the "Worker"
 *          thread, this thread runs with regular priority.
 *          This thread is also created from
 *          cmnInitializeXFolder.
 */

void _System fntQuickThread(ULONG ulDummy)
{
    HAB                   habQuickThread = NULLHANDLE;
    HMQ                   hmqQuickThread = NULLHANDLE;
    QMSG                  qmsg;

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    APIRET                rc;
    PSZ                   pszErrMsg = NULL;

    // set our extended exception handler
    RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
    if ( rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulQuickFunc2 = 500;
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt)
    {
        HWND hwndDesktop;

        // the thread exception handler puts us here if an exception occured:
        ulQuickFunc2 = 510;
        // first get the handler off the chain
        if (rc = DosUnsetExceptionHandler( (PEXCEPTIONREGISTRATIONRECORD)&RegRec2 ))
            DosBeep(100, 1000);

        // clean up
        ulQuickFunc2 = 515;
        WinDestroyWindow(hwndQuickObject);
        hwndQuickObject = NULLHANDLE;
        WinDestroyMsgQueue(hmqQuickThread);
        WinTerminate(habQuickThread);

        // check for the killed-thread case
        /* switch (ulExcpt) {
            case XCPT_PROCESS_TERMINATE:
            case XCPT_ASYNC_PROCESS_TERMINATE:
                DosExit( EXIT_THREAD, 0 );
                break;
        } */

        hwndDesktop = _wpclsQueryActiveDesktopHWND(_WPDesktop);
        if (    (hwndDesktop)
             && (pszErrMsg == NULL)
           )
        {
            // only report the first error, or otherwise we will
            // jam the system with msg boxes
            ulQuickFunc2 = 516;
            pszErrMsg = malloc(1000);
            if (pszErrMsg) {
                strcpy(pszErrMsg, "An error occured in the XFolder Quick thread. "
                        "\n\nThe additional XFolder system sounds will be disabled for the "
                        "rest of this Workplace Shell session. You will need to restart "
                        "the WPS in order to re-enable them. "
                        "\n\nIf errors like these persist, you might want to disable the "
                        "additional XFolder system sounds again. For doing this, execute "
                        "SOUNDOFF.CMD in the BIN subdirectory of the XFolder installation "
                        "directory. "
                        "\n\nYou will find additional debug information "
                        "in the XFLDTRAP.LOG file in the root directory of your boot drive. ");
                cmnPostObjectMsg(XOM_EXCEPTIONCAUGHT, (MPARAM)pszErrMsg, MPNULL);
            }
        }

        DosExit(EXIT_THREAD, 0);
    }

    ulQuickFunc2 = 1;
    if (habQuickThread = WinInitialize(0))
    {
        if (hmqQuickThread = WinCreateMsgQueue(habQuickThread, 3000))
        {
            CHAR szQuickClass[] = "XFolderQuickObject";

            ulQuickFunc2 = 2;
            WinCancelShutdown(hmqQuickThread, TRUE);

            WinRegisterClass(habQuickThread,
                             szQuickClass,    // class name
                             (PFNWP)fnwpQuickObject,    // Window procedure
                             0,                  // class style
                             0);                 // extra window words

            // create object window
            hwndQuickObject = WinCreateWindow(
                                HWND_OBJECT,
                                szQuickClass,
                                (PSZ)"",     // title
                                0,           // style
                                0,0,0,0,     // position
                                0,           // owner
                                HWND_BOTTOM, // z-order
                                ID_WORKEROBJECT,  // window id
                                NULL,        // create params
                                NULL);       // pres params

            if (!hwndQuickObject)
                DebugBox("XFolder: Error", "XFolder failed to create the Quick thread object window.");

            // set ourselves to higher regular priority
            DosSetPriority(PRTYS_THREAD,
                           PRTYC_REGULAR,
                           +31, // priority delta
                           0);

            // now enter the message loop
            while (WinGetMsg(habQuickThread, &qmsg, NULLHANDLE, 0, 0)) {
                ulQuickFunc2 = 9;
                WinDispatchMsg(habQuickThread, &qmsg);
            }
            // loop until WM_QUIT

            ulQuickFunc2 = 99993;
            WinDestroyWindow(hwndQuickObject);
            hwndQuickObject = NULLHANDLE;
            WinDestroyMsgQueue(hmqQuickThread);
            hmqQuickThread = NULLHANDLE;
            ulQuickFunc2 = 99994;
        }
        WinTerminate(habQuickThread);
        ulQuickFunc2 = 99995;
    }
    ulQuickFunc2 = 99997;
    tidQuickThread = NULLHANDLE;
    // _endthread();
}


/*
 * fnXFolderExitList:
 *      routine for DosExitList in case PMSHELL.EXE gets killed or traps;
 *      this is initialized in XFldObjM_wpclsInitData
 */

void APIENTRY fnXFolderExitList(VOID)
{
    ULONG   ul = 1;

    DosExitCritSec();

    #ifdef DEBUG_SHUTDOWN
        DosBeep(1000, 100);
    #endif

    if (tidWorkerThread) {
        DosKillThread(tidWorkerThread);
        tidWorkerThread = NULLHANDLE;
    }

    if (tidQuickThread) {
        DosKillThread(tidQuickThread);
        tidQuickThread = NULLHANDLE;
    }

    if (PreventNextStartup)
        PrfWriteProfileData(HINI_USERPROFILE, INIAPP, INIKEY_DONTDOSTARTUP,
                &ul, sizeof(ul));

    #ifdef DEBUG_SHUTDOWN
        DosBeep(5000, 100);
    #endif
    DosExitList(EXLST_EXIT, (PFNEXITLIST)NULL);
}

/*
 * cmnInitializeXFolder:
 *      this starts both the Worker and Quick threads; called
 *      from M_WPObject_wpclsInitData
 */

VOID cmnInitializeXFolder(VOID)
{
    if (tidWorkerThread == NULLHANDLE)
    {
        // PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
        PTIB        ptib;
        PPIB        ppib;
        if (DosGetInfoBlocks(&ptib, &ppib) == NO_ERROR)
            if (ptib)
                if (ptib->tib_ptib2)
                    tidWorkplaceThread = ptib->tib_ptib2->tib2_ultid;

        DosEnterCritSec();
        // if (pGlobalSettings->NoWorkerThread == 0)
        {
            PreventNextStartup = TRUE;

            DosExitList(EXLST_ADD | 0x00000000,
                (PFNEXITLIST)fnXFolderExitList);

            ulAwakeMsgCount = 0;
            DosCreateThread(&tidWorkerThread,
                            fntWorkerThread,
                            0,
                            CREATE_READY |
                            STACK_COMMITTED,
                            2*65536);

            DosCreateThread(&tidQuickThread,
                            fntQuickThread,
                            0,
                            CREATE_READY |
                            STACK_COMMITTED,
                            2*65536);
        }
        DosExitCritSec();
    }
}

/********************************************************************
 *                                                                  *
 *   thread exception handlers                                      *
 *                                                                  *
 ********************************************************************/

/*
 * The following code is based on code written by Monte Copeland,
 * IBM Boca Ration, Florida, USA (1993). I have enhanced it to
 * be able to write logfiles etc.
 *
 */

static PSZ pszRegisters =
    "\nRegister dump:\n"
    "EAX: 0x%08X  EBX: 0x%08X  ECX: 0x%08X  EDX: 0x%08X\n"
    "EBP: 0x%08X  EDI: 0x%08X  ESI: 0x%08X  ESP: 0x%08X\n"
    " CS: 0x%04X  DS: 0x%04X  ES: 0x%04X  FS: 0x%04X\n"
    " GS: 0x%04X  SS: 0x%04X\n";

/*
 * cmnExplainException:
 *      used by the exception handlers below to print
 *      information about the exception into pszBuffer.
 */

VOID cmnExplainException(PSZ pszHandler,               // in: descriptive string
                         PSZ pszBuffer,                // out: buffer
                         PEXCEPTIONREPORTRECORD pReportRec, // in: excpt info
                         PCONTEXTRECORD pContextRec)   // in: excpt info
{
    switch (pReportRec->ExceptionNum)
    {
        case XCPT_ACCESS_VIOLATION: {
            sprintf(pszBuffer, "\n%s: XFolder detected an access violation.\n", pszHandler);
            if (pReportRec->ExceptionInfo[0] & XCPT_READ_ACCESS)
                sprintf(pszBuffer+strlen(pszBuffer), "Invalid read access occurred at 0x%04X:%08X.\n",
                            pContextRec->ctx_SegDs, pReportRec->ExceptionInfo[1]);
            else if (pReportRec->ExceptionInfo[0] & XCPT_WRITE_ACCESS)
                sprintf(pszBuffer+strlen(pszBuffer), "Invalid write access occurred at 0x%04X:%08X.\n",
                            pContextRec->ctx_SegDs, pReportRec->ExceptionInfo[1]);
            else if (pReportRec->ExceptionInfo[0] & XCPT_SPACE_ACCESS)
                sprintf(pszBuffer+strlen(pszBuffer), "Invalid space access occurred with 0x%04X.\n",
                            pReportRec->ExceptionInfo[1]);
            else if (pReportRec->ExceptionInfo[0] & XCPT_LIMIT_ACCESS)
                sprintf(pszBuffer+strlen(pszBuffer), "Invalid limit access occurred.\n");
            else if (pReportRec->ExceptionInfo[0] == XCPT_UNKNOWN_ACCESS)
                sprintf(pszBuffer+strlen(pszBuffer), "Access unknown; occurred at 0x%04X:%08X\n",
                            pContextRec->ctx_SegDs, pReportRec->ExceptionInfo[1]);
            strcat(pszBuffer,
                        "Explanation: An attempt was made to access a memory object which does\n"
                        "             not belong to the current process. Most probable causes\n"
                        "             for this are that a program uses invalid pointers, got\n"
                        "             confused with administering its memory or does not check\n"
                        "             properly for error conditions.\n");
            break;
        }

        case XCPT_INTEGER_DIVIDE_BY_ZERO: {
            sprintf(pszBuffer, "\n%s: Integer division by zero.\n", pszHandler);
            strcat(pszBuffer,
                        "Explanation: An attempt was made to divide an integer value by zero,\n"
                        "             which is not defined.\n");
            break;
        }

        case XCPT_ILLEGAL_INSTRUCTION: {
            sprintf(pszBuffer, "\n%s: Illegal instruction found.\n", pszHandler);
            strcat(pszBuffer,
                        "Explanation: An attempt was made to execute an instruction that\n"
                        "             is not defined on this machine's architecture.\n");
            break;
        }

        case XCPT_PRIVILEGED_INSTRUCTION: {
            sprintf(pszBuffer, "\n%s: Privileged instruction found.\n", pszHandler);
            strcat(pszBuffer,
                        "Explanation: An attempt was made to execute an instruction that\n"
                        "             is not permitted in the current machine mode or that\n"
                        "             the application has no permission to execute.\n");
            break;
        }

        case XCPT_INTEGER_OVERFLOW:
            sprintf(pszBuffer, "\n%s: Integer overflow.\n", pszHandler);
            strcat(pszBuffer,
                        "Explanation: An integer operation generated a carry-out of the most\n"
                        "             significant bit. This is a sign of an attempt to store\n"
                        "             a value which does not fit into an integer variable.\n");
    }
}

/*
 * ExceptionHandler:
 *     this is the "simple" exception handler, which can write
 *     a trap log, but will then terminate PMSHELL.EXE
 */

ULONG _System ExcptHandler(PEXCEPTIONREPORTRECORD pReportRec,
                                    PEXCEPTIONREGISTRATIONRECORD pRegRec,
                                    PCONTEXTRECORD pContextRec,
                                    PVOID pv)
{
    CHAR  szBuffer[3000] = "";

    /* From the VAC++3 docs:
     * The first thing an exception handler should do is check the
     * exception flags.  If EH_EXIT_UNWIND is set, meaning
     * the thread is ending, the handler tells the operating system
     * to pass the exception to the next exception handler. It does the
     * same if the EH_UNWINDING flag is set, the flag that indicates
     * this exception handler is being removed.
     * The EH_NESTED_CALL flag indicates whether the exception
     * occurred within an exception handler. If the handler does
     * not check this flag, recursive exceptions could occur until
     * there is no stack remaining.
     */
    if (pReportRec->fHandlerFlags & EH_EXIT_UNWIND)
       return (XCPT_CONTINUE_SEARCH);
    if (pReportRec->fHandlerFlags & EH_UNWINDING)
       return (XCPT_CONTINUE_SEARCH);
    if (pReportRec->fHandlerFlags & EH_NESTED_CALL)
       return (XCPT_CONTINUE_SEARCH);

    switch (pReportRec->ExceptionNum)
    {
        case XCPT_ACCESS_VIOLATION:
        case XCPT_INTEGER_DIVIDE_BY_ZERO:
        case XCPT_ILLEGAL_INSTRUCTION:
        case XCPT_PRIVILEGED_INSTRUCTION:
        case XCPT_INTEGER_OVERFLOW:
            cmnExplainException("ThreadExcptHandler",
                                szBuffer,
                                pReportRec,
                                pContextRec);

            cmnOpenDebugFile();
            if (pContextRec)
                sprintf(szBuffer+strlen(szBuffer), pszRegisters,
                       pContextRec->ctx_RegEax, pContextRec->ctx_RegEbx, pContextRec->ctx_RegEcx, pContextRec->ctx_RegEdx,
                       pContextRec->ctx_RegEbp, pContextRec->ctx_RegEdi, pContextRec->ctx_RegEsi, pContextRec->ctx_RegEsp,
                       pContextRec->ctx_SegCs, pContextRec->ctx_SegDs, pContextRec->ctx_SegEs, pContextRec->ctx_SegFs,
                       pContextRec->ctx_SegGs, pContextRec->ctx_SegSs);
            cmnDescribeThreadInfo(&(szBuffer[strlen(szBuffer)]));
            cmnWriteAndCloseDebugFile(szBuffer);
    }

    return (XCPT_CONTINUE_SEARCH);
}

/*
 * ExcptHandlerPlus:
 *      this is the "sophisticated" exception handler, which writes
 *      the trap log also, but then returns back to the thread to
 *      continue execution, i.e. PMSHELL.EXE will not be terminated.
 *      This procedure is taken from some package I found at Hobbes,
 *      I believe, by Monte Copeland, IBM, from 10-15-1993.
 *
 *      This excpt handler must be installed using the REGREC2
 *      structure instead of the default PEXCEPTIONREGISTRATIONRECORD:
 *
 *      REGREC2 RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
 *      APIRET  rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
 *
 *      // store a known thread state
 *      ULONG   ulExcpt = setjmp(RegRec2.jmpThread);
 *
 *      if (ulExcpt)
 *      {  // ulExcpt is != 0 if an exception occurred; this excpt
 *         // handler will then jump to the setjmp location above,
 *         // so here you should do some cleanup
 *      } else { // no excpt: normal thread initialization
 */

ULONG _System ExcptHandlerPlus(PEXCEPTIONREPORTRECORD pReportRec,
                                     PREGREC2 pRegRec2,
                                     PCONTEXTRECORD pContextRec,
                                     PVOID pv)
{
    CHAR  szBuffer[5000] = "";

    /* From the VAC++3 docs:
     * The first thing an exception handler should do is check the
     * exception flags.  If EH_EXIT_UNWIND is set, meaning
     * the thread is ending, the handler tells the operating system
     * to pass the exception to the next exception handler. It does the
     * same if the EH_UNWINDING flag is set, the flag that indicates
     * this exception handler is being removed.
     * The EH_NESTED_CALL flag indicates whether the exception
     * occurred within an exception handler. If the handler does
     * not check this flag, recursive exceptions could occur until
     * there is no stack remaining.
     */
    if (pReportRec->fHandlerFlags & EH_EXIT_UNWIND)
       return (XCPT_CONTINUE_SEARCH);
    if (pReportRec->fHandlerFlags & EH_UNWINDING)
       return (XCPT_CONTINUE_SEARCH);
    if (pReportRec->fHandlerFlags & EH_NESTED_CALL)
       return (XCPT_CONTINUE_SEARCH);

    switch (pReportRec->ExceptionNum)
    {
        // thread was killed: no error log
        /* case XCPT_PROCESS_TERMINATE:
        case XCPT_ASYNC_PROCESS_TERMINATE:
            longjmp(pRegRec2->jmpThread, pReportRec->ExceptionNum); */

        // "real" exceptions:
        // write error log
        case XCPT_ACCESS_VIOLATION:
        case XCPT_INTEGER_DIVIDE_BY_ZERO:
        case XCPT_ILLEGAL_INSTRUCTION:
        case XCPT_PRIVILEGED_INSTRUCTION:
        case XCPT_INTEGER_OVERFLOW:
            cmnExplainException("ExcptHandlerPlus",
                                szBuffer,
                                pReportRec,
                                pContextRec);

            cmnOpenDebugFile();
            if (pContextRec)
                sprintf(szBuffer+strlen(szBuffer), pszRegisters,
                       pContextRec->ctx_RegEax, pContextRec->ctx_RegEbx, pContextRec->ctx_RegEcx, pContextRec->ctx_RegEdx,
                       pContextRec->ctx_RegEbp, pContextRec->ctx_RegEdi, pContextRec->ctx_RegEsi, pContextRec->ctx_RegEsp,
                       pContextRec->ctx_SegCs, pContextRec->ctx_SegDs, pContextRec->ctx_SegEs, pContextRec->ctx_SegFs,
                       pContextRec->ctx_SegGs, pContextRec->ctx_SegSs);
            cmnDescribeThreadInfo(&(szBuffer[strlen(szBuffer)]));
            cmnWriteAndCloseDebugFile(szBuffer);
            // jump back to failing routine
            longjmp(pRegRec2->jmpThread, pReportRec->ExceptionNum);
        break;

        default:
            break;
    }

    // not handled
    return (XCPT_CONTINUE_SEARCH);
}

/*
 * ExcptHandlerQuiet:
 *      "quiet" xcpt handler, which simply suppresses exceptions;
 *      writes no error log, but only returns to the calling
 *      function. Must be installed the same way as
 *      ExcptHandlerPlus. Used by cmnCheckObject to implement
 *      a fail-safe SOM object check.
 */

ULONG _System ExcptHandlerQuiet(PEXCEPTIONREPORTRECORD pReportRec,
                                     PREGREC2 pRegRec2,
                                     PCONTEXTRECORD pContextRec,
                                     PVOID pv)
{
    // CHAR  szBuffer[5000] = "";

    /* From the VAC++3 docs:
     * The first thing an exception handler should do is check the
     * exception flags.  If EH_EXIT_UNWIND is set, meaning
     * the thread is ending, the handler tells the operating system
     * to pass the exception to the next exception handler. It does the
     * same if the EH_UNWINDING flag is set, the flag that indicates
     * this exception handler is being removed.
     * The EH_NESTED_CALL flag indicates whether the exception
     * occurred within an exception handler. If the handler does
     * not check this flag, recursive exceptions could occur until
     * there is no stack remaining.
     */
    if (pReportRec->fHandlerFlags & EH_EXIT_UNWIND)
       return (XCPT_CONTINUE_SEARCH);
    if (pReportRec->fHandlerFlags & EH_UNWINDING)
       return (XCPT_CONTINUE_SEARCH);
    if (pReportRec->fHandlerFlags & EH_NESTED_CALL)
       return (XCPT_CONTINUE_SEARCH);

    switch (pReportRec->ExceptionNum)
    {
         case XCPT_ACCESS_VIOLATION:
         case XCPT_INTEGER_DIVIDE_BY_ZERO:
         case XCPT_INTEGER_OVERFLOW:
             // jump back to failing routine
             longjmp(pRegRec2->jmpThread, pReportRec->ExceptionNum);
         break;

         default:
             break;
    }
    return (XCPT_CONTINUE_SEARCH);
}



