
/*
 * xfldr.c:
 *      This file contains the following major XFolder parts:
 *
 *      -  XFolder class
 *      -  XFldStartup (startup folder) class
 *      -  XFldShutdown (shutdown folder) class
 *      -  a number of common functions (with a "cmn" func prefix)
 *         which are used by several XFolder routines
 *
 *      Check the other files starting with xf* for the
 *      other XFolder classes.
 *
 *      Copyright (C) 1997-98 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of the XFolder main distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

/*
 *  This file was generated by the SOM Compiler and Emitter Framework.
 *  Generated using:
 *      SOM Emitter emitctm: 2.41
 */

#pragma langlvl(extended)

#ifndef SOM_Module_xfldr_Source
#define SOM_Module_xfldr_Source
#endif
#define XFolder_Class_Source
#define M_XFolder_Class_Source

#define INCL_DOSFILEMGR
#define INCL_DOSMODULEMGR
#define INCL_DOSMEMMGR
#define INCL_DOSPROCESS
#define INCL_DOSSEMAPHORES
#define INCL_DOSEXCEPTIONS
#define INCL_DOSERRORS
#define INCL_DOSMISC

// pmwin.h
#define INCL_WINWINDOWMGR
#define INCL_WINFRAMEMGR        // WM_FORMATFRAME etc.
#define INCL_WINSYS             // presparams
#define INCL_WINPOINTERS
#define INCL_WININPUT           // WM_BUTTON1DOWN etc.

#define INCL_WINSHELLDATA       // profile funcs
#define INCL_WINPROGRAMLIST     // PROGDETAILS etc.
#define INCL_WINCOUNTRY         // WinCompareStrings
#define INCL_WINTIMER
#define INCL_WINERRORS
#define INCL_WINRECTANGLES      // rcl functions
#define INCL_WINCLIPBOARD
#define INCL_WINMESSAGEMGR      // sophisticated msg funcs

// dialogs and controls therein
#define INCL_WINDIALOGS
#define INCL_WINSTATICS
#define INCL_WINMENUS
#define INCL_WINENTRYFIELDS
#define INCL_WINBUTTONS
#define INCL_WINLISTBOXES

// pmstddlg.h
#define INCL_WINSTDCNR          // container
#define INCL_WINSTDBOOK         // notebooks
#define INCL_WINSTDFILE         // file dialog

#define INCL_WINWORKPLACE

// pmmle.h
#define INCL_WINMLE             // multi-line entry field

// GPI
#define INCL_GPILOGCOLORTABLE
#define INCL_GPIPRIMITIVES

#include <os2.h>

#include "xfldr.ih"
#include "xfobj.h"
#include "xfdesk.h"
#include "xfdisk.h"
#include  <wppgm.h>         // WPProgram
#include  <wprootf.h>       // WPRootFolder
#include  <wpshadow.h>      // WPShadow

#include "common.h"           // this includes other libs

#include "xfsort.h"
#include "wphandles.h"        // Henk Kelder's object handles stuff

#pragma  linkage(main, optlink)

/********************************************************************
 *                                                                  *
 *   Global variables                                               *
 *                                                                  *
 ********************************************************************/

// linked lists / counts for variable context menu items
// pliVarItems contains ALL variable items ever inserted
// (i.e. config folder items AND folder content items)
PVARMENULISTITEM        pliVarItems = NULL;
// counts for providing unique menu id's
ULONG       ulVarItemCount = 0;    // number of inserted menu items
SHORT       sNextMenuId = 0;      // next menu item ID to use
// pcmliContentMenus contains ONLY folder content menus
PCONTENTMENULISTITEM    pcmliContentMenus = NULL;

// root of linked list to remember original window
// procedures when subclassing frame windows for folder hotkeys;
// this list is maintained by wpOpen of XFolder
PSUBCLASSEDLISTITEM      psliSubclassed = NULL;
// mutex semaphore for access to this list
HMTX                  hmtxSubclassed = NULLHANDLE;

// Thread management, error handling
ULONG   ulWorkplaceFunc = 0,
        ulWorkplaceFunc2 = 0;

BOOL    IsXFolderMInitialized = FALSE;

MRESULT EXPENTRY fnwpSettingsDlg_Fldr1(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY fnwpSettingsDlg_FldrSort(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);

/********************************************************************
 *                                                                  *
 *   Common routines used by several XFolder methods                *
 *                                                                  *
 ********************************************************************/

/*
 * IS_CONFIG_FOLDER:
 *      this macro returns TRUE is somSelf is either the
 *      XFolder Config folder or resides below it
 */

#define IS_CONFIG_FOLDER(somSelf) (_xfResidesBelow(somSelf, \
    _wpclsQueryFolder(_WPFolder, XFOLDER_CONFIGID, TRUE)))


/*
 * cmnGetWorkplaceFunc2:
 *      return &ulWorkplaceFunc2 for exception handlers
 */

PULONG cmnGetWorkplaceFunc2(VOID)
{
    return (&ulWorkplaceFunc2);
}

/*
 * cmnCheckIfPopulated:
 *           this populates a folder if it's not fully populated yet.
 *           Saves you from querying the full path and all that.
 */

BOOL cmnCheckIfPopulated(XFolder *somSelf)
{
    CHAR        szRealName[CCHMAXPATH];
    ULONG       cb = sizeof(szRealName);

    if ((_wpQueryFldrFlags(somSelf) & FOI_POPULATEDWITHALL) == 0)
    {
        _wpQueryRealName(somSelf, szRealName, &cb, TRUE);
        _wpPopulate(somSelf, 0, szRealName, FALSE);
        return (TRUE);
    }
    return (FALSE);
}

/*
 * cmnPopulateTree:
 *           this will populate a given folder and all of
 *           its subfolders; used by the Worker thread to
 *           populate the Config folders in the background
 *           after the desktop is ready
 */

BOOL cmnPopulateTree(XFolder *somSelf)
{
    WPObject    *pObject;

    if (somSelf) {
        cmnCheckIfPopulated(somSelf);
        for (   pObject = _wpQueryContent(somSelf, NULL, (ULONG)QC_FIRST);
                (pObject);
                pObject = _wpQueryContent(somSelf, pObject, (ULONG)QC_NEXT)
            )
        {
            if (_somIsA(pObject, _WPFolder))
                cmnPopulateTree(pObject); // recurse
        }
    }
    return (TRUE);
}

/*
 * cmnAppendMi2List:
 *      this stores a variable XFolder menu item in the
 *      respective global linked list (pliVarItems)
 *      and increases sNextMenuId for the next item.
 *      Returns FALSE if too many items have already been
 *      used and menus should be closed.
 */

BOOL cmnAppendMi2List(WPObject *pObject, USHORT usObjType)
{
    // CHAR* p;
    /* remember program object's data for later use in wpMenuItemSelected
       itemCount is unique for each inserted object */
    PVARMENULISTITEM pNewItem = (PVARMENULISTITEM)malloc(sizeof(VARMENULISTITEM));
    pNewItem->pObject = pObject;
    strncpy(pNewItem->szTitle, _wpQueryTitle(pObject), sizeof(pNewItem->szTitle)-1);

    strBeautifyTitle(pNewItem->szTitle);

    pNewItem->usObjType = usObjType;
    lstAppendItem((PLISTITEM*)&pliVarItems, NULL, (PLISTITEM)pNewItem, 0, 0);

    if (sNextMenuId < 0x7F00) {
        sNextMenuId++;
        ulVarItemCount++;
        return (TRUE);
    } else {
        cmnPostObjectMsg(XOM_LIMITREACHED, MPNULL, MPNULL);
        return (FALSE);
    }
}

/*
 * cmnAppendFldr2ContentList:
 *      this stores a folder / menu item id in a global linked
 *      list so that the subclassed window procedure can find
 *      it later for populating the folder and displaying its
 *      contents in the menu
 */

VOID cmnAppendFldr2ContentList(WPFolder *pFolder, SHORT sMenuId)
{
    PCONTENTMENULISTITEM pNewItem = (PCONTENTMENULISTITEM)malloc(sizeof(CONTENTMENULISTITEM));
    pNewItem->pFolder = pFolder;
    pNewItem->sMenuId = sMenuId;
    lstAppendItem((PLISTITEM*)&pcmliContentMenus, NULL, (PLISTITEM)pNewItem, 0, 0);
}

/*
 * cmnFindAlphPosInMenu:
 *      this searches a given hwndMenu and returns the position
 *      at which pszItemString will have to be inserted alphabetically;
 *      usFirst and usLast specify the first and the last item to
 *      search; specify 0 and MIT_LAST to search the whole menu.
 *      This allows separating menus into several sections which are
 *      each alphabetically sorted (as in the folder content menus).
 */

SHORT cmnFindAlphPosInMenu(HAB hab, HWND hwndMenu, PSZ pszItemString,
            SHORT sFirst1, SHORT sLast1)
{
    SHORT  sFirst2, sLast2, s, sItemCount, sItem;
    // CHAR   szBuffer[100];
    PVARMENULISTITEM pItem;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    SHORT  sFirstVarMenuId = (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_VARIABLE);

    sItemCount = (SHORT)WinSendMsg(hwndMenu, MM_QUERYITEMCOUNT, MPNULL, MPNULL);
    sFirst2 = sFirst1;
    if (sLast1 == MIT_END)
        sLast2 = sItemCount;
    else sLast2 = sLast1;

    if (sLast2 < sFirst2) {
        return (MIT_END);
    }

    // we're doing a quick check if the new item will be the last alphabetically;
    // this is a lot quicker than going through the whole menu and helps a lot
    // on HPFS drives, because these are sorted alphabetically already
    sItem = (SHORT)WinSendMsg(hwndMenu, MM_ITEMIDFROMPOSITION, (MPARAM)(sLast2-1), MPNULL);
    pItem = (PVARMENULISTITEM)lstItemFromIndex(
                (PLISTITEM)pliVarItems, (sItem - sFirstVarMenuId), 0, 0);
    if (pItem)
        if (WinCompareStrings(hab, 0, 0, pItem->szTitle, pszItemString, 0) == WCS_LT)
        {
            return (sLast2);
        }

    // if the quicky didn't work, we now need to check more thoroughly
    // by going thru the specified menu items
    for (s = sFirst2;
         s <= sLast2;
         s++)
    {
        sItem = (SHORT)WinSendMsg(hwndMenu, MM_ITEMIDFROMPOSITION, (MPARAM)s, MPNULL);

        pItem = (PVARMENULISTITEM)lstItemFromIndex(
                    (PLISTITEM)pliVarItems, (sItem - sFirstVarMenuId), 0, 0);

        if (pItem) {
            if (WinCompareStrings(hab, 0, 0, pItem->szTitle, pszItemString, 0) == WCS_GT)
            {
                return (s);
            }
        } else {
            return (sLast2);
        }

    }
    // still no greater item found: the new item must be last then
    return (sLast2);
}

/*
 * cmnQueryPSLI:
 *      this retrieves the PSUBCLASSEDLISTITEM from the
 *      global linked list of subclassed frame windows,
 *      according to a given frame wnd handle. One of these
 *      structs is maintained for each open folder view
 *      to store window data which is needed everywhere.
 */

PSUBCLASSEDLISTITEM cmnQueryPSLI(HWND hwndFrame)
{
    PSUBCLASSEDLISTITEM psli = NULL;

    if (hwndFrame) {
        if (DosRequestMutexSem(hmtxSubclassed, 4000) == NO_ERROR)
        {
            psli = psliSubclassed;
            while (psli) {
                if (psli->hwndFrame == hwndFrame) {
                    /* pfnwpLast = pfnwpOriginal;
                    hwndLast = hwndFrame;
                    somSelfLast = somSelf; */
                    break; // while
                }
                else
                    psli = psli->pNext;
            }
            DosReleaseMutexSem(hmtxSubclassed);
        }
    }

    return (psli);
}

/*
 * cmnQueryFrameFromView:
 *           this routine gets the frame window handle of the
 *           specified object view; returns 0 if the specified view is
 *           not currently open.
 */

HWND cmnQueryFrameFromView(XFolder *somSelf, ULONG ulView)
{
    PVIEWITEM           pViewItem;
    // PUSEITEM            pUseFile;
    HWND                hwndFrame = 0;

    if (_wpFindUseItem(somSelf, USAGE_OPENVIEW, NULL))
    { // folder has an open view
        // now we go search the open views of the folder and get the
        // frame handle of the desired view (ulView)
        for (pViewItem = _wpFindViewItem(somSelf, VIEW_ANY, NULL);
            pViewItem;
            pViewItem = _wpFindViewItem(somSelf, VIEW_ANY, pViewItem))
        {
            if (pViewItem->view == ulView) {
                 hwndFrame = pViewItem->handle;
            } // end if
        } // end for
    } // end if
    return (hwndFrame);
}

/*
 * cmnSubclassFrameWnd:
 *      this routine is used by the wpOpen methods of both
 *      XFolder and XFldDisk to subclass a new frame window
 *      to allow for extra XFolder features.
 *      For this, this routines maintains a global linked list of
 *      subclassed windows, whose first item is stored in psliSubclassed.
 *      Every list item contains (among other things) the original
 *      wnd proc of the subclassed window, which the subclassed wnd
 *      proc (fnwpSubclassedFolderFrame) needs to call the the original folder
 *      wnd proc for a given frame window, because these procs might differ
 *      depending on the class or view type or installed WPS enhancers.
 *      These list items are also used for storing status bar data.
 *      We cannot store this data elsewhere, because QWL_USER in the
 *      wnd data seems to be used by the WPS already.
 *      This func returns the newly created PSUBCLASSEDLISTITEM.
 */

PSUBCLASSEDLISTITEM cmnSubclassFrameWnd(HWND hwndFrame, XFolder *somSelf, ULONG ulView)
{
    PFNWP               pfnwpCurrent, pfnwpOriginal;
    HWND                hwndCnr;
    PSUBCLASSEDLISTITEM    pNewItem = NULL;
    PGLOBALSETTINGS     pGlobalSettings = cmnQueryGlobalSettings();

    if (pGlobalSettings->NoSubclassing == 0)

        // exclude other views, such as settings notebooks and
        // possible user views
        if (    (hwndFrame)
             && (   (ulView == OPEN_CONTENTS)
                 || (ulView == OPEN_TREE)
                 || (ulView == OPEN_DETAILS)
                )
           )
        {
            // get container wnd handle
            hwndCnr = WinQueryCnrFromFrame(hwndFrame);
            if (hwndCnr)
            // only subclass frame window if it contains a container;
            // just a security check
            {
                // now check if frame wnd has already been subclassed;
                // just another security check
                pfnwpCurrent = (PFNWP)WinQueryWindowPtr(hwndFrame, QWP_PFNWP);
                if (pfnwpCurrent != (PFNWP)fnwpSubclassedFolderFrame) {
                    if (!(pfnwpOriginal = WinSubclassWindow(hwndFrame, (PFNWP)fnwpSubclassedFolderFrame)))
                        // error occured
                        DebugBox("XFolder", "Folder subclassing failed.");
                        // should never happen
                    else {
                        // subclassing succeeded: now remember the old wnd proc
                        // and other data in the global linked list
                        pNewItem = malloc(sizeof(SUBCLASSEDLISTITEM));
                        pNewItem->hwndFrame = hwndFrame;
                        pNewItem->pfnwpOriginal = pfnwpOriginal;
                        pNewItem->somSelf = somSelf;
                        pNewItem->hwndCnr = hwndCnr;
                        pNewItem->hwndStatusBar = NULLHANDLE;
                        pNewItem->ulView = ulView;
                        pNewItem->fStatusBarContextMenu = FALSE;
                        // append to global list
                        lstAppendItem((PLISTITEM*)&psliSubclassed, NULL,
                            (PLISTITEM)pNewItem,
                            hmtxSubclassed, 4000);
                    }
                }
            }
        }
    return (pNewItem);
}

/*
 * cmnSetCnrSort:
 *      this resets the cnr of an open folder view to the
 *      settings that XFolder needs to implement proper
 *      extended sorting; the WPS keeps messing with the
 *      container settings, so we need to override this
 */

VOID cmnSetCnrSort(XFolder *somSelf, HWND hwndCnr)
{
    if (hwndCnr) {
        // get instance data
        XFolderData     *somThis = XFolderGetData(somSelf);
        // get pointer to WPFolder sort info found by
        // wpRestoreData (cheap trick)
        PFDRSORTINFO    pFdrSortInfo = (PFDRSORTINFO)_pFdrSortInfo;
        PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
        // macro for determining this folder's always-sort flag
        BOOL            AlwaysSort = ALWAYS_SORT;
        // get our sort comparison func
        PFN             pfnSort =  (PFN)((AlwaysSort)
                                       ? (_xfclsQuerySortFunc(_XFolder, DEFAULT_SORT))
                                       : NULL
                                   );

        CNRINFO         CnrInfo;
        ULONG           ulStyle = 0;
        BOOL            Update = FALSE;

        WinQueryCnrInfo(hwndCnr, CnrInfo);
        ulStyle = WinQueryWindowULong(hwndCnr, QWL_STYLE);

        #ifdef DEBUG_SORT
            _Pmpf(( "cmnSetCnrSort: %s", _wpQueryTitle(somSelf) ));
            _Pmpf(( "  _Always: %d, Global->Always: %d", _AlwaysSort, pGlobalSettings->AlwaysSort ));
            _Pmpf(( "  _Default: %d, Global->Default: %d", _DefaultSort, pGlobalSettings->DefaultSort ));
        #endif

        if ((CnrInfo.flWindowAttr & (CV_ICON | CV_TREE)) == CV_ICON)
        {
            // for some reason, cnr icon views need to have "auto arrange" on
            // when sorting, or icons will not be "fixed" when dragging
            if (AlwaysSort) {
                // always sort: we need to set CCS_AUTOPOSITION, if not set
                if ((ulStyle & CCS_AUTOPOSITION) == 0) {
                    #ifdef DEBUG_SORT
                        _Pmpf(( "  New ulStyle = %lX", ulStyle | CCS_AUTOPOSITION ));
                    #endif
                    WinSetWindowULong(hwndCnr, QWL_STYLE, (ulStyle | CCS_AUTOPOSITION));
                    Update = TRUE;
                }
            } else {
                // NO always sort: we need to unset CCS_AUTOPOSITION, if set
                if ((ulStyle & CCS_AUTOPOSITION) != 0) {
                    #ifdef DEBUG_SORT
                        _Pmpf(( "  New ulStyle = %lX", ulStyle & (~CCS_AUTOPOSITION) ));
                    #endif
                    WinSetWindowULong(hwndCnr, QWL_STYLE, (ulStyle & (~CCS_AUTOPOSITION)));
                    Update = TRUE;
                }
            }

            // now also update the internal WPFolder sort info, because otherwise
            // the WPS will keep reverting the cnr attrs
            if (pFdrSortInfo)
                // pFdrSortInfo->fAlwaysSort = AlwaysSort;
                pFdrSortInfo->fAlwaysSort = FALSE;
        }

        // now work on the sort function: we perform these checks
        // to avoid cnr flickering
        if (    (CnrInfo.pSortRecord != (PVOID)pfnSort)
             || (Update)        // was set to TRUE if CCS_AUTOPOSITION was changed
           )
        {
            CnrInfo.pSortRecord = (PVOID)pfnSort;

            #ifdef DEBUG_SORT
                _Pmpf(( "  Resetting pSortRecord to %lX", CnrInfo.pSortRecord ));
            #endif
            // now update the CnrInfo, which will repaint the cnr also
            WinSendMsg(hwndCnr, CM_SETCNRINFO,
                (MPARAM)&CnrInfo, (MPARAM)CMA_PSORTRECORD);
        }
    }
}

/********************************************************************
 *                                                                  *
 *   here come the XFolder instance methods                         *
 *                                                                  *
 ********************************************************************/

/*
 * cmnSetOneFrameWndTitle:
 *           this changes the window title of a given folder frame window
 *           to the full path of the folder; this method does NOT check
 *           the respective XFolder settings any more, so this has to be
 *           done by the caller.
 */

BOOL cmnSetOneFrameWndTitle(XFolder *somSelf, HWND hwndFrame)
{
    PSZ                 pFirstSlash, pSrchSlash, pNextSlash;
    CHAR                szTemp[CCHMAXPATH];
    ULONG               cb  = sizeof(szTemp);   // Size of object
    XFolderData         *somThis = XFolderGetData(somSelf);
    PGLOBALSETTINGS     pGlobalSettings = cmnQueryGlobalSettings();
    BOOL                brc;

    if (    (_FullPath == 1)
         || ((_FullPath == 2) && (pGlobalSettings->FullPath))
       )
    {
        // settings allow full path in title for this folder:

        // get real name (= full path), if allowed)
        _wpQueryRealName(somSelf, szTemp, &cb, TRUE);

        // now truncate path if it's longer than allowed by user
        pFirstSlash = strchr(szTemp, '\\');
        if ((pFirstSlash) && (pGlobalSettings->MaxPathChars > 10)) {
            pSrchSlash = pFirstSlash+3;
            while (strlen(szTemp) > pGlobalSettings->MaxPathChars) {
                pNextSlash = strchr(pSrchSlash, '\\');
                if (pNextSlash) {
                    strcpy(pFirstSlash+4, pNextSlash);
                    pFirstSlash[1] = '.';
                    pFirstSlash[2] = '.';
                    pFirstSlash[3] = '.';
                    pSrchSlash = pFirstSlash+5;
                }
                else break;
            }
        }

        // now either append the full path in brackets to or replace window title
        if (pGlobalSettings->KeepTitle) {
            CHAR szFullPathTitle[CCHMAXPATH*2];
            sprintf(szFullPathTitle, "%s (%s)",
                    _wpQueryTitle(somSelf),
                    szTemp);
            WinSetWindowText(hwndFrame, szFullPathTitle);
        }
        else
            WinSetWindowText(hwndFrame, szTemp);

        brc = TRUE;
    } else {
        // settings DON'T allow full path in title for this folder:
        // set to title only
        WinSetWindowText(hwndFrame, _wpQueryTitle(somSelf));
        brc = FALSE;
    }
    return (brc);
}

/*
 * xfContainsFile:
 *           this new instance returns an object if the folder contains the
 *           file pszRealName. This uses DosFindFirst to find the file,
 *           including a subfolder of the same name. Abstract objects
 *           are not found. If such an object does not exist, NULL is
 *           returned.
 */

SOM_Scope WPFileSystem*  SOMLINK xf_xfContainsFile(XFolder *somSelf,
                                                   PSZ pszRealName)
{
    CHAR    szRealName[2*CCHMAXPATH];
    ULONG   cbRealName = sizeof(szRealName);
    WPObject *prc = NULL;
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfContainsFile");

    if (_wpQueryRealName(somSelf, szRealName, &cbRealName, TRUE))
    {
        HDIR          hdirFindHandle = HDIR_SYSTEM;
        FILEFINDBUF3  FindBuffer     = {0};      // Returned from FindFirst/Next
        ULONG         ulResultBufLen = sizeof(FILEFINDBUF3);
        ULONG         ulFindCount    = 1;        // Look for 1 file at a time
        APIRET        rc             = NO_ERROR; // Return code
        szRealName[cbRealName] =  '\\';
        strcpy(szRealName+cbRealName+1, pszRealName);

        rc = DosFindFirst( szRealName,            // File pattern
                            &hdirFindHandle,      // Directory search handle
                            FILE_DIRECTORY
                                | FILE_SYSTEM
                                | FILE_ARCHIVED
                                | FILE_HIDDEN
                                | FILE_READONLY,
                            // FILE_NORMAL,          // Search attribute
                            &FindBuffer,          // Result buffer
                            ulResultBufLen,       // Result buffer length
                            &ulFindCount,         // Number of entries to find
                            FIL_STANDARD);        // Return level 1 file info
        if (rc == NO_ERROR)
            prc = _wpclsQueryObjectFromPath(_WPFileSystem, szRealName);
    }
    return (prc);
}

/*
 * xfUpdateAllFrameWndTitles:
 *           this method sets the frame wnd titles for all currently
 *           open views of a given folder to the full path;
 *           it is called on the Worker thread after XFolder's replacements
 *           of wpMoveObject, wpSetTitle, or wpRefresh have been called.
 *           It does check the respective Global / folder settings.
 */

SOM_Scope BOOL     SOMLINK xf_xfUpdateAllFrameWndTitles(XFolder *somSelf)
{
    HWND        hwndFrame;
    BOOL        brc = FALSE;

    if (hwndFrame = cmnQueryFrameFromView(somSelf, OPEN_CONTENTS)) {
        cmnSetOneFrameWndTitle(somSelf, hwndFrame);
        brc = TRUE;
    }
    if (hwndFrame = cmnQueryFrameFromView(somSelf, OPEN_DETAILS)) {
        cmnSetOneFrameWndTitle(somSelf, hwndFrame);
        brc = TRUE;
    }
    if (hwndFrame = cmnQueryFrameFromView(somSelf, OPEN_TREE)) {
        cmnSetOneFrameWndTitle(somSelf, hwndFrame);
        brc = TRUE;
    }

    return (brc);
}

/*
 * xfSnapToGrid:
 *           makes all objects in the folder "snap" on a grid whose
 *           coordinates are to be defined in the "system" object.
 *           This method checks if an Icon view of the folder is
 *           currently open; if not and fNotify == TRUE, it displays
 *           a message box.
 */

SOM_Scope BOOL  SOMLINK xf_xfSnapToGrid(XFolder *somSelf,
                                           BOOL fNotify)
{
    WPObject            *obj;
    HWND                hwndFrame = 0,
                        hwndCnr = 0;
    PMINIRECORDCORE     pMRC;
    // RECORDINSERT        RecordInsert;
    SWP                 swp;
    LONG                lNewX, lNewY;

    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfSnapToGrid");

    /* first we need the frame handle of a currently open icon view;
       all others don't make sense */
    hwndFrame = cmnQueryFrameFromView(somSelf, OPEN_CONTENTS);

    if (hwndFrame) {

        // now get the container handle
        hwndCnr = WinQueryCnrFromFrame(hwndFrame);

        if (hwndCnr)
        {
            // now begin iteration over config folder's objects
            for (   obj = _wpQueryContent(somSelf, NULL, (ULONG)QC_FIRST);
                    (obj);
                    obj = _wpQueryContent(somSelf, obj, (ULONG)QC_NEXT)
                )
            {
                pMRC = _wpQueryCoreRecord(obj);
                if (pMRC)
                {
                    // now play with the objects coordinates
                    lNewX =
                        ( (
                            ( (pMRC->ptlIcon.x - pGlobalSettings->GridX)
                              + (pGlobalSettings->GridCX / 2)
                            )
                        / pGlobalSettings->GridCX ) * pGlobalSettings->GridCX )
                        + pGlobalSettings->GridX;
                    lNewY =
                        ( (
                            ( (pMRC->ptlIcon.y - pGlobalSettings->GridY)
                              + (pGlobalSettings->GridCY / 2)
                            )
                        / pGlobalSettings->GridCY ) * pGlobalSettings->GridCY )
                        + pGlobalSettings->GridY;
                    if (lNewX)
                        pMRC->ptlIcon.x = lNewX;         // X
                    if (lNewY)
                        pMRC->ptlIcon.y = lNewY;         // Y
                }
            }
            WinSendMsg(hwndCnr, CM_INVALIDATERECORD,
                MPNULL,
                MPFROM2SHORT(0, CMA_REPOSITION));
            // now send WM_SIZE to frame window with the present
            // window size: this will not resize the window, but
            // we need to re-adjust the scrollbars, if present */
            WinQueryWindowPos(hwndFrame, &swp);
            WinSendMsg(hwndFrame, WM_SIZE, MPFROM2SHORT(swp.x, swp.y), MPFROM2SHORT(swp.x, swp.y));
            WinInvalidateRect(hwndFrame, NULL, FALSE);
            return TRUE;
        } // end if (hwndContainer)
        else {
            DebugBox("XFolder", "Container window not found.");
            return FALSE;
        }
    } // end if (hwndFrame)
    else { // no open icon view: complain
        cmnSetHelpPanel(-1);
        WinDlgBox(HWND_DESKTOP,                // parent is desktop
                     HWND_DESKTOP,             // owner is desktop
                     (PFNWP)fnwpDlgGeneric,    // dialog procedure, defd. at bottom
                     NLS_MODULE, // load from resource file
                     ID_XFD_NOICONVIEW,        // dialog resource id
                     (PVOID)NULL);             // no dialog parameters
        return FALSE;
    }
}

/*
 * xfQueryFldrSort:
 *           this returns the folder's sort settings into the specified
 *           USHORT variables. These are set to SET_DEFAULT if no instance
 *           data has been defined; you will then need to query the Global
 *           Settings' values.
 */

SOM_Scope BOOL  SOMLINK xf_xfQueryFldrSort(XFolder *somSelf,
                                              PUSHORT pusDefaultSort,
                                              PUSHORT pusAlwaysSort)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfQueryFldrSort");

    if ((pusDefaultSort) && (pusAlwaysSort)) {
        *pusDefaultSort = (USHORT)_DefaultSort;
        *pusAlwaysSort  = (USHORT)_AlwaysSort;
        return (TRUE);
    } else
        return (FALSE);
}

/*
 * fncbSortAllViews:
 *      callback function for sorting all folder views.
 *      This is called by xf(cls)ForEachOpenView, which also passes
 *      the parameters to this func.
 */

MRESULT EXPENTRY fncbSortAllViews(HWND hwndView, ULONG ulSort,
        MPARAM mpView, MPARAM mpFolder)
{
    XFolder     *somSelf = (XFolder*)mpFolder;
    MRESULT     mrc = (MPARAM)FALSE;

    if (   ((ULONG)mpView == OPEN_CONTENTS)
        || ((ULONG)mpView == OPEN_TREE)
        || ((ULONG)mpView == OPEN_DETAILS)
       )
    {
        _xfSortView(somSelf, hwndView, ulSort);
        mrc = (MPARAM)TRUE;
    }
    return (mrc);
}

/*
 * fncbUpdateFolderSorts:
 *      callback function for updating all folder sorts.
 *      This is called by xf(cls)ForEachOpenView, which also passes
 *      the parameters to this func.
 */

MRESULT EXPENTRY fncbUpdateFolderSorts(HWND hwndView,   // frame wnd handle
                        ULONG ulDummy,
                        MPARAM mpView,                  // OPEN_xxx flag for this view
                        MPARAM mpFolder)                // somSelf
{
    XFolder     *somSelf = (XFolder*)mpFolder;
    MRESULT     mrc = (MPARAM)FALSE;

    #ifdef DEBUG_SORT
        _Pmpf(( "fncbUpdateFolderSorts: %s", _wpQueryTitle(somSelf) ));
    #endif

    if (   ((ULONG)mpView == OPEN_CONTENTS)
        || ((ULONG)mpView == OPEN_TREE)
        || ((ULONG)mpView == OPEN_DETAILS)
       )
    {
        cmnSetCnrSort(somSelf, WinQueryCnrFromFrame(hwndView));
        mrc = (MPARAM)TRUE;
    }
    return (mrc);
}

/*
 * xfSetFldrSort:
 *           this is the new XFolder method for setting the sort data
 *           for a certain folder.
 *           usDefaultSort should be one of the SV_ constants as in
 *           xfSortView or SET_DEFAULT for resetting the folder's
 *           default sort criterion to the GLobal Settings's value.
 *           usAlwaysSort can be 0 or 1 or SET_DEFAULT also.
 *           This method returns TRUE if any visible change occured as
 *           a result to the new settings, and if thus any open folder
 *           views have or would have been updated.
 */

SOM_Scope BOOL  SOMLINK xf_xfSetFldrSort(XFolder *somSelf,
                                            USHORT usDefaultSort,
                                            USHORT usAlwaysSort)
{
    BOOL Update = FALSE;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfSetFldrSort");

    #ifdef DEBUG_SORT
        _Pmpf(("xfSetFldrSort %s", _wpQueryTitle(somSelf)));
        _Pmpf(("  Old: Default %d, Always %d", _DefaultSort, _AlwaysSort));
        _Pmpf(("  New: Default %d, Always %d", usDefaultSort, usAlwaysSort));
    #endif

    if (usDefaultSort != _DefaultSort) {
        _DefaultSort = usDefaultSort;
        Update = TRUE;
    }
    if (usAlwaysSort != _AlwaysSort) {
        _AlwaysSort = usAlwaysSort;
        // if _wpRestoreState has found the pointer to the
        // WPFOlder-internal sort structure, we will update
        // this one also, because otherwise the WPS keeps
        // messing with the container attributes
        if (_pFdrSortInfo)
            ((PFDRSORTINFO)_pFdrSortInfo)->fAlwaysSort = ALWAYS_SORT;
        Update = TRUE;
    }

    if (Update) {
        // update open views of this folder
        _xfForEachOpenView(somSelf, 0, &fncbUpdateFolderSorts);
        _wpSaveDeferred(somSelf);
    }

    return (Update);
}

/*
 * xfSortView:
 *           sorts the content of given folder. As opposed to the
 *           regular sort function, this sorts one folder view
 *           (specified by hwndFrame) only. ulSort may be one of the
 *           following:
 *               SV_NAME
 *               SV_TYPE
 *               SV_CLASS
 *               SV_REALNAME
 *               SV_SIZE
 *               SV_LASTWRITEDATE
 *               SV_LASTACCESSDATE
 *               SV_CREATIONDATE
 *               SV_EXT                  new: sort by extension
 *               SV_FOLDERSFIRST         new: sort folders first
 */

SOM_Scope BOOL  SOMLINK xf_xfSortView(XFolder *somSelf, HWND hwndFrame,
                                         USHORT usSort)
{
    BOOL rc = FALSE;
    HWND hwndCnr = WinQueryCnrFromFrame(hwndFrame);
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfSortByExt");

    if (hwndCnr) {
        CNRINFO CnrInfo;
        ULONG   ulStyle = 0;

        WinQueryCnrInfo(hwndCnr, CnrInfo);

        if ((CnrInfo.flWindowAttr & (CV_ICON | CV_TREE)) == CV_ICON) {
            // for some reason, icon views need to have "auto arrange" on,
            // or nothing will happen
            ulStyle = WinQueryWindowULong(hwndCnr, QWL_STYLE);
            WinSetWindowULong(hwndCnr, QWL_STYLE, ulStyle | CCS_AUTOPOSITION);
        }

        // send sort msg with proper sort (comparison) func
        WinSendMsg(hwndCnr, CM_SORTRECORD,
                (MPARAM)(PFN)_xfclsQuerySortFunc(_XFolder, usSort),
                MPNULL);

        if ((CnrInfo.flWindowAttr & (CV_ICON | CV_TREE)) == CV_ICON)
            // restore old cnr style
            WinSetWindowULong(hwndCnr, QWL_STYLE, ulStyle);

        rc = TRUE;
    }
    return (rc);
}

/*
 * xfQueryIconPos:
 *           this new instance method retrieves the icon position of an
 *           object in a currently populated folder; you need to initialize
 *           the pICONPOS structure first, whose size you need to pass in
 *           ulICONPOSSize; the ICONPOS data will be copied to pipReturn.
 *           The method returns FALSE if something went wrong.
 */

SOM_Scope BOOL  SOMLINK xf_xfGetIconPos(XFolder *somSelf,
                                           WPObject *pObject,
                                           PBYTE pICONPOS, USHORT usICONPOSSize,
                                           PICONPOS pipReturn)
{
    USHORT   usStartPos;
    PICONPOS pip;

    CHAR     szKey[100],
             szPath[CCHMAXPATH];
    ULONG    ulBufSize = sizeof(szPath);

    HOBJECT  hObject = _wpQueryHandle(pObject);
    PSZ      pszClass = _somGetClassName(pObject);

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfGetIconPos");

    usStartPos = 21; // with OS/2 2.1 and above, Henk Kelder says

    /* first step: the icon position of each object within a given
       .ICONPOS EA starts with a string identifying the object; so
       first, we need to compose this string depending on the type
       of the passed object */
    if (IsObjectAbstract(hObject))
        sprintf(szKey, "%s:A%lX", pszClass, LOUSHORT(hObject));
    else
    {   // file system object
        WPFileSystem    *pobjFile;
        if (pobjFile = _wpclsQueryObject(_WPObject, hObject)) {
            ulBufSize = sizeof(szPath)-1;
            if (_wpQueryRealName(pobjFile, szPath, &ulBufSize, FALSE)) {
                sprintf(szKey, "%s:%c%s", pszClass,
                   (_somIsA(pobjFile, _WPFolder) ? 'D' : 'F'),
                   szPath);
            } else
                return FALSE;
        } else
            return FALSE;

    }

    // now we have the key to search for within the .ICONPOS EA

    if ((pICONPOS) && (pipReturn))
    {
        /* now we go through the .ICONPOS data that was given to us
           and check each item in there if it matches the key we
           composed above */
        for (pip = (PICONPOS)( pICONPOS + usStartPos );
            (PBYTE)pip < pICONPOS + usICONPOSSize; )
        {
            if (!stricmp(pip->szIdentity, szKey))
            {
                *pipReturn = *pip;
                return TRUE;
            }
            pip = (PICONPOS)( (PBYTE)pip + sizeof(POINTL) + strlen(pip->szIdentity) + 1 );
        }
    }

    return FALSE;
}

/*
 * xfInvalidateOrderedContent:
 *           this deletes the ordered content list so that it will
 *           be rebuilt when it's needed; this is neccessary if
 *           objects in the folder have changed
 */

SOM_Scope BOOL  SOMLINK xf_xfInvalidateOrderedContent(XFolder *somSelf)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfInvalidateOrderedContent");

    if (_pliOrderedContentFirst)
        lstClear((PLISTITEM*)&(_pliOrderedContentFirst),
                 (PLISTITEM*)&(_pliOrderedContentLast),
                 _hmtxOrderedContent, 4000);
    return (TRUE);
}

typedef struct _SORTBYICONPOS {
    CHAR    szRealName[CCHMAXPATH];
    PBYTE   pICONPOS;
    USHORT  usICONPOSSize;
} SORTBYICONPOS, *PSORTBYICONPOS;

/*
 * cmnGetICONPOS:
 *
 */

PICONPOS cmnGetICONPOS(PORDEREDLISTITEM poli, PSORTBYICONPOS psip)
{
    PICONPOS                    pip;
    CHAR                        *p;
    USHORT usStartPos = 21;  // OS/2 2.1 and above, says Henk

    // ICONPOS is defined in PMWP.H as folllows:
    //     typedef struct _ICONPOS
    //     {
    //        POINTL  ptlIcon;
    //        CHAR    szIdentity[1];
    //     } ICONPOS;
    //     typedef ICONPOS *PICONPOS;

    // now compare all the objects in the .ICONPOS structure
    // to the identity string of the search object

    for (   pip = (PICONPOS)( psip->pICONPOS + usStartPos );
            (PBYTE)pip < (psip->pICONPOS + psip->usICONPOSSize);
        )
    {   // pip now points to an ICONPOS structure

        // go beyond the class name
        p = strchr(pip->szIdentity, ':');
        if (p) {

            #ifdef DEBUG_ORDEREDLIST
                _Pmpf(("      Identities: %s and %s...", p, poli->szIdentity));
            #endif

            if (stricmp(p, poli->szIdentity) == 0)
                // object found: return the ICONPOS address
                return (pip);
            else
                // not identical: go to next ICONPOS structure
                pip = (PICONPOS)( (PBYTE)pip + sizeof(POINTL) + strlen(pip->szIdentity) + 1 );
        } else
            break;
    }
    return (NULL);
}

/*
 * fnSortByICONPOS:
 *      callback sort function for lstSort to sort the
 *      menu items according to a folder's ICONPOS EAs.
 *      pICONPOS points to the ICONPOS data.
 */

SHORT EXPENTRY fnSortByICONPOS(PVOID pItem1, PVOID pItem2, PVOID psip)
{
    #ifdef DEBUG_ORDEREDLIST
        _Pmpf(("    Comparing %s and %s...",
            _wpQueryTitle(((PORDEREDLISTITEM)pItem1)->pObj),
            _wpQueryTitle(((PORDEREDLISTITEM)pItem2)->pObj)
        ));
    #endif
    if ((pItem1) && (pItem2)) {
        PICONPOS pip1 = cmnGetICONPOS(((PORDEREDLISTITEM)pItem1), psip),
                 pip2 = cmnGetICONPOS(((PORDEREDLISTITEM)pItem2), psip);

        if ((pip1) && (pip2))
            if (pip1 < pip2)
                return (-1);
            else return (1);
        else if (pip1)
            return (-1);
        else if (pip2)
            return (1);
    }

    return (0);
}

/*
 * xfUpdateOrderedContent:
 *           this new instance methods builds or updates a list of the
 *           folder contents in memory. The root of the list is saved
 *           in the folder instance data.
 *           This method is called by xfQueryOrderedContent if the list in
 *           memory has not been built yet. You do not need to call this
 *           method manually.
 */

SOM_Scope BOOL  SOMLINK xf_xfUpdateOrderedContent(XFolder *somSelf)
{
    BOOL                        rc = FALSE;
    APIRET                      drc;
    PORDEREDLISTITEM             poliNew;
    PEABINDING                  peab;
    WPObject                    *pObj;
    SORTBYICONPOS               sip;
    ULONG                       ulSize = CCHMAXPATH,
                                ulrc;

    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfUpdateOrderedContent");

    #ifdef DEBUG_ORDEREDLIST
        _Pmpf(("Entering xf_xfUpdateOrderedContent for %s", _wpQueryTitle(somSelf) ));
    #endif

    if (_hmtxOrderedContent == NULLHANDLE) {
        DosCreateMutexSem(NULL, // unnamed
            &(_hmtxOrderedContent),
            0,                  // unshared
            FALSE);             // unowned
    }

    drc = DosRequestMutexSem(_hmtxOrderedContent, 4000);
    if (drc == NO_ERROR) {

        if (_pliOrderedContentFirst)
            lstClear((PLISTITEM*)&(_pliOrderedContentFirst),
                     (PLISTITEM*)&(_pliOrderedContentLast),
                     0, 0);

        if (_pICONPOS) {
            _wpFreeMem(somSelf, _pICONPOS);
            _pICONPOS = NULL;
        }

        // first get the folder's content as the WPS delivers
        // it (this is unsorted)
        for (   pObj = _wpQueryContent(somSelf, NULL, (ULONG)QC_FIRST);
                (pObj);
                pObj = _wpQueryContent(somSelf, pObj, (ULONG)QC_NEXT)
            )
        {
            poliNew = malloc(sizeof(ORDEREDLISTITEM));
            poliNew->pObj = pObj;

            // Each ICONPOS struct's identity string has the following format:
            // <class>:<t><identity>
            // with: <class> being the class of the object
            //       <t> == A for abstracts, D for folders, F for files
            //       <identity> for abstract objects: the handle
            //                  for file-system objects: the filename

            // now create the identity string for the search object
            if (_somIsA(pObj, _WPAbstract)) {
                // for abstract objects, this is the low word
                // of the object handle
                HOBJECT hobjSearch = _wpQueryHandle(pObj);
                sprintf(poliNew->szIdentity, ":A%lX", (hobjSearch & 0xFFFF));
            } else {
                // for file-system objects, this is the object's real name
                ULONG   ulSize = sizeof(poliNew->szIdentity)-2;
                if (_somIsA(pObj, _WPFolder))
                    strcpy(poliNew->szIdentity, ":D");
                else
                    strcpy(poliNew->szIdentity, ":F");
                // append real name
                _wpQueryRealName(pObj, (poliNew->szIdentity)+2, &ulSize, FALSE);
            }

            lstAppendItem((PLISTITEM*)&(_pliOrderedContentFirst),
                     (PLISTITEM*)&(_pliOrderedContentLast),
                     (PLISTITEM)poliNew,
                     0, 0);
        }

        // read .ICONPOS extended attributes of this folder
        _wpQueryRealName(somSelf, sip.szRealName, &ulSize, TRUE);
        if (peab = EAPathReadOneByName(sip.szRealName, ".ICONPOS")) {

            //  typedef struct
            //  {
            //    BYTE bFlags;
            //    BYTE bNameLength;
            //    USHORT usValueLength;
            //    PSZ pszName;
            //    PSZ pszValue;
            //  } EABINDING, *PEABINDING;

            _pICONPOS = _wpAllocMem(somSelf, peab->usValueLength+100, &ulrc);

            if (_pICONPOS) {
                memcpy(_pICONPOS, peab->pszValue+4, peab->usValueLength-3);
                _usICONPOSSize = (peab->usValueLength)-5;
                EAFreeBinding(peab);

                // finally, we have the ICONPOS data in _pICONPOS;
                // now we pass the ICONPOS data to the sort function
                // defined above

                #ifdef DEBUG_ORDEREDLIST
                    _Pmpf(("  Sorting..."));
                #endif

                sip.pICONPOS = _pICONPOS;
                sip.usICONPOSSize = _usICONPOSSize;

                lstQuickSort((PLISTITEM*)&(_pliOrderedContentFirst),
                         (PLISTITEM*)&(_pliOrderedContentLast),
                         fnSortByICONPOS,
                         &sip,
                         0, 0);

                rc = TRUE;
            }
        }
    }
    DosReleaseMutexSem(_hmtxOrderedContent);

    return (rc);
}

/*
 * xfQueryOrderedContent:
 *           this new instance method has the same parameters als wpQueryContent,
 *           but it returns items from the list built by xfUpdateOrderedContent.
 *           As opposed to the list returned by wpQueryContent, the objects
 *           in this list are in the same order as in the name or details views,
 *           because the list is built according to the .ICONPOS EAs.
 */

SOM_Scope WPObject*  SOMLINK xf_xfQueryOrderedContent(XFolder *somSelf,
                                                         WPObject* Object,
                                                         ULONG ulOption)
{
    WPObject                    *pobjReturn = NULL;
    APIRET                      drc;
    PORDEREDLISTITEM            poli;

    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfQueryOrderedContent");

    cmnCheckIfPopulated(somSelf);

    if (    (ulOption == QC_FIRST)
         && (_pliOrderedContentFirst == NULL)
       )
    {
        _xfUpdateOrderedContent(somSelf);
    }

    if (_pliOrderedContentFirst)
    {
        drc = DosRequestMutexSem(_hmtxOrderedContent, 4000);
        if (drc == NO_ERROR) {
            poli = _pliOrderedContentFirst;
            if (poli) {
                switch (ulOption) {
                    case QC_FIRST: {
                        pobjReturn = poli->pObj;
                    break; }

                    case QC_NEXT: {
                        while (poli)
                            if (poli->pObj == Object) {
                                BOOL fValid = TRUE;
                                if (poli->pNext) {
                                    pobjReturn = (poli->pNext->pObj);

                                    // check validity of object
                                    if (!cmnCheckObject(pobjReturn))
                                    {
                                        // if the object we found is not a valid SOM object
                                        // any more, we will delete it from the list
                                        lstRemoveItem((PLISTITEM*)&(_pliOrderedContentFirst),
                                                 (PLISTITEM*)&(_pliOrderedContentLast),
                                                 (PLISTITEM)poli,
                                                 0, 0); // we already have the semaphore
                                        // continue search w/ next object
                                        fValid = FALSE;
                                    }
                                }
                                if (fValid)
                                    break;
                                else
                                    poli = poli->pNext;
                            }
                            else
                                poli = poli->pNext;
                    break; }
                }
            }
        }
        DosReleaseMutexSem(_hmtxOrderedContent);
    }
    else
        pobjReturn = _wpQueryContent(somSelf, Object, ulOption);

    return (pobjReturn);
}

/*
 * xfBeginProcessOrderedContent:
 *           this new instance method uses xfQueryOrderedContent to open all
 *           the objects in the given folder. This method returns immediately;
 *           further processing is done in the background Worker thread.
 *
 *           Parameters:
 *              ULONG ulTiming      the time to wait between starting objects
 *                                  in milliseconds; if set to 0, this function
 *                                  will work in "Wait" mode, i.e. the next
 *                                  object will open the next object only if
 *                                  the previous one has been closed again
 *                                  (e.g. for XShutdown folder)
 *              PFNWP pfnwpCallback a callback routine which will be called on
 *                                  every object which is started;
 *              ULONG ulCallbackParam the first parameter to pass to this proc.
 *
 *              pfnwpCallback will be passed the following parameters:
 *                  HWND  hwnd      will not be a hwnd, but ulCallbackParam;
 *                  ULONG msg       will contain the current object of the
 *                                  folder on which this mthd is invoked;
 *                                  this must be cast manually to (WPObject*)
 *                  MPARAM mp1      contains the current object count
 *                                  (starting with 1 and increasing for each
 *                                  subsequent object);
 *                  MPARAM mp2      the (constant) number of objects in the folder.
 *
 *           This callback func may be used for implementing a progress bar;
 *           you can use ulCallbackParam for the msg window handle and display
 *           the progress according to mp1 and mp2; the object title may be
 *           obtained by calling _wpQueryTitle((WPObject*)msg).
 *
 *           After all objects have been processed, the callback will called
 *           once more with msg == 0 to allow cleaning up. If the folder does
 *           not contain any objects, callback will only be called this one time.
 *
 *           If you're running in "Wait" mode, the callback must return the
 *           hwnd of the object which you must have opened with wpViewObject,
 *           so that XFolder can wait for this window to be closed.
 *
 *           Note that although the processing of the folder is done in the
 *           Worker thread, the callback func is actually called on the main
 *           (Workplace) thread, so you can create your window there and have
 *           work done on it without having to worry about thread safety. This
 *           is implemented because wpViewObject is having problems when not
 *           being called in the Workplace thread.
 *
 *           This method returns a handle which may be used in
 *           xfCancelProcessOrderedContent to abort processing. The function
 *           returns 0 upon errors.
 */

SOM_Scope ULONG  SOMLINK xf_xfBeginProcessOrderedContent(XFolder *somSelf,
                                                           ULONG ulTiming,
                                                           PFNWP pfnwpCallback,
                                                           ULONG ulCallbackParam)
{
    PPROCESSCONTENTINFO pPCI;
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfBeginProcessOrderedContent");

    pPCI = (PPROCESSCONTENTINFO)malloc(sizeof(PROCESSCONTENTINFO));

    if (pPCI) {
        pPCI->ulObjectNow = 0;
        pPCI->ulTiming = ulTiming;
        pPCI->pfnwpCallback = pfnwpCallback;
        pPCI->ulCallbackParam = ulCallbackParam;
        pPCI->fCancelled = FALSE;

        cmnPostWorkerMsg(WOM_PROCESSORDEREDCONTENT, (MPARAM)somSelf, pPCI);

        return ((ULONG)pPCI);
    }
    return (0);
}

/*
 * xfCancelProcessOrderedContent:
 *           this method cancels a folder content process started with
 *           xfBeginProcessOrderedContent; hPOC must the handle returned by
 *           that function.
 *           Warning: After folder content processing has finished, hPOC is
 *           no longer valid, and calling this function then will simply
 *           crash the WPS. As a result, your callback function MUST keep
 *           track if hPOC is still valid; after the callback func has been
 *           called with msg == 0, calling this function is no longer allowed.
 */

SOM_Scope BOOL  SOMLINK xf_xfCancelProcessOrderedContent(XFolder *somSelf,
                                                            ULONG hPOC)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfCancelProcessOrderedContent");

    ((PPROCESSCONTENTINFO)hPOC)->fCancelled = TRUE;

    return (TRUE);
}

/*
 * xfMakeFavoriteFolder:
 *           if fInsert is TRUE, this folder will be made a "favorite folder",
 *           i.e. added to all context menus; if FALSE, it will be removed
 */

SOM_Scope ULONG  SOMLINK xf_xfMakeFavoriteFolder(XFolder *somSelf,
                                                    BOOL fInsert)
{
    CHAR    szFavoriteFolders[1000] = "";
    PCONTENTMENULISTITEM pcmli, pFound = NULL;

    // XFolderData     *somThis = XFolderGetData(somSelf);
    M_XFolderData   *somThat = M_XFolderGetData(_XFolder);

    XFolderMethodDebug("XFolder","xf_xfMakeFavoriteFolder");

    pcmli = (PCONTENTMENULISTITEM)somThat->pFavoriteFolders;
    while (pcmli)
        if (pcmli->pFolder == somSelf) {
            pFound = pcmli;
            break;
        } else
            pcmli = pcmli->pNext;

    if (fInsert)
    {
        if (!pFound) {
            pcmli = malloc(sizeof(CONTENTMENULISTITEM));
            pcmli->pFolder = somSelf;
            lstAppendItem((PLISTITEM*)&(somThat->pFavoriteFolders), NULL,
                (PLISTITEM)pcmli, 0, 0);
        }
    }
    else
        lstRemoveItem((PLISTITEM*)&(somThat->pFavoriteFolders), NULL,
                (PLISTITEM)pFound, 0, 0);

    pcmli = (somThat->pFavoriteFolders);
    if (pcmli) {
        while (pcmli) {
            sprintf(szFavoriteFolders+strlen(szFavoriteFolders),
                "%lX ", _wpQueryHandle(pcmli->pFolder));
            pcmli = pcmli->pNext;
        }
        PrfWriteProfileString(HINI_USERPROFILE, INIAPP, INIKEY_FAVORITEFOLDERS,
                    szFavoriteFolders);
    } else {
        PrfWriteProfileData(HINI_USERPROFILE, INIAPP, INIKEY_FAVORITEFOLDERS,
                    NULL, 0);
    }

    return 0;
}

/*
 * xfIsFavoriteFolder:
 *           returns TRUE if somSelf is a favorite folder
 */

SOM_Scope BOOL  SOMLINK xf_xfIsFavoriteFolder(XFolder *somSelf)
{
    PCONTENTMENULISTITEM pcmli;
    BOOL                 rc = FALSE;

    // XFolderData     *somThis = XFolderGetData(somSelf);
    M_XFolderData   *somThat = M_XFolderGetData(_XFolder);
    XFolderMethodDebug("XFolder","xf_xfIsFavoriteFolder");

    pcmli = (PCONTENTMENULISTITEM)somThat->pFavoriteFolders;
    while (pcmli)
        if (pcmli->pFolder == somSelf) {
            rc = TRUE;
            break;
        } else
            pcmli = pcmli->pNext;

    return (rc);
}

/*
 * xfSetQuickOpen:
 *           if fQuickOpen == TRUE, somSelf will automatically be
 *           populated at WPS bootup
 */

SOM_Scope ULONG  SOMLINK xf_xfSetQuickOpen(XFolder *somSelf,
                                           BOOL fQuickOpen)
{
    CHAR    szQuickOpenFolders[1000] = "";
    PCONTENTMENULISTITEM pcmli, pFound = NULL;

    // XFolderData     *somThis = XFolderGetData(somSelf);
    M_XFolderData   *somThat = M_XFolderGetData(_XFolder);
    XFolderMethodDebug("XFolder","xf_xfSetQuickOpen");

    pcmli = (PCONTENTMENULISTITEM)somThat->pQuickOpenFolders;
    while (pcmli)
        if (pcmli->pFolder == somSelf) {
            pFound = pcmli;
            break;
        } else
            pcmli = pcmli->pNext;

    if (fQuickOpen)
    {
        if (!pFound) {
            pcmli = malloc(sizeof(CONTENTMENULISTITEM));
            pcmli->pFolder = somSelf;
            lstAppendItem((PLISTITEM*)&(somThat->pQuickOpenFolders), NULL,
                (PLISTITEM)pcmli, 0, 0);
        }
    }
    else
        lstRemoveItem((PLISTITEM*)&(somThat->pQuickOpenFolders), NULL,
                (PLISTITEM)pFound, 0, 0);

    pcmli = (somThat->pQuickOpenFolders);
    if (pcmli) {
        while (pcmli) {
            sprintf(szQuickOpenFolders+strlen(szQuickOpenFolders),
                "%lX ", _wpQueryHandle(pcmli->pFolder));
            pcmli = pcmli->pNext;
        }
        PrfWriteProfileString(HINI_USERPROFILE, INIAPP, INIKEY_QUICKOPENFOLDERS,
                    szQuickOpenFolders);
    } else {
        PrfWriteProfileData(HINI_USERPROFILE, INIAPP, INIKEY_QUICKOPENFOLDERS,
                    NULL, 0);
    }

    return 0;
}

/*
 * xfQueryQuickOpen:
 *           returns TRUE if somSelf has the QuickOpen feature ON
 */

SOM_Scope BOOL  SOMLINK xf_xfQueryQuickOpen(XFolder *somSelf)
{
    PCONTENTMENULISTITEM pcmli;
    BOOL                 rc = FALSE;

    // XFolderData     *somThis = XFolderGetData(somSelf);
    M_XFolderData   *somThat = M_XFolderGetData(_XFolder);
    XFolderMethodDebug("XFolder","xf_xfQueryQuickOpen");

    pcmli = (PCONTENTMENULISTITEM)somThat->pQuickOpenFolders;
    while (pcmli)
        if (pcmli->pFolder == somSelf) {
            rc = TRUE;
            break;
        } else
            pcmli = pcmli->pNext;

    return (rc);
}

/*
 * cmnFSFreeFromFolder:
 *      returns the free space on a drive from a given
 *      folder
 */

ULONG cmnFSFreeFromFolder(XFolder *somSelf)
{
    if (somSelf) {
        CHAR        szRealName[CCHMAXPATH];
        ULONG       ulSize = sizeof(szRealName)-1;
        ULONG       ulDrive;
        FSALLOCATE  fsa;

        _wpQueryRealName(somSelf, szRealName, &ulSize, TRUE);
        ulDrive = (szRealName[0] - 'A' + 1); // = 1 for "A", 2 for "B" etc.
        DosQueryFSInfo(ulDrive, FSIL_ALLOC, &fsa, sizeof(fsa));
        return (fsa.cSectorUnit * fsa.cbSector * fsa.cUnitAvail);
    } else
        return (0);
}

/*
 * cmnComposeStatusBarText:
 *      sets up the status bar text; this func queries
 *      the mnemonic string depending on how many objects
 *      are currently selected and does the translation into
 *      meaningful information. If only one object is selected,
 *      the respective class / instance methods are called
 *      to do class-specific mnemonic translation.
 */

/* status bar text mnemonics:

     $t      object title
     $w      WPS class default title (e.g. "Data file")
     $W      WPS class name (e.g. WPDataFile)

     $c      no. of selected objects
     $C      total object count

     $fb     free space on drive in bytes
     $fk     free space on drive in kBytes
     $fK     free space on drive in KBytes
     $fm     free space on drive in mBytes
     $fM     free space on drive in MBytes

     $sb     size of selected objects in bytes
     $sk     size of selected objects in kBytes
     $sK     size of selected objects in KBytes
     $sm     size of selected objects in mBytes
     $sM     size of selected objects in MBytes

     $Sb     size of folder content in bytes
     $Sk     size of folder content in kBytes
     $SK     size of folder content in KBytes
     $Sm     size of folder content in mBytes
     $SM     size of folder content in MBytes
 */

BOOL cmnComposeStatusBarText(XFolder* somSelf, HWND hwndCnr, PSZ pszText, ULONG cbText)
{
    CNRINFO     CnrInfo;
    PMINIRECORDCORE pmrcSelected, pmrc2;
    ULONG       ulSelectedCount = 0,
                ulSizeSelected = 0,
                ulSizeTotal = 0;
    WPObject    *pObject = NULL,
                *pObject2 = NULL;

    CHAR        // szSizeSelected[20] = "0",
                szThousand[10],
                szTemp[300];
    BOOL        fCont = TRUE;
    PSZ         p;

    // now go thru all the selected objects in the container
    // and sum up the size of the corresponding objects in
    // ulSizeSelected
    pmrcSelected = (PMINIRECORDCORE)CMA_FIRST;
    do {
        pmrcSelected =
            (PMINIRECORDCORE)WinSendMsg(hwndCnr,
                    CM_QUERYRECORDEMPHASIS,
                    (MPARAM)pmrcSelected,
                    (MPARAM)CRA_SELECTED);
        if (pmrcSelected) {
            ulSelectedCount++;
            pObject = OBJECT_FROM_PREC(pmrcSelected);
            if (pObject)
                if (cmnCheckObject(pObject)) {
                    if (_somIsA(pObject, _WPFileSystem))
                        ulSizeSelected += _wpQueryFileSize(pObject);
                } else pObject = NULL;
        }
    } while (pmrcSelected);

    // now get the template text which has been set by the
    // user in the Global Settings, depending on how many
    // objects are selected
    if ( (ulSelectedCount == 0) || (pObject == NULL) )
        strcpy(pszText,
                cmnQueryStatusBarSetting(SBS_TEXTNONESEL));
    else if (ulSelectedCount == 1) {
        // single-object mode: query the text to translate
        // from the object, because we can implement
        // different mnemonics for different WPS classes
        _xfclsQueryStatusBarMnemonics(_somGetClass(pObject), pszText, cbText);
        _xfTranslateStatusBarMnemonics(pObject, pszText, cbText);
    } else
        strcpy(pszText,
                cmnQueryStatusBarSetting(SBS_TEXTMULTISEL));

    // query total object count (CnrInfo.cRecords)
    WinQueryCnrInfo(hwndCnr, CnrInfo);

    // if we have a "total size" query, also sum up the size of
    // the whole folder into ulSizeTotal, which will be handled
    // in detail below
    if (strstr(pszText, "$S")) {
        pmrc2 = NULL;
        do {
            pmrc2 =
                (PMINIRECORDCORE)WinSendMsg(hwndCnr,
                        CM_QUERYRECORD,
                        (MPARAM)pmrc2,
                        MPFROM2SHORT(
                            (pmrc2) ? CMA_NEXT : CMA_FIRST,
                            CMA_ITEMORDER)
                        );
            if (pmrc2) {
                pObject2 = OBJECT_FROM_PREC(pmrc2);
                if (pObject2)
                    if (_somIsA(pObject2, _WPFileSystem))
                        ulSizeTotal += _wpQueryFileSize(pObject2);
            }
        } while (pmrc2);
    }

    PrfQueryProfileString(HINI_USER, "PM_Default_National", "sThousand",
            ",", szThousand, sizeof(szThousand)-1);

    do {
        strcpy(szTemp, "?");
        fCont = FALSE;

        if (p = strstr(pszText, "$c")) // selected objs count
        {
            sprintf(szTemp, "%d", ulSelectedCount);
            strrpl(pszText, "$c", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$C")) // total obj count
        {
            sprintf(szTemp, "%d", CnrInfo.cRecords);
            strrpl(pszText, "$C", szTemp);
            fCont = TRUE;
        }

        // the following are for free space on drive
        else if (p = strstr(pszText, "$fb"))
        {
            strths(szTemp,
                    cmnFSFreeFromFolder(somSelf),
                    szThousand[0]);
            strrpl(pszText, "$fb", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fk"))
        {
            strths(szTemp,
                    ((cmnFSFreeFromFolder(somSelf) + 500) / 1000),
                    szThousand[0]);
            strrpl(pszText, "$fk", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fK"))
        {
            strths(szTemp,
                    ((cmnFSFreeFromFolder(somSelf) + 512) / 1024),
                    szThousand[0]);
            strrpl(pszText, "$fK", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fm"))
        {
            strths(szTemp,
                    ((cmnFSFreeFromFolder(somSelf) +500000) / 1000000),
                    szThousand[0]);
            strrpl(pszText, "$fm", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$fM"))
        {
            strths(szTemp,
                    ((cmnFSFreeFromFolder(somSelf) + (1024*1024/2)) / (1024*1024)),
                    szThousand[0]);
            strrpl(pszText, "$fM", szTemp);
            fCont = TRUE;
        }

        // the following are for SELECTED size
        else if (p = strstr(pszText, "$sb"))
        {
            strths(szTemp, ulSizeSelected, szThousand[0]);
            strrpl(pszText, "$sb", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$sk"))
        {
            strths(szTemp, ((ulSizeSelected+500) / 1000), szThousand[0]);
            strrpl(pszText, "$sk", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$sK"))
        {
            strths(szTemp, ((ulSizeSelected+512) / 1024), szThousand[0]);
            strrpl(pszText, "$sK", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$sm"))
        {
            strths(szTemp, ((ulSizeSelected+500000) / 1000000), szThousand[0]);
            strrpl(pszText, "$sm", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$sM"))
        {
            strths(szTemp, ((ulSizeSelected+(1024*1024/2)) / (1024*1024)), szThousand[0]);
            strrpl(pszText, "$sM", szTemp);
            fCont = TRUE;
        }
        // the following are for TOTAL folder size
        else if (p = strstr(pszText, "$Sb"))
        {
            strths(szTemp, ulSizeTotal, szThousand[0]);
            strrpl(pszText, "$Sb", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$Sk"))
        {
            strths(szTemp, ((ulSizeTotal+500) / 1000), szThousand[0]);
            strrpl(pszText, "$Sk", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$SK"))
        {
            strths(szTemp, ((ulSizeTotal+512) / 1024), szThousand[0]);
            strrpl(pszText, "$SK", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$Sm"))
        {
            strths(szTemp, ((ulSizeTotal+500000) / 1000000), szThousand[0]);
            strrpl(pszText, "$Sm", szTemp);
            fCont = TRUE;
        }
        else if (p = strstr(pszText, "$SM"))
        {
            strths(szTemp, ((ulSizeTotal+(1024*1024/2)) / (1024*1024)), szThousand[0]);
            strrpl(pszText, "$SM", szTemp);
            fCont = TRUE;
        }
    } while (fCont);

    return (TRUE);
}

/*
 * xfShowStatusBar:
 *           depending on fShow, this shows or hides the folder status
 *           bar for a certain folder. Note that this does not change the
 *           folder's visibility settings, but only shows or hides the status
 *           bar "factually". You should thus call xfSetStatusBarVisibility
 *           instead.
 *           Parameters:
 *               hwndFrame: frame window of an open Icon or Details view of
 *                          somSelf
 *               psli:     window procs list item from global linked list
 *                          (of type PSUBCLASSEDLISTITEM)
 *               fShow:     show / hide flag
 *           This func returns the hwnd of the status bar or NULL if calling
 *           the func was useless, because the status bar was already
 *           (de)activated.
 */

SOM_Scope HWND  SOMLINK xf_xfShowStatusBar(XFolder *somSelf,
                                           HWND hwndFrame, PVOID psli,
                                           BOOL fShow)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfShowStatusBar");

    #ifdef DEBUG_STATUSBARS
        _Pmpf(("xfShowStatusBar %d", fShow));
    #endif

    if (psli) {
        // we had to declare psli as a PVOID because
        // SOM does not know the PSUBCLASSEDLISTITEM struct;
        // so we do the conversion per macro
        #define psli2 ((PSUBCLASSEDLISTITEM)psli)
        // CHAR szINIKey[50];
        // HOBJECT hObj = _wpQueryHandle(somSelf);
        // ULONG ulDummy;

        if (fShow)
        {   // activate status bar

            if (psli2->hwndStatusBar) { // already activated: update only
                WinPostMsg(psli2->hwndStatusBar, STBM_UPDATESTATUSBAR, MPNULL, MPNULL);
                // and quit
                return (NULLHANDLE);
            }

            // else create status bar as a static control
            // (which will be subclassed below)
            if (psli2->hwndStatusBar = WinCreateWindow(hwndFrame,
                                          WC_STATIC,
                                          (cmnQueryNLSStrings())->pszPopulating,
                                          (SS_TEXT | DT_LEFT | DT_VCENTER |
                                              DT_ERASERECT | WS_VISIBLE | WS_CLIPSIBLINGS),
                                          0L, 0L, -1L, -1L,
                                          hwndFrame,
                                          HWND_BOTTOM,
                                          0x9001,
                                          (PVOID)NULL,
                                          (PVOID)NULL))
            {
                SWP swp;
                PSTATUSBARDATA psbd = malloc(sizeof(STATUSBARDATA));
                PSZ pszStatusBarFont = cmnQueryStatusBarSetting(SBS_STATUSBARFONT);

                // set up window data (QWL_USER) for status bar
                psbd->somSelf    = somSelf;
                psbd->psli       = psli2;
                psbd->idTimer    = 0;
                psbd->fDontBroadcast = TRUE;
                            // prevents broadcasting of WM_PRESPARAMSCHANGED
                psbd->fFolderPopulated = FALSE;
                            // suspends updating until folder is populated;
                WinSetWindowULong(psli2->hwndStatusBar, QWL_USER, (ULONG)psbd);

                // subclass static control to make it a status bar
                psbd->pfnwpStatusBarOriginal = WinSubclassWindow(psli2->hwndStatusBar,
                        fnwpStatusBar);
                WinSetPresParam(psli2->hwndStatusBar, PP_FONTNAMESIZE,
                     (ULONG)strlen(pszStatusBarFont) + 1, (PVOID)pszStatusBarFont);

                // now "inflate" the folder frame window if this is the first
                // time that this folder view has been opened with a status bar;
                // if we didn't do this, the folder frame would be too small
                // for the status bar and scroll bars would appear. We store
                // a flag in the folder's instance data for each different view
                // that we've inflated the folder frame, so that this happens
                // only once per view.

                // From wpobject.h:
                // #define VIEW_CONTENTS      0x00000001
                // #define VIEW_SETTINGS      0x00000002
                // #define VIEW_HELP          0x00000004
                // #define VIEW_RUNNING       0x00000008
                // #define VIEW_DETAILS       0x00000010
                // #define VIEW_TREE          0x00000020

                // _Pmpf(( "Old _ulSBInflatedFrame: %lX", _ulSBInflatedFrame ));
                if (    (   (  _ulSBInflatedFrame
                          & (   (psli2->ulView == OPEN_CONTENTS) ? VIEW_CONTENTS
                              : (psli2->ulView == OPEN_TREE) ? VIEW_TREE
                              : VIEW_DETAILS
                            )
                        ) == 0)
                   )
                {
                    // this view has not been inflated yet:
                    // inflate now and set flag for this view

                    ULONG ulStatusBarHeight = cmnQueryStatusBarHeight();
                    WinQueryWindowPos(hwndFrame, &swp);
                    // inflate folder frame
                    WinSetWindowPos(hwndFrame, 0,
                                swp.x, (swp.y - ulStatusBarHeight),
                                swp.cx, (swp.cy + ulStatusBarHeight),
                                SWP_MOVE | SWP_SIZE);
                    // set flag in instance data for this view
                    _ulSBInflatedFrame |= (   (psli2->ulView == OPEN_CONTENTS) ? VIEW_CONTENTS
                                            : (psli2->ulView == OPEN_TREE) ? VIEW_TREE
                                            : VIEW_DETAILS
                                          );
                    _wpSaveDeferred(somSelf);
                    // _Pmpf(( "New _ulSBInflatedFrame: %lX", _ulSBInflatedFrame ));

                    // finally, set a flag for the subclassed folder frame
                    // window proc that this folder view needs no additional scrolling
                    // (this is evaluated in WM_FORMATFRAME msgs)
                    psli2->fNeedCnrScroll = FALSE;
                }
                else
                    // if the folder frame has been inflated already,
                    // WM_FORMATFRAME _will_ have to scroll the container.
                    // We do this for icon views only.
                    if (psli2->ulView == OPEN_CONTENTS)
                        psli2->fNeedCnrScroll = TRUE;

                // enforce reformatting / repaint of frame window
                WinSendMsg(hwndFrame, WM_UPDATEFRAME, (MPARAM)0, MPNULL);

                // update status bar contents
                WinPostMsg(psli2->hwndStatusBar, STBM_UPDATESTATUSBAR, MPNULL, MPNULL);

                return (psli2->hwndStatusBar);
            }
        } else { // deactivate
            if (psli2->hwndStatusBar) {
                SWP swp;
                HWND hwndStatus = psli2->hwndStatusBar;

                psli2->hwndStatusBar = 0;
                WinSendMsg(hwndFrame, WM_UPDATEFRAME, (MPARAM)0, MPNULL);
                WinDestroyWindow(hwndStatus);

                // decrease the size of the frame window by the status bar height,
                // if we did this before
                // _Pmpf(( "Old _ulSBInflatedFrame: %lX", _ulSBInflatedFrame ));
                if ( (  _ulSBInflatedFrame
                      & (   (psli2->ulView == OPEN_CONTENTS) ? VIEW_CONTENTS
                          : (psli2->ulView == OPEN_TREE) ? VIEW_TREE
                          : VIEW_DETAILS
                        )
                     ) != 0)
                {
                    ULONG ulStatusBarHeight = cmnQueryStatusBarHeight();
                    WinQueryWindowPos(hwndFrame, &swp);
                    WinSetWindowPos(hwndFrame, 0,
                                swp.x, (swp.y + ulStatusBarHeight),
                                swp.cx, (swp.cy - ulStatusBarHeight),
                                SWP_MOVE | SWP_SIZE);
                    // now unset the "inflated" flag for this view
                    _ulSBInflatedFrame &= ~(    (psli2->ulView == OPEN_CONTENTS) ? VIEW_CONTENTS
                                              : (psli2->ulView == OPEN_TREE) ? VIEW_TREE
                                              : VIEW_DETAILS
                                           );

                    _wpSaveDeferred(somSelf);
                    // _Pmpf(( "New _ulSBInflatedFrame: %lX", _ulSBInflatedFrame ));
                }

                return (psli2->hwndStatusBar);
            }
        }
    }
    return (NULLHANDLE);
}

/*
 * cmnXomActivateStatusBar:
 *      this func is called from the XFolder Object window
 *      proc only when XOM_ACTIVATESTATUSBAR is received.
 *      Unfortunately, this extra message is neccessary
 *      because PM will hang if we reformat folder windows
 *      from the worker thread.
 *      Handling this msg has been moved to xfldr.c
 *      although the object window handling is in xfobj.c,
 *      because the neccessary structs are visible here only.
 *      Parameters:
 *          ULONG mp1   0: hide status bar
 *                      1: show status bar
 *                      2: update (reformat) status bar
 *          HWND  mp2:  hwndView (frame) to update
 */

VOID cmnXomActivateStatusBar(MPARAM mp1, MPARAM mp2)
{
    HWND hwndFrame = (HWND)mp2;
    PSUBCLASSEDLISTITEM psli = cmnQueryPSLI(hwndFrame);

    #ifdef DEBUG_STATUSBARS
        _Pmpf(("cmnXomActivateStatusBar, mp1: %lX", mp1));
    #endif

    if (psli)
        switch ((ULONG)mp1) {
            case 0:
                _xfShowStatusBar(psli->somSelf, hwndFrame, psli, FALSE);
            break;
            case 1:
                _xfShowStatusBar(psli->somSelf, hwndFrame, psli, TRUE);
            break;
            default: { // == 2 => update status bars
                PSZ pszStatusBarFont =
                        cmnQueryStatusBarSetting(SBS_STATUSBARFONT);
                WinSendMsg(psli->hwndStatusBar, STBM_PROHIBITBROADCASTING,
                    (MPARAM)TRUE, MPNULL);
                WinSetPresParam(psli->hwndStatusBar, PP_FONTNAMESIZE,
                     (ULONG)strlen(pszStatusBarFont) + 1, (PVOID)pszStatusBarFont);
                WinSendMsg(hwndFrame, WM_UPDATEFRAME, MPNULL, MPNULL);
                WinSendMsg(psli->hwndStatusBar, STBM_UPDATESTATUSBAR, MPNULL, MPNULL);
            break; }
        }
}

/*
 * fncbUpdateStatusBars:
 *      callback func for WOM_UPDATEALLSTATUSBARS in
 *      XFolder Worker thread (xfdesk.c); should not
 *      be called directly (it must reside in this file
 *      to be able to access folder instance data).
 *      This is called by xf(cls)ForEachOpenView, which also passes
 *      the parameters to this func.
 */

MRESULT EXPENTRY fncbUpdateStatusBars(HWND hwndView,        // folder frame
                        ULONG ulActivate,
                                // 1: show/hide status bars according to
                                //    folder/Global settings
                                // 2: reformat status bars (e.g. because
                                //    fonts have changed)
                        MPARAM mpView,                      // OPEN_xxx flag
                        MPARAM mpFolder)                    // folder object
{
    BOOL                fEnable;
    PGLOBALSETTINGS     pGlobalSettings = cmnQueryGlobalSettings();
    MRESULT             mrc = (MPARAM)FALSE;

    #ifdef DEBUG_STATUSBARS
        _Pmpf(("fncbUpdateStatusBars ulActivate = %d", ulActivate));
    #endif

    if (    (mpFolder != _wpclsQueryActiveDesktop(_WPDesktop))
         && (   ((ULONG)mpView == OPEN_CONTENTS)
             || ((ULONG)mpView == OPEN_TREE)
             || ((ULONG)mpView == OPEN_DETAILS)
            )
       )
    {
       if (    (   ((ULONG)mpView == OPEN_CONTENTS)
                && (pGlobalSettings->SBForViews & SBV_ICON)
               )
            || (   ((ULONG)mpView == OPEN_TREE)
                && (pGlobalSettings->SBForViews & SBV_TREE)
               )
            || (   ((ULONG)mpView == OPEN_DETAILS)
                && (pGlobalSettings->SBForViews & SBV_DETAILS)
               )
           )
        {
            XFolderData *somThis = XFolderGetData(mpFolder);

            fEnable = (    (_StatusBar == STATUSBAR_ON)
                        || (    (_StatusBar == STATUSBAR_DEFAULT)
                             && (pGlobalSettings->StatusBar)
                           )
                      );

            // we now post a message to the XFolder Object
            // window; this message will call cmnXomActivateStatusBar
            // on the main PM thread, because we cannot mess
            // with the windows on the Worker thread, which would
            // lead to PM hangs
            cmnPostObjectMsg(XOM_ACTIVATESTATUSBAR,
                (MPARAM)(ulActivate == 2
                         ?  2                      // reformat
                         :  (fEnable ? 1 : 0)      // show or hide
                        ),
                (MPARAM)hwndView);
            mrc = (MPARAM)TRUE;
            mrc = (MPARAM)TRUE;
        }
    }
    return (mrc);
}

/*
 * xfSetStatusBarVisibility:
 *           this new instance method sets the status bar visibility of
 *           a folder and updates open folder views accordingly.
 *           ulVisibility may be:
 *                STATUSBAR_ON        show status bars
 *                STATUSBAR_OFF       hide status bars
 *                STATUSBAR_DEFAULT   use Global Setting for this folder
 *           Returns TRUE if successful.
 */

SOM_Scope BOOL  SOMLINK xf_xfSetStatusBarVisibility(XFolder *somSelf,
                                                    ULONG ulVisibility)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfSetStatusBarVisibility");

    if (_StatusBar != ulVisibility) {
        #ifdef DEBUG_STATUSBARS
            _Pmpf(( "xfSetStatusBarVisibility: %d", ulVisibility));
        #endif
        _StatusBar = ulVisibility;
        // update open folder views in Worker thread
        cmnPostWorkerMsg(WOM_UPDATEALLSTATUSBARS,
                    (MPARAM)1,      // show/hide flag
                    MPNULL);
    }
    return (TRUE);
}

/*
 * xfQueryStatusBarVisibility:
 *           this new instance method returns the status bar visibility of
 *           a folder:
 *                STATUSBAR_ON        status bars visible
 *                STATUSBAR_OFF       status bars invisible
 *                STATUSBAR_DEFAULT   Global Setting used for this folder
 */

SOM_Scope BOOL  SOMLINK xf_xfQueryStatusBarVisibility(XFolder *somSelf)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfQueryStatusBarVisibility");

    return (_StatusBar);
}

/*
 * xfForEachOpenView:
 *           this instance method goes through all open views of a folder and calls
 *           pfnwpCallback for each them (as opposed to xfclsForEachOpenView, which
 *           goes through all open folders on the system); the following
 *           params are passed to pfnwpCallback:
 *                hwnd    HWND       hwndView   the hwnd of the view frame window;
 *                mp1     ULONG      ulView     the view type (as def'd in wpOpen)
 *                mp2     XFolder*   pFolder    somSelf.
 *           This method does not return until all views have been processed.
 *           You might want to call this method in a different thread if the task
 *           will take long.
 *           This method returns TRUE if the callback returned TRUE at least once.
 *           Note on disk objects/root folders: the WPS does not maintain an open
 *           view list for root folders, but only for the corresponding disk object.
 *           xfForEachOpenView will call open disk views also, but the callback
 *           will still be passed the root folder in pFolder!
 */

SOM_Scope BOOL  SOMLINK xf_xfForEachOpenView(XFolder *somSelf,
                                                ULONG ulMsg,
                                                PFNWP pfnwpCallback)
{
    BOOL brc = FALSE;
    WPObject *somSelf2;
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfForEachOpenView");

    if (_somIsA(somSelf, _WPRootFolder)) {
        // for disk/root folder views: root folders have no
        // open view, instead the disk object is registered
        // to have the open view. Duh. So we need to find
        // the disk object first
        somSelf2 = _xfclsQueryDiskObject(_XFldDisk, somSelf);
    } else
        somSelf2 = somSelf;

    if (somSelf2) {
        if (_wpFindUseItem(somSelf2, USAGE_OPENVIEW, NULL))
        {   // folder has an open view;
            // now we go search the open views of the folder and get the
            // frame handle of the desired view (ulView) */
            PVIEWITEM   pViewItem;
            for (pViewItem = _wpFindViewItem(somSelf2, VIEW_ANY, NULL);
                pViewItem;
                pViewItem = _wpFindViewItem(somSelf2, VIEW_ANY, pViewItem))
            {
                if ((*pfnwpCallback)(
                        pViewItem->handle,
                        ulMsg,
                        (MPARAM)pViewItem->view,
                        // but even if we have found a disk object
                        // above, we need to pass it the root folder
                        // pointer, because otherwise the callback
                        // might get into trouble
                        (MPARAM)somSelf)
                    == (MPARAM)TRUE)
                brc = TRUE;
            } // end for
        } // end if
    }
    return (brc);
}

/*
 * xfAddXFolderPages:
 *           this actually adds the "XFolder" pages into all folder notebooks
 */

SOM_Scope ULONG  SOMLINK xf_xfAddXFolderPages(XFolder *somSelf,
                                                    HWND hwndDlg)
{
    PAGEINFO        pi;
    CHAR            szXFolderVersion[100];
    PSZ pszHelpLibrary = cmnQueryHelpLibrary();

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfAddXFolderPages");

    strcpy(szXFolderVersion, "~XFolder ");
    strcat(szXFolderVersion, XFOLDER_VERSION);

    // insert Shutdown settings page
    memset((PCH)&pi, 0, sizeof(PAGEINFO));
    pi.cb                  = sizeof(PAGEINFO);
    pi.hwndPage            = NULLHANDLE;
    pi.pfnwp               = fnwpSettingsDlg_Fldr1;
    pi.resid               = NLS_MODULE;
    pi.pCreateParams       = somSelf;
    pi.dlgid               = ID_XSD_SETTINGS_FLDR1;
    pi.usPageStyleFlags    = BKA_STATUSTEXTON | BKA_MAJOR;   // major tab;
    pi.usPageInsertFlags   = BKA_FIRST;
    pi.usSettingsFlags     = 0; // don't enumerate in status line
    pi.pszName             = szXFolderVersion;

    pi.pszHelpLibraryName  = pszHelpLibrary;
    pi.idDefaultHelpPanel  = ID_XSH_SETTINGS_FLDR1;

    return (_wpInsertSettingsPage(somSelf, hwndDlg, &pi));
}

/*
 * wpFilterPopupMenu:
 *           remove default entries according to Global Settings
 */

SOM_Scope ULONG  SOMLINK xf_wpFilterPopupMenu(XFolder *somSelf,
                                                 ULONG ulFlags,
                                                 HWND hwndCnr,
                                                 BOOL fMultiSelect)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    XFolderMethodDebug("XFolder","xf_wpFilterPopupMenu");

    return (XFolder_parent_WPFolder_wpFilterPopupMenu(somSelf,
                                                          ulFlags,
                                                          hwndCnr,
                                                          fMultiSelect)
                & ~(pGlobalSettings->DefaultMenuItems)
        );
}

/*
 * cmnInsertOneObjectMenuItem:
 *          this sub-subroutine is called by xfFillOneSubmenu
 *          whenever a single menu item is to be inserted
 *          (all objects except folders); returns the menu
 *          item id
 */

ULONG cmnInsertOneObjectMenuItem(HWND       hAddToMenu,   // hwnd of menu to add to
                    USHORT     iPosition,
                    PSZ        pszNewItemString,          // title of new item
                    USHORT     afStyle,
                    WPObject   *pObject,                  // pointer to corresponding object
                    ULONG      usObjType)
{
    ULONG               rc = sNextMenuId;

    WinInsertMenuItem(hAddToMenu, iPosition,
            sNextMenuId, pszNewItemString,
            afStyle, 0);

    if (cmnAppendMi2List(pObject, usObjType))
        // give signal for calling function that we found
        //   something */
        return (rc);
    else
        return (0); // return error
}

/*
 * cmnPrepareContentSubmenu:
 *      prepares a "folder content" submenu by inserting a
 *      submenu menu item with pszTitle into hwndMenu; this
 *      submenu now only contains the "empty" menu item,
 *      however, this menu will be filled with objects if
 *      the user opens it; this is then done by answering
 *      the WM_INITMENU message in fnwpSubclassedFolderFrame.
 *      Parameters:
 *          somSelf     folder whose content is to be displayed
 *          hwndMenu    menu to insert submenu in
 *          pszTitle    submenu item title
 *          iPosition   position to insert at (may be MIT_END)
 *          fOwnerDraw  owner-draw style flag (for submenu item,
 *                      not its contents); if set, an icon will
 *                      be displayed for the submenu title
 *      This function returns the submenu item ID.
 */

SHORT cmnPrepareContentSubmenu(XFolder *somSelf, HWND hwndMenu, PSZ pszTitle,
                USHORT iPosition, BOOL fOwnerDraw)
{
    HWND    hwndNewMenu;
    SHORT   sId = sNextMenuId;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    if (hwndNewMenu = WinInsertSubmenu(hwndMenu, iPosition,
                sId, pszTitle, (fOwnerDraw ? MIS_OWNERDRAW : 0),
                (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_DUMMY),
                    (cmnQueryNLSStrings())->pszFldrEmpty,
                    MIS_TEXT, MIA_DISABLED))
    {
        cmnAppendFldr2ContentList(somSelf, sId);
        cmnAppendMi2List(somSelf, OC_CONTENTFOLDER);
        return(sId);
    }

    return (0);
}

/*
 * fnSortContentMenuItems:
 *      callback sort func for sorting the folder content
 *      menu items alphabetically
 */

typedef struct _MENULISTITEM {
    struct _MENULISTITEM   *pNext, *pPrevious;
    ULONG       ulSize;
    WPObject    *pObject;
    CHAR        szItemString[256];
} MENULISTITEM, *PMENULISTITEM;

SHORT EXPENTRY fnSortContentMenuItems(PVOID pItem1, PVOID pItem2, PVOID hab)
{
    switch (WinCompareStrings((HAB)hab, 0, 0,
                ((PMENULISTITEM)pItem1)->szItemString,
                ((PMENULISTITEM)pItem2)->szItemString, 0))
    {
        case WCS_LT:    return (-1);
        case WCS_GT:    return (1);
        default:        return (0);
    }

}

/*
 * cmnFillContentSubmenu:
 *      this fills a submenu created with cmnPrepareContentMenu
 *      with the contents of the corresponding folder; this
 *      func only needs the sMenuId returned by that function
 *      and the hwndMenu of the submenu which will be filled;
 *      it automatically finds the folder contents from the
 *      somSelf which was specified with cmnPrepareContentMenu.
 *      The submenu will be subclassed in order to allow opening
 *      objects with MB2.
 */

PFNWP   pfnwpFolderContentMenuOriginal = NULL;

VOID cmnFillContentSubmenu(SHORT sMenuId, HWND hwndMenu)
{
    // tons of declarations...
    WPObject    *pObject, *pObject2;
    WPFolder    *pFolder = NULL;
    MENUITEM    mi;
    HPOINTER    hptrOld, hptrWait;
    RECTL       rtlItem;

    PCONTENTMENULISTITEM pcmli = pcmliContentMenus;

    HAB         habDesktop = WinQueryAnchorBlock(HWND_DESKTOP);
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    BOOL        fInsert;

    // first check if the menu contains the "[empty]" item;
    // this means that it's one of the folder content menus
    // and hasn't been filled yet (it could have been re-clicked,
    // and we don't want double menu items)

    if ((ULONG)WinSendMsg(hwndMenu, MM_ITEMIDFROMPOSITION, (MPARAM)0, MPNULL)
        == (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_DUMMY))
    {
        // get folder to be populated from the linked list
        // (pcmliContentMenus)
        ulWorkplaceFunc2 = 1000;
        while (pcmli)
            if (pcmli->sMenuId == sMenuId)
            {
                pFolder = pcmli->pFolder;
                break;
            }
            else
                pcmli = pcmli->pNext;

        // pFolder now contains the folder, pcmli contains
        // the CONTENTMENULISTITEM
        ulWorkplaceFunc2 = 1010;
        if (pFolder)
        {   // folder found: populate

            // show "Wait" pointer
            hptrOld = WinQueryPointer(HWND_DESKTOP);
            hptrWait = WinQuerySysPointer(HWND_DESKTOP,
                   SPTR_WAIT, FALSE);
            WinSetPointer(HWND_DESKTOP, hptrWait);

            // if pFolder is a disk object: get root folder
            ulWorkplaceFunc2 = 1020;
            if (_somIsA(pFolder, _WPDisk))
                pFolder = _wpQueryRootFolder(pFolder);

            ulWorkplaceFunc2 = 1030;
            if (pFolder)
            {
                // populate
                cmnCheckIfPopulated(pFolder);

                ulWorkplaceFunc2 = 1040;
                if (_wpQueryContent(pFolder, NULL, QC_FIRST))
                {
                    // folder does contain objects: go!
                    // The menu is separated into two sections (for
                    // folders and other objects), each of which
                    // is sorted alphabetically.
                    // We will first create two lists in memory
                    // for all folders and non-folders; we will
                    // then sort these lists alphabetically and
                    // finally insert their content into the menu
                    PMENULISTITEM   pmliFoldersFirst = NULL,
                                    pmliFoldersLast = NULL,   // folder list
                                    pmliObjectsFirst = NULL,
                                    pmliObjectsLast = NULL,   // non-folder list
                                    pmli;

                    SHORT       sItemSize, sItemsPerColumn, sItemId, sItemCount,
                                sColumns,
                                s;

                    // subclass menu window to allow MB2 clicks
                    ulWorkplaceFunc2 = 1050;
                    pfnwpFolderContentMenuOriginal = WinSubclassWindow(hwndMenu, fnwpFolderContentMenu);

                    // remove "empty" item
                    ulWorkplaceFunc2 = 1060;
                    WinSendMsg(hwndMenu, MM_REMOVEITEM,
                        MPFROM2SHORT((pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_DUMMY), TRUE), NULL);

                    ulWorkplaceFunc2 = 1070;
                    // now collect all objects in folder
                    for (pObject = _wpQueryContent(pFolder, NULL, QC_FIRST);
                         pObject;
                         pObject = _wpQueryContent(pFolder, pObject, QC_Next))
                    {
                        ulWorkplaceFunc2 = 1080;
                        // dereference shadows, if neccessary
                        pObject2 = pObject;
                        while (pObject2) {
                            if (_somIsA(pObject2, _WPShadow))
                                pObject2 = _wpQueryShadowedObject(pObject2, TRUE);
                            else
                                break;
                        }

                        ulWorkplaceFunc2 = 1090;
                        if (pObject2)
                        {
                            // CHAR    *p;
                            fInsert = TRUE;

                            // exclude hidden file system objects
                            if (_somIsA(pObject, _WPFileSystem)) {
                                // _Pmpf(( "%s attr: %lX", szNewItemString, _wpQueryAttr(pObject2) ));
                                if (_wpQueryAttr(pObject2)
                                     & (FILE_HIDDEN)
                                   )
                                   fInsert = FALSE;
                            }

                            // exclude hidden WPS objects
                            if (_wpQueryStyle(pObject2) & OBJSTYLE_NOTVISIBLE)
                                   fInsert = FALSE;

                            if (fInsert) {
                                PMENULISTITEM pmliNew = malloc(sizeof(MENULISTITEM));
                                pmliNew->pObject = pObject2;
                                strcpy(pmliNew->szItemString, _wpQueryTitle(pObject2));
                                // remove line breaks
                                strBeautifyTitle(pmliNew->szItemString);

                                if (    (_somIsA(pObject2, _WPFolder))
                                     || (_somIsA(pObject2, _WPDisk))
                                   )
                                {
                                    // folder/disk: append to folder list
                                    lstAppendItem((PLISTITEM*)&pmliFoldersFirst,
                                                  (PLISTITEM*)&pmliFoldersLast,
                                                  (PLISTITEM)pmliNew,
                                                  0, 0);
                                } else {
                                    // folder/disk: append to folder list
                                    lstAppendItem((PLISTITEM*)&pmliObjectsFirst,
                                                  (PLISTITEM*)&pmliObjectsLast,
                                                  (PLISTITEM)pmliNew,
                                                  0, 0);
                                }
                            }
                        }
                    } // end for pObject

                    // now sort the lists alphabetically
                    lstQuickSort((PLISTITEM*)&pmliFoldersFirst,
                            (PLISTITEM*)&pmliFoldersLast,
                            fnSortContentMenuItems,
                            (PVOID)habDesktop,
                            0, 0);
                    lstQuickSort((PLISTITEM*)&pmliObjectsFirst,
                            (PLISTITEM*)&pmliObjectsLast,
                            fnSortContentMenuItems,
                            (PVOID)habDesktop,
                            0, 0);

                    // insert folders into menu
                    pmli = pmliFoldersFirst;
                    while (pmli) {
                        // folder items
                        sItemId = cmnPrepareContentSubmenu(pmli->pObject,
                            hwndMenu,
                            pmli->szItemString,
                            MIT_END,
                            pGlobalSettings->FCShowIcons); // OwnerDraw flag

                        // next folder
                        pmli = pmli->pNext;
                    }

                    // if we have both objects and folders:
                    // insert separator between them
                    if ((pmliFoldersFirst) && (pmliObjectsFirst))
                       WinInsertMenuSeparator(hwndMenu, MIT_END,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));

                    // insert objects into menu
                    pmli = pmliObjectsFirst;
                    while (pmli) {
                        sItemId = cmnInsertOneObjectMenuItem(hwndMenu,
                                     MIT_END,
                                     pmli->szItemString,
                                     ((pGlobalSettings->FCShowIcons)
                                        ? MIS_OWNERDRAW
                                        : MIS_TEXT),
                                     pmli->pObject,
                                     OC_CONTENT);
                        if (sItemId) {
                            pmli = pmli->pNext;
                        } else
                            pmli = NULL; // error: abort loop
                    }

                    // calculate maximum number of items per column by looking
                    // at the screen and item sizes
                    WinSendMsg(hwndMenu, MM_QUERYITEMRECT,
                        MPFROM2SHORT(sItemId, FALSE),
                        (MPARAM)&rtlItem);
                    sItemSize = (rtlItem.yTop-rtlItem.yBottom);
                    if (sItemSize == 0) sItemSize = 20;
                    sItemsPerColumn = (USHORT)(
                        (WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN)-30) / sItemSize );
                    if (sItemsPerColumn == 0)
                        sItemsPerColumn = 20;

                    // sItemsPerColumn now contains the items which will
                    // beautifully fit into one column; we now reduce this
                    // number if the last column would contain white space
                    sItemCount = (USHORT)WinSendMsg(hwndMenu, MM_QUERYITEMCOUNT, MPNULL, MPNULL);
                    // calculate number of resulting columns
                    sColumns = (sItemCount / sItemsPerColumn) + 1;
                    // distribute remainder in last column to all columns
                    sItemsPerColumn -= (sItemsPerColumn - (sItemCount % sItemsPerColumn)) / sColumns;

                    // now, for through every (sItemsPerColumn)'th menu item,
                    // set MIS_BREAK style
                    ulWorkplaceFunc2 = 1200;
                    for (s = sItemsPerColumn;
                         s < sItemCount;
                         s += sItemsPerColumn)
                    {
                        sItemId = (USHORT)WinSendMsg(
                                    hwndMenu, MM_ITEMIDFROMPOSITION, (MPARAM)s, MPNULL);
                        WinSendMsg(hwndMenu, MM_QUERYITEM,
                            MPFROM2SHORT(sItemId, FALSE),
                            &mi);
                        mi.afStyle |= MIS_BREAK;
                        WinSendMsg(hwndMenu, MM_SETITEM,
                            MPFROM2SHORT(sItemId, FALSE),
                            &mi);
                    }

                    // clean up
                    lstClear((PLISTITEM*)&pmliFoldersFirst,
                            (PLISTITEM*)&pmliFoldersLast,
                            0, 0);
                    lstClear((PLISTITEM*)&pmliObjectsFirst,
                            (PLISTITEM*)&pmliObjectsLast,
                            0, 0);
                }
            }

            ulWorkplaceFunc2 = 1299;
            // reset the mouse pointer
            WinSetPointer(HWND_DESKTOP, hptrOld);
        }
    }
}

/*
 * cmnFillMenuWithObjects:
 *      this subroutine is called by xfModifyPopupMenu with
 *      *objFolder pointing to the XFolder config folder;
 *      it calls itself recursively when folders are encountered
 *      in the config folder, passing their pointers in *objFolder
 *      again
 */

LONG cmnFillMenuWithObjects(WPObject      *somSelf,     // needed for wpInsertPopupMenuItems called in here
                        WPFolder   *pFolder,   // the config folder to process
                        HWND       hAddToMenu,   // the menu handle to add items to
                        HWND       hwndCnr)      // needed for wpInsertPopupMenuItems called in here
{
    // MENUITEM    mi;
    ULONG       ulStyle;
    // CHAR        szRealName[CCHMAXPATH];    // Buffer for wpQueryRealName()
    // ULONG       cb  = sizeof(szRealName);  // Size of object
    WPObject    *pObject, *pObject2;
    PSZ         pszNewItemString = NULL;
    // BOOL        fSuccess;
    HWND        hNewMenu;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    LONG       lDefaultItem = 0;
    LONG       rc, lReturnDefaultItem = 0;

    // we iterate over the content of *pFolder;
    // remember, on the first call to this routine,
    // *pFolder points to the config folder itself;
    // this routine will call itself recursively for each
    // folder encountered in the config folder, then passing
    // this folder in *objfolder

    for ( pObject = _xfQueryOrderedContent(pFolder, NULL, (ULONG)QC_FIRST);
          pObject;
          pObject = _xfQueryOrderedContent(pFolder, pObject, (ULONG)QC_NEXT)
        )
    {
        pszNewItemString = _wpQueryTitle(pObject);

        // if the object is a shadow, we will only de-reference
        //   it if it's a template
        if (_somIsA(pObject, _WPShadow)) {
            pObject2 = _wpQueryShadowedObject(pObject, FALSE);
            if (pObject2)
                if ((_wpQueryStyle(pObject2) & OBJSTYLE_TEMPLATE) == 0)
                    pObject2 = pObject;
        }
        else pObject2 = pObject;

        if (pObject2) { // eliminate broken shadows
            // now we check on the type of the class that we found and
            // remember the relevant types in the linked object list:
            // -- template: insert name, mark as QC_TEMPLATE
            // -- WPProgram: insert name into menu and mark as QC_PROGRAM;
            // -- WPFolder: insert submenu and recurse this routine;
            // -- others: insert name, mark as QC_OTHER;
            // we mark WPPrograms separately, since we will perform some
            // tricks on them in cmnMenuItemSelected when selected

            if (_wpQueryStyle(pObject2) & OBJSTYLE_TEMPLATE) {

                rc = cmnInsertOneObjectMenuItem(hAddToMenu,
                             MIT_END,
                             pszNewItemString,
                             MIS_TEXT,
                             pObject2,
                             OC_TEMPLATE);

                if (lReturnDefaultItem == 0)
                    lReturnDefaultItem = rc;

            } // end template
            else if (_somIsA(pObject2, _WPProgram)) {
                /* program object: get the title of it and insert it at the end of
                   the popup menu we're currently working on (can either be the
                   folder's context menu or submenu, when this routine is called
                   recursively; it's contained in hAddToMenu);
                   mark this item as OC_PROGRAM so we can perform tricks on it */

                if (strncmp(pszNewItemString, "---", 3) != 0)
                    rc = cmnInsertOneObjectMenuItem(hAddToMenu,
                                         MIT_END,
                                         pszNewItemString,
                                         MIS_TEXT,
                                         pObject2,
                                         OC_PROGRAM);
                else // title == "---": insert separator
                    rc = cmnInsertOneObjectMenuItem(hAddToMenu,
                                         MIT_END,
                                         pszNewItemString,
                                         MIS_SEPARATOR,
                                         pObject2,
                                         OC_SEPARATOR);

                if (lReturnDefaultItem == 0)
                    lReturnDefaultItem = rc;

            } // end else if (_somIsA(pObject2, _WPProgram))
            else if (_somIsA(pObject2, _WPFolder)) {
                //  if we find another folder, we add a submenu containing
                //  "[Config folder empty]"), which will be removed if more
                //  objects are found in the respective config folder

                hNewMenu = WinInsertSubmenu(hAddToMenu, MIT_END,
                            sNextMenuId, _wpQueryTitle(pObject2), MIS_TEXT,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_BORED),
                            (cmnQueryNLSStrings())->pszBored,
                                MIS_TEXT, 0);

                cmnAppendMi2List(pObject2, OC_FOLDER);

                // now call this function recursively with the new
                // folder and the new submenu handle */
                lDefaultItem = cmnFillMenuWithObjects(somSelf, pObject2, hNewMenu, hwndCnr);

                // now we're back: check if error occured; if so, exit
                // immediately to stop recursing
                if (lDefaultItem == -1)
                    return (-1);

                if (lDefaultItem) {
                    // items were inserted:
                    // remove static "config folder empty" menu item
                    WinSendMsg(hNewMenu,            // hAddToMenu,
                            MM_DELETEITEM,
                            MPFROM2SHORT((pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_BORED), TRUE),
                            (MPARAM)NULL);

                    if ((pGlobalSettings->Cascade) && (lDefaultItem != 1)) {
                        // make the XFolder submenu cascading;
                        // stolen from the Warp Toolkit WPS Guide */
                        ulStyle = WinQueryWindowULong(hNewMenu, QWL_STYLE);
                        ulStyle |= MS_CONDITIONALCASCADE;
                        WinSetWindowULong(hNewMenu, QWL_STYLE, ulStyle);

                        // make the first item in the subfolder
                        // the default of cascading submenu */
                        WinSendMsg(hNewMenu,
                                MM_SETDEFAULTITEMID,
                                (MPARAM)(lDefaultItem), 0L);
                    }
                }

                if (lReturnDefaultItem == 0)
                    lReturnDefaultItem = 1;

            } // end else if (_somIsA(pObject2, _WPFolder))
            else {
                // types other than WPFolder and WPProgram:
                // simply add them to menu and mark them as OC_OTHER
                // (will simply be opened)
                rc = cmnInsertOneObjectMenuItem(hAddToMenu,
                             MIT_END,
                             pszNewItemString,
                             MIS_TEXT,
                             pObject2,
                             OC_OTHER);

                if (lReturnDefaultItem == 0)
                    lReturnDefaultItem = rc;
            }
        } // end if (pObject2)
    } // end for
    return (lReturnDefaultItem);
}

/*
 * cmnCheckDefaultSortItem:
 *      checks/unchecks a sort item in the "Sort" submenu
 */

VOID cmnCheckDefaultSortItem(PGLOBALSETTINGS pGlobalSettings,
                            HWND hwndSortMenu,
                            ULONG ulDefaultSort)
{
    WinSetMenuItemChecked(hwndSortMenu,
            WinSendMsg(hwndSortMenu, MM_QUERYDEFAULTITEMID,
                    MPNULL, MPNULL),        // find current default
            FALSE);                         // uncheck

    WinSendMsg(hwndSortMenu,
            MM_SETDEFAULTITEMID,
            (MPARAM)
                    ((ulDefaultSort == SV_NAME) ? ID_WPMI_SORTBYNAME
                    : (ulDefaultSort == SV_TYPE) ? ID_WPMI_SORTBYTYPE
                    : (ulDefaultSort == SV_CLASS) ? pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTBYCLASS
                    : (ulDefaultSort == SV_REALNAME) ? ID_WPMI_SORTBYREALNAME
                    : (ulDefaultSort == SV_SIZE) ? ID_WPMI_SORTBYSIZE
                    : (ulDefaultSort == SV_LASTWRITEDATE) ? ID_WPMI_SORTBYWRITEDATE
                    : (ulDefaultSort == SV_LASTACCESSDATE) ? ID_WPMI_SORTBYACCESSDATE
                    : (ulDefaultSort == SV_CREATIONDATE) ? ID_WPMI_SORTBYCREATIONDATE
                    : (ulDefaultSort == SV_EXT) ? pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTBYEXT
                    : pGlobalSettings->VarMenuOffset +ID_XFMI_OFS_SORTFOLDERSFIRST),
            MPNULL);
}

/*
 * cmnModifySortMenu:
 *      this modifies the "Sort" submenu. Used for both
 *      folder context menus (cmnModifyPopupMenu below)
 *      and folder menu bars (WM_INITMENU message in
 *      fnwpSubclassedFolderFrame). For speedier operation,
 *      this func takes a lot of parameters.
 *      Note that hwndMenu is NOT the window handle of the
 *      "Sort" menu, but the one of the parent menu, i.e.
 *      the context menu itself or the "View" menu in
 *      menu bars.
 */

VOID cmnModifySortMenu(HWND hwndMenu,               // menu to modify
                XFolderData *somThis,               // XFolderGetData
                PGLOBALSETTINGS pGlobalSettings,    // cmnQueryGlobalSettings
                PNLSSTRINGS pNLSStrings)            // cmnQueryNLSStrings
{
    // work on "Sort" menu, if allowed
    if (pGlobalSettings->ReplaceSort)
    {
        if ((pGlobalSettings->DefaultMenuItems & CTXT_SORT) == 0)
        {
            HWND        hwndSortMenu;
            // SHORT       sItem, sItemCount;
            ULONG       ulDefaultSort = DEFAULT_SORT;
            MENUITEM    mi;

            if ((BOOL)WinSendMsg(hwndMenu,
                MM_QUERYITEM,
                MPFROM2SHORT(WPMENUID_SORT, TRUE),
                (MPARAM)&mi))
            {
                hwndSortMenu = mi.hwndSubMenu;
                // now contains "Sort" submenu handle;
                // insert the XFolder sort criteria
                WinInsertMenuItem(hwndSortMenu, 2, // after "type"
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTBYCLASS),
                        pNLSStrings->pszSortByClass,
                        MIS_TEXT, 0);
                WinInsertMenuItem(hwndSortMenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTBYEXT),
                        pNLSStrings->pszSortByExt,
                        MIS_TEXT, 0);
                WinInsertMenuItem(hwndSortMenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTFOLDERSFIRST),
                        pNLSStrings->pszSortFoldersFirst,
                        MIS_TEXT, 0);

                WinInsertMenuSeparator(hwndSortMenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));

                // insert "Always sort"
                WinInsertMenuItem(hwndSortMenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_ALWAYSSORT),
                        pNLSStrings->pszAlwaysSort,
                        MIS_TEXT,
                        (ALWAYS_SORT)           // check item if "Always sort" is on
                            ? (MIA_CHECKED)
                            : 0);

                cmnCheckDefaultSortItem(pGlobalSettings,
                                        hwndSortMenu,
                                        ulDefaultSort);

            } // end if MM_QUERYITEM
        }
    }
}

/*
 * cmnModifyPopupMenu:
 *      this routine gets called by the wpModifyPopupMenu methods of
 *      both XFolder and XFldDisk; first it removes and inserts menu
 *      items according to the Global / folder settings and then starts
 *      working on folder content and Config Folders
 */

BOOL cmnModifyPopupMenu(XFolder *somSelf, HWND hwndMenu, HWND hwndCnr, ULONG iPosition)
{
    XFolder             *pConfigFolder;
    XFolder             *pFavorite;
    BOOL                rc = TRUE;
    MENUITEM            mi;

    XFolderData         *somThis;
    BOOL                fIsWarp4 = DosIsWarp4();

    PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    APIRET                arc;
    PSZ                   pszErrMsg = NULL;

    begin:

    // set our extended exception handler
    RegRec2.pfnHandler = (PFN)ExcptHandlerQuiet;
    if ( arc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt)
    {
        pszErrMsg = malloc(1000);
        if (pszErrMsg) {
            strcpy(pszErrMsg, "An error occured while XFolder was trying to build "
                    "a folder's context menu. This might either be due to the fact "
                    "that you have deleted objects from these folders, but you did "
                    "not have these folders opened in the Icon or Details views "
                    "while doing so. As a result, the icon positions of the "
                    "configuration folder have not been updated. You can simply open "
                    "and close the configuration folders to solve this problem. "
                    "Another possible cause for the error might be problems with "
                    "the icon handling in folder content menus, which have occured "
                    "with some video drivers in the past. If these errors "
                    "persist, you might want to disable the icons for folder "
                    "content menus in the Global Settings.");
            cmnPostObjectMsg(XOM_EXCEPTIONCAUGHT, (MPARAM)pszErrMsg, MPNULL);
        }
    } else {
        if (somSelf) {
            somThis = XFolderGetData(somSelf);

            // store mouse pointer position for creating objects from templates
            WinQueryMsgPos(WinQueryAnchorBlock(HWND_DESKTOP), &(_ptlMenuMousePos));

            // remove a few menu items which we cannot remove
            // in wpFilterPopupMenu, because no codes are provided;
            // we only do this if the Global Settings allow it
            if (pGlobalSettings->RemoveView) {
                WinSendMsg(hwndMenu, MM_REMOVEITEM,
                    MPFROM2SHORT(ID_WPM_VIEW, FALSE),
                    0);
            }

            if (pGlobalSettings->RemoveInsert) {
                WinSendMsg(hwndMenu, MM_REMOVEITEM,
                    MPFROM2SHORT(ID_WPMI_INSERT, FALSE),
                    0);
            }

            // add product info item to the help menu
            if ((pGlobalSettings->DefaultMenuItems & CTXT_HELP) == 0) {
                // get handle to the WPObject's "Help" submenu in the
                // the folder's popup menu; the wp functions do not support
                // dynamic menus, so we use Win functions */
                WinSendMsg(hwndMenu,
                    MM_QUERYITEM,
                    MPFROM2SHORT(WPMENUID_HELP, TRUE),
                    (MPARAM)&mi);
                // mi.hwndSubMenu now contains "Help" submenu handle,
                // which we add an item to now

                WinInsertMenuSeparator(mi.hwndSubMenu, MIT_END, (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));

                WinInsertMenuItem(mi.hwndSubMenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_PRODINFO),
                        pNLSStrings->pszProductInfo,
                        MIS_TEXT, 0);
            }

            // add "Select by name"
            if (    (pGlobalSettings->SelectSome)
                 && (   ( (fIsWarp4)  && (pGlobalSettings->RemoveView == FALSE) )
                     || ( (!fIsWarp4) && (pGlobalSettings->DefaultMenuItems & CTXT_SELECT) == 0) )
               )
            {
                // in Warp 3, "Select" is  a separate submenu;
                // in Warp 4, these items are in the "View submenu"
                SHORT sPos = MIT_END;
                WinSendMsg(hwndMenu,
                    MM_QUERYITEM,
                    MPFROM2SHORT( (fIsWarp4)
                                ? ID_WPM_VIEW
                                : WPMENUID_SELECT,
                            TRUE),
                    (MPARAM)&mi);
                // mi.hwndSubMenu now contains "Select"/"View" submenu handle,
                // which we add an item to now

                if (DosIsWarp4())
                    sPos = (SHORT)WinSendMsg(mi.hwndSubMenu, MM_ITEMPOSITIONFROMID,
                                    MPFROM2SHORT(WPMENUID_REFRESH, FALSE),
                                    MPNULL);
                WinInsertMenuItem(mi.hwndSubMenu,
                        sPos,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SELECTSOME),
                        pNLSStrings->pszSelectSome,
                        MIS_TEXT, 0);

                if (sPos != MIT_END)
                    // for Warp 4, add an additional separator
                    WinInsertMenuSeparator(mi.hwndSubMenu, sPos+1,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));
            }

            // work on "Sort" menu; we have put this
            // into a subroutine, because this is also
            // needed for folder menu _bars_
            cmnModifySortMenu(hwndMenu, somThis, pGlobalSettings, pNLSStrings);

            // insert the "Refresh now" and "Snap to grid" items only
            // when the folder is open */
            if (_wpFindUseItem(somSelf, USAGE_OPENVIEW, NULL))
            {
                // "Refresh now"
                if (pGlobalSettings->Move4Refresh) {
                    WinInsertMenuItem(hwndMenu, MIT_END,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_REFRESH),
                            pNLSStrings->pszRefreshNow,
                            MIS_TEXT, 0);
                }

                // "Snap to grid"
                if (    (_SnapToGrid == 1)
                     || ((_SnapToGrid == 2) && (pGlobalSettings->SnapToGrid))
                   )
                {
                    // insert only when sorting is off
                    if (!(ALWAYS_SORT))
                    {
                        CNRINFO     CnrInfo;
                        WinQueryCnrInfo(hwndCnr, CnrInfo);

                        if ((CnrInfo.flWindowAttr & (CV_ICON | CV_TREE)) == CV_ICON)
                        {
                            // insert "Snap to grid" only for open icon views
                            WinInsertMenuItem(hwndMenu, MIT_END,
                                    (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SNAPTOGRID),
                                    pNLSStrings->pszSnapToGrid,
                                    MIS_TEXT, 0);
                        }

                        /* // disable "Snap to grid" if always-sort is on
                        if (ALWAYS_SORT) {
                            WinEnableMenuItem(hwndMenu,
                                (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SNAPTOGRID),
                                FALSE);
                        } */
                    }
                }
            } // end if view open

            // now do neccessary preparations for all variable menu items
            // (i.e. folder content and config folder items)
            sNextMenuId = (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_VARIABLE);
            ulVarItemCount = 0;         // reset the number of variable items to 0

            lstClear((PLISTITEM*)&pcmliContentMenus, NULL, 0, 0);
            lstClear((PLISTITEM*)&pliVarItems, NULL, 0, 0);

            // folder content / favorite folders
            pFavorite = _xfclsQueryFavoriteFolder(_XFolder, NULL);

            if (    (pGlobalSettings->NoSubclassing == 0)
                 && (   (pGlobalSettings->FolderContent)
                     || (pFavorite)
                    )
               )
            {
                WinInsertMenuSeparator(hwndMenu,
                        MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));

                if (pGlobalSettings->FCShowIcons) {
                    // before actually inserting the content submenus, we need a real
                    // awful cheat, because otherwise the owner draw items won't work
                    // right (I don't know why the hell they are not being sent
                    // WM_MEASUREITEM msgs if we don't do this); so we insert a
                    // content submenu and remove it again right away
                    WinSendMsg(hwndMenu, MM_REMOVEITEM,
                        MPFROM2SHORT(cmnPrepareContentSubmenu(somSelf, hwndMenu,
                                 pNLSStrings->pszFldrContent,
                                 MIT_END,
                                 FALSE), // no owner draw
                            FALSE),
                        MPNULL);
                }

                if (pGlobalSettings->FolderContent)
                {
                    // add "Folder content" only if somSelf is not a favorite folder,
                    // because then we will insert the folder content anyway
                    if (!_xfIsFavoriteFolder(somSelf))
                        // somself is not in favorites list: add "Folder content"
                        cmnPrepareContentSubmenu(somSelf, hwndMenu,
                                pNLSStrings->pszFldrContent,
                                MIT_END,
                                FALSE); // no owner draw in main context menu
                }

                // now add favorite folders
                for (pFavorite = _xfclsQueryFavoriteFolder(_XFolder, NULL);
                     pFavorite;
                     pFavorite = _xfclsQueryFavoriteFolder(_XFolder, pFavorite))
                {
                    cmnPrepareContentSubmenu(pFavorite, hwndMenu,
                            _wpQueryTitle(pFavorite), MIT_END,
                            FALSE); // no owner draw in main context menu
                }
            } // end folder contents

            // now, this will start searching for user-defined stuff
            // in the XFolder config folder, which we need to locate first */
            pConfigFolder = _wpclsQueryFolder(_WPFolder, XFOLDER_CONFIGID, TRUE);

            if (pConfigFolder != NULL) {
                // config folder found: append another separator first
                WinInsertMenuSeparator(hwndMenu, MIT_END, (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));

                // now insert items in pConfigFolder into main context menu (hwndMenu);
                // this routine will call itself recursively if it finds subfolders
                cmnFillMenuWithObjects(somSelf, pConfigFolder,
                            hwndMenu, hwndCnr);
            } else {
                // config folder not found: give message and create it anew
                cmnPostWorkerMsg(WOM_RECREATECONFIGFOLDER, (MPARAM)RCF_QUERYACTION, MPNULL);
                rc = FALSE;
            }
        } // end if (somSelf)
            else rc = FALSE;
    }

    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
    return (rc);
}

/*
 * wpModifyPopupMenu:
 *           add the variable XFolder menu entries;
 *           have this done by the common XFolder function
 */

SOM_Scope BOOL  SOMLINK xf_wpModifyPopupMenu(XFolder   *somSelf,
                                                    HWND  hwndMenu,
                                                    HWND  hwndCnr,
                                                    ULONG iPosition)
{
    // HPOINTER            hptrOld, hptrWait, hptrCurrent;
    BOOL                rc;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpModifyPopupMenu");

    // change the mouse pointer to wait state
/*    hptrOld = WinQueryPointer(HWND_DESKTOP);
    hptrWait = WinQuerySysPointer(HWND_DESKTOP,
           SPTR_WAIT, FALSE);
    hptrCurrent = hptrWait;
    WinSetPointer(HWND_DESKTOP, hptrWait); */

    XFolder_parent_WPFolder_wpModifyPopupMenu(somSelf,hwndMenu,hwndCnr,iPosition);

    rc = cmnModifyPopupMenu(somSelf, hwndMenu, hwndCnr, iPosition);

    // reset the mouse pointer
    // WinSetPointer(HWND_DESKTOP, hptrOld);

    return (rc);
}

/*
 *  cmnProgramObjectSelected:
 *      this subroutine is called by cmnMenuItemSelected whenever a
 *      program object is to be handled; it does all the neccessary
 *      fuddling with the program object's data before opening it
 */

BOOL cmnProgramObjectSelected(WPObject *somSelf, WPProgram *pProgram)
{
    // HWND            openHWnd;
    PPROGDETAILS    pProgDetails;
    ULONG           ulSize;

    WPFolder        *pFolder = NULL;
    CHAR            szRealName[CCHMAXPATH];    // Buffer for wpQueryRealName()
    ULONG           cb  = sizeof(szRealName);  // Size of object

    BOOL            ValidRealName,
                    StartupChanged = FALSE,
                    ParamsChanged = FALSE,
                    TitleChanged = FALSE,
                    brc = TRUE;

    PSZ             pszOldParams = NULL,
                    pszOldTitle = NULL;
    CHAR            szPassRealName[CCHMAXPATH];
    CHAR            szNewParams[1024] = "";
    CHAR            szNewTitle[1024] = "";

    /* PSZ             pszPos,                         // temp pointer
                    pszClipText;                    // pointer for getting clipboard
                    */
    CHAR            szClipBuf[CCHMAXPATH];          // buffer for copying data
    ULONG           Ofs = 0;

    HAB             hab;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    // get program object data
    if ((_wpQueryProgDetails(pProgram, (PPROGDETAILS)NULL, &ulSize))) {
        if ((pProgDetails = (PPROGDETAILS)_wpAllocMem(somSelf, ulSize, NULL)) != NULL) {
            if ((_wpQueryProgDetails(pProgram, pProgDetails, &ulSize))) {

                // dereference folder shadows
                if (_somIsA(somSelf, _WPFolder))
                    pFolder = somSelf;
                // dereference disk objects
                else if (_somIsA(somSelf, _WPDisk))
                    pFolder = _wpQueryRootFolder(somSelf);

                if (pFolder)
                    ValidRealName = _wpQueryRealName(pFolder, szRealName, &cb, TRUE);
                // now we have the folder's full path

                // there seems to be a bug in wpQueryRealName for
                // root folders, so we might need to append a "\" */
                if (strlen(szRealName) == 2)
                    strcat(szRealName, "\\");

                // *** first trick:
                // if the program object's startup dir has not been
                // set, we will set it to szRealName
                // temporarily; this will start the
                // program object in the directory
                // of the folder whose context menu was selected */
                if (ValidRealName && (pProgDetails->pszStartupDir == NULL)) {
                    StartupChanged = TRUE;
                    pProgDetails->pszStartupDir = szRealName;
                }

                // start playing with the object's parameter list,
                // if the global settings allow it */
                if (pGlobalSettings->AppdParam) {
                    // if the folder's real name contains spaces,
                    // we need to enclose it in quotes */
                    if (strchr(szRealName, ' ')) {
                        strcpy(szPassRealName, "\"");
                        strcat(szPassRealName, szRealName);
                        strcat(szPassRealName, "\"");
                    } else
                        strcpy(szPassRealName, szRealName);

                    // backup prog data for later restore
                    pszOldParams = pProgDetails->pszParameters;

                    if (pszOldParams) { // parameter list not empty
                        // *** second trick:
                        // we will append the current folder path to the parameters
                        // if the program object's parameter list does not
                        // end in "%" ("Netscape support") */
                        if (ValidRealName && (pProgDetails->pszParameters[strlen(pProgDetails->pszParameters)-1] != '%')) {
                            ParamsChanged = TRUE;

                            strcpy(szNewParams, pszOldParams);
                            strcat(szNewParams, " ");
                            strcat(szNewParams, szPassRealName);
                        }

                        // *** third trick:
                        // replace an existing "%**C" in the parameters
                        // with the contents of the clipboard */
                        if (strstr(pszOldParams, CLIPBOARDKEY)) {

                            hab = WinQueryAnchorBlock(HWND_DESKTOP);
                            if (WinOpenClipbrd(hab)) {
                                PSZ pszClipText;
                                if (pszClipText = (PSZ)WinQueryClipbrdData(hab, CF_TEXT))
                                {
                                    PSZ pszPos = NULL;
                                    // Copy text from the shared memory object to local memory.
                                    strncpy(szClipBuf, pszClipText, CCHMAXPATH);
                                    szClipBuf[CCHMAXPATH-2] = '\0'; // make sure the string is terminated
                                    WinCloseClipbrd(hab);

                                    if (ParamsChanged == FALSE) // did we copy already?
                                        strcpy(szNewParams, pszOldParams);
                                    pszPos = strstr(szNewParams, CLIPBOARDKEY);
                                    Ofs = strlen(szClipBuf);
                                    if (Ofs + strlen(szNewParams) > CCHMAXPATH)
                                        Ofs -= strlen(szNewParams);
                                    strcpy(szClipBuf+Ofs, pszPos+strlen(CLIPBOARDKEY));
                                    strcpy(pszPos, szClipBuf);

                                    ParamsChanged = TRUE;
                                }
                                else { // no text data in clipboard
                                    WinCloseClipbrd(hab);
                                    cmnSetHelpPanel(ID_XFH_NOTEXTCLIP);
                                    if (WinDlgBox(HWND_DESKTOP,         // parent is desktop
                                          HWND_DESKTOP,             // owner is desktop
                                          (PFNWP)fnwpDlgGeneric,    // dialog procedure, defd. at bottom
                                          NLS_MODULE,  // from resource file
                                          ID_XFD_NOTEXTCLIP,        // dialog resource id
                                          (PVOID)NULL)             // no dialog parameters
                                       == DID_CANCEL)
                                          brc = FALSE;
                                }
                            }
                            else
                                DebugBox("XFolder", "The clipboard could not be opened.");
                        }
                        if (ParamsChanged)
                            pProgDetails->pszParameters = szNewParams;

                    } else
                        // parameter list is empty: simply set params
                        if (ValidRealName)
                        {
                            ParamsChanged = TRUE;
                            // set parameter list to folder name
                            pProgDetails->pszParameters = szPassRealName;
                            // since parameter list is empty, we need not
                            // search for the clipboard key ("%**C") */
                        }
                } // end if (pGlobalSettings->AppdParam)

                // now remove "~" from title, if allowed
                pszOldTitle = pProgDetails->pszTitle;
                if ((pszOldTitle) && (pGlobalSettings->RemoveX)) {
                    PSZ pszPos = strchr(pszOldTitle, '~');
                    if (pszPos)  {
                        TitleChanged = TRUE;
                        strncpy(szNewTitle, pszOldTitle, (pszPos-pszOldTitle));
                        strcat(szNewTitle, pszPos+1);
                        pProgDetails->pszTitle = szNewTitle;
                    }
                }

                // now apply new settings, if neccessary
                if (StartupChanged || ParamsChanged || TitleChanged) {
                    if (!_wpSetProgDetails(pProgram, pProgDetails)) {
                        DebugBox("XFolder", "Unable to set new startup directory.");
                        brc = FALSE;
                    }
                }

                // open the object with new settings
                _wpViewObject(pProgram, NULLHANDLE, OPEN_DEFAULT, 0);

                // now restore the old settings, if neccessary
                if (StartupChanged)
                    pProgDetails->pszStartupDir = NULL;
                if (ParamsChanged)
                    pProgDetails->pszParameters = pszOldParams;
                if (TitleChanged)
                    pProgDetails->pszTitle = pszOldTitle;
                if (StartupChanged || ParamsChanged || TitleChanged)
                    _wpSetProgDetails(pProgram, pProgDetails);

            } else
                brc = FALSE;

            _wpFreeMem(somSelf, (PBYTE)pProgDetails);
        } else
            brc = FALSE;
    } else
        brc = FALSE;

    return (brc);
}

/*
 * cmnIsSortMenuItemSelected:
 *      this is used by both cmnMenuItemSelected and
 *      xfSelectingMenuItem for checking if the selected
 *      menu item is one of the folder things and, if so,
 *      setting the folder sort settings accordingly.
 *      We need to have a separate proc for this because
 *      xfSelectingMenuItem needs this if the user uses
 *      the mouse and cmnMenuItemSelected gets the folder
 *      hotkeys.
 *      This returns TRUE if the menu item was processed
 *      and then sets *pbDismiss to whether the menu should
 *      be dismissed. If pbDismiss is NULL, this assumes
 *      that we're dealing with hotkeys.
 */

BOOL cmnIsSortMenuItemSelected(XFolder* somSelf,
                        HWND hwndFrame,
                        HWND hwndMenu,      // may be NULLHANDLE if
                                            // pbDismiss is NULL also
                        ULONG ulMenuId,
                        PGLOBALSETTINGS pGlobalSettings,
                        PBOOL pbDismiss)    // out: dismiss flag for xfSelectingMenuItem
{
    BOOL        brc = FALSE;
    ULONG       ulMenuId2 = ulMenuId - pGlobalSettings->VarMenuOffset;
    USHORT      usAlwaysSort, usDefaultSort;

    switch (ulMenuId2) {
        // new sort items
        case ID_XFMI_OFS_SORTBYCLASS:
        case ID_XFMI_OFS_SORTBYEXT:
        case ID_XFMI_OFS_SORTFOLDERSFIRST:
        {
            USHORT usSort;
            BOOL   fShiftPressed = DosQueryShiftState();
            switch (ulMenuId2) {
                case ID_XFMI_OFS_SORTBYCLASS:       usSort = SV_CLASS;        break;
                case ID_XFMI_OFS_SORTFOLDERSFIRST:  usSort = SV_FOLDERSFIRST; break;
                case ID_XFMI_OFS_SORTBYEXT:         usSort = SV_EXT;          break;
            }
            if ((fShiftPressed) && (pbDismiss))
            {
                _xfQueryFldrSort(somSelf, &usDefaultSort, &usAlwaysSort);
                _xfSetFldrSort(somSelf,   usSort,  usAlwaysSort);
                cmnCheckDefaultSortItem(pGlobalSettings,
                                        hwndMenu,
                                        usSort);
            } else
                _xfSortView(somSelf,
                        hwndFrame,
                        usSort);

            // do not dismiss menu when shift was pressed
            if (pbDismiss)
                *pbDismiss = (!fShiftPressed);
            brc = TRUE;
        break; }

        default:
            switch (ulMenuId) { // check original menu id

                // one of the original WPS "sort" menu items:
                case ID_WPMI_SORTBYNAME:
                case ID_WPMI_SORTBYTYPE:
                case ID_WPMI_SORTBYREALNAME:
                case ID_WPMI_SORTBYSIZE:
                case ID_WPMI_SORTBYWRITEDATE:
                case ID_WPMI_SORTBYACCESSDATE:
                case ID_WPMI_SORTBYCREATIONDATE:
                    if (pGlobalSettings->ReplaceSort) { // extended sorting?
                        BOOL   fShiftPressed = DosQueryShiftState();
                        USHORT usSort;
                        switch (ulMenuId) {
                            case ID_WPMI_SORTBYNAME:            usSort = SV_NAME;         break;
                            case ID_WPMI_SORTBYTYPE:            usSort = SV_TYPE;         break;
                            case ID_WPMI_SORTBYREALNAME:        usSort = SV_REALNAME;     break;
                            case ID_WPMI_SORTBYSIZE:            usSort = SV_SIZE;         break;
                            case ID_WPMI_SORTBYWRITEDATE:       usSort = SV_LASTWRITEDATE;    break;
                            case ID_WPMI_SORTBYACCESSDATE:      usSort = SV_LASTACCESSDATE;   break;
                            case ID_WPMI_SORTBYCREATIONDATE:    usSort = SV_CREATIONDATE; break;
                        }
                        if ((fShiftPressed) && (pbDismiss))
                        {
                            // SHIFT pressed when menu item was selected:
                            // set default sort criterion
                            _xfQueryFldrSort(somSelf, &usDefaultSort, &usAlwaysSort);
                            _xfSetFldrSort(somSelf,   usSort,         usAlwaysSort);
                            cmnCheckDefaultSortItem(pGlobalSettings,
                                                    hwndMenu,
                                                    usSort);
                        } else {
                            // not SHIFT pressed: simply sort
                            _xfSortView(somSelf,
                                    hwndFrame,
                                    usSort);
                        }

                        // do not dismiss menu when shift was pressed
                        if (pbDismiss)
                            *pbDismiss = (!fShiftPressed);
                        // found flag
                        brc = TRUE;
                    }
                break;
            }
    }
    return (brc);
}

/*
 * cmnMenuItemSelected:
 *           checks for whether one of XFolder's menu items
 *           was selected; if so, executes corresponding action
 *           and returns TRUE, otherwise does nothing and
 *           returns FALSE, upon which the calling method
 *           (which should be wpMenuItemSelected) should
 *           call its parent method to process the menu item
 */

BOOL cmnMenuItemSelected(XFolder *somSelf, HWND hwndFrame, ULONG ulMenuId)
{
    WPObject            *pObject = NULL;
    ULONG               ulFirstVarMenuId;
    PVARMENULISTITEM    pItem;
    PGLOBALSETTINGS     pGlobalSettings = cmnQueryGlobalSettings();
    ULONG               ulMenuId2 = ulMenuId - pGlobalSettings->VarMenuOffset;
    BOOL                brc = FALSE;     // "not processed" flag

    if (somSelf) {
        BOOL        fDummy;

        // check for sort menu items first
        if (cmnIsSortMenuItemSelected(somSelf,
                        hwndFrame,
                        NULLHANDLE,     // we don't know the menu hwnd
                        ulMenuId,
                        pGlobalSettings, &fDummy))
            brc = TRUE;

        // else: no sort menu item
        else switch (ulMenuId2)
        {
            // "Product Information"
            case ID_XFMI_OFS_PRODINFO:
            {
                // advertise for myself
                CHAR szGPLInfo[2000];
                HWND hwndInfo = WinLoadDlg(HWND_DESKTOP, HWND_DESKTOP,
                                  fnwpDlgGeneric,
                                  NLS_MODULE,
                                  ID_XFD_PRODINFO,
                                  NULL);
                cmnPlaySystemSound(MMSOUND_SYSTEMSTARTUP);

                // load GPL info message into prodinfo MLE
                cmnGetMessage(NULL, 0,
                        szGPLInfo, sizeof(szGPLInfo),
                        140);
                WinSetDlgItemText(hwndInfo, ID_XSDI_SC_TEXT2, szGPLInfo);
                WinSendDlgItemMsg(hwndInfo, ID_XSDI_SC_TEXT2,
                        MLM_SETFIRSTCHAR,       // scroll MLE to top
                        (MPARAM)0,
                        MPNULL);

                cmnSetHelpPanel(0);
                WinCenterWindow(hwndInfo);
                WinProcessDlg(hwndInfo);
                WinDestroyWindow(hwndInfo);
                brc = TRUE;
            break; }

            // "Select by name"
            case ID_XFMI_OFS_SELECTSOME: {
                HWND hwndSelectSome = WinLoadDlg(HWND_DESKTOP,    // parent
                        hwndFrame,         // owner
                        (PFNWP)fnwpSelectSome,
                        NLS_MODULE,
                        ID_XFD_SELECTSOME,
                        (PVOID)hwndFrame);    // dlg params
                cmnSetHelpPanel(ID_XFH_SELECTSOME);
                WinShowWindow(hwndSelectSome, TRUE);
                brc = TRUE;
            break; }

            // copy filename:
            // this is no menu item, but only posted here by folder hotkeys
            case ID_XFMI_OFS_COPYFILENAME_SHORT:
            case ID_XFMI_OFS_COPYFILENAME_FULL: {
                // if the user presses hotkeys for "copy filename",
                // he doesn't want the filename of the folder
                // (which somSelf points to here...), but of the
                // selected objects, so we repost the msg to
                // the first selected object, which will handle
                // the rest
                HWND hwndCnr = WinQueryCnrFromFrame(hwndFrame);
                if (hwndCnr) {
                    PMINIRECORDCORE pmrc = WinSendMsg(hwndCnr,
                                CM_QUERYRECORDEMPHASIS,
                                (MPARAM)CMA_FIRST, // query first
                                (MPARAM)CRA_SELECTED);
                    if ((pmrc != NULL) && ((ULONG)pmrc != -1)) {
                        WPObject *pObject = OBJECT_FROM_PREC(pmrc);
                        if (pObject) {
                            cmnCopyObjectFileName(pObject, hwndFrame,
                                (ulMenuId2 == ID_XFMI_OFS_COPYFILENAME_FULL));
                        }
                    }
                }
            break; }

            // "Snap to grid"
            case ID_XFMI_OFS_SNAPTOGRID: {
                _xfSnapToGrid(somSelf, TRUE);
                brc = TRUE;
            break; }

            // "Open parent":
            // this is no menu item, but only used by folder hotkeys
            case ID_XFMI_OFS_OPENPARENT: {
                _wpViewObject(_wpQueryFolder(somSelf), NULLHANDLE, OPEN_DEFAULT, 0);
                brc = TRUE;
            break; }

            // "Open parent and close":
            // this is no menu item, but only used by folder hotkeys
            case ID_XFMI_OFS_OPENPARENTANDCLOSE: {
                _wpViewObject(_wpQueryFolder(somSelf), NULLHANDLE, OPEN_DEFAULT, 0);
                _wpClose(somSelf);
                brc = TRUE;
            break; }

            // "Context menu":
            // this is no menu item, but only used by folder hotkeys
            case ID_XFMI_OFS_CONTEXTMENU: {
                HWND hwndCnr = WinQueryCnrFromFrame(hwndFrame);
                POINTS pts = {0, 0};
                WinPostMsg(hwndCnr, WM_CONTEXTMENU,
                        (MPARAM)&pts,
                        MPFROM2SHORT(0, TRUE));
                brc = TRUE;
            break; }

            // "Refresh now": translate and pass to original method
            case ID_XFMI_OFS_REFRESH: {
                _wpMenuItemSelected(somSelf, hwndFrame, WPMENUID_REFRESH);
                brc = TRUE;
            break; }

            // close window
            case ID_XFMI_OFS_CLOSE: {
                WinPostMsg(hwndFrame, WM_SYSCOMMAND, (MPARAM)SC_CLOSE,
                        MPFROM2SHORT(CMDSRC_MENU, FALSE));
                brc = TRUE;
            break; }

            // "[Config folder empty]"
            case ID_XFMI_OFS_BORED: {
                // explain how to configure XFolder
                cmnMessageBoxMsg(HWND_DESKTOP, 116, 135, MB_OK);
                brc = TRUE;
            break; }

            default: {
                // anything else: check if it's one of our variable menu items
                ulFirstVarMenuId = pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_VARIABLE;
                if ( (ulMenuId >= ulFirstVarMenuId)
                      && (ulMenuId <  ulFirstVarMenuId + ulVarItemCount)
                   )
                {
                    // variable menu item selected:
                    // get corresponding menu list item from the list that
                    // was created by cmnModifyPopupMenu
                    pItem = (PVARMENULISTITEM)lstItemFromIndex(
                                (PLISTITEM)pliVarItems, (ulMenuId - ulFirstVarMenuId), 0, 0);

                    if (pItem)
                        pObject = pItem->pObject;

                    if (pObject) { // defaults to NULL
                        // OK, we've found the corresponding object
                        switch (pItem->usObjType) {
                            // this data has previously been saved by cmnFillMenuWithObjects when
                            // the context menu was created; it contains a flag telling us
                            // what kind of menu item we're dealing with

                            case OC_TEMPLATE: {
                                // if the object is a template, we create a new object from it
                                // WPObject        *pNewObject;
                                // ULONG           ulCreateFlags = 0;
                                XFolderData     *somThis = XFolderGetData(somSelf);

                                _xfCreateFromTemplate(pObject, somSelf,
                                        hwndFrame,
                                        pGlobalSettings->OpenSettings,
                                        &(_ptlMenuMousePos));

                            break; } // end OC_TEMPLATE

                            case OC_PROGRAM: {
                                // WPPrograms are handled separately, for we will perform
                                // tricks on the startup directory and parameters */
                                cmnProgramObjectSelected(somSelf, pObject);
                            break; } // end OC_PROGRAM

                            default:
                                // objects other than WPProgram and WPFolder (which is handled by
                                // the OS/2 menu handling) will simply be opened without further
                                // discussion.
                                // This includes folder content menu items,
                                // which are marked as OC_CONTENT; MB2 clicks into
                                // content menus are handled by the subclassed folder wnd proc
                                _wpViewObject(pObject, NULLHANDLE, OPEN_DEFAULT, 0);
                            break;
                        } // end switch
                    } //end else (pObject == NULL)
                    brc = TRUE;
                } // end if ((ulMenuId >= ID_XFM_VARIABLE) && (ulMenuId < ID_XFM_VARIABLE+varItemCount))
                else { // none of our variable menu items:
                    brc = FALSE;  // "not processed" flag
                }
            } // end default;
        } // end switch;
    } // end if (somSelf)

    return (brc);
    // this flag is FALSE by default; it signals to the caller (which
    // is wpMenuItemSelected of either XFolder or XFldDisk) whether the
    // parent method still needs to be called. If TRUE, we have processed
    // something, if FALSE, we havent, then call the parent
}

/*
 * xfModifySettingsSubmenu:
 *           this new method is called from XFldObject's override
 *           of wpModifyPopupMenu every time a context menu is
 *           about to be opened. In this method, WPS classes may
 *           add new items to the new "Settings" submenu, whose
 *           wnd handle is passed to this method in
 *           hwndSettingsSubmenu. This method must return TRUE if
 *           any items were added, or FALSE otherwise.
 */

SOM_Scope BOOL  SOMLINK xf_xfModifySettingsSubmenu(XFolder *somSelf,
                                                   HWND hwndSettingsSubmenu,
                                                   HWND hwndCnr)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
    HWND        hwndViewSubmenu;
    CNRINFO     CnrInfo;
    BOOL        brc;
    PUSEITEM    pUseItem = NULL;
    BOOL        fAddViewSettings = TRUE;

    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_xfModifySettingsSubmenu");

    brc = XFolder_parent_XFldFileSystem_xfModifySettingsSubmenu(somSelf,
                                                              hwndSettingsSubmenu,
                                                              hwndCnr);

    // insert the view settings items only if the context
    // menu is requested from an open view of somSelf, not
    // from the parent; in order to find out about this, we
    // go through the almost-documented USAGE_RECORD structures,
    // which give us all the cnrs into which somSelf has been
    // inserted;
    if (hwndCnr)
    {
        WinQueryCnrInfo(hwndCnr, CnrInfo);

        for (pUseItem = _wpFindUseItem(somSelf, USAGE_RECORD, NULL);
            pUseItem;
            pUseItem = _wpFindUseItem(somSelf, USAGE_RECORD, pUseItem))
        {
            PRECORDITEM pRecordItem = (PRECORDITEM)(pUseItem+1);
            if (hwndCnr == pRecordItem->hwndCnr)
                // somSelf has been inserted into hwndCnr: this
                // means that the context menu has NOT been
                // opened from an open view of somSelf,
                // UNLESS it is the top item in a tree view...
                if (    (WinSendMsg(hwndCnr, CM_QUERYRECORD,
                                (MPARAM)NULL,
                                MPFROM2SHORT(CMA_FIRST, CMA_ITEMORDER))
                            != _wpQueryCoreRecord(somSelf))
                    || ((CnrInfo.flWindowAttr & CV_TREE) == 0)
                   )
                {
                    fAddViewSettings = FALSE;
                    break;
                }
        }

        if (fAddViewSettings) {
            ULONG ulAttr = 0;
            // we have an open view:
            hwndViewSubmenu = WinInsertSubmenu(hwndSettingsSubmenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFM_OFS_FLDRSETTINGS),
                                pNLSStrings->pszFldrSettings, 0,
                    // "small icons" item (for all view types)
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SMALLICONS),
                                pNLSStrings->pszSmallIcons, MIS_TEXT,
                                    (CnrInfo.flWindowAttr & CV_DETAIL)
                                      // Details view: disable and check
                                      // "mini icons"
                                        ? (MIA_DISABLED | MIA_CHECKED)
                                    : // other than Details view:
                                        (CnrInfo.flWindowAttr & CV_MINI)
                                            ? MIA_CHECKED
                                            : 0);
            brc = TRUE; // modified flag

            if (    ((CnrInfo.flWindowAttr & (CV_ICON | CV_TREE)) == CV_ICON)
                 || ((CnrInfo.flWindowAttr & CV_NAME) == CV_NAME)
                 || ((CnrInfo.flWindowAttr & CV_TEXT) == CV_TEXT)
               )
            {
                // icon view:
                WinInsertMenuSeparator(hwndViewSubmenu, MIT_END,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));


                WinInsertMenuItem(hwndViewSubmenu, MIT_END,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_NOGRID),
                                    pNLSStrings->pszNoGrid, MIS_TEXT,
                                    ((CnrInfo.flWindowAttr & (CV_ICON | CV_TREE)) == CV_ICON)
                                        ? MIA_CHECKED
                                        : 0);
                WinInsertMenuItem(hwndViewSubmenu, MIT_END,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_FLOWED),
                                    pNLSStrings->pszFlowed, MIS_TEXT,
                                    ((CnrInfo.flWindowAttr & (CV_NAME | CV_FLOW)) == (CV_NAME | CV_FLOW))
                                        ? MIA_CHECKED
                                        : 0);
                WinInsertMenuItem(hwndViewSubmenu, MIT_END,
                            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_NONFLOWED),
                                    pNLSStrings->pszNonFlowed, MIS_TEXT,
                                    ((CnrInfo.flWindowAttr & (CV_NAME | CV_FLOW)) == (CV_NAME))
                                        ? MIA_CHECKED
                                        : 0);
            }

            WinInsertMenuSeparator(hwndViewSubmenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));


            if (_somIsA(somSelf, _WPDesktop))
                ulAttr = MIA_DISABLED;
            else
                ulAttr = (
                           (_StatusBar == STATUSBAR_ON)
                        || (    (_StatusBar == STATUSBAR_DEFAULT)
                             && (pGlobalSettings->StatusBar)
                           )
                         )
                            ? MIA_CHECKED
                            : 0;
            WinInsertMenuItem(hwndViewSubmenu, MIT_END,
                        (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SHOWSTATUSBAR),
                                pNLSStrings->pszShowStatusBar, MIS_TEXT,
                                    ulAttr);
        }
    }
    return (brc);
}

/*
 * xfSelectingMenuItem:
 *           this new method is called on an object even before
 *           wpMenuItemSelected is called, i.e. right after a
 *           menu item has been selected by the user, and before
 *           the menu is dismissed.
 *           This call is the result
 *           of a WM_MENUSELECT intercept of the subclassed frame
 *           window procedure of an open folder. An object may
 *           intercept certain menu selections in order to process
 *           them without the menu being dismissed afterwards.
 *           Parameters:
 *              usItem        selected menu item
 *              fPostCommand  if this flag is TRUE, wpMenuItemSelected
 *                            will be called afterwards (do not modify)
 *              hwndMenu      wnd handle of the current menu control
 *              hwndCnr       wnd handle of the current container
 *              ulSelection   one of the following:
 *                    SEL_WHITESPACE the context menu was opened on the
 *                                   whitespace in an open container view
 *                                   of somSelf (which is a folder then)
 *                    SEL_SINGLESEL  the context menu was opened for a
 *                                   single selected object: somSelf can
 *                                   be any object then, including folders
 *                    SEL_MULTISEL   the context menu was opened on one
 *                                   of a multitude of selected objects;
 *                                   again, somSelf can be any object
 *                    SEL_SINGLEOTHER the context menu was opened for a
 *                                   single object _other_ than the selected
 *                                   objects
 *
 *           You must return one of the following:
 *              TRUE          dismiss the menu and, if (fPostCommand),
 *                            call wpMenuItemSelected
 *              FALSE         do not dismiss the menu and do not call
 *                            wpMenuItemSelected
 *           For menu items which have no meaning for this instance method,
 *           the return value of the parent method must be returned.
 */

SOM_Scope BOOL  SOMLINK xf_xfSelectingMenuItem(XFolder *somSelf,
                                               USHORT usItem,
                                               BOOL fPostCommand,
                                               HWND hwndMenu,
                                               HWND hwndCnr,
                                               ULONG ulSelection)
{
    // XFolderData *somThis = XFolderGetData(somSelf);

    PGLOBALSETTINGS     pGlobalSettings = cmnQueryGlobalSettings();
    ULONG               ulMenuId2 = usItem - pGlobalSettings->VarMenuOffset;
    BOOL                brc = FALSE;     // "not processed" flag
    USHORT              usAlwaysSort, usDefaultSort;

    XFolderMethodDebug("XFolder","xf_xfSelectingMenuItem");

    if (cmnIsSortMenuItemSelected(somSelf,
                    WinQueryWindow(hwndCnr, QW_PARENT), // frame window
                    hwndMenu,
                    usItem,
                    pGlobalSettings,
                    &brc))              // dismiss flag == return value
        ;
        // brc = TRUE;
    // else: no sort menu item
    else switch (ulMenuId2)
    {
        case ID_XFMI_OFS_SMALLICONS: {
            // toggle small icons for folder; this menu item
            // only exists for open Icon and Tree views
            CNRINFO CnrInfo;
            ULONG ulViewAttr, ulCnrView;
            ULONG ulMenuAttr = (ULONG)WinSendMsg(hwndMenu, MM_QUERYITEMATTR,
                                        MPFROM2SHORT(usItem, FALSE),
                                        (MPARAM)MIA_CHECKED);
            // toggle "checked" flag in menu
            ulMenuAttr ^= MIA_CHECKED;  // XOR checked flag;
            WinSendMsg(hwndMenu, MM_SETITEMATTR,
                                        MPFROM2SHORT(usItem, FALSE),
                                        MPFROM2SHORT(MIA_CHECKED, ulMenuAttr));

            // toggle cnr flags
            WinQueryCnrInfo(hwndCnr, CnrInfo);
            ulCnrView = (CnrInfo.flWindowAttr & CV_TREE) ? OPEN_TREE : OPEN_CONTENTS;
            ulViewAttr = _wpQueryFldrAttr(somSelf, ulCnrView);
            ulViewAttr ^= CV_MINI;      // XOR mini-icons flag
            _wpSetFldrAttr(somSelf,
                    ulViewAttr,
                    ulCnrView);
            brc = FALSE;
        break; }

        case ID_XFMI_OFS_FLOWED:
        case ID_XFMI_OFS_NONFLOWED:
        case ID_XFMI_OFS_NOGRID: {
            // these items exist for icon views only
            ULONG ulViewAttr = _wpQueryFldrAttr(somSelf, OPEN_CONTENTS);
            switch (ulMenuId2) {
                case ID_XFMI_OFS_FLOWED:
                    // == CV_NAME | CV_FLOW; not CV_ICON
                    ulViewAttr = (ulViewAttr & ~CV_ICON) | CV_NAME | CV_FLOW;
                break;

                case ID_XFMI_OFS_NONFLOWED:
                    // == CV_NAME only; not CV_ICON
                    ulViewAttr = (ulViewAttr & ~(CV_ICON | CV_FLOW)) | CV_NAME;
                break;

                case ID_XFMI_OFS_NOGRID:
                    ulViewAttr = (ulViewAttr & ~(CV_NAME | CV_FLOW)) | CV_ICON;
                break;
            }
            _wpSetFldrAttr(somSelf,
                    ulViewAttr,
                    OPEN_CONTENTS);

            WinSetMenuItemChecked(hwndMenu,
                        pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_FLOWED,
                        (ulMenuId2 == ID_XFMI_OFS_FLOWED));
            WinSetMenuItemChecked(hwndMenu,
                        pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_NONFLOWED,
                        (ulMenuId2 == ID_XFMI_OFS_NONFLOWED));
            WinSetMenuItemChecked(hwndMenu,
                        pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_NOGRID,
                        (ulMenuId2 == ID_XFMI_OFS_NOGRID));

            // do not dismiss menu
            brc = FALSE;
        break; }

        case ID_XFMI_OFS_SHOWSTATUSBAR: {
            // toggle status bar for folder
            ULONG ulMenuAttr = (ULONG)WinSendMsg(hwndMenu, MM_QUERYITEMATTR,
                                        MPFROM2SHORT(usItem, FALSE),
                                        (MPARAM)MIA_CHECKED);
            _xfSetStatusBarVisibility(somSelf,
                        (ulMenuAttr & MIA_CHECKED)
                            ? STATUSBAR_OFF
                            : STATUSBAR_ON);
            // toggle "checked" flag in menu
            ulMenuAttr ^= MIA_CHECKED;  // XOR checked flag;
            WinSendMsg(hwndMenu, MM_SETITEMATTR,
                                        MPFROM2SHORT(usItem, FALSE),
                                        MPFROM2SHORT(MIA_CHECKED, ulMenuAttr));

            // do not dismiss menu
            brc = FALSE;
        break; }

        // "Always sort"
        case ID_XFMI_OFS_ALWAYSSORT: {
            XFolderData         *somThis = XFolderGetData(somSelf);
            BOOL                fAlwaysSort = ALWAYS_SORT;
            _xfQueryFldrSort(somSelf, &usDefaultSort, &usAlwaysSort);
            _xfSetFldrSort(somSelf, usDefaultSort,
                        !fAlwaysSort);

            WinSetMenuItemChecked(hwndMenu,
                        usItem,
                        !fAlwaysSort);

            // do not dismiss menu
            brc = FALSE;
        break; }

        default:
            brc =  XFolder_parent_XFldFileSystem_xfSelectingMenuItem(somSelf,
                                                      usItem,
                                                      fPostCommand,
                                                      hwndMenu,
                                                      hwndCnr,
                                                      ulSelection);
    }
    return (brc);
}

/*
 * wpMenuItemSelected:
 *           process input when any menu item was selected;
 *           pass input to common XFolder function
 */

SOM_Scope BOOL  SOMLINK xf_wpMenuItemSelected(XFolder *somSelf,
                                                 HWND hwndFrame,
                                                 ULONG ulMenuId)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpMenuItemSelected");

    if (cmnMenuItemSelected(somSelf, hwndFrame, ulMenuId))
        return (TRUE);
    else
        return (XFolder_parent_WPFolder_wpMenuItemSelected(somSelf, hwndFrame, ulMenuId));
}

/*
 * cmnMenuItemHelpSelected:
 *           display help for a context menu item; this routine is
 *           shared by all XFolder classes also, so you'll find
 *           XFldDesktop items in here too
 */

BOOL cmnMenuItemHelpSelected(XFolder *somSelf, ULONG MenuId)
{
    ULONG   ulFirstVarMenuId;
    ULONG   ulPanel = 0;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    if ((MenuId == WPMENUID_SHUTDOWN) && (pGlobalSettings->XShutdown & XSD_ENABLED))
        ulPanel = ID_XMH_XSHUTDOWN;
    else if (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_RESTARTWPS)
        ulPanel = ID_XMH_RESTARTWPS;
    else if (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SNAPTOGRID)
        ulPanel = ID_XMH_SNAPTOGRID;
    else  if (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_REFRESH)
        ulPanel = ID_XMH_REFRESH;
    else  if (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SELECTSOME)
        ulPanel = ID_XFH_SELECTSOME;
    else if (   (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTBYEXT)
             || (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTBYCLASS)
             || (MenuId == pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SORTFOLDERSFIRST)
             || (   (pGlobalSettings->ReplaceSort)
                 && (   (MenuId == ID_WPMI_SORTBYNAME)
                     || (MenuId == ID_WPMI_SORTBYREALNAME)
                     || (MenuId == ID_WPMI_SORTBYTYPE)
                     || (MenuId == ID_WPMI_SORTBYSIZE)
                     || (MenuId == ID_WPMI_SORTBYWRITEDATE)
                     || (MenuId == ID_WPMI_SORTBYACCESSDATE)
                     || (MenuId == ID_WPMI_SORTBYCREATIONDATE)
                    )
                )
            )
        ulPanel = ID_XSH_SETTINGS_FLDRSORT;
    else {
        // if F1 was pressed over one of the variable menu items,
        // open a help panel with generic help on XFolder */
        ulFirstVarMenuId = (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_VARIABLE);
        if ( (MenuId >= ulFirstVarMenuId)
                && (MenuId < ulFirstVarMenuId + ulVarItemCount)
             )
        {
            PVARMENULISTITEM pItem = (PVARMENULISTITEM)lstItemFromIndex(
                        (PLISTITEM)pliVarItems, (MenuId - ulFirstVarMenuId), 0, 0);

            if (pItem) {
                // OK, we've found the corresponding object
                switch (pItem->usObjType) {
                    // this data has previously been saved by cmnFillMenuWithObjects when
                    // the context menu was created; it contains a flag telling us
                    // what kind of menu item we're dealing with

                    case OC_CONTENTFOLDER:
                    case OC_CONTENT:
                        ulPanel = ID_XMH_FOLDERCONTENT;
                    break;

                    default:
                        ulPanel = ID_XMH_VARIABLE;
                    break;
                }
            }
        }
    }

    if (ulPanel)
    {
        PSZ pszHelpLibrary;
        BOOL fProcessed = FALSE;
        // now open the help panel we've set above
        if (pszHelpLibrary = cmnQueryHelpLibrary())
            // path found: display help panel
            if (_wpDisplayHelp(somSelf, ulPanel, pszHelpLibrary))
                fProcessed = TRUE;

        if (!fProcessed)
            cmnMessageBoxMsg(HWND_DESKTOP, 104, 134, MB_OK);
        return TRUE;
    } else // none of our items: pass on to parent
        return FALSE;
}

/*
 * wpMenuItemHelpSelected:
 *           display help for a context menu item
 */

SOM_Scope BOOL  SOMLINK xf_wpMenuItemHelpSelected(XFolder *somSelf,
                                                     ULONG MenuId)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpMenuItemHelpSelected");

    if (cmnMenuItemHelpSelected(somSelf, MenuId))
        return TRUE;
    else
        return (XFolder_parent_WPFolder_wpMenuItemHelpSelected(somSelf,
                                                               MenuId));
}

/*
 * wpOpen:
 *           we call the parent first and then subclass the
 *           resulting frame window and change the window title
 */

SOM_Scope HWND  SOMLINK xf_wpOpen(XFolder *somSelf, HWND hwndCnr,
                                     ULONG ulView, ULONG param)
{
    HWND                hwndNewFrame;
    PSUBCLASSEDLISTITEM    psli;
    // LONG                lBgnColor = SYSCLR_DIALOGBACKGROUND;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpOpen");

    // have parent do the window creation
    hwndNewFrame = XFolder_parent_WPFolder_wpOpen(somSelf, hwndCnr, ulView, param);

    // subclass the new folder frame window
    psli = cmnSubclassFrameWnd(hwndNewFrame, somSelf, ulView);

    // change the window title to full path, if allowed
    if (    (_FullPath == 1)
         || ((_FullPath == 2) && (pGlobalSettings->FullPath))
       )
        cmnSetOneFrameWndTitle(somSelf, hwndNewFrame);

    // add status bar, if allowed:
    // 1) status bar only if allowed for the current folder
    if (    (_StatusBar == STATUSBAR_ON)
         || (   (_StatusBar == STATUSBAR_DEFAULT)
             && (pGlobalSettings->StatusBar)
            )
       )
        // 2) no status bar for active Desktop
        if (somSelf != _wpclsQueryActiveDesktop(_WPDesktop))
            // 3) check that subclassed list item is valid
            if (psli)
                // 4) status bar only if allowed for the current view type
                if (    (   (ulView == OPEN_CONTENTS)
                         && (pGlobalSettings->SBForViews & SBV_ICON)
                        )
                     || (   (ulView == OPEN_TREE)
                         && (pGlobalSettings->SBForViews & SBV_TREE)
                        )
                     || (   (ulView == OPEN_DETAILS)
                         && (pGlobalSettings->SBForViews & SBV_DETAILS)
                        )
                    )
                    _xfShowStatusBar(somSelf, hwndNewFrame, psli, TRUE);

    // replace sort stuff
    if (    (pGlobalSettings->ReplaceSort)
         && (   (ulView == OPEN_CONTENTS)
             || (ulView == OPEN_TREE)
             || (ulView == OPEN_DETAILS)
            )
       )
    {
        hwndCnr = WinQueryCnrFromFrame(hwndNewFrame);
        cmnSetCnrSort(somSelf, hwndCnr);
        // let Worker thread reset the cnr sort, because the WPS messes
        // with it somewhere
        cmnPostWorkerMsg(WOM_CORRECTCNRSORT, (MPARAM)somSelf, (MPARAM)hwndCnr);
    }

    return (hwndNewFrame);
}

/*
 * wpRefresh:
 *           this method updates a folder; after doing
 *           this, we will also update the title of the
 *           window
 */

SOM_Scope BOOL  SOMLINK xf_wpRefresh(XFolder *somSelf, ULONG ulView,
                                        PVOID pReserved)
{
    BOOL        rc;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpRefresh");

    rc = XFolder_parent_WPFolder_wpRefresh(somSelf, ulView, pReserved);

    _xfForEachOpenView(somSelf,
                (ULONG)2,           // update
                (PFNWP)fncbUpdateStatusBars);

    cmnPostWorkerMsg(WOM_REFRESHFOLDERVIEWS, (MPARAM)somSelf, MPNULL);

    return rc;
}

/*
 * wpAddToContent:
 *           this method should be overridden to intercept the
 *           notification of the "Added an object to a folder"
 *           event for subclasses that define their own folder view.
 *           The parent must always be called.
 */

SOM_Scope BOOL  SOMLINK xf_wpAddToContent(XFolder *somSelf,
                                             WPObject* Object)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpAddToContent");

    #ifdef DEBUG_CNRCONTENT
         _Pmpf(("wpAddToContent, folder: %s, object: %s",
             _wpQueryTitle(somSelf),
             _wpQueryTitle(Object)));
    #endif

    return (XFolder_parent_WPFolder_wpAddToContent(somSelf, Object));
}

/*
 * wpDeleteFromContent:
 *           this method should be overridden to intercept the
 *           notification of the "Removed an object from a folder"
 *           event for subclasses that define their own folder view.
 *           The parent must always be called.
 */

SOM_Scope BOOL  SOMLINK xf_wpDeleteFromContent(XFolder *somSelf,
                                                  WPObject* Object)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpDeleteFromContent");

    #ifdef DEBUG_CNRCONTENT
         _Pmpf(("wpDeleteFromContent, folder: %s, object: %s",
             _wpQueryTitle(somSelf),
             _wpQueryTitle(Object)));
    #endif

    return (XFolder_parent_WPFolder_wpDeleteFromContent(somSelf,
                                                        Object));
}

/*
 * wpStoreIconPosData:
 *           this method is documented only for Warp 4
 *           (but exists in Warp 3 also, see WPFOLDER.H);
 *           it is called when an open folder in icon or
 *           details view is closed.
 *           The WPS apparently then saves the .ICONPOS
 *           data to disk; we only override this method to be
 *           notified that we need to to update our
 *           icon order data (xfUpdateOrderedContent).
 */

SOM_Scope BOOL  SOMLINK xf_wpStoreIconPosData(XFolder *somSelf,
                                                 PICONPOS pIconPos,
                                                 ULONG cbSize)
{
    BOOL rc;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpStoreIconPosData");

    rc =  (XFolder_parent_WPFolder_wpStoreIconPosData(somSelf,
                                                       pIconPos,
                                                       cbSize));

    #ifdef DEBUG_SORT
        _Pmpf(("wpStoreIconPosData -- pIconPos: 0x%lX, ulSize: 0x%lX",
            pIconPos, cbSize));
    #endif
    cmnPostWorkerMsg(WOM_INVALIDATEORDEREDCONTENT,
        (MPARAM)somSelf, MPNULL);

    return (rc);
}

/*
 * wpMoveObject:
 *           this is called when the folder is moved to a
 *           different location; we then need to update
 *           the titles of this folder AND of possibly open
 *           subfolders with the full path; we pass this
 *           to the Worker thread
 */

SOM_Scope BOOL  SOMLINK xf_wpMoveObject(XFolder *somSelf,
                                           WPFolder* Folder)
{
    BOOL rc;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpMoveObject");

    /* call the parent method first, which will actually move the
       folder */
    rc = XFolder_parent_WPFolder_wpMoveObject(somSelf, Folder);

    cmnPostWorkerMsg(WOM_REFRESHFOLDERVIEWS, (MPARAM)somSelf, MPNULL);

    return rc;
}

/*
 * wpSetTitle:
 *           this is called when the folder is renamed;
 *           we then need to update the titles of this
 *           folder AND of possibly open subfolders with
 *           the full path; we pass this task to the Worker
 *           thread, since it may take a while
 */

SOM_Scope BOOL  SOMLINK xf_wpSetTitle(XFolder *somSelf, PSZ pszNewTitle)
{
    BOOL rc;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpSetTitle");

    rc = XFolder_parent_WPFolder_wpSetTitle(somSelf, pszNewTitle);

    if (_wpFindUseItem(somSelf, USAGE_OPENVIEW, NULL))
        cmnPostWorkerMsg(WOM_REFRESHFOLDERVIEWS, (MPARAM)somSelf, MPNULL);

    return (rc);
}

/*
 * wpDrop:
 *           this instance method is called to
 *           inform an object that another object
 *           has been dropped on it
 */

SOM_Scope MRESULT  SOMLINK xf_wpDrop(XFolder *somSelf, HWND hwndCnr,
                                        PDRAGINFO pdrgInfo, PDRAGITEM pdrgItem)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpDrop");

    #ifdef DEBUG_DRAGNDROP
    {
        PSZ         pszTitle;
        CHAR        szBuffer[1000] = "Error";
        PDRAGITEM   pItem2;
        ULONG   i;

        switch (pdrgInfo->usOperation) {
            case DO_DEFAULT: pszTitle = "Default action:"; break;
            case DO_COPY:    pszTitle = "Copy:"          ; break;
            case DO_LINK:    pszTitle = "Link:"          ; break;
            case DO_MOVE:    pszTitle = "Move:"          ; break;
            case DO_UNKNOWN: pszTitle = "Unknown action:"; break;
            default:         pszTitle = "Unknown code:"  ; break;
        }

        for (i = 0; i < pdrgInfo->cditem; i++)
        {
            pItem2 = DrgQueryDragitemPtr(pdrgInfo, i);
            if (DrgVerifyRMF(pItem2, "DRM_OS2FILE", NULL))
            {
                strcpy(szBuffer, "File:          ");
                // Get the path
                DrgQueryStrName(pItem2->hstrContainerName,
                    sizeof(szBuffer)-strlen(szBuffer),
                    &(szBuffer[strlen(szBuffer)]));

                // Append the name of the object to the path
                DrgQueryStrName(pItem2->hstrSourceName,
                    sizeof(szBuffer)-strlen(szBuffer),
                    &(szBuffer[strlen(szBuffer)]));
            }
            DebugBox(pszTitle, szBuffer);
        }
    }
    #endif
    return (XFolder_parent_WPFolder_wpDrop(somSelf, hwndCnr,
                                           pdrgInfo, pdrgItem));
}

/*
 * wpQueryFldrSortClass:
 *           this instance method returns the sort class object
 *           whose details are currently being used as sort criteria
 *           for a given folder. Normally, this method returns
 *           WPFileSystem per default; we will set this to
 *           WPObject, which has less sort criteria,
 *           to speed up processing
 */

SOM_Scope M_WPObject*  SOMLINK xf_wpQueryFldrSortClass(XFolder *somSelf)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpQueryFldrSortClass");

    if (pGlobalSettings->ReplaceSort)
        return (_WPFileSystem);
    else
        return (XFolder_parent_WPFolder_wpQueryFldrSortClass(somSelf));
}

/*
 * wpQueryDefaultHelp:
 *           this instance method specifies the default
 *           help panel for this instance; XFolder will
 *           return something different for the
 *           Config folder and its subfolders
 */

SOM_Scope BOOL  SOMLINK xf_wpQueryDefaultHelp(XFolder *somSelf,
                                                 PULONG pHelpPanelId,
                                                 PSZ HelpLibrary)
{
    BOOL        rc;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpQueryDefaultHelp");

    if (IS_CONFIG_FOLDER(somSelf)) {
        strncpy(HelpLibrary, cmnQueryHelpLibrary(), CCHMAXPATH);
        *pHelpPanelId = ID_XMH_CONFIGFOLDER;
        rc = TRUE;
    } else if (  (somSelf == _wpclsQueryFolder(_WPFolder, XFOLDER_STARTUPID, TRUE))
              || (somSelf == _wpclsQueryFolder(_WPFolder, XFOLDER_SHUTDOWNID, TRUE))
              )
    {
        strncpy(HelpLibrary, cmnQueryHelpLibrary(), CCHMAXPATH);
        *pHelpPanelId = ID_XMH_STARTUPSHUTDOWN;
        rc = TRUE;
    } else
        rc = (XFolder_parent_WPFolder_wpQueryDefaultHelp(somSelf,
                                                           pHelpPanelId,
                                                           HelpLibrary));

    return (rc);
}

/*
 * wpInitData:
 *           here XFolder will initialize its instance data
 */

SOM_Scope void  SOMLINK xf_wpInitData(XFolder *somSelf)
{
    // PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpInitData");

    XFolder_parent_WPFolder_wpInitData(somSelf);

    _pliOrderedContentFirst = NULL;
    _pliOrderedContentLast = NULL;
    _hmtxOrderedContent = NULLHANDLE;
    _pICONPOS = NULL;
    _usICONPOSSize = 0;
    _SnapToGrid = 2;
    _FullPath = 2;
    _Accelerators = 2;
    _StatusBar = STATUSBAR_DEFAULT;
    _AlwaysSort = SET_DEFAULT;
    _DefaultSort = SET_DEFAULT;
    _ulSBInflatedFrame = 0;
    _fUnInitCalled = FALSE;

    _pFdrSortInfo = NULL;

}

/*
 * wpUnInitData:
 *           clean up when object is freed / made dormant
 */

SOM_Scope void  SOMLINK xf_wpUnInitData(XFolder *somSelf)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpUnInitData");

    if (!_fUnInitCalled) {

        if (_pliOrderedContentFirst)
            lstClear(_pliOrderedContentFirst, _pliOrderedContentLast, _hmtxOrderedContent, 4000);
        if (_hmtxOrderedContent)
            DosCloseMutexSem(_hmtxOrderedContent);

        if (_pICONPOS) {
            _wpFreeMem(somSelf, _pICONPOS);
            _pICONPOS = NULL;
        }

        XFolder_parent_WPFolder_wpUnInitData(somSelf);

        _fUnInitCalled = TRUE;
    }
}

/*
 * wpFree:
 *           this WPObject method destroys the persistent form of the object
 *           and  then frees the memory that represented that object. For
 *           WPFolders, this is called when a folder is actually to be
 *           deleted. We will call the parent method and then also remove
 *           those darn PMWorkplace:FolderPos entries which the WPS never
 *           deletes
 */

SOM_Scope BOOL  SOMLINK xf_wpFree(XFolder *somSelf)
{
    BOOL brc;
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpFree");

    brc = XFolder_parent_WPFolder_wpFree(somSelf);
    return (brc);
}

/* SOM_Scope void  SOMLINK xf_wpUnInitData(XFolder *somSelf)
{
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpUnInitData");

    // according to WPS docs, the parent method should be called
    // AFTER additional processing; probably somSelf becomes invalid
    // after this
    XFolder_parent_WPFolder_wpUnInitData(somSelf);
} */

/*
 * wpFree:
 *           this WPObject method destroys the persistent form of the object
 *           and  then frees the memory that represented that object. For
 *           WPFolders, this is called when a folder is actually to be
 *           deleted. We will call the parent method and then also remove
 *           those darn PMWorkplace:FolderPos entries which the WPS never
 *           deletes
 */

/* SOM_Scope BOOL  SOMLINK xf_wpFree(XFolder *somSelf)
{
    DosBeep(6000, 30);
    {
        PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
        // XFolderData *somThis = XFolderGetData(somSelf);
        XFolderMethodDebug("XFolder","xf_wpFree");

        if (pGlobalSettings->CleanupINIs) {

            {
                CHAR szKey[50];
                HOBJECT hObj = _wpQueryHandle(somSelf);
                // according to Henk Kelder:
                // for each open folder the WPS creates an INI entry, which
                // key is the decimal value of the HOBJECT, followed by
                // some wicked "@" codes, which probably stand for the
                // various folder views

                sprintf(szKey, "%d@", hObj);
                PrfWriteProfileData(HINI_USER, WPINIKEY_FOLDERPOS, szKey,
                        NULL, 0);

                sprintf(szKey, "%d@10", hObj);
                PrfWriteProfileData(HINI_USER, WPINIKEY_FOLDERPOS, szKey,
                        NULL, 0);

                sprintf(szKey, "%d@20", hObj);
                PrfWriteProfileData(HINI_USER, WPINIKEY_FOLDERPOS, szKey,
                        NULL, 0);

                sprintf(szKey, "%d@1010", hObj);
                PrfWriteProfileData(HINI_USER, WPINIKEY_FOLDERPOS, szKey,
                        NULL, 0);

                sprintf(szKey, "%d@1020", hObj);
                PrfWriteProfileData(HINI_USER, WPINIKEY_FOLDERPOS, szKey,
                        NULL, 0);
            }
        }

        // the parent must be called LAST because somSelf is
        // undefined afterwards
        return (XFolder_parent_WPFolder_wpFree(somSelf));
    }
    DosBeep(5000, 30);
}
   */

/*
 * wpAddSettingsPages:
 *           call xfAddXFolderPages
 */

SOM_Scope BOOL  SOMLINK xf_wpAddSettingsPages(XFolder *somSelf,
                                                 HWND hwndNotebook)
{
    BOOL            rc;
    // PAGEINFO        pi;

    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpAddSettingsPages");

    rc = (XFolder_parent_WPFolder_wpAddSettingsPages(somSelf,
                                                       hwndNotebook));

    _xfAddXFolderPages(somSelf, hwndNotebook);

    return (rc);
}

/*
 * wpAddFolderSortPage:
 *           this normally adds the "Sort" page to the folder
 *           settings notebook; if allowed, we will replace this
 *           by the new XFolder version of it
 */

SOM_Scope ULONG  SOMLINK xf_wpAddFolderSortPage(XFolder *somSelf,
                                                   HWND hwndNotebook)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpAddFolderSortPage");

    if (pGlobalSettings->ReplaceSort) {
        PAGEINFO        pi;
        PSZ pszHelpLibrary = cmnQueryHelpLibrary();

        // insert "Sort" settings page
        memset((PCH)&pi, 0, sizeof(PAGEINFO));
        pi.cb                  = sizeof(PAGEINFO);
        pi.hwndPage            = NULLHANDLE;
        pi.pfnwp               = fnwpSettingsDlg_FldrSort;
        pi.resid               = NLS_MODULE;
        pi.pCreateParams       = somSelf;
        pi.dlgid               = ID_XSD_SETTINGS_FLDRSORT;
        pi.usPageStyleFlags    = BKA_STATUSTEXTON | BKA_MAJOR;   // major tab;
        pi.usPageInsertFlags   = BKA_FIRST;
        pi.usSettingsFlags     = 0; // don't enumerate in status line
        pi.pszName             = pNLSStrings->pszSort;

        pi.pszHelpLibraryName  = pszHelpLibrary;
        pi.idDefaultHelpPanel  = ID_XSH_SETTINGS_FLDRSORT;

        _wpInsertSettingsPage(somSelf, hwndNotebook, &pi);

        return (SETTINGS_PAGE_REMOVED);
    }
    else
        return (XFolder_parent_WPFolder_wpAddFolderSortPage(somSelf,
                                                        hwndNotebook));
}

/*
 * wpSetup:
 *           this instance method is called to allow a
 *           newly created object to initialize itself.
 *           XFolder will examine its setup strings here
 */

SOM_Scope BOOL  SOMLINK xf_wpSetup(XFolder *somSelf, PSZ pszSetupString)
{
    BOOL        rc;
    CHAR        szValue[CCHMAXPATH+1];
    ULONG       ulSize;
    USHORT      usDefaultSort, usAlwaysSort;

    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpSetup");

    rc = (XFolder_parent_WPFolder_wpSetup(somSelf, pszSetupString));

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "SNAPTOGRID", szValue, &ulSize))
    {
        if (strnicmp(szValue, "NO", 2) == 0)
            _SnapToGrid = 0;
        else if (strnicmp(szValue, "YES", 3) == 0)
            _SnapToGrid = 1;
        else if (strnicmp(szValue, "DEFAULT", 7) == 0)
            _SnapToGrid = 2;
        else if (strnicmp(szValue, "EXEC", 4) == 0)
            _xfSnapToGrid(somSelf, FALSE);
    }

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "FULLPATH", szValue, &ulSize))
    {
        if (strnicmp(szValue, "NO", 2) == 0)
            _FullPath = 0;
        else if (strnicmp(szValue, "YES", 3) == 0)
            _FullPath = 1;
        else if (strnicmp(szValue, "DEFAULT", 7) == 0)
            _FullPath = 2;

        _xfUpdateAllFrameWndTitles(somSelf);
    }

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "ACCELERATORS", szValue, &ulSize))
    {
        if (strnicmp(szValue, "NO", 2) == 0)
            _Accelerators = 0;
        else if (strnicmp(szValue, "YES", 3) == 0)
            _Accelerators = 1;
        else if (strnicmp(szValue, "DEFAULT", 7) == 0)
            _Accelerators = 2;
    }

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "FAVORITEFOLDER", szValue, &ulSize))
    {
        if (strnicmp(szValue, "NO", 2) == 0)
            _xfMakeFavoriteFolder(somSelf, FALSE);
        else if (strnicmp(szValue, "YES", 3) == 0)
            _xfMakeFavoriteFolder(somSelf, TRUE);
    }

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "QUICKOPEN", szValue, &ulSize))
    {
        if (strnicmp(szValue, "NO", 2) == 0)
            _xfSetQuickOpen(somSelf, FALSE);
        else if (strnicmp(szValue, "YES", 3) == 0)
            _xfSetQuickOpen(somSelf, TRUE);
    }

    if (somSelf != _wpclsQueryActiveDesktop(_WPDesktop))
    {
        ulSize = sizeof(szValue);
        if (_wpScanSetupString(somSelf, pszSetupString,
                    "STATUSBAR", szValue, &ulSize))
        {
            if (strnicmp(szValue, "NO", 2) == 0)
                _StatusBar = STATUSBAR_OFF;
            else if (strnicmp(szValue, "YES", 3) == 0)
                _StatusBar = STATUSBAR_ON;
            else if (strnicmp(szValue, "DEFAULT", 7) == 0)
                _StatusBar = STATUSBAR_DEFAULT;
        }
        cmnPostWorkerMsg(WOM_UPDATEALLSTATUSBARS,
                (MPARAM)1,  // show/hide flag
                MPNULL);
    }

    _xfQueryFldrSort(somSelf, &usDefaultSort, &usAlwaysSort);

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "ALWAYSSORT", szValue, &ulSize))
    {
        if (strnicmp(szValue, "NO", 2) == 0)
            usAlwaysSort = 0;
        else if (strnicmp(szValue, "YES", 3) == 0)
            usAlwaysSort = 1;
        else if (strnicmp(szValue, "DEFAULT", 7) == 0)
            usAlwaysSort = SET_DEFAULT;
        _xfSetFldrSort(somSelf, usDefaultSort, usAlwaysSort);
    }

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "DEFAULTSORT", szValue, &ulSize))
    {
        LONG lValue;
        sscanf(szValue, "%d", &lValue);
        if ( (lValue >=0) && (lValue <= SV_LAST) )
            usDefaultSort = lValue;
        else
            usDefaultSort = SET_DEFAULT;
        _xfSetFldrSort(somSelf, usDefaultSort, usAlwaysSort);
    }

    ulSize = sizeof(szValue);
    if (_wpScanSetupString(somSelf, pszSetupString,
                "SORTNOW", szValue, &ulSize))
    {
        LONG lValue;
        sscanf(szValue, "%d", &lValue);
        if ( (lValue >=0) && (lValue <= SV_LAST) )
            usDefaultSort = lValue;
        else
            usDefaultSort = SET_DEFAULT;
        _xfclsForEachOpenView(_XFolder, 0, &fncbSortAllViews);
    }

    _wpSaveDeferred(somSelf);
    return (rc);
}

/*
 * wpSaveState:
 *           this instance method is called to allow an
 *           an object to save its state; XFolder will
 *           save its instance settings (NB page) here
 */

SOM_Scope BOOL  SOMLINK xf_wpSaveState(XFolder *somSelf)
{
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpSaveState");

    if (_SnapToGrid != 2)
        _wpSaveLong(somSelf, "XFolder", 1, (ULONG)_SnapToGrid);
    if (_FullPath != 2)
        _wpSaveLong(somSelf, "XFolder", 2, (ULONG)_FullPath);
    if (_Accelerators != 2)
        _wpSaveLong(somSelf, "XFolder", 3, (ULONG)_Accelerators);
    if (_StatusBar != STATUSBAR_DEFAULT)
        _wpSaveLong(somSelf, "XFolder", 4, (ULONG)_StatusBar);
    if (_ulSBInflatedFrame)
        _wpSaveLong(somSelf, "XFolder", 5, (ULONG)_ulSBInflatedFrame);
    if (_AlwaysSort != SET_DEFAULT)
        _wpSaveLong(somSelf, "XFolder", 6, (ULONG)_AlwaysSort);
    if (_DefaultSort != SET_DEFAULT)
        _wpSaveLong(somSelf, "XFolder", 7, (ULONG)_DefaultSort);

    return (XFolder_parent_WPFolder_wpSaveState(somSelf));
}

/*
 * wpRestoreState:
 *           this instance method is called to allow an
 *           an object to restore its state; XFolder will
 *           restore its instance settings (NB page) here
 */

SOM_Scope BOOL  SOMLINK xf_wpRestoreState(XFolder *somSelf,
                                             ULONG ulReserved)
{
    ULONG   ul;
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpRestoreState");

    if (_wpRestoreLong(somSelf, "XFolder", 1, &ul))
        _SnapToGrid = (BYTE)ul;
    else _SnapToGrid = 2;

    if (_wpRestoreLong(somSelf, "XFolder", 2, &ul))
        _FullPath = (BYTE)ul;
    else _FullPath = 2;

    if (_wpRestoreLong(somSelf, "XFolder", 3, &ul))
        _Accelerators = (BYTE)ul;
    else _Accelerators = 2;

    if (somSelf != _wpclsQueryActiveDesktop(_WPDesktop))
        if (_wpRestoreLong(somSelf, "XFolder", 4, &ul))
            _StatusBar = (BYTE)ul;
        // not found: set to default
        else _StatusBar = STATUSBAR_DEFAULT;
    else
        // always disable status bars for Desktop
        _StatusBar = STATUSBAR_OFF;

    if (_wpRestoreLong(somSelf, "XFolder", 5, &ul))
        _ulSBInflatedFrame = (BYTE)ul;
    else _ulSBInflatedFrame = 0;

    if (_wpRestoreLong(somSelf, "XFolder", 6, &ul))
        _AlwaysSort = (USHORT)ul;

    if (_wpRestoreLong(somSelf, "XFolder", 7, &ul))
        _DefaultSort = (USHORT)ul;

    // _xfSetFldrSort(somSelf, usDefaultSort, usAlwaysSort);

    return (XFolder_parent_WPFolder_wpRestoreState(somSelf, ulReserved));
}

/*
 * wpSaveData: this instance method saves binary
 *           instance data to the folder EAs
 */

SOM_Scope BOOL  SOMLINK xf_wpSaveData(XFolder *somSelf, PSZ pszClass,
                                         ULONG ulKey, PBYTE pValue,
                                         ULONG cbValue)
{
    /*
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    // XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpSaveData");

    // if the sort methods are replaced, we modify the WPS sort
    // info, because otherwise the WPS keeps messing with the
    // cnr data
    if (    (ulKey == IDKEY_FDRSORTINFO)
         && (pGlobalSettings->ReplaceSort)
       )
    {
    }  */

    // have data written by parent method
    return (XFolder_parent_WPFolder_wpSaveData(somSelf, pszClass,
                                               ulKey, pValue,
                                               cbValue));
}

/*
 * wpRestoreData:
 *           this instance method restores binary instance
 *           data from the folder EAs upon object awakeing;
 *           we check for whether the WPS
 *           tries to restore the folder sort settings and
 *           suppress these, because otherwise the WPS will
 *           keep messing with our own sort functions
 */

SOM_Scope BOOL  SOMLINK xf_wpRestoreData(XFolder *somSelf,
                                            PSZ pszClass, ULONG ulKey,
                                            PBYTE pValue, PULONG pcbValue)
{
    BOOL        brc;
    ULONG       cbOrigValue = *pcbValue;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    XFolderData *somThis = XFolderGetData(somSelf);
    XFolderMethodDebug("XFolder","xf_wpRestoreData");

    /* #ifdef DEBUG_SORT
        _Pmpf(( "wpRestoreData, ulKey: %lX (%s ))", ulKey,
            (ulKey == IDKEY_FDRSORTATTRIBS) ? "IDKEY_FDRSORTATTRIBS"
            : (ulKey == IDKEY_FDRSORTINFO) ? "IDKEY_FDRSORTINFO"
            : "unknown");
    #endif */

    // always call parent, even for the sort data, or
    // the WPFolder original gets confused
    brc = XFolder_parent_WPFolder_wpRestoreData(somSelf, pszClass,
                                              ulKey, pValue,
                                              pcbValue);

    // now check if WPFolder was trying to restore the
    // folder sort info structure, which has the
    // IDKEY_FDRSORTINFO key; if so, we check some more
    if (ulKey == IDKEY_FDRSORTINFO)
    {
        // the pointer given to this method (pValue) must be
        // the pointer to the WPFolder-internal FDRSORTINFO
        // structure (undocumented, I've declared it in
        // common.h); we store this pointer in the instance
        // data so that we can manipulate it later
        if (cbOrigValue == sizeof(FDRSORTINFO))
        {
            _pFdrSortInfo = pValue;

            if (brc)
                // if the parent method has found sort data,
                // we need to update our own structure, but
                // only update this if this hasn't been set
                // by wpRestoreState yet; as a result, the
                // XFolder sort settings will follow the WPFolder
                // sort settings, but can be overridden
                if (_AlwaysSort == SET_DEFAULT) // not set yet?
                    _AlwaysSort = ((PFDRSORTINFO)pValue)->fAlwaysSort;
        }
    }
    return (brc);
}

/********************************************************************
 *                                                                  *
 *   here come the XFolder class methods                            *
 *                                                                  *
 ********************************************************************/

/*
 * xfclsForEachOpenView:
 *           this class method goes through all open folder windows and calls
 *           pfnwpCallback for each open view of each open folder; the following
 *           params are passed to pfnwpCallback:
 *                hwnd    HWND       hwndView   the hwnd of the view frame window;
 *                mp1     ULONG      ulView     the view type (as def'd in wpOpen)
 *                mp2     XFolder*   pFolder    the currently open folder.
 *           This method does not return until all views have been processed.
 *           You might want to call this method in a different thread if the task
 *           will take long.
 */

SOM_Scope BOOL  SOMLINK xfM_xfclsForEachOpenView(M_XFolder *somSelf,
                                                    ULONG ulMsg,
                                                    PFNWP pfnwpCallback)
{
    XFolder     *pFolder;
    // M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_xfclsForEachOpenView");

    for ( pFolder = _wpclsQueryOpenFolders(somSelf, NULL, QC_FIRST, FALSE);
          pFolder;
          pFolder = _wpclsQueryOpenFolders(somSelf, pFolder, QC_NEXT, FALSE))
    {
        if (_somIsA(pFolder, _WPFolder))
        {
            _xfForEachOpenView(pFolder, ulMsg, pfnwpCallback);
        }
    }
    return TRUE;
}

/*
 * xfclsQueryFavoriteFolder:
 *           returns a favorite folder; if pFolder == NULL,
 *           the first favorite folder is returned, otherwise the
 *           favorite folder which comes after pFolder in the favorite
 *           folder list; returns NULL if no more folders are found
 */

SOM_Scope XFolder*  SOMLINK xfM_xfclsQueryFavoriteFolder(M_XFolder *somSelf,
                                                            XFolder* pFolder)
{
    M_XFolderData *somThis = M_XFolderGetData(somSelf);
    PCONTENTMENULISTITEM    pliFavoriteFolders = _pFavoriteFolders,
                            pItem;
    M_XFolderMethodDebug("M_XFolder","xfM_xfclsQueryFavoriteFolder");

    if (pliFavoriteFolders == NULL)
    {
        // if the list of favorite folders has not yet been built
        // in the class data, we will do this now
        CHAR        szFavorites[1000], szDummy[1000];
        PSZ         pszFavorites;
        // ULONG       rc;

        PrfQueryProfileString(HINI_USERPROFILE, INIAPP, INIKEY_FAVORITEFOLDERS,
                    "", &szFavorites, sizeof(szFavorites));
        pszFavorites = szFavorites;
        do {
            HOBJECT          hObject;
            WPFolder         *pFolder2;
            sscanf(pszFavorites, "%lX %s", &hObject, &szDummy);
            pFolder2 = _wpclsQueryObject(_WPFolder, hObject);
            if (pFolder2) {
                if (_somIsA(pFolder2, _WPFolder))
                {
                     PCONTENTMENULISTITEM pNew = malloc(sizeof(CONTENTMENULISTITEM));
                     pNew->pFolder = pFolder2;
                     lstAppendItem((PLISTITEM*)&pliFavoriteFolders, NULL,
                         (PLISTITEM)pNew, 0, 0);
                }
            }
            pszFavorites += 6;
        } while (strlen(pszFavorites) > 0);

        _pFavoriteFolders = pliFavoriteFolders;
    }

    pItem = pliFavoriteFolders;
    if (pFolder) {
        // folder given as param: look for this folder
        // and return the following in the list
        while (pItem)
            if (pItem->pFolder == pFolder) {
                pItem = pItem->pNext;
                break;
            }
            else
                pItem = pItem->pNext;
    } // else: pItem == pliFavoriteFolders

    if (pItem)
        return (pItem->pFolder);
    else
        return (NULL);
}

/*
 * xfclsQueryFavoriteFolder:
 *           returns a folder from the list of folders that have the
 *           QuickOpen feature ON; if pFolder == NULL,
 *           the first folder is returned, otherwise the
 *           folder which comes after pFolder in the QuickOpen
 *           folder list; returns NULL if no more folders are found
 */

/*
 * xfclsQueryQuickOpenFolder:
 *           returns a folder from the list of folders that have the
 *           QuickOpen feature ON; if pFolder == NULL,
 *           the first folder is returned, otherwise the
 *           folder which comes after pFolder in the QuickOpen
 *           folder list; returns NULL if no more folders are found
 */

SOM_Scope XFolder*  SOMLINK xfM_xfclsQueryQuickOpenFolder(M_XFolder *somSelf,
                                                          XFolder* pFolder)
{
    M_XFolderData *somThis = M_XFolderGetData(somSelf);
    PCONTENTMENULISTITEM    pliQuickOpenFolders = _pQuickOpenFolders,
                            pItem;
    M_XFolderMethodDebug("M_XFolder","xfM_xfclsQueryQuickOpenFolder");

    if (pliQuickOpenFolders == NULL)
    {
        // if the list of QuickOpen folders has not yet been built
        // in the class data, we will do this now
        CHAR        szQuickOpen[1000], szDummy[1000];
        PSZ         pszQuickOpen;
        // ULONG       rc;

        PrfQueryProfileString(HINI_USERPROFILE, INIAPP, INIKEY_QUICKOPENFOLDERS,
                    "", &szQuickOpen, sizeof(szQuickOpen));
        pszQuickOpen = szQuickOpen;
        do {
            HOBJECT          hObject;
            WPFolder         *pFolder2;
            sscanf(pszQuickOpen, "%lX %s", &hObject, &szDummy);
            pFolder2 = _wpclsQueryObject(_WPFolder, hObject);
            if (pFolder2) {
                if (_somIsA(pFolder2, _WPFolder))
                {
                     PCONTENTMENULISTITEM pNew = malloc(sizeof(CONTENTMENULISTITEM));
                     pNew->pFolder = pFolder2;
                     lstAppendItem((PLISTITEM*)&pliQuickOpenFolders, NULL,
                         (PLISTITEM)pNew, 0, 0);
                }
            }
            pszQuickOpen += 6;
        } while (strlen(pszQuickOpen) > 0);

        _pQuickOpenFolders = pliQuickOpenFolders;
    }

    pItem = pliQuickOpenFolders;
    if (pFolder) {
        // folder given as param: look for this folder
        // and return the following in the list
        while (pItem)
            if (pItem->pFolder == pFolder) {
                pItem = pItem->pNext;
                break;
            }
            else
                pItem = pItem->pNext;
    } // else: pItem == pliQuickOpenFolders

    if (pItem)
        return (pItem->pFolder);
    else
        return (NULL);
}

/*
 * xfclsSetDefaultTitle:
 *           changes the default title to pszNewTitle
 */

SOM_Scope BOOL SOMLINK xfM_xfclsSetDefaultTitle(M_XFolder *somSelf,
                                                   PSZ pszNewTitle)
{
    M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_xfclsSetDefaultTItle");

    strncpy(_szDefaultTitle, pszNewTitle, sizeof(_szDefaultTitle)-1);
    return (PrfWriteProfileString(HINI_USERPROFILE, INIAPP, INIKEY_DEFAULTTITLE,
                _szDefaultTitle));
}

/*
 * xfclsQuerySortFunc:
 *           this new instance method returns the sort comparison
 *           function for the specified sort method (as with
 *           xfSortView).
 */

SOM_Scope PVOID  SOMLINK xfM_xfclsQuerySortFunc(M_XFolder *somSelf,
                                                   USHORT usSort)
{
    USHORT usSort2 = usSort;
    // M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_xfclsQuerySortFunc");

    if (usSort == SET_DEFAULT) {
        PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
        usSort2 =  pGlobalSettings->DefaultSort;
    }

    switch (usSort2) {
        case SV_TYPE:           return ((PVOID)(PFN)fnCompareType);
        case SV_CLASS:          return ((PVOID)(PFN)fnCompareClass);
        case SV_REALNAME:       return ((PVOID)(PFN)fnCompareRealName);
        case SV_SIZE:           return ((PVOID)(PFN)fnCompareSize);
        case SV_LASTWRITEDATE:  return ((PVOID)(PFN)fnCompareLastWriteDate);
        case SV_LASTACCESSDATE: return ((PVOID)(PFN)fnCompareLastAccessDate);
        case SV_CREATIONDATE:   return ((PVOID)(PFN)fnCompareCreationDate);
        case SV_EXT:            return ((PVOID)(PFN)fnCompareExt);
        case SV_FOLDERSFIRST:   return ((PVOID)(PFN)fnCompareFoldersFirst);
    };

    return ((PVOID)(PFN)fnCompareName);
}

/*
 * wpclsInitData:
 *           this initializes the WPFolder / XFolder class as a whole;
 *           we need to call the parent and then set some XFolder
 *           data
 */

SOM_Scope void  SOMLINK xfM_wpclsInitData(M_XFolder *somSelf)
{
    // CHAR szDummy[100];
    // ULONG ulCopied2;

    M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_wpclsInitData");

    M_XFolder_parent_M_WPFolder_wpclsInitData(somSelf);

    _szStatusBarMnemonics[0] = '\0';

    if (!IsXFolderMInitialized) {
        IsXFolderMInitialized = TRUE;

        _pFavoriteFolders = NULL;
        _pQuickOpenFolders = NULL;

        PrfQueryProfileString(HINI_USERPROFILE, INIAPP, INIKEY_DEFAULTTITLE,
                 "XFolder", &(_szDefaultTitle), sizeof(_szDefaultTitle));

        cmnLoadAccelerators();

        DosCreateMutexSem("\\sem32\\WndProcList",
                            &hmtxSubclassed, 0, FALSE);
    }
}

/*
 * wpclsQueryStyle:
 *           we return a flag so that no more templates are created
 *           for folders; otherwise we would get new templates every
 *           time the user changes the class default title
 */

SOM_Scope ULONG  SOMLINK xfM_wpclsQueryStyle(M_XFolder *somSelf)
{
    // M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_wpclsQueryStyle");

    return (M_XFolder_parent_M_WPFolder_wpclsQueryStyle(somSelf)
            | CLSSTYLE_NEVERTEMPLATE);
}

/*
 * wpclsQueryTitle:
 *           tell the WPS the new class name: XFolder or whatever was
 *           specified on page 5 of the Global Settings
 */

SOM_Scope PSZ  SOMLINK xfM_wpclsQueryTitle(M_XFolder *somSelf)
{
    M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xf_wpclsQueryTitle");

    return (_szDefaultTitle);
}

/*
 * wpclsQueryIconData:
 *           give XFolders a new closed icon
 */

SOM_Scope ULONG  SOMLINK xfM_wpclsQueryIconData(M_XFolder *somSelf,
                                                   PICONINFO pIconInfo)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    // M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_wpclsQueryIconData");

    if (pGlobalSettings->ReplIcons) {
        if (pIconInfo) {
           pIconInfo->fFormat = ICON_RESOURCE;
           pIconInfo->resid   = ID_ICON1;
           pIconInfo->hmod    = _xfclsQueryModuleHandle(_XFldObject);
        }
        return (sizeof(ICONINFO));
    } else
        return (M_XFolder_parent_M_WPFolder_wpclsQueryIconData(somSelf, pIconInfo));
}

/*
 * wpclsQueryIconDataN:
 *           give XFolders a new animated icon
 */

SOM_Scope ULONG  SOMLINK xfM_wpclsQueryIconDataN(M_XFolder *somSelf,
                                                    ICONINFO* pIconInfo,
                                                    ULONG ulIconIndex)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    // M_XFolderData *somThis = M_XFolderGetData(somSelf);
    M_XFolderMethodDebug("M_XFolder","xfM_wpclsQueryIconDataN");

    if (pGlobalSettings->ReplIcons) {
        if (pIconInfo) {
           pIconInfo->fFormat = ICON_RESOURCE;
           pIconInfo->resid   = ID_ICON2;
           pIconInfo->hmod    = _xfclsQueryModuleHandle(_XFldObject);
        }

        return (sizeof(ICONINFO));
    } else
        return (M_XFolder_parent_M_WPFolder_wpclsQueryIconDataN(somSelf, pIconInfo, ulIconIndex));
}

/********************************************************************
 *                                                                  *
 *   here come the XFldStartup methods                              *
 *                                                                  *
 ********************************************************************/

/*
 * wpModifyPopupMenu:
 *           add a "Process content" menu item to this
 *           popup menu; the other menu items are inherited
 *           from XFolder
 */

SOM_Scope BOOL  SOMLINK xfstup_wpModifyPopupMenu(XFldStartup *somSelf,
                                                 HWND hwndMenu,
                                                 HWND hwndCnr,
                                                 ULONG iPosition)
{
    BOOL rc;
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
    /* XFldStartupData *somThis = XFldStartupGetData(somSelf); */
    XFldStartupMethodDebug("XFldStartup","xfstup_wpModifyPopupMenu");

    rc = XFldStartup_parent_XFolder_wpModifyPopupMenu(somSelf,
                                                         hwndMenu,
                                                         hwndCnr,
                                                         iPosition);

    WinInsertMenuSeparator(hwndMenu, MIT_END,
            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_SEPARATOR));

    WinInsertMenuItem(hwndMenu,
            MIT_END,
            (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_PROCESSCONTENT),
            pNLSStrings->pszProcessContent,
            MIS_TEXT, 0);

    return (rc);
}

/*
 * wpMenuItemSelected:
 *           react to selection of "Process content"
 *           menu item
 */

SOM_Scope BOOL  SOMLINK xfstup_wpMenuItemSelected(XFldStartup *somSelf,
                                                  HWND hwndFrame,
                                                  ULONG ulMenuId)
{
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    /* XFldStartupData *somThis = XFldStartupGetData(somSelf); */
    XFldStartupMethodDebug("XFldStartup","xfstup_wpMenuItemSelected");

    if ( (ulMenuId - pGlobalSettings->VarMenuOffset) == ID_XFMI_OFS_PROCESSCONTENT ) {
        if (cmnMessageBoxMsg((hwndFrame) ? hwndFrame : HWND_DESKTOP,
                            116, 138, MB_YESNO | MB_DEFBUTTON2)
                        == MBID_YES) {
                cmnPostObjectMsg(XOM_BEGINSTARTUP, MPNULL, MPNULL);
        }
        return (TRUE);
    } else
        return (XFldStartup_parent_XFolder_wpMenuItemSelected(somSelf,
                                                          hwndFrame,
                                                          ulMenuId));
}

/*
 * wpMenuItemHelpSelected:
 *           display help for "Process content"
 *           menu item
 */

SOM_Scope BOOL  SOMLINK xfstup_wpMenuItemHelpSelected(XFldStartup *somSelf,
                                                      ULONG MenuId)
{
    /* XFldStartupData *somThis = XFldStartupGetData(somSelf); */
    XFldStartupMethodDebug("XFldStartup","xfstup_wpMenuItemHelpSelected");

    return (XFldStartup_parent_XFolder_wpMenuItemHelpSelected(somSelf,
                                                              MenuId));
}

/*
 * wpQueryDefaultHelp:
 *           this instance method specifies the default
 *           help panel for this instance; display some
 *           help for the Startup folder
 */

SOM_Scope BOOL  SOMLINK xfstup_wpQueryDefaultHelp(XFldStartup *somSelf,
                                                  PULONG pHelpPanelId,
                                                  PSZ HelpLibrary)
{
    /* XFldStartupData *somThis = XFldStartupGetData(somSelf); */
    XFldStartupMethodDebug("XFldStartup","xfstup_wpQueryDefaultHelp");

    return (XFldStartup_parent_XFolder_wpQueryDefaultHelp(somSelf,
                                                          pHelpPanelId,
                                                          HelpLibrary));
}

/*
 * wpclsQueryTitle:
 *           tell the WPS the new class name: "XFolder Startup"
 */

SOM_Scope PSZ  SOMLINK xfstupM_wpclsQueryTitle(M_XFldStartup *somSelf)
{
    /* M_XFldStartupData *somThis = M_XFldStartupGetData(somSelf); */
    M_XFldStartupMethodDebug("M_XFldStartup","xfstupM_wpclsQueryTitle");

    return ("XFolder Startup");
    // return (M_XFldStartup_parent_M_XFolder_wpclsQueryTitle(somSelf));
}

/*
 * wpclsQueryStyle:
 *           we return a flag so that no templates are created
 *           for the Startup folder
 */

SOM_Scope ULONG  SOMLINK xfstupM_wpclsQueryStyle(M_XFldStartup *somSelf)
{
    /* M_XFldStartupData *somThis = M_XFldStartupGetData(somSelf); */
    M_XFldStartupMethodDebug("M_XFldStartup","xfstupM_wpclsQueryStyle");

    return (M_XFldStartup_parent_M_XFolder_wpclsQueryStyle(somSelf)
                | CLSSTYLE_NEVERTEMPLATE
                | CLSSTYLE_NEVERCOPY
                // | CLSSTYLE_NEVERDELETE
           );

}

/*
 * wpclsQueryIconData:
 *           give the Startup folder a new closed icon
 */

SOM_Scope ULONG  SOMLINK xfstupM_wpclsQueryIconData(M_XFldStartup *somSelf,
                                                    PICONINFO pIconInfo)
{
    /* M_XFldStartupData *somThis = M_XFldStartupGetData(somSelf); */
    M_XFldStartupMethodDebug("M_XFldStartup","xfstupM_wpclsQueryIconData");

    if (pIconInfo) {
       pIconInfo->fFormat = ICON_RESOURCE;
       pIconInfo->resid   = ID_STARTICON1;
       pIconInfo->hmod    = _xfclsQueryModuleHandle(_XFldObject);
    }

    return (sizeof(ICONINFO));

    /* return (M_XFldStartup_parent_M_XFolder_wpclsQueryIconData(somSelf,
                                                              pIconInfo)); */
}

/*
 * wpclsQueryIconDataN:
 *           give the Startup folder a new animated icon
 */

SOM_Scope ULONG  SOMLINK xfstupM_wpclsQueryIconDataN(M_XFldStartup *somSelf,
                                                     ICONINFO* pIconInfo,
                                                     ULONG ulIconIndex)
{
    /* M_XFldStartupData *somThis = M_XFldStartupGetData(somSelf); */
    M_XFldStartupMethodDebug("M_XFldStartup","xfstupM_wpclsQueryIconDataN");

    if (pIconInfo) {
       pIconInfo->fFormat = ICON_RESOURCE;
       pIconInfo->resid   = ID_STARTICON2;
       pIconInfo->hmod    = _xfclsQueryModuleHandle(_XFldObject);
    }

    return (sizeof(ICONINFO));

    /* return (M_XFldStartup_parent_M_XFolder_wpclsQueryIconDataN(somSelf,
                                                               pIconInfo,
                                                               ulIconIndex)); */
}


/********************************************************************
 *                                                                  *
 *   here come the XFldShutdown methods                             *
 *                                                                  *
 ********************************************************************/

/*
 * wpQueryDefaultHelp:
 *           this instance method specifies the default
 *           help panel for this instance; display some
 *           help for the Shutdown folder
 */

SOM_Scope BOOL  SOMLINK xfshut_wpQueryDefaultHelp(XFldShutdown *somSelf,
                                                  PULONG pHelpPanelId,
                                                  PSZ HelpLibrary)
{
    /* XFldShutdownData *somThis = XFldShutdownGetData(somSelf); */
    XFldShutdownMethodDebug("XFldShutdown","xfshut_wpQueryDefaultHelp");

    return (XFldShutdown_parent_XFolder_wpQueryDefaultHelp(somSelf,
                                                           pHelpPanelId,
                                                           HelpLibrary));
}


/*
 * wpclsQueryTitle:
 *           tell the WPS the new class name: "XFolderShutdown"
 */

SOM_Scope PSZ  SOMLINK xfshutM_wpclsQueryTitle(M_XFldShutdown *somSelf)
{
    /* M_XFldShutdownData *somThis = M_XFldShutdownGetData(somSelf); */
    M_XFldShutdownMethodDebug("M_XFldShutdown","xfshutM_wpclsQueryTitle");

    return ("XFolder Shutdown");
    // return (M_XFldShutdown_parent_M_XFolder_wpclsQueryTitle(somSelf));
}

/*
 * wpclsQueryStyle:
 *           we return a flag so that no templates are created
 *           for the Shutdown folder
 */

SOM_Scope ULONG  SOMLINK xfshutM_wpclsQueryStyle(M_XFldShutdown *somSelf)
{
    /* M_XFldShutdownData *somThis = M_XFldShutdownGetData(somSelf); */
    M_XFldShutdownMethodDebug("M_XFldShutdown","xfshutM_wpclsQueryStyle");

    return (M_XFldShutdown_parent_M_XFolder_wpclsQueryStyle(somSelf)
                | CLSSTYLE_NEVERTEMPLATE
                | CLSSTYLE_NEVERCOPY
                // | CLSSTYLE_NEVERDELETE
           );
}

/*
 * wpclsQueryIconData:
 *           give the Shutdown folder a new closed icon
 */

SOM_Scope ULONG  SOMLINK xfshutM_wpclsQueryIconData(M_XFldShutdown *somSelf,
                                                    PICONINFO pIconInfo)
{
    /* M_XFldShutdownData *somThis = M_XFldShutdownGetData(somSelf); */
    M_XFldShutdownMethodDebug("M_XFldShutdown","xfshutM_wpclsQueryIconData");

    if (pIconInfo) {
       pIconInfo->fFormat = ICON_RESOURCE;
       pIconInfo->resid   = ID_SHUTICON1;
       pIconInfo->hmod    = _xfclsQueryModuleHandle(_XFldObject);
    }

    return (sizeof(ICONINFO));

    /* return (M_XFldShutdown_parent_M_XFolder_wpclsQueryIconData(somSelf,
                                                               pIconInfo)); */
}

/*
 * wpclsQueryIconDataN:
 *           give the Shutdown folder a new animated icon
 */

SOM_Scope ULONG  SOMLINK xfshutM_wpclsQueryIconDataN(M_XFldShutdown *somSelf,
                                                     ICONINFO* pIconInfo,
                                                     ULONG ulIconIndex)
{
    /* M_XFldShutdownData *somThis = M_XFldShutdownGetData(somSelf); */
    M_XFldShutdownMethodDebug("M_XFldShutdown","xfshutM_wpclsQueryIconDataN");

    if (pIconInfo) {
       pIconInfo->fFormat = ICON_RESOURCE;
       pIconInfo->resid   = ID_SHUTICON2;
       pIconInfo->hmod    = _xfclsQueryModuleHandle(_XFldObject);
    }

    return (sizeof(ICONINFO));

    /* return (M_XFldShutdown_parent_M_XFolder_wpclsQueryIconDataN(somSelf,
                                                                pIconInfo,
                                                                ulIconIndex)); */
}


/********************************************************************
 *                                                                  *
 *   here come all the XFolder window procedures                    *
 *                                                                  *
 ********************************************************************/

/*
 * fnwpStatusBar:
 *      since the status bar is just created as a static frame
 *      control, it needs to be subclassed (by xfShowStatusBar),
 *      and this is the wnd proc for this
 */

MRESULT EXPENTRY fnwpStatusBar(HWND hwndBar, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    PSTATUSBARDATA psbd = (PSTATUSBARDATA)WinQueryWindowULong(hwndBar, QWL_USER);
    MRESULT        mrc;

    PGLOBALSETTINGS pGlobalSettings =
            cmnQueryGlobalSettings();

    if (psbd)
    {
        PFNWP      pfnwpStatusBarOriginal = psbd->pfnwpStatusBarOriginal;

        switch(msg)
        {
            /*
             * STBM_UPDATESTATUSBAR:
             *      mp1: MPNULL,
             *      mp2: MPNULL
             *      Update status bar text. We will set a timer
             *      for a short delay to filter out repetitive
             *      messages here.
             */

            case STBM_UPDATESTATUSBAR: {
                #ifdef DEBUG_STATUSBARS
                    _Pmpf(( "STBM_UPDATESTATUSBAR received" ));
                #endif
                if (psbd->idTimer == 0) {
                    // timer not yet running: start it
                    psbd->idTimer = WinStartTimer(WinQueryAnchorBlock(hwndBar), hwndBar,
                        1, 100);
                    #ifdef DEBUG_STATUSBARS
                        _Pmpf(( "  Timer started, ID: %lX", psbd->idTimer ));
                        if (psbd->idTimer == 0)
                            _Pmpf(( "  Error %d", WinGetLastError(WinQueryAnchorBlock(hwndBar)) ));
                    #endif
                }
            break; }

            /*
             * WM_TIMER:
             *      timer is started by STBM_UPDATESTATUSBAR to avoid flickering;
             *      we now compose the actual text to be displayed
             */

            case WM_TIMER: {
                CHAR szText[1000];
                // stop timer (it's just for one shot)
                WinStopTimer(WinQueryAnchorBlock(hwndBar), hwndBar, 1);
                psbd->idTimer = 0;

                // if we're not fully populated yet, start timer again and quit;
                // otherwise we would display false information.
                // We need an extra flag in the status bar data because the
                // FOI_POPULATEDWITHALL is reset to 0 by the WPS for some reason
                // when an object is deleted from an open folder, and no further
                // status bar updates would occur then
                if (psbd->fFolderPopulated == FALSE) {
                    ULONG ulFlags = _wpQueryFldrFlags(psbd->somSelf);
                    #ifdef DEBUG_STATUSBARS
                        _Pmpf(( "  Folder flags: %lX", ulFlags ));
                        _Pmpf(( "  View: %s",
                                (psbd->psli->ulView == OPEN_TREE) ? "Tree"
                                : (psbd->psli->ulView == OPEN_CONTENTS) ? "Content"
                                : (psbd->psli->ulView == OPEN_DETAILS) ? "Details"
                                : "unknown"
                             ));
                    #endif

                    // for tree views, check if folder is populated with folders;
                    // otherwise check for populated with all
                    if (    (   (psbd->psli->ulView == OPEN_TREE)
                             && ((ulFlags & FOI_POPULATEDWITHFOLDERS) !=0)
                            )
                        || ((ulFlags & FOI_POPULATEDWITHALL) != 0)
                       )
                    {
                        psbd->fFolderPopulated = TRUE;
                    } else {
                        // folder not yet populated:
                        // restart timer with a lower frequency
                        // to have this checked again
                        psbd->idTimer = WinStartTimer(WinQueryAnchorBlock(hwndBar),
                                hwndBar,
                                1, 300);
                        // and stop
                        break;
                    }
                }

                // now translate the template text (szText) into something
                // meaningful (this func is just above)
                cmnComposeStatusBarText(psbd->somSelf,
                        psbd->psli->hwndCnr,    // cnr hwnd in frame wnd struct
                        szText, sizeof(szText));

                // set window text (provokes WM_PAINT) and quit
                WinSetWindowText(hwndBar, szText);
            break; }

            /*
             * WM_PAINT:
             *      this, well, paints
             */

            case WM_PAINT: {
                RECTL   rclBar, rcl;
                POINTL  ptl1;
                CHAR    szText[1000], szTemp[100] = "0";
                USHORT  usLength;
                LONG    lNextX;
                PSZ     p1, p2, p3;
                // FATTRS fat;

                HPS hps = WinBeginPaint(hwndBar, NULLHANDLE, &rcl);

                /* HRGN hrgnUpdate, hrgnOld;
                HPS hps = WinGetClipPS(hwndBar,
                        NULLHANDLE,     // clip all siblings
                        PSF_CLIPSIBLINGS);
                WinQueryUpdateRect(hwndBar, &rclUpdate);
                WinValidateRect(hwndBar, &rclUpdate, FALSE);

                hrgnUpdate = GpiCreateRegion(hps, 1, &rclUpdate);
                GpiSetClipRegion(hps, hrgnUpdate, &hrgnOld); */

                WinQueryWindowRect(hwndBar, &rclBar);

                // switch to RGB mode
                GpiCreateLogColorTable(hps, 0,
                    LCOLF_RGB,
                    0, 0, NULL);

                // draw background
                WinFillRect(hps, &rclBar, pGlobalSettings->lSBBgndColor);

                // draw 3D frame in selected style
                if (pGlobalSettings->SBStyle == SBSTYLE_WARP3RAISED)
                    // Warp 3 style, raised
                    GpiDraw3DFrame(hps, &rclBar, 1, TRUE);
                else if (pGlobalSettings->SBStyle == SBSTYLE_WARP3SUNKEN)
                    // Warp 3 style, sunken
                    GpiDraw3DFrame(hps, &rclBar, 1, FALSE);
                else if (pGlobalSettings->SBStyle == SBSTYLE_WARP4MENU) {
                    // Warp 4 menu style: draw 3D line at top only
                    rcl = rclBar;
                    rcl.yBottom = rcl.yTop-2;
                    GpiDraw3DFrame(hps, &rcl, 1, FALSE);
                } else {
                    // Warp 4 button style
                    rcl = rclBar;
                    // draw "sunken" outer rect
                    GpiDraw3DFrame(hps, &rclBar, 2, FALSE);
                    // draw "raised" inner rect
                    WinInflateRect(WinQueryAnchorBlock(hwndBar),
                            &rcl, -1, -1);
                    GpiDraw3DFrame(hps, &rcl, 2, TRUE);
                }

                // now start working on text; we do "simple" GpiCharString
                // if no tabulators are defined, but work with subrectangles
                // otherwise
                WinQueryWindowText(hwndBar, sizeof(szText)-1, &szText[0]);
                // usOfs = 0;
                p1 = szText;
                p2 = NULL;
                ptl1.x = 7;
                do {
                    // check for "tabulators"
                    if (p2 = strstr(p1, "$x(")) {
                        // tab found: calculate next x position into lNextX
                        usLength = (p2-p1);
                        strcpy(szTemp, "100");
                        if (p3 = strchr(p2, ')')) {
                            PSZ p4 = strchr(p2, '%');
                            strncpy(szTemp, p2+3, p3-p2-3);
                            // _Pmpf(( "  szTemp: %s", szTemp ));
                            sscanf(szTemp, "%d", &lNextX);

                            if (lNextX < 0) {
                                // if the result is negative, it's probably
                                // meant to be an offset from the right
                                // status bar border
                                lNextX = (rclBar.xRight + lNextX); // lNextX is negative
                            }
                            else if ((p4) && (p4 < p3)) {
                                // if we have a '%' char before the closing
                                // bracket, consider lNextX a percentage
                                // parameter and now translate it into an
                                // absolute x position using the status bar's
                                // width
                                lNextX = (rclBar.xRight * lNextX) / 100;
                            }
                        } else
                            p2 = NULL;
                    } else
                        usLength = strlen(p1);

                    // now print text; the font is already set via presparams
                    ptl1.y = (pGlobalSettings->SBStyle == SBSTYLE_WARP4MENU) ? 5 : 7;
                    GpiSetColor(hps, pGlobalSettings->lSBTextColor);
                    if (p2) {
                        // did we have tabs? if so, print text clipped to rect
                        rcl.xLeft   = 0;
                        rcl.yBottom = 0; // ptl1.y;
                        rcl.xRight  = lNextX-10; // 10 pels space
                        rcl.yTop    = rclBar.yTop;
                        GpiCharStringPosAt(hps, &ptl1, &rcl, CHS_CLIP,
                                usLength, p1, NULL);
                    } else {
                        // no (more) tabs: just print
                        GpiMove(hps, &ptl1);
                        GpiCharString(hps, usLength, p1);
                    }

                    if (p2) { // "tabulator" was found: set next x pos
                        ptl1.x = lNextX;
                        p1 = p3+1;
                    }

                } while (p2);

                WinEndPaint(hps);
                // WinReleasePS(hps);
            break; }

            /*
             * STBM_PROHIBITBROADCASTING:
             *      this msg is sent to us by the subclassed
             *      desktop frame wnd proc to tell us we will
             *      be given new presentation parameters soon;
             *      we will set a flag to TRUE in order to
             *      prevent broadcasting the pres params again.
             *      This is neccessary to tell apart
             *      WM_PRESPARAMCHANGED which result from updating
             *      all status bars from those which result from
             *      dropping fonts/colors on the status bars;
             *      in the latter case, we WILL broadcast!
             *      (see WM_PRESPARAMCHANGED below)
             */

            case STBM_PROHIBITBROADCASTING: {
                psbd->fDontBroadcast = TRUE;
            break; }

            /*
             * WM_PRESPARAMCHANGED:
             *      if fonts or colors were dropped on the bar, update
             *      GlobalSettings and have this message broadcast to all
             *      other status bars on the system.
             *      This is also posted to us by the Worker thread
             *      after some OTHER status bar has been dropped
             *      fonts or colors upon; in this case, we will only update
             *      our own display and NOT broadcast, because this is
             *      already being done.
             */

            case WM_PRESPARAMCHANGED: {
                mrc = (MRESULT)(*pfnwpStatusBarOriginal)(hwndBar, msg, mp1, mp2);

                if (psbd->fDontBroadcast) {
                    // this flag has been set if it was not this status
                    // bar whose presparams have changed, but some other
                    // status bar; in this case, update only
                    psbd->fDontBroadcast = FALSE;
                    // update parent's frame controls (because font size
                    // might have changed)
                    WinSendMsg(WinQueryWindow(hwndBar, QW_PARENT), WM_UPDATEFRAME, MPNULL, MPNULL);
                    // update ourselves
                    WinPostMsg(hwndBar, STBM_UPDATESTATUSBAR, MPNULL, MPNULL);
                    // and quit
                    break;
                }

                // else: it was us that the presparam has been set for
                #ifdef DEBUG_STATUSBARS
                    _Pmpf(( "WM_PRESPARAMCHANGED: %lX", mp1 ));
                #endif

                // now check what has changed
                switch ((ULONG)mp1) {
                    case PP_FONTNAMESIZE: {
                        ULONG attrFound;
                        // CHAR  szDummy[200];
                        CHAR  szNewFont[100];
                        WinQueryPresParam(hwndBar,
                                    PP_FONTNAMESIZE,
                                    0,
                                    &attrFound,
                                    (ULONG)sizeof(szNewFont),
                                    (PVOID)&szNewFont,
                                    0);
                        cmnSetStatusBarSetting(SBS_STATUSBARFONT,
                                szNewFont);
                        // update parent's frame controls (because font size
                        // might have changed)
                        WinSendMsg(WinQueryWindow(hwndBar, QW_PARENT),
                                WM_UPDATEFRAME, MPNULL, MPNULL);
                    break; }

                    case PP_FOREGROUNDCOLOR:
                    case PP_BACKGROUNDCOLOR: {
                        ULONG ul = 0, attrFound = 0;
                        WinQueryPresParam(hwndBar,
                                    (ULONG)mp1,
                                    0,
                                    &attrFound,
                                    (ULONG)sizeof(ul),
                                    (PVOID)&ul,
                                    0);
                        if ((ULONG)mp1 == PP_FOREGROUNDCOLOR)
                            pGlobalSettings->lSBTextColor = ul;
                        else
                            pGlobalSettings->lSBBgndColor = ul;
                        PrfWriteProfileData(HINI_USERPROFILE, INIAPP, INIKEY_GLOBALSETTINGS,
                                    pGlobalSettings, sizeof(GLOBALSETTINGS));
                        WinPostMsg(hwndBar, STBM_UPDATESTATUSBAR, MPNULL, MPNULL);
                    break; }
                }

                // finally, broadcast this message to all other status bars;
                // this is handled by the Worker thread
                cmnPostWorkerMsg(WOM_UPDATEALLSTATUSBARS,
                        (MPARAM)2,      // update display
                        MPNULL);
            break; }

            /*
             * WM_DESTROY:
             *      call original wnd proc, then free psbd
             */

            case WM_DESTROY:
                mrc = (MRESULT)(*pfnwpStatusBarOriginal)(hwndBar, msg, mp1, mp2);
                WinSetWindowULong(hwndBar, QWL_USER, 0);
                free(psbd);
            break;

            /*
             * WM_CONTEXTMENU:
             *      if the user right-clicks on status bar,
             *      display folder's context menu. Parameters:
             *      mp1:
             *          POINTS mp1      pointer position
             *      mp2:
             *          USHORT usReserved  should be 0
             *          USHORT fPointer    if TRUE: results from keyboard
             */

            case WM_CONTEXTMENU: {
                if (psbd->psli) {
                    POINTL ptl;
                    // give the cnr source emphasis
                    // (needed for some wpSelectingMenu funcs)
                    WinSendMsg(psbd->psli->hwndCnr,
                                CM_SETRECORDEMPHASIS,
                                (MPARAM)NULL,   // undocumented: if precc == NULL,
                                                // the whole cnr is given emphasis
                                MPFROM2SHORT(TRUE,  // set emphasis
                                        CRA_SOURCE));

                    ptl.x = SHORT1FROMMP(mp1)-30;
                    ptl.y = SHORT2FROMMP(mp1);
                    _wpDisplayMenu(psbd->somSelf,
                                WinQueryWindow(hwndBar, QW_PARENT), // owner: folder frame
                                hwndBar,                            // parent: status bar
                                &ptl,
                                MENU_OBJECTPOPUP,
                                0);

                    // set flag so that the fnwpSubclassedFolderFrame
                    // can remove the cnr source emphasis after the
                    // menu has been terminated
                    psbd->psli->fStatusBarContextMenu = TRUE;
                }
            break; }

            default:
                mrc = (MRESULT)(*pfnwpStatusBarOriginal)(hwndBar, msg, mp1, mp2);
            break;
        } // end switch
    } // end if (psbd)
    /* else // error: call default
        mrc = (MRESULT)(*pfnwpStatusBarOriginal)(hwndBar, msg, mp1, mp2); */

    return (mrc);
}

/*
 * fnwpSubclassedFolderFrame:
 *          New window proc for subclassed folder frame window.
 *          This one handles WM_INITMENU messages for folder content
 *          submenus, and it intercepts all WM_CHAR messages, all others
 *          are passed to the original wnd proc.
 */

RECTL   rtlMenuItem;
ULONG   ulMiniIconSize;
LONG    lHiliteBackground, lBackground, lHiliteText, lText;
PSZ     pszFontName = NULL;
BOOL    fFolderContentWindowPosChanged = FALSE,
        fFolderContentButtonDown = FALSE;

#define CX_ARROW 25

MRESULT EXPENTRY fnwpSubclassedFolderFrame(HWND hwndFrame, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    // CHAR            szDebug[2000];
    // CHAR            szBuf[1000] = "";

    PSUBCLASSEDLISTITEM psli = NULL;
    PFNWP           pfnwpOriginal = NULL;

    XFolder         *somSelf;
    MRESULT         mrc = MRFALSE;
    APIRET          arc;

    REGREC2         RegRec2;
    ULONG           ulExcpt;

    ulWorkplaceFunc = 0;
    ulWorkplaceFunc2 = 100;

    // set the extended exception handler (see xfdesk.c)
    RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
    if ( arc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt) {
        // *** exception occurred:
        // get the handler off the chain
        DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
        // clean up here: free memory allocations, release mutex sems, etc.

        // check for the killed-thread case
        /* switch (ulExcpt) {
            case XCPT_PROCESS_TERMINATE:
            case XCPT_ASYNC_PROCESS_TERMINATE:
                DosExit( EXIT_THREAD, 0 );
                break;
        } */
        return (0);
    }
    else {
        // if not: find the original wnd proc in the
        // global linked list, so we can pass messages
        // on to it */
        ulWorkplaceFunc2 = 110;
        // check if hwndFrame is the same as last time
        /* if (hwndFrame == hwndLast) {
            pfnwpOriginal = pfnwpLast;
            somSelf = somSelfLast;
        } else */

        psli = cmnQueryPSLI(hwndFrame);
        if (psli) {
            pfnwpOriginal = psli->pfnwpOriginal;
            somSelf = psli->somSelf;
        }

        if (pfnwpOriginal)
        {
            ulWorkplaceFunc2 = 120;
            switch(msg) {

                /***************************
                 *                         *
                 * Status bar:             *
                 *                         *
                 **************************/

                case WM_QUERYFRAMECTLCOUNT: {
                    //  query the standard frame controls count and increment to make
                    //  PM aware of our new status bar
                    ULONG ulrc = (ULONG)((*pfnwpOriginal)(hwndFrame, msg, mp1, mp2));

                    if (psli->hwndStatusBar)
                        ulrc++;

                    mrc = (MPARAM)ulrc;
                break; }

                /*
                 * WM_FORMATFRAME:
                 *    this message is sent to a frame window to calculate the sizes
                 *    and positions of all of the frame controls and the client window:
                 *          mp1     PSWP    pswp        structure array
                 *          mp2     PRECTL  pprectl     pointer to client window rectangle
                 *          returns USHORT  ccount      count of the number of SWP arrays returned
                 */

                case WM_FORMATFRAME:
                {
                    //  query the number of standard frame controls
                    ULONG ulCount = (ULONG)((*pfnwpOriginal)(hwndFrame, msg, mp1, mp2));

                    #ifdef DEBUG_STATUSBARS
                        _Pmpf(( "WM_FORMATFRAME ulCount = %d", ulCount ));
                    #endif

                    if (psli->hwndStatusBar)
                    {
                        // access the SWP array that is passed to us
                        // and search all the controls for the container child window,
                        // which for folders always has the ID 0x8008
                        ULONG       ul;
                        PSWP        swpArr = (PSWP)mp1;
                        CNRINFO     CnrInfo;

                        for (ul = 0; ul < ulCount; ul++)
                        {
                            if ( WinQueryWindowUShort( swpArr[ul].hwnd, QWS_ID ) == 0x8008 ) // FID_CLIENT )
                            {
                                // container found: reduce size of container by
                                // status bar height
                                POINTL          ptlBorderSizes;
                                // XFolderData     *somThis = XFolderGetData(psli->somSelf);
                                ULONG ulStatusBarHeight = cmnQueryStatusBarHeight();
                                WinSendMsg(hwndFrame, WM_QUERYBORDERSIZE,
                                        (MPARAM)&ptlBorderSizes, MPNULL);

                                // first initialize the _new_ SWP for the status bar;
                                // since the SWP array for the std frame controls is
                                // zero-based and the standard frame controls occupy
                                // indices 0 thru ulCount-1 (where ulCount is the total
                                // count), we use ulCount for our static text control.
                                swpArr[ulCount].fl = SWP_MOVE | SWP_SIZE | SWP_NOADJUST | SWP_ZORDER;
                                swpArr[ulCount].x  = ptlBorderSizes.x;
                                swpArr[ulCount].y  = ptlBorderSizes.y;
                                swpArr[ulCount].cx = swpArr[ul].cx;
                                swpArr[ulCount].cy = ulStatusBarHeight;
                                swpArr[ulCount].hwndInsertBehind = HWND_BOTTOM; // HWND_TOP;
                                swpArr[ulCount].hwnd = psli->hwndStatusBar;

                                // adjust the origin and height of the container to
                                // accomodate our static text control
                                swpArr[ul].y  += swpArr[ulCount].cy;
                                swpArr[ul].cy -= swpArr[ulCount].cy;

                                // now we need to adjust the workspace origin of the cnr
                                // accordingly, or otherwise the folder icons will appear
                                // outside the visible cnr workspace and scroll bars will
                                // show up.
                                // We only do this the first time we're arriving here
                                // (which should be before the WPS is populating the folder);
                                // psli->fNeedCnrScroll has been initially set to TRUE
                                // by xfShowStatusBar
                                if (psli->fNeedCnrScroll)
                                {
                                    WinQueryCnrInfo(swpArr[ul].hwnd, CnrInfo);
                                    #ifdef DEBUG_STATUSBARS
                                        _Pmpf(( "Old CnrInfo.ptlOrigin.y: %lX", CnrInfo.ptlOrigin.y ));
                                    #endif
                                    if (    ((LONG)CnrInfo.ptlOrigin.y >= (LONG)ulStatusBarHeight)
                                       )
                                    {
                                        CnrInfo.ptlOrigin.y -= ulStatusBarHeight;
                                        #ifdef DEBUG_STATUSBARS
                                            _Pmpf(( "New CnrInfo.ptlOrigin.y: %lX", CnrInfo.ptlOrigin.y ));
                                        #endif
                                        WinSendMsg(swpArr[ul].hwnd, CM_SETCNRINFO,
                                                (MPARAM)&CnrInfo, (MPARAM)CMA_PTLORIGIN);
                                        // set flag to FALSE to prevent a second adjustment
                                        psli->fNeedCnrScroll = FALSE;
                                    }
                                } // end if (psli->fNeedCnrScroll)
                            } // end if WinQueryWindowUShort
                        } // end for (ul = 0; ul < ulCount; ul++)

                        // increment the number of frame controls
                        // to include our status bar
                        mrc = (MRESULT)(ulCount + 1);
                    } // end if (psli->hwndStatusBar)
                    else
                        mrc = (MRESULT)ulCount;
                break; }

                /*
                 * WM_CALCFRAMERECT:
                 *     this message occurs when an application uses the
                 *     WinCalcFrameRect function:
                 *          mp1     PRECTL  pRect      rectangle structure
                 *          mp2     USHORT  usFrame    frame indicator
                 *          returns BOOL    rc         rectangle-calculated indicator
                 */

                case WM_CALCFRAMERECT:
                {
                    //  calculate the rectl of the client control
                    if (psli->hwndStatusBar)
                    {
                        PRECTL prclPassed = (PRECTL)mp1;
                        BOOL brc = TRUE;
                        ULONG ulStatusBarHeight = cmnQueryStatusBarHeight();
                        if (SHORT1FROMMP(mp2))
                            //     TRUE:  Frame rectangle provided, calculate client
                            //     FALSE: Client area rectangle provided, calculate frame
                        {
                            //  TRUE: calculate the rectl of the client;
                            //  call default window procedure to subtract child frame
                            //  controls from the rectangle's height
                            LONG lClientHeight;
                            brc = (BOOL)((*pfnwpOriginal)(hwndFrame, msg, mp1, mp2));

                            //  position the static text frame extension below the client
                            lClientHeight = prclPassed->yTop - prclPassed->yBottom;
                            if ( ulStatusBarHeight  > lClientHeight  )
                            {
                                // extension is taller than client, so set client height to 0
                                prclPassed->yTop = prclPassed->yBottom;
                            }
                            else
                            {
                                //  set the origin of the client and shrink it based upon the
                                //  static text control's height
                                prclPassed->yBottom += ulStatusBarHeight;
                                prclPassed->yTop -= ulStatusBarHeight;
                            }
                        }
                        else
                        {
                            //  FALSE: calculate the rectl of the frame;
                            //  call default window procedure to subtract child frame
                            //  controls from the rectangle's height;
                            //  set the origin of the frame and increase it based upon the
                            //  static text control's height
                            prclPassed->yBottom -= ulStatusBarHeight;
                            prclPassed->yTop += ulStatusBarHeight;
                        }
                        mrc = (MRESULT)brc;
                    }
                    else
                        mrc = (*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                break; }

                /***************************
                 *                         *
                 * Owner-draw menu items:  *
                 *                         *
                 **************************/

                /*
                 * WM_INITMENU:
                 *      this message is sent to a frame whenever a menu
                 *      is about to be displayed; parameters:
                 *          SHORT mp1   menu item id
                 *          HWND  mp2   menu window handle
                 *      We need to intercept this for two reasons:
                 *      1)  for folder content menus, because these
                 *          are initially empty and only filled if the user
                 *          clicks on them.
                 *          We will query a bunch of data first, which we need
                 *          later for drawing our items, and then call
                 *          cmnFillContentSubmenu, which populates the folder
                 *          and fills the menu with the items therein
                 *      2)  for the menu system sounds.
                 */

                case WM_INITMENU: {
                    PGLOBALSETTINGS pGlobalSettings =
                            cmnQueryGlobalSettings();

                    #ifdef DEBUG_MENUS
                        _Pmpf(( "WM_INITMENU: mp1 = %lX, mp2 = %lX", mp1, mp2 ));
                    #endif

                    // play system sound
                    if (    ((USHORT)mp1 <  0x8000) // avoid system menu
                         || ((USHORT)mp1 == 0x8020) // include context menu
                       )
                        cmnPlaySystemSound(MMSOUND_XFLD_CTXTOPEN);

                    ulWorkplaceFunc2 = 200;
                    // find out whether the menu of which we are notified
                    // is a folder content menu; if so (and it is not filled
                    // yet), the first menu item is ID_XFMI_OFS_DUMMY
                    if ((ULONG)WinSendMsg((HWND)mp2, MM_ITEMIDFROMPOSITION, (MPARAM)0, MPNULL)
                        == (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_DUMMY))
                    {
                        // okay, let's go
                        ULONG abValue[24];
                        ULONG attrFound;

                        if (pGlobalSettings->FCShowIcons)
                        {
                            // show folder content icons:

                            // query bounding rectangle of "[empty]" item, according to
                            // which we will format our own items
                            WinSendMsg((HWND)mp2, MM_QUERYITEMRECT,
                                MPFROM2SHORT(WinSendMsg((HWND)mp2, MM_ITEMIDFROMPOSITION, (MPARAM)0, MPNULL),
                                        FALSE),
                                &rtlMenuItem);
                            ulMiniIconSize = WinQuerySysValue(HWND_DESKTOP, SV_CYICON) / 2;

                            // query presentation parameters (colors and font) for menu
                            ulWorkplaceFunc2 = 220;
                            lBackground = WinQueryPresColor((HWND)mp2,
                                    PP_MENUBACKGROUNDCOLOR, "Menu", "130 130 130");
                            ulWorkplaceFunc2 = 230;
                            lHiliteBackground = WinQueryPresColor((HWND)mp2,
                                    PP_MENUHILITEBGNDCOLOR, "MenuHilite", "0 0 128");
                            ulWorkplaceFunc2 = 240;
                            lText = WinQueryPresColor((HWND)mp2,
                                    PP_MENUFOREGROUNDCOLOR, "MenuText", "0 0 0");
                            ulWorkplaceFunc2 = 250;
                            lHiliteText = WinQueryPresColor((HWND)mp2,
                                    PP_MENUHILITEFGNDCOLOR, "MenuHiliteText", "255 255 255");

                            ulWorkplaceFunc2 = 260;
                            WinQueryPresParam((HWND)mp2,
                                            PP_FONTNAMESIZE,
                                            0,
                                            &attrFound,
                                            (ULONG)sizeof(abValue),
                                            (PVOID)&abValue,
                                            0);
                            pszFontName = (PSZ)attrFound;
                        }

                        ulWorkplaceFunc2 = 200;
                        cmnFillContentSubmenu((SHORT)mp1, (HWND)mp2);
                        mrc = MPNULL;
                        break;
                    } else {
                        #ifdef DEBUG_MENUS
                            _Pmpf(("Checking menu bar"));
                        #endif
                        // on Warp 4, check if the folder has a menu bar
                        if (DosIsWarp4()) {
                            PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
                            mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);

                            if (SHORT1FROMMP(mp1) == 0x8005) {
                                // get first item in menu; we need to
                                // find out which menu we're dealing with,
                                // and the WPS seems to be using the same
                                // menu IDs for all folder bar submenus,
                                // so there's no other way. We use a
                                // ULONG instead of a SHORT because some
                                // of these have a negative value.
                                ULONG ulFirstItemId = ((SHORT)WinSendMsg((HWND)mp2,
                                            MM_ITEMIDFROMPOSITION,
                                            (MPARAM)0,
                                            MPNULL) & 0xFFFF);
                                #ifdef DEBUG_MENUS
                                    _Pmpf(("  Item at pos 1 has id 0x%lX", ulFirstItemId));
                                #endif

                                switch (ulFirstItemId) {
                                    case 0x8013: // "Help index" item --> "Help" submenu
                                        #ifdef DEBUG_MENUS
                                            _Pmpf(("  Help menu found"));
                                        #endif
                                        WinInsertMenuSeparator((HWND)mp2, MIT_END,
                                                (pGlobalSettings->VarMenuOffset
                                                        + ID_XFMI_OFS_SEPARATOR));
                                        WinInsertMenuItem((HWND)mp2, MIT_END,
                                                (pGlobalSettings->VarMenuOffset
                                                        + ID_XFMI_OFS_PRODINFO),
                                                pNLSStrings->pszProductInfo,
                                                MIS_TEXT, 0);
                                    break;

                                    case 0x65: { // "Create" item --> "Edit" submenu
                                        // find position of "Deselect all" item
                                        SHORT sPos = (SHORT)WinSendMsg((HWND)mp2,
                                                    MM_ITEMPOSITIONFROMID,
                                                    MPFROM2SHORT(0x73, FALSE),
                                                    MPNULL);
                                        // insert "Select by name" after that item
                                        WinInsertMenuItem((HWND)mp2,
                                                sPos+1,
                                                (pGlobalSettings->VarMenuOffset
                                                        + ID_XFMI_OFS_SELECTSOME),
                                                pNLSStrings->pszSelectSome,
                                                MIS_TEXT, 0);
                                    break; }

                                    case 0x2CC: { // "Icon view" item --> "View" submenu
                                        // modify the "Sort" menu, as we would
                                        // do it for context menus also
                                        cmnModifySortMenu((HWND)mp2,
                                                XFolderGetData(psli->somSelf),
                                                pGlobalSettings,
                                                pNLSStrings);
                                    break; }
                                }
                            }

                            // now quit WM_INITMENU, we already have called
                            // the original window proc
                            break;
                        }
                    }

                    mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                break; }

                /*
                 * WM_MEASUREITEM:
                 *      this msg is sent only once per owner-draw item when
                 *      PM needs to know its size; the height of our items
                 *      will be the same as with non-owner-draw ones, but
                 *      we need to calculate the width according to the item
                 *      text
                 */

                case WM_MEASUREITEM: {
                    PGLOBALSETTINGS pGlobalSettings =
                            cmnQueryGlobalSettings();
                    ulWorkplaceFunc2 = 300;
                    if ( (SHORT)mp1 > (pGlobalSettings->VarMenuOffset+ID_XFMI_OFS_VARIABLE) )
                    {
                        POWNERITEM poi = (POWNERITEM)mp2;
                        // HWND hwndParent = WinQueryWindow(poi->hwnd, QW_PARENT);
                        POINTL aptlText[TXTBOX_COUNT];

                        // get the item from the linked list of variable menu items
                        // which corresponds to the menu item whose size is being queried
                        PVARMENULISTITEM pItem = (PVARMENULISTITEM)lstItemFromIndex(
                                    (PLISTITEM)pliVarItems,
                                    (poi->idItem - (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_VARIABLE)),
                                    0, 0);

                        if (pItem) {
                            ulWorkplaceFunc2 = 305;
                            // find out the space required for drawing this item with
                            // the current font and fill the owner draw structure (mp2)
                            // accordingly
                            GpiQueryTextBox(poi->hps, strlen(pItem->szTitle), pItem->szTitle,
                                    TXTBOX_COUNT, (PPOINTL)&aptlText);
                            poi->rclItem.xLeft = 0;
                            poi->rclItem.yBottom = 0;
                            poi->rclItem.xRight = aptlText[TXTBOX_TOPRIGHT].x+ulMiniIconSize-15+CX_ARROW;

                            poi->rclItem.yTop = rtlMenuItem.yTop-rtlMenuItem.yBottom;

                        }
                        ulWorkplaceFunc2 = 310;
                        mrc = MRFROMSHORT(poi->rclItem.yTop); //(MPARAM)poi->rclItem.yTop;
                    }
                    else
                        // none of our items: pass to original wnd proc
                        mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                break; }

                /*
                 * WM_DRAWITEM:
                 *      this msg is sent for each item every time it needs to be
                 *      redrawn, so this is what we'll do
                 */

                case WM_DRAWITEM: {
                    PGLOBALSETTINGS pGlobalSettings =
                            cmnQueryGlobalSettings();
                    ulWorkplaceFunc2 = 400;
                    if ( (SHORT)mp1 > (pGlobalSettings->VarMenuOffset+ID_XFMI_OFS_VARIABLE) )
                    {
                        HBITMAP hbm;
                        RECTL      rcl;
                        // ULONG      x;
                        LONG       lColor, lBmpBackground;
                        POWNERITEM poi = (POWNERITEM)mp2;
                        POINTL     ptl;

                        // get the item from the linked list of variable menu items
                        // which corresponds to the menu item being drawn
                        PVARMENULISTITEM pItem = (PVARMENULISTITEM)lstItemFromIndex(
                                   (PLISTITEM)pliVarItems,
                                   (poi->idItem - (pGlobalSettings->VarMenuOffset + ID_XFMI_OFS_VARIABLE)),
                                   0, 0);
                        HPOINTER hIcon;

                        if (pItem) {
                            ulWorkplaceFunc2 = 405;

                            // get the item's (object's) icon
                            hIcon = _wpQueryIcon(pItem->pObject);

                            // switch to RGB mode
                            GpiCreateLogColorTable(poi->hps, 0,
                                LCOLF_RGB,
                                0, 0, NULL);

                            ulWorkplaceFunc2 = 410;
                            // find out the background color, which depends
                            // on whether the item is highlighted (= selected);
                            // these colors have been initialized by WM_INITMENU
                            // above
                            if (poi->fsAttribute & MIA_HILITED)
                            {
                                lColor = lHiliteBackground;
                                lBmpBackground = lBackground;
                            }
                            else {
                                lColor = lBackground;
                                lBmpBackground = lHiliteBackground;
                            }

                            if (poi->fsAttribute != poi->fsAttributeOld)
                            {   // hilite has CHANGED for this item since last draw:
                                // for folder menu items (= submenus) only, we will
                                // now capture the funny arrow which indicates that
                                // the item has another submenu.
                                // We do this by actually stealing a rectangle from
                                // the screen and storing it in a bitmap, which we
                                // can put back on the screen later

                                ulWorkplaceFunc2 = 420;
                                if (pItem->usObjType == OC_CONTENTFOLDER) {
                                    rcl = poi->rclItem;
                                    rcl.xLeft = rcl.xRight - CX_ARROW;

                                    ulWorkplaceFunc2 = 430;
                                    // capture bitmap
                                    hbm = GpiCreateBmpFromPS(
                                               WinQueryAnchorBlock(HWND_DESKTOP),
                                               poi->hps, &rcl);
                                }
                            }

                            ulWorkplaceFunc2 = 440;
                            // draw rectangle in lColor, size of whole item
                            rcl = poi->rclItem;
                            WinFillRect(poi->hps, &rcl, lColor);

                            // print the item's text
                            ptl.x = poi->rclItem.xLeft+5+(ulMiniIconSize);
                            ptl.y = poi->rclItem.yBottom+4;
                            GpiMove(poi->hps, &ptl);
                            GpiSetColor(poi->hps,
                                (poi->fsAttribute & MIA_HILITED) ? lHiliteText : lText);
                            ulWorkplaceFunc2 = 450;
                            GpiCharString(poi->hps, strlen(pItem->szTitle), pItem->szTitle);

                            // draw the item's icon
                            ulWorkplaceFunc2 = 460;
                            WinDrawPointer(poi->hps, poi->rclItem.xLeft+2,
                                    poi->rclItem.yBottom
                                    +(
                                        (rtlMenuItem.yTop-rtlMenuItem.yBottom-ulMiniIconSize) / 2
                                    ),
                                    hIcon, DP_MINI);

                            ulWorkplaceFunc2 = 470;
                            if (poi->fsAttribute != poi->fsAttributeOld)
                                if (pItem->usObjType == OC_CONTENTFOLDER)
                                    // now, for folders, put the bitmap we captured
                                    // earlier back on the screen
                                    if (hbm) {
                                        rcl = poi->rclItem;
                                        rcl.xLeft = rcl.xRight - CX_ARROW;

                                        // declare the color which used to be the
                                        // background color TRANSPARENT! so our
                                        // beautiful background drawn above will not
                                        // be overpainted
                                        GpiSetBackColor(poi->hps, lBmpBackground);
                                        GpiSetBackMix(poi->hps, BM_SRCTRANSPARENT);

                                        ptl.x = rcl.xLeft;
                                        ptl.y = rcl.yBottom;
                                        ulWorkplaceFunc2 = 480;
                                        WinDrawBitmap(poi->hps, hbm,
                                            NULL, // draw whole bitmap
                                            &ptl,
                                            0, 0,
                                            DBM_IMAGEATTRS); // use HPS settings

                                        ulWorkplaceFunc2 = 490;
                                        // discard the bitmap, we don't need it any more
                                        GpiDeleteBitmap(hbm);
                                    }

                            // now, this is funny: we need to ALWAYS delete the
                            // MIA_HILITED flag in both the old and new attributes,
                            // or PM will invert the item again for some reason;
                            // this must be code from the stone ages (i.e. Microsoft)
                            poi->fsAttributeOld = (poi->fsAttribute &= ~MIA_HILITED);

                            mrc = (MRESULT)TRUE; // TRUE means the item is drawn
                        }
                        else
                            mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                    }
                    else
                        mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                    ulWorkplaceFunc2 = 499;
                break; }

                /*
                 * WM_MENUSELECT:
                 *      this is SENT to a menu owner by the menu control to
                 *      determine what to do right after a menu item has been
                 *      selected. If we return TRUE, the menu will be dismissed.
                 *      Parameters:
                 *          mp1 USHORT usItem - selected menu item
                 *              USHORT usPostCommand - TRUE: if we return TRUE,
                 *                  a message will be posted to the owner.
                 *          mp2 HWND - menu control wnd handle
                 *      We need this for two reasons:
                 *      1) we will play a system sound, if desired;
                 *      2) we need to swallow this for very large folder
                 *         content menus, because for some reason, PM will
                 *         select a random menu item after we have moved
                 *         these menus on the screen
                 *      3) we call the new xfSelectingMenuItem method on
                 *         the selected object. The return value of this
                 *         message is that of that method. This way, an
                 *         instance method can react to a menu item
                 *         selection (in that method, similar to
                 *         wpMenuItemSelected) and return FALSE still,
                 *         which will not dismiss the menu. ;-)
                 *         Examples of this are the new extended settings
                 *         menus.
                 */

                case WM_MENUSELECT: {
                    #ifdef DEBUG_MENUS
                        _Pmpf(( "WM_MENUSELECT: mp1 = %lX/%lX, mp2 = %lX",
                                SHORT1FROMMP(mp1),
                                SHORT2FROMMP(mp1),
                                mp2 ));
                        _Pmpf(( "  fFolderContentWindowPosChanged: %d",
                                fFolderContentWindowPosChanged));
                        _Pmpf(( "  fFolderContentButtonDown: %d",
                                fFolderContentButtonDown));
                    #endif

                    // check if we have moved a folder content menu
                    // (this flag is set by fnwpStatusBar)
                    if (    (fFolderContentWindowPosChanged)
                         // && (SHORT2FROMMP(mp1) == TRUE)
                       )
                    {
                        if (!fFolderContentButtonDown) {
                            // _Pmpf(( "  !fFolderContentButtonDown"));
                            mrc = (MPARAM)FALSE;
                        } else {
                            mrc = (MPARAM)TRUE;
                            fFolderContentButtonDown = FALSE;
                            fFolderContentWindowPosChanged = FALSE;
                        }
                    } else {

                        if (    (SHORT2FROMMP(mp1) == TRUE)
                            && (    ((USHORT)mp1 <  0x8000) // avoid system menu
                                 || ((USHORT)mp1 == 0x8020) // include context menu
                               )
                           )
                        {
                            HWND hwndCnr = WinQueryCnrFromFrame(hwndFrame);

                            // play system sound
                            cmnPlaySystemSound(MMSOUND_XFLD_CTXTSELECT);

                            if (hwndCnr) {
                                // we need to call xfSelectingMenuItem on all
                                // the objects for which the context menu was
                                // opened

                                PMINIRECORDCORE pmrcSource = (PMINIRECORDCORE)
                                        WinSendMsg(hwndCnr,
                                            CM_QUERYRECORDEMPHASIS,
                                            (MPARAM)CMA_FIRST,
                                            (MPARAM)CRA_SOURCE);
                                XFldObject *pObject = NULL;
                                ULONG      ulSelection;

                                if (pmrcSource == NULL) {
                                    // if CM_QUERYRECORDEMPHASIS returns NULL,
                                    // this means the whole container has source
                                    // emphasis --> context menu on folder whitespace
                                    pObject = somSelf;   // folder
                                    ulSelection = SEL_WHITESPACE;
                                }
                                else if (((LONG)pmrcSource) != -1) // no error?
                                {
                                    // check if first source object is equal to
                                    // first selected object
                                    PMINIRECORDCORE pmrcSelected = (PMINIRECORDCORE)
                                        WinSendMsg(hwndCnr,
                                            CM_QUERYRECORDEMPHASIS,
                                            (MPARAM)CMA_FIRST,
                                            (MPARAM)CRA_SELECTED);
                                    pObject = OBJECT_FROM_PREC(pmrcSource);

                                    if (pmrcSelected == pmrcSource) {
                                        // OK, menu on selected object:
                                        // are several objects selected?
                                        pmrcSelected = (PMINIRECORDCORE)
                                            WinSendMsg(hwndCnr,
                                                CM_QUERYRECORDEMPHASIS,
                                                (MPARAM)pmrcSelected,
                                                (MPARAM)CRA_SELECTED);
                                        if (pmrcSelected)
                                            // several objects
                                            ulSelection = SEL_MULTISEL;
                                        else
                                            // only one object
                                            ulSelection = SEL_SINGLESEL;
                                    } else
                                        // only one object, but not one of
                                        // the selected ones
                                        ulSelection = SEL_SINGLEOTHER;
                                }

                                // dereference shadows
                                if (pObject)
                                    if (_somIsA(pObject, _WPShadow))
                                        pObject = _wpQueryShadowedObject(pObject, TRUE);

                                // now call the object's method for this:
                                // XFldObject returns TRUE per default
                                if (pObject)
                                    mrc = (MRESULT)_xfSelectingMenuItem(pObject,
                                            SHORT1FROMMP(mp1), // usItem
                                            (BOOL)SHORT2FROMMP(mp1), // fPostCommand
                                            (HWND)mp2,         // hwndMenu,
                                            hwndCnr,
                                            ulSelection);    // of wnd proc
                                    if (!mrc)
                                        // menu not dismissed: set the flag
                                        // which will remove cnr source
                                        // emphasis when the menu is closed
                                        psli->fStatusBarContextMenu = TRUE;
                                else
                                    mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                            }
                        } else
                            // for suspicious menu items, call original proc
                            mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                    }
                break; }

                /*
                 * WM_MENUEND:
                 *      this message occurs when a menu control is about to
                 *      terminate. We need to remove cnr source emphasis
                 *      if the user has requested a context menu from a
                 *      status bar.
                 */

                case WM_MENUEND: {
                    // menu opened from status bar?
                    if (psli->fStatusBarContextMenu) {
                        // if so, remove cnr source emphasis
                        WinSendMsg(psli->hwndCnr,
                                    CM_SETRECORDEMPHASIS,
                                    (MPARAM)NULL,   // undocumented: if precc == NULL,
                                                    // the whole cnr is given emphasis
                                    MPFROM2SHORT(FALSE,  // remove emphasis
                                            CRA_SOURCE));
                        // reset flag for next context menu
                        psli->fStatusBarContextMenu = FALSE;
                    }
                    mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                break; }

                /***************************
                 *                         *
                 * Miscellaneae:           *
                 *                         *
                 **************************/

                /*
                 * WM_CHAR:
                 *      this is intercepted to provide folder hotkeys
                 */

                case WM_CHAR: {
                    XFolderData         *somThis = XFolderGetData(somSelf);
                    PGLOBALSETTINGS pGlobalSettings =
                                        cmnQueryGlobalSettings();

                    if  (   (_Accelerators == 1)
                        || ((_Accelerators == 2) && (pGlobalSettings->Accelerators))
                        )
                    {
                        if (cmnProcessAccel(hwndFrame, mp1, mp2)) {
                            mrc = (MRESULT)TRUE;
                            break;
                        }
                    }

                    mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                break; }

                /*
                 * WM_CONTROL:
                 *      this is intercepted to check for container
                 *      notifications we might be interested in
                 */

                case WM_CONTROL: {
                    ulWorkplaceFunc2 = 950;
                    if (SHORT1FROMMP(mp1) /* id */ == 0x8008) // container!!
                    {
                        #ifdef DEBUG_CNRCNTRL
                            CHAR szTemp2[30];
                            sprintf(szTemp2, "unknown: %d", SHORT2FROMMP(mp1));
                            _Pmpf(("Cnr cntrl msg: %s, mp2: %lX",
                                (SHORT2FROMMP(mp1) == CN_BEGINEDIT) ? "CN_BEGINEDIT"
                                    : (SHORT2FROMMP(mp1) == CN_COLLAPSETREE) ? "CN_COLLAPSETREE"
                                    : (SHORT2FROMMP(mp1) == CN_CONTEXTMENU) ? "CN_CONTEXTMENU"
                                    : (SHORT2FROMMP(mp1) == CN_DRAGAFTER) ? "CN_DRAGAFTER"
                                    : (SHORT2FROMMP(mp1) == CN_DRAGLEAVE) ? "CN_DRAGLEAVE"
                                    : (SHORT2FROMMP(mp1) == CN_DRAGOVER) ? "CN_DRAGOVER"
                                    : (SHORT2FROMMP(mp1) == CN_DROP) ? "CN_DROP"
                                    : (SHORT2FROMMP(mp1) == CN_DROPNOTIFY) ? "CN_DROPNOTIFY"
                                    : (SHORT2FROMMP(mp1) == CN_DROPHELP) ? "CN_DROPHELP"
                                    : (SHORT2FROMMP(mp1) == CN_EMPHASIS) ? "CN_EMPHASIS"
                                    : (SHORT2FROMMP(mp1) == CN_ENDEDIT) ? "CN_ENDEDIT"
                                    : (SHORT2FROMMP(mp1) == CN_ENTER) ? "CN_ENTER"
                                    : (SHORT2FROMMP(mp1) == CN_EXPANDTREE) ? "CN_EXPANDTREE"
                                    : (SHORT2FROMMP(mp1) == CN_HELP) ? "CN_HELP"
                                    : (SHORT2FROMMP(mp1) == CN_INITDRAG) ? "CN_INITDRAG"
                                    : (SHORT2FROMMP(mp1) == CN_KILLFOCUS) ? "CN_KILLFOCUS"
                                    : (SHORT2FROMMP(mp1) == CN_PICKUP) ? "CN_PICKUP"
                                    : (SHORT2FROMMP(mp1) == CN_QUERYDELTA) ? "CN_QUERYDELTA"
                                    : (SHORT2FROMMP(mp1) == CN_REALLOCPSZ) ? "CN_REALLOCPSZ"
                                    : (SHORT2FROMMP(mp1) == CN_SCROLL) ? "CN_SCROLL"
                                    : (SHORT2FROMMP(mp1) == CN_SETFOCUS) ? "CN_SETFOCUS"
                                    : szTemp2,
                                mp2));
                        #endif

                        switch (SHORT2FROMMP(mp1)) {
/*                            case CN_DRAGAFTER:
                                // dropping in name or details view: invalidate
                                // our ICONPOS list
                                cmnPostWorkerMsg(WM_INVALIDATEORDEREDCONTENT,
                                    (MPARAM)psli->somSelf, MPNULL);
                                break; */

                            /*
                             * CN_ENTER:
                             *      double-click or enter key
                             */

                            case CN_ENTER:
                                cmnPlaySystemSound(MMSOUND_XFLD_CNRDBLCLK);
                                mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                            break;

                            /*
                             * CN_EMPHASIS:
                             *      selection changed
                             */

                            case CN_EMPHASIS:
                                mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                                if (psli->hwndStatusBar) {
                                    #ifdef DEBUG_STATUSBARS
                                        _Pmpf(( "CN_EMPHASIS: posting PM_UPDATESTATUSBAR to hwnd %lX", psli->hwndStatusBar ));
                                    #endif
                                    WinPostMsg(psli->hwndStatusBar,
                                             STBM_UPDATESTATUSBAR,
                                             MPNULL,
                                             MPNULL);
                                }
                            break;

                            /*
                             * CN_EXPANDTREE:
                             *      tree view has been expanded:
                             *      do cnr auto-scroll
                             */

                            case CN_EXPANDTREE: {
                                PGLOBALSETTINGS pGlobalSettings =
                                        cmnQueryGlobalSettings();
                                mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                                if (pGlobalSettings->TreeViewAutoScroll) {
                                    cmnPostWorkerMsg(WOM_TREEVIEWAUTOSCROLL,
                                            (MPARAM)hwndFrame,
                                            mp2); // PMINIRECORDCORE
                                }
                            break; }

                            default:
                                mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                            break;
                        }
                    }
                break; }

                /*
                 * WM_CLOSE:
                 *      upon this, we need to clean up our linked list
                 *      of subclassed windows
                 */

                case WM_CLOSE: {
                    // first remove the status bar to store the correct wnd size
                    // xfShowStatusBar(hwndFrame, psli->somSelf, psli, FALSE);
                    // then do the default stuff
                    ulWorkplaceFunc2 = 990;
                    mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                    // upon closing the window, undo the subclassing, in case
                    // some other message still comes in
                    WinSubclassWindow(hwndFrame, pfnwpOriginal);
                    // and remove this window from our subclassing linked list
                    lstRemoveItem((PLISTITEM*)&psliSubclassed, NULL,
                        (PLISTITEM)psli,
                        hmtxSubclassed, 4000);
                break; }

                default:
                    ulWorkplaceFunc2 = 997;
                    mrc = (MRESULT)(*pfnwpOriginal)(hwndFrame, msg, mp1, mp2);
                    ulWorkplaceFunc2 = 998;
                break;

            } // end switch
        } // end if (pfnwpOriginal)
    } // end if (!ulExcpt)

    ulWorkplaceFunc2 = 999;
    // _Pmpf(("  Exiting fnwpSubclassedFolderFrame"));
    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
    return (mrc);
}

/*
 * fnwpFolderContentMenu:
 *      this is the subclassed wnd proc for folder content menus;
 *      we need to intercept mouse button 2 msgs to open a folder
 *      (WarpCenter behavior).
 */

MRESULT EXPENTRY fnwpFolderContentMenu(HWND hwndMenu, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    USHORT  sSelected;
    POINTL  ptlMouse;
    RECTL   rtlItem;
    // PVARMENULISTITEM pItem;

    REGREC2               RegRec2;
    ULONG                 ulExcpt;
    MRESULT               mrc;
    APIRET                rc;

    ulWorkplaceFunc = 0;
    ulWorkplaceFunc2 = 10000;

    // set the extended exception handler (see xfdesk.c)
    RegRec2.pfnHandler = (PFN)ExcptHandlerPlus;
    if ( rc = DosSetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2) )
        DosBeep(100, 1000);
    // store a known thread state
    ulExcpt = setjmp(RegRec2.jmpThread);
    if (ulExcpt) {
        // *** exception occurred:
        // get the handler off the chain
        DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
        // clean up here: free memory allocations, release mutex sems, etc.

        // check for the killed-thread case
        /* switch (ulExcpt) {
            case XCPT_PROCESS_TERMINATE:
            case XCPT_ASYNC_PROCESS_TERMINATE:
                DosExit( EXIT_THREAD, 0 );
                break;
        } */
        return (MRESULT)0; // keep compiler happy
    } else {
        // no exception:
        switch(msg) {

            case WM_ADJUSTWINDOWPOS: {
                PSWP pswp = (PSWP)mp1;
                BOOL fAdjusted = FALSE;
                #ifdef DEBUG_MENUS
                    _Pmpf(("WM_ADJUSTWINDOWPOS"));
                #endif

                if ((pswp->fl & (SWP_MOVE)) == (SWP_MOVE)) {
                    #ifdef DEBUG_MENUS
                        _Pmpf(("  SWP_MOVE set"));
                    #endif

                    if (!fFolderContentWindowPosChanged) {
                        #ifdef DEBUG_MENUS
                            _Pmpf(("    Checking bounds"));
                        #endif
                        if ((pswp->x + pswp->cx) >
                                WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN))
                        {
                            pswp->x = 0;
                            #ifdef DEBUG_MENUS
                                _Pmpf(("    Changed x pos"));
                            #endif
                            // avoid several changes for this menu;
                            // this flag is reset by WM_INITMENU in
                            // fnwpSubclassedFolderFrame
                            fFolderContentWindowPosChanged = TRUE;
                            fAdjusted = TRUE;
                        }
                        if ((pswp->y + pswp->cy) >
                                WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN))
                        {
                            pswp->y = 0;
                            #ifdef DEBUG_MENUS
                                _Pmpf(("    Changed y pos"));
                            #endif
                            // avoid several changes for this menu;
                            // this flag is reset by WM_INITMENU in
                            // fnwpSubclassedFolderFrame
                            fFolderContentWindowPosChanged = TRUE;
                            fAdjusted = TRUE;
                        }
                    }
                }
                if (fAdjusted)
                    pswp->fl |= (SWP_NOADJUST);
                mrc = (MRESULT)(*pfnwpFolderContentMenuOriginal)(hwndMenu, msg, mp1, mp2);
                fFolderContentButtonDown = FALSE;
            break; }

            #ifdef DEBUG_MENUS
                case MM_SELECTITEM: {
                    _Pmpf(( "MM_SELECTITEM: mp1 = %lX/%lX, mp2 = %lX",
                        SHORT1FROMMP(mp1),
                        SHORT2FROMMP(mp1),
                        mp2 ));
                    mrc = (MRESULT)(*pfnwpFolderContentMenuOriginal)(hwndMenu, msg, mp1, mp2);
                break; }
            #endif

            case WM_BUTTON2DOWN:
                #ifdef DEBUG_MENUS
                    _Pmpf(("WM_BUTTON2DOWN"));
                #endif

                ptlMouse.x = SHORT1FROMMP(mp1);
                ptlMouse.y = SHORT2FROMMP(mp1);
                WinSendMsg(hwndMenu, MM_SELECTITEM,
                    MPFROM2SHORT(MIT_NONE, FALSE),
                    MPFROM2SHORT(0, FALSE));
                sSelected = WinQueryItemUnderMouse(hwndMenu, &ptlMouse, &rtlItem);
                WinSendMsg(hwndMenu, MM_SETITEMATTR,
                        MPFROM2SHORT(sSelected,
                            FALSE),
                        MPFROM2SHORT(MIA_HILITED, MIA_HILITED)
                    );
            break;

            case WM_BUTTON1DOWN:
                // let this be handled by the default proc
                #ifdef DEBUG_MENUS
                    _Pmpf(("WM_BUTTON1DOWN"));
                #endif
                fFolderContentButtonDown = TRUE;
                mrc = (MRESULT)(*pfnwpFolderContentMenuOriginal)(hwndMenu, msg, mp1, mp2);
            break;

            case WM_BUTTON1DBLCLK:
            case WM_BUTTON2UP: {
                // upon receiving these, we will open the object directly; we need to
                // cheat a little bit because sending MM_SELECTITEM would open the submenu
                #ifdef DEBUG_MENUS
                    _Pmpf(("WM_BUTTON2UP"));
                #endif
                fFolderContentButtonDown = TRUE;
                ulWorkplaceFunc2 = 10100;
                ptlMouse.x = SHORT1FROMMP(mp1);
                ptlMouse.y = SHORT2FROMMP(mp1);
                WinSendMsg(hwndMenu, MM_SELECTITEM,
                    MPFROM2SHORT(MIT_NONE, FALSE),
                    MPFROM2SHORT(0, FALSE));
                sSelected = WinQueryItemUnderMouse(hwndMenu, &ptlMouse, &rtlItem);

                cmnPlaySystemSound(MMSOUND_XFLD_CTXTSELECT);

                WinPostMsg(WinQueryWindow(hwndMenu, QW_OWNER),
                    WM_COMMAND,
                    (MPARAM)sSelected,
                    MPFROM2SHORT(CMDSRC_MENU, FALSE));
            break; }

            default:
                ulWorkplaceFunc2 = 11000;
                mrc = (MRESULT)(*pfnwpFolderContentMenuOriginal)(hwndMenu, msg, mp1, mp2);
                ulWorkplaceFunc2 = 11001;
            break;
        }
    }

    ulWorkplaceFunc = 0;
    ulWorkplaceFunc2 = 0;
    DosUnsetExceptionHandler((PEXCEPTIONREGISTRATIONRECORD)&RegRec2);
    return (mrc);
}

/*
 * fnwpSelectSome:
 *      dlg proc for "Select by name" window
 */

MRESULT EXPENTRY fnwpSelectSome(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = MPNULL;

    switch (msg) {
        case WM_INITDLG:  {
            CHAR szTitle[CCHMAXPATH];
            WinSetWindowULong(hwndDlg, QWL_USER, (ULONG)mp2); // Owner frame hwnd;
            WinQueryWindowText((HWND)mp2,
                    sizeof(szTitle),
                    szTitle);
            WinSetWindowText(hwndDlg, szTitle);

            WinSetDlgItemText(hwndDlg, ID_XFDI_SOME_ENTRYFIELD, "*");
            WinSendDlgItemMsg(hwndDlg, ID_XFDI_SOME_ENTRYFIELD,
                    EM_SETSEL,
                    MPFROM2SHORT(0, 1000), // select all
                    MPNULL);

            mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
        break; }

        case WM_COMMAND: {
            switch (SHORT1FROMMP(mp1)) {

                /*
                 * ID_XFDI_SOME_SELECT / DESELECT:
                 *      these are the "select" / "deselect" buttons
                 */

                case ID_XFDI_SOME_SELECT:
                case ID_XFDI_SOME_DESELECT: {
                    CHAR szMask[CCHMAXPATH];
                    HWND hwndFrame = WinQueryWindowULong(hwndDlg, QWL_USER);
                    if (hwndFrame) {
                        HWND hwndCnr = WinQueryCnrFromFrame(hwndFrame);
                        if (hwndCnr) {
                            WinQueryDlgItemText(hwndDlg, ID_XFDI_SOME_ENTRYFIELD,
                                sizeof(szMask),
                                szMask);
                            if (strlen(szMask)) {
                                // now go through all the container items in hwndCnr
                                // and select / deselct them accordingly
                                PMINIRECORDCORE pmrc = NULL;
                                do {
                                    pmrc =
                                        (PMINIRECORDCORE)WinSendMsg(hwndCnr,
                                                CM_QUERYRECORD,
                                                (MPARAM)pmrc,
                                                MPFROM2SHORT(
                                                    (pmrc) ? CMA_NEXT : CMA_FIRST,
                                                    CMA_ITEMORDER)
                                                );
                                    if (pmrc) {
                                        CHAR szTarget[CCHMAXPATH];
                                        // CHAR szTemp[3000];
                                        DosEditName(1,
                                            pmrc->pszIcon,
                                            szMask,
                                            szTarget,
                                            sizeof(szTarget)-1);
                                        /* printf("Mask: %s, Item: %s, Result: %s\n",
                                                szMask, pmrc->pszIcon, szTarget); */
                                        if (stricmp(pmrc->pszIcon, szTarget) == 0)
                                            // if the two are equal, the cnr item matches
                                            // the search mask
                                            WinSendMsg(hwndCnr,
                                                CM_SETRECORDEMPHASIS,
                                                pmrc,
                                                MPFROM2SHORT(
                                                    // select or deselect flag
                                                    (SHORT1FROMMP(mp1) == ID_XFDI_SOME_SELECT),
                                                    CRA_SELECTED
                                                ));
                                    }
                                } while (pmrc);
                            }

                            WinSetDlgItemFocus(hwndDlg, ID_XFDI_SOME_ENTRYFIELD);
                            WinSendDlgItemMsg(hwndDlg, ID_XFDI_SOME_ENTRYFIELD,
                                    EM_SETSEL,
                                    MPFROM2SHORT(0, 1000), // select all
                                    MPNULL);
                        }
                    }
                break; }

                case ID_XFDI_SOME_SELECTALL:
                case ID_XFDI_SOME_DESELECTALL: {
                    HWND hwndFrame = WinQueryWindowULong(hwndDlg, QWL_USER);
                    if (hwndFrame) {
                        HWND hwndCnr = WinQueryCnrFromFrame(hwndFrame);
                        if (hwndCnr) {
                            PMINIRECORDCORE pmrc = NULL;
                            do {
                                pmrc =
                                    (PMINIRECORDCORE)WinSendMsg(hwndCnr,
                                            CM_QUERYRECORD,
                                            (MPARAM)pmrc,
                                            MPFROM2SHORT(
                                                (pmrc) ? CMA_NEXT : CMA_FIRST,
                                                CMA_ITEMORDER)
                                            );
                                if (pmrc) {
                                    WinSendMsg(hwndCnr,
                                        CM_SETRECORDEMPHASIS,
                                        pmrc,
                                        MPFROM2SHORT(
                                            // select or deselect flag
                                            (SHORT1FROMMP(mp1) == ID_XFDI_SOME_SELECTALL),
                                            CRA_SELECTED
                                        ));
                                }
                            } while (pmrc);

                            WinSetDlgItemFocus(hwndDlg, ID_XFDI_SOME_ENTRYFIELD);
                            WinSendDlgItemMsg(hwndDlg, ID_XFDI_SOME_ENTRYFIELD,
                                    EM_SETSEL,
                                    MPFROM2SHORT(0, 1000), // select all
                                    MPNULL);
                        }
                    }
                break; }

                default:
                    mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
            }
        break; }

        default:
            mrc = fnwpDlgGeneric(hwndDlg, msg, mp1, mp2);
    }
    return (mrc);
}

/*
 * fnwpSettingsDlg_Fldr1:
 *           dlg function for Folder settings page 1
 */

/* XFLDWINDATA:
   structure for data exchange with XFolder instance data */
typedef struct _XFLDWINDATA
{
   SOMAny           *somSelf;
   XFolderData      *somThis;
   XFolderData      Backup;    // no pointer, but real data!
} XFLDWINDATA, *PXFLDWINDATA;

MRESULT EXPENTRY fnwpSettingsDlg_Fldr1(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    // USHORT              idItem;
    MRESULT             mrc = NULL;
    // LONG                i;

    // PCONTENTMENULISTITEM pcmli;

    PXFLDWINDATA        pWinData;
    XFolderData         *somThis;

    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();

    pWinData = (PXFLDWINDATA)WinQueryWindowPtr(hwndDlg, QWL_USER);

    switch(msg)
    {
        case WM_INITDLG: {
            /* we need to initialize SOM stuff to be able to
            access instance variables later */
            pWinData = (PXFLDWINDATA)_wpAllocMem((SOMAny*)mp2, sizeof(XFLDWINDATA), NULL);
            WinSetWindowPtr(hwndDlg, QWL_USER, pWinData);

            pWinData->somSelf     = (SOMAny*)mp2;
            somThis = XFolderGetData(pWinData->somSelf);
            pWinData->somThis     = somThis;

            // now backup instance data
            pWinData->Backup = *somThis;

            if (_xfIsFavoriteFolder(pWinData->somSelf))
                WinSetDlgItemChecked(hwndDlg, ID_XSDI_FAVORITEFOLDER, TRUE);

            if (_xfQueryQuickOpen(pWinData->somSelf))
                WinSetDlgItemChecked(hwndDlg, ID_XSDI_QUICKOPEN, TRUE);

            WinPostMsg(hwndDlg, WM_SETTINGS2DLG, 0, 0);
        break; }

        case WM_SETTINGS2DLG: {
            somThis = pWinData->somThis;
            WinSendDlgItemMsg(hwndDlg, ID_XSDI_SNAPTOGRID, BM_SETCHECK,
                   (MPARAM)( (_SnapToGrid == 2) ? pGlobalSettings->SnapToGrid : _SnapToGrid ),
                   MPNULL);
            WinSendDlgItemMsg(hwndDlg, ID_XSDI_FULLPATH, BM_SETCHECK,
                   (MPARAM)( (_FullPath == 2) ? pGlobalSettings->FullPath : _FullPath ),
                   MPNULL);
            WinSendDlgItemMsg(hwndDlg, ID_XSDI_ACCELERATORS, BM_SETCHECK,
                   (MPARAM)( (_Accelerators == 2) ? pGlobalSettings->Accelerators : _Accelerators ),
                   MPNULL);
            WinSendDlgItemMsg(hwndDlg, ID_XSDI_ENABLESTATUSBAR, BM_SETCHECK,
                   (MPARAM)( ( (_StatusBar == STATUSBAR_DEFAULT)
                                        ? pGlobalSettings->StatusBar
                                        : _StatusBar )
                             // always uncheck for Desktop
                             && (pWinData->somSelf != _wpclsQueryActiveDesktop(_WPDesktop))
                           ),
                   MPNULL);
            goto enableitems2;
        }   // continue, no break

        case WM_ENABLEITEMS:
        enableitems2:
            WinEnableDlgItem(hwndDlg, ID_XSDI_ACCELERATORS, (!(pGlobalSettings->NoSubclassing)));
            // disable for Desktop
            WinEnableDlgItem(hwndDlg, ID_XSDI_ENABLESTATUSBAR,
                (pWinData->somSelf != _wpclsQueryActiveDesktop(_WPDesktop)));
        break;

        case WM_DLG2SETTINGS: {
            somThis = pWinData->somThis;
            _SnapToGrid =   WinIsDlgItemChecked(hwndDlg, ID_XSDI_SNAPTOGRID);
            _FullPath   =   WinIsDlgItemChecked(hwndDlg, ID_XSDI_FULLPATH);
            _Accelerators = WinIsDlgItemChecked(hwndDlg, ID_XSDI_ACCELERATORS);
            _xfSetStatusBarVisibility(pWinData->somSelf,
                        WinIsDlgItemChecked(hwndDlg, ID_XSDI_ENABLESTATUSBAR));
        break; }

        case WM_CONTROL:
        {
            switch (SHORT2FROMMP(mp1)) {

                case BN_CLICKED: {
                    WinSendMsg(hwndDlg, WM_DLG2SETTINGS, MPNULL, MPNULL);
                    WinSendMsg(hwndDlg, WM_ENABLEITEMS, MPNULL, MPNULL);
                    _wpSaveDeferred(pWinData->somSelf);

                    switch (SHORT1FROMMP(mp1)) {
                        case ID_XSDI_FAVORITEFOLDER:
                            _xfMakeFavoriteFolder(pWinData->somSelf,
                                WinIsDlgItemChecked(hwndDlg, ID_XSDI_FAVORITEFOLDER));
                        break;

                        case ID_XSDI_QUICKOPEN:
                            _xfSetQuickOpen(pWinData->somSelf,
                                WinIsDlgItemChecked(hwndDlg, ID_XSDI_QUICKOPEN));
                        break;

                        case ID_XSDI_FULLPATH:
                            _xfUpdateAllFrameWndTitles(pWinData->somSelf);
                    }
                break; }

            } // end switch
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; } // end of WM_CONTROL

        case WM_COMMAND: {
            somThis = pWinData->somThis;

            switch (SHORT1FROMMP(mp1)) {
                case DID_UNDO: { // exists on every page
                    *somThis = pWinData->Backup;
                break; }

                case DID_DEFAULT: { // exists on every page
                    _FullPath = 2;
                    _SnapToGrid = 2;
                    _Accelerators = 2;
                    _StatusBar = STATUSBAR_DEFAULT;
                    WinPostMsg(hwndDlg, WM_SETTINGS2DLG, MPNULL, MPNULL);
                break; }
            }

            WinPostMsg(hwndDlg, WM_SETTINGS2DLG, MPNULL, MPNULL);
            _wpSaveDeferred(pWinData->somSelf);

        break; } // end of WM_COMMAND

        case WM_HELP: {
            PSZ pszHelpLibrary = cmnQueryHelpLibrary();
            // always display help for the whole page, not for single items
            if (!_wpDisplayHelp(pWinData->somSelf, ID_XSH_SETTINGS_FLDR1,
                        pszHelpLibrary))
            {
                cmnMessageBoxMsg(HWND_DESKTOP, 104, 134, MB_OK);
            }
        break; }

        case WM_DESTROY: {
            // WinSendMsg(hwndDlg, WM_DLG2SETTINGS, MPNULL, MPNULL);
            // _wpSaveDeferred(pWinData->somSelf);
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; }

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }
    return (mrc);
}

/*
 * fnwpSettingsDlg_FldrSort:
 *      dlg proc for "Sort" page;
 *      this proc is used both for the folder settings notebook page
 *      AND the respective "Sort" page in the "System" object, so we
 *      need to keep instance data and the XFolder GLobal Settings apart
 */

MRESULT EXPENTRY fnwpSettingsDlg_FldrSort(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    PXFLDWINDATA    pWinData = (PXFLDWINDATA)WinQueryWindowPtr(hwndDlg, QWL_USER);
    PGLOBALSETTINGS pGlobalSettings = cmnQueryGlobalSettings();
    MRESULT         mrc = NULL;

    switch(msg)
    {
        case WM_INITDLG: {
            // HAB         habDesktop = WinQueryAnchorBlock(HWND_DESKTOP);
            // HMODULE     hmodResource = NLS_MODULE;
            HWND        hwndListbox;
            PNLSSTRINGS pNLSStrings = cmnQueryNLSStrings();
            if (mp2) {
                // folder instance mode:
                // we need to initialize SOM stuff to be able to
                // access instance variables later
                XFolderData         *somThis;
                pWinData = (PXFLDWINDATA)_wpAllocMem((SOMAny*)mp2, sizeof(XFLDWINDATA), NULL);
                memset(pWinData, 0, sizeof(XFLDWINDATA));
                WinSetWindowPtr(hwndDlg, QWL_USER, pWinData);

                pWinData->somSelf     = (SOMAny*)mp2;
                somThis = XFolderGetData(pWinData->somSelf);
                pWinData->somThis     = somThis;

                // now backup instance data
                pWinData->Backup = *somThis;

                WinShowWindow(WinWindowFromID(hwndDlg, ID_XSDI_REPLACESORT), FALSE);
            } else {
                // Global Settings mode:
                // set QWL_USER to NULL, so we know we need to access GLobalSettings
                WinSetWindowPtr(hwndDlg, QWL_USER, NULL);
            }

            hwndListbox = WinWindowFromID(hwndDlg, ID_XSDI_SORTLISTBOX);

            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByName);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByType);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByClass);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByRealName);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortBySize);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByWriteDate);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByAccessDate);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByCreationDate);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortByExt);
            WinInsertLboxItem(hwndListbox, LIT_END, pNLSStrings->pszSortFoldersFirst);

            WinPostMsg(hwndDlg, WM_SETTINGS2DLG, 0, 0);
        break; }

        case WM_SETTINGS2DLG: {
            if (pWinData) {
                XFolderData *somThis = pWinData->somThis;
                WinSendDlgItemMsg(hwndDlg, ID_XSDI_ALWAYSSORT, BM_SETCHECK,
                       (MPARAM)ALWAYS_SORT,
                       MPNULL);
                WinSendDlgItemMsg(hwndDlg, ID_XSDI_SORTLISTBOX,
                        LM_SELECTITEM,
                        (MPARAM)DEFAULT_SORT,
                        (MPARAM)TRUE);
            } else {
                WinSetDlgItemChecked(hwndDlg, ID_XSDI_REPLACESORT,
                       pGlobalSettings->ReplaceSort);
                WinSetDlgItemChecked(hwndDlg, ID_XSDI_ALWAYSSORT,
                       pGlobalSettings->AlwaysSort);

                WinSendDlgItemMsg(hwndDlg, ID_XSDI_SORTLISTBOX,
                        LM_SELECTITEM,
                        (MPARAM)(pGlobalSettings->DefaultSort),
                        (MPARAM)TRUE);
            }
            goto enableitems4;
        }

        case WM_ENABLEITEMS:
        enableitems4:
            if (!pWinData) {
                // global settings only
                WinEnableDlgItem(hwndDlg, ID_XSDI_SORTTEXT,
                        pGlobalSettings->ReplaceSort);
                WinEnableDlgItem(hwndDlg, ID_XSDI_SORTLISTBOX,
                        pGlobalSettings->ReplaceSort);
                WinEnableDlgItem(hwndDlg, ID_XSDI_ALWAYSSORT,
                        pGlobalSettings->ReplaceSort);
            }
        break;

        case WM_DLG2SETTINGS: {
            if (pWinData) {
                _xfSetFldrSort(pWinData->somSelf,
                            // DefaultSort:
                                (USHORT)(WinSendDlgItemMsg(hwndDlg, ID_XSDI_SORTLISTBOX,
                                        LM_QUERYSELECTION,
                                        (MPARAM)LIT_CURSOR,
                                        MPNULL)),
                            // AlwaysSort:
                                (USHORT)(WinIsDlgItemChecked(hwndDlg, ID_XSDI_ALWAYSSORT))
                            );
            } else {
                BOOL bTemp;
                pGlobalSettings->ReplaceSort =
                    WinIsDlgItemChecked(hwndDlg, ID_XSDI_REPLACESORT);
                pGlobalSettings->DefaultSort =
                        (BYTE)WinSendDlgItemMsg(hwndDlg, ID_XSDI_SORTLISTBOX,
                            LM_QUERYSELECTION,
                            (MPARAM)LIT_CURSOR,
                            MPNULL);
                bTemp = WinIsDlgItemChecked(hwndDlg, ID_XSDI_ALWAYSSORT);
                if (bTemp) {
                    USHORT usDefaultSort, usAlwaysSort;
                    _xfQueryFldrSort(_wpclsQueryActiveDesktop(_WPDesktop),
                            &usDefaultSort, &usAlwaysSort);
                    if (usAlwaysSort != 0)
                    {
                        // issue warning that this might also sort the Desktop
                        if (cmnMessageBoxMsg(hwndDlg, 116, 133, MB_YESNO) == MBID_YES)
                        /* cmnSetHelpPanel(0);
                        if (WinDlgBox(HWND_DESKTOP, HWND_DESKTOP, fnwpDlgGeneric,
                                    NLS_MODULE, ID_XFD_CONFIRMSORT, NULL)
                                == DID_OK) */
                            _xfSetFldrSort(_wpclsQueryActiveDesktop(_WPDesktop),
                                usDefaultSort, 0);
                    }
                }
                pGlobalSettings->AlwaysSort = bTemp;
            }
        break; }

        case WM_CONTROL:
        {
            /* upon receiving WM_CONTROL, we might need to save the dialog
               status in the Global Settings and into OS2.INI; this is
               neccessary in the following situations:
               -- a checkbox has been clicked upon;
               -- an entry field loses focus;
               -- a spin button changes its value. */

            switch (SHORT2FROMMP(mp1)) {

                case BN_CLICKED: { // == LN_SELECT
                    WinSendMsg(hwndDlg, WM_DLG2SETTINGS, MPNULL, MPNULL);
                    WinSendMsg(hwndDlg, WM_ENABLEITEMS, MPNULL, MPNULL);
                    if (    (SHORT1FROMMP(mp1) == ID_XSDI_SORTLISTBOX)
                         || (SHORT1FROMMP(mp1) == ID_XSDI_ALWAYSSORT)
                       )
                    {
                        if (pWinData) {
                            _wpSaveDeferred(pWinData->somSelf);
                            // update our folder only
                            _xfForEachOpenView(pWinData->somSelf, 0,
                                    &fncbUpdateFolderSorts);
                        } else {
                            cmnStoreGlobalSettings();
                            // update all open folders
                            _xfclsForEachOpenView(_XFolder, 0,
                                    &fncbUpdateFolderSorts);
                        }
                    }
                break; }

            } // end switch
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; } // end of WM_CONTROL

        case WM_COMMAND: {
            if (pWinData) {
                XFolderData *somThis = pWinData->somThis;

                switch (SHORT1FROMMP(mp1)) {
                    case DID_UNDO: { // exists on every page
                        *somThis = pWinData->Backup;
                        _xfSetFldrSort(pWinData->somSelf, _DefaultSort, _AlwaysSort);
                        WinPostMsg(hwndDlg, WM_SETTINGS2DLG, MPNULL, MPNULL);
                    break; }

                    case DID_DEFAULT: { // exists on every page
                        _xfSetFldrSort(pWinData->somSelf, SET_DEFAULT, SET_DEFAULT);
                        WinPostMsg(hwndDlg, WM_SETTINGS2DLG, MPNULL, MPNULL);
                    break; }
                }
                _wpSaveDeferred(pWinData->somSelf);
            } else {
                cmnSetDefaultSettings(SP_FLDRSORT);
                cmnStoreGlobalSettings();
            }

            WinPostMsg(hwndDlg, WM_SETTINGS2DLG, MPNULL, MPNULL);

        break; } // end of WM_COMMAND

        case WM_HELP: {
            PSZ pszHelpLibrary = cmnQueryHelpLibrary();
            WPObject *pHelpSomSelf = (pWinData)
                    ? pWinData->somSelf
                    : _wpclsQueryActiveDesktop(_WPDesktop);
            // always display help for the whole page, not for single items
            if (!_wpDisplayHelp(pHelpSomSelf, ID_XSH_SETTINGS_FLDRSORT,
                pszHelpLibrary))
            {
                cmnMessageBoxMsg(HWND_DESKTOP, 104, 134, MB_OK);
            }
        break; }

        case WM_DESTROY: {
            // WinSendMsg(hwndDlg, WM_DLG2SETTINGS, MPNULL, MPNULL);
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; }

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }
    return (mrc);
}


